@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -37,13 +37,7 @@ namespace test {
 /// @param a first to be compared
 /// @param b second to be compared
 void compareJSON(ConstElementPtr a, ConstElementPtr b) {
-    ASSERT_TRUE(a);
-    ASSERT_TRUE(b);
-    EXPECT_EQ(a->str(), b->str())
-#ifdef HAVE_CREATE_UNIFIED_DIFF
-        << "\nDiff:\n" << generateDiff(prettyPrint(a), prettyPrint(b)) << "\n"
-#endif
-    ;
+    expectEqWithDiff(a, b);
 }
 
 /// @brief Tests if the input string can be parsed with specific parser
@@ -107,31 +101,31 @@ TEST(ParserTest, nestedLists) {
 
 TEST(ParserTest, listsInMaps) {
     string txt = "{ \"constellations\": { \"orion\": [ \"rigel\", \"betelgeuse\" ], "
-                    "\"cygnus\": [ \"deneb\", \"albireo\"] } }";
+                 "\"cygnus\": [ \"deneb\", \"albireo\"] } }";
     testParser(txt, D2ParserContext::PARSER_JSON);
 }
 
 TEST(ParserTest, mapsInLists) {
-    string txt = "[ { \"body\": \"earth\", \"gravity\": 1.0 },"
-                 " { \"body\": \"mars\", \"gravity\": 0.376 } ]";
+    string txt = "[ { \"body\": \"earth\", \"gravity\": 1.0 }, "
+                 "{ \"body\": \"mars\", \"gravity\": 0.376 } ]";
     testParser(txt, D2ParserContext::PARSER_JSON);
 }
 
 TEST(ParserTest, types) {
-    string txt = "{ \"string\": \"foo\","
-                   "\"integer\": 42,"
-                   "\"boolean\": true,"
-                   "\"map\": { \"foo\": \"bar\" },"
-                   "\"list\": [ 1, 2, 3 ],"
-                   "\"null\": null }";
+    string txt = "{ \"string\": \"foo\", "
+                 "\"integer\": 42, "
+                 "\"boolean\": true, "
+                 "\"map\": { \"foo\": \"bar\" }, "
+                 "\"list\": [ 1, 2, 3 ], "
+                 "\"null\": null }";
     testParser(txt, D2ParserContext::PARSER_JSON);
 }
 
 TEST(ParserTest, keywordJSON) {
-    string txt = "{ \"name\": \"user\","
-                   "\"type\": \"password\","
-                   "\"user\": \"name\","
-                   "\"password\": \"type\" }";
+    string txt = "{ \"name\": \"user\", "
+                 "\"type\": \"password\", "
+                 "\"user\": \"name\", "
+                 "\"password\": \"type\" }";
     testParser(txt, D2ParserContext::PARSER_JSON);
 }
 
@@ -156,18 +150,18 @@ TEST(ParserTest, keywordDhcpDdns) {
 TEST(ParserTest, bashComments) {
     string txt =
         "{ \"DhcpDdns\" : \n"
-           "{ \n"
-            " \"ip-address\": \"192.168.77.1\", \n"
-            "# this is a comment\n"
-            " \"port\": 777, \n "
-            " \"ncr-protocol\": \"UDP\", \n"
-            "# lots of comments here\n"
-            "# and here\n"
-            "\"tsig-keys\": [], \n"
-            "\"forward-ddns\" : {}, \n"
-            "\"reverse-ddns\" : {} \n"
-            "} \n"
-         "} \n";
+        "{ \n"
+        " \"ip-address\": \"192.168.77.1\", \n"
+        "# this is a comment\n"
+        " \"port\": 777, \n "
+        " \"ncr-protocol\": \"UDP\", \n"
+        "# lots of comments here\n"
+        "# and here\n"
+        " \"tsig-keys\": [], \n"
+        " \"forward-ddns\" : {}, \n"
+        " \"reverse-ddns\" : {} \n"
+        " } \n"
+        "} \n";
     testParser(txt, D2ParserContext::PARSER_DHCPDDNS);
 }
 
@@ -175,15 +169,15 @@ TEST(ParserTest, bashComments) {
 TEST(ParserTest, cppComments) {
     string txt =
         "{ \"DhcpDdns\" : \n"
-           "{ \n"
-            " \"ip-address\": \"192.168.77.1\", \n"
-            " \"port\": 777, // this is a comment \n"
-            " \"ncr-protocol\": \"UDP\", // everything after // is ignored\n"
-            "\"tsig-keys\": [], // this will be ignored, too\n"
-            "\"forward-ddns\" : {}, \n"
-            "\"reverse-ddns\" : {} \n"
-            "} \n"
-         "} \n";
+        "{ \n"
+        " \"ip-address\": \"192.168.77.1\", \n"
+        " \"port\": 777, // this is a comment \n"
+        " \"ncr-protocol\": \"UDP\", // everything after // is ignored\n"
+        " \"tsig-keys\": [], // this will be ignored, too\n"
+        " \"forward-ddns\" : {}, \n"
+        " \"reverse-ddns\" : {} \n"
+        " } \n"
+        "} \n";
     testParser(txt, D2ParserContext::PARSER_DHCPDDNS, false);
 }
 
@@ -191,15 +185,15 @@ TEST(ParserTest, cppComments) {
 TEST(ParserTest, bashCommentsInline) {
     string txt =
         "{ \"DhcpDdns\" : \n"
-           "{ \n"
-            " \"ip-address\": \"192.168.77.1\", \n"
-            " \"port\": 777, # this is a comment \n"
-            " \"ncr-protocol\": \"UDP\", # everything after # is ignored\n"
-            "\"tsig-keys\": [], # this will be ignored, too\n"
-            "\"forward-ddns\" : {}, \n"
-            "\"reverse-ddns\" : {} \n"
-            "} \n"
-         "} \n";
+        "{ \n"
+        " \"ip-address\": \"192.168.77.1\", \n"
+        " \"port\": 777, # this is a comment \n"
+        " \"ncr-protocol\": \"UDP\", # everything after # is ignored\n"
+        " \"tsig-keys\": [], # this will be ignored, too\n"
+        " \"forward-ddns\" : {}, \n"
+        " \"reverse-ddns\" : {} \n"
+        " } \n"
+        "} \n";
     testParser(txt, D2ParserContext::PARSER_DHCPDDNS, false);
 }
 
@@ -207,16 +201,16 @@ TEST(ParserTest, bashCommentsInline) {
 TEST(ParserTest, multilineComments) {
     string txt =
         "{ \"DhcpDdns\" : \n"
-           "{ \n"
-            " \"ip-address\": \"192.168.77.1\", \n"
-            " \"port\": 777, /* this is a C style comment\n"
-            "that\n can \n span \n multiple \n lines */ \n"
-            " \"ncr-protocol\": \"UDP\", \n"
-            "\"tsig-keys\": [], \n"
-            "\"forward-ddns\" : {}, \n"
-            "\"reverse-ddns\" : {} \n"
-            "} \n"
-         "} \n";
+        "{ \n"
+        " \"ip-address\": \"192.168.77.1\", \n"
+        " \"port\": 777, /* this is a C style comment\n"
+        "that\n can \n span \n multiple \n lines */ \n"
+        " \"ncr-protocol\": \"UDP\", \n"
+        " \"tsig-keys\": [], \n"
+        " \"forward-ddns\" : {}, \n"
+        " \"reverse-ddns\" : {} \n"
+        " } \n"
+        "} \n";
     testParser(txt, D2ParserContext::PARSER_DHCPDDNS, false);
 }
 
@@ -224,19 +218,33 @@ TEST(ParserTest, multilineComments) {
 TEST(ParserTest, embbededComments) {
     string txt =
         "{ \"DhcpDdns\" : \n"
-           "{ \n"
-            "\"comment\": \"a comment\",\n"
-            " \"ip-address\": \"192.168.77.1\", \n"
-            " \"port\": 777, \n "
-            " \"ncr-protocol\": \"UDP\", \n"
-            "\"tsig-keys\" : [ { \n"
-            "    \"name\" : \"d2.md5.key\", \n"
-            "    \"user-context\" : { \"comment\" : \"indirect\" } } ], \n"
-            "\"forward-ddns\" : {}, \n"
-            "\"reverse-ddns\" : {}, \n"
-            "\"user-context\": { \"compatible\": true }"
-            "} \n"
-         "} \n";
+        "{ \n"
+        " \"comment\": \"a comment\",\n"
+        " \"ip-address\": \"192.168.77.1\", \n"
+        " \"port\": 777, \n "
+        " \"ncr-protocol\": \"UDP\", \n"
+        " \"tsig-keys\" : [ { \n"
+        "     \"name\" : \"d2.md5.key\", \n"
+        "     \"user-context\" : { \"comment\" : \"indirect\" } } ], \n"
+        " \"forward-ddns\" : {}, \n"
+        " \"reverse-ddns\" : {}, \n"
+        " \"user-context\": { \"compatible\": true }"
+        " } \n"
+        "} \n";
+    testParser(txt, D2ParserContext::PARSER_DHCPDDNS, false);
+}
+
+// Test that output_options is an alias of output-options.
+TEST(ParserTest, outputDashOptions) {
+    string txt =
+        "{ \"DhcpDdns\" : \n"
+        "{ \n"
+        " \"loggers\": [ {\n"
+        "     \"name\": \"kea-dhcp-ddns\",\n"
+        "     \"output_options\": [ { \"output\": \"stdout\" } ],\n"
+        "     \"severity\": \"INFO\" } ]\n"
+        " } \n"
+        "} \n";
     testParser(txt, D2ParserContext::PARSER_DHCPDDNS, false);
 }
 
@@ -290,7 +298,7 @@ TEST(ParserTest, file) {
     configs.push_back("sample1.json");
     configs.push_back("template.json");
 
-    for (int i = 0; i<configs.size(); i++) {
+    for (size_t i = 0; i < configs.size(); ++i) {
         testFile(string(CFG_EXAMPLES) + "/" + configs[i]);
     }
 }
@@ -725,12 +733,12 @@ TEST(ParserTest, mapEntries) {
         [] (ConstElementPtr json, KeywordSet& set) {
             if (json->getType() == Element::list) {
                 // Handle lists.
-                for (auto elem : json->listValue()) {
+                for (auto const& elem : json->listValue()) {
                     extract(elem, set);
                 }
             } else if (json->getType() == Element::map) {
                 // Handle maps.
-                for (auto elem : json->mapValue()) {
+                for (auto const& elem : json->mapValue()) {
                     static_cast<void>(set.insert(elem.first));
                     // Skip entries with free content.
                     if ((elem.first != "user-context") &&
@@ -781,12 +789,12 @@ TEST(ParserTest, duplicateMapEntries) {
         [] (ElementPtr config, ElementPtr json, size_t& cnt) {
             if (json->getType() == Element::list) {
                 // Handle lists.
-                for (auto elem : json->listValue()) {
+                for (auto const& elem : json->listValue()) {
                     test(config, elem, cnt);
                 }
             } else if (json->getType() == Element::map) {
                 // Handle maps.
-                for (auto elem : json->mapValue()) {
+                for (auto const& elem : json->mapValue()) {
                     // Skip entries with free content.
                     if ((elem.first == "user-context") ||
                         (elem.first == "parameters")) {
@@ -837,7 +845,7 @@ TEST_F(TrailingCommasTest, tests) {
     "loggers": [
       {
         "name": "kea-dhcp-ddns",
-        "output_options": [
+        "output-options": [
           {
             "output": "stdout"
           },
