@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2025 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -11,11 +11,14 @@
 #include <dhcp4/dhcp4_log.h>
 #include <dhcp4/parser_context.h>
 #include <dhcp4/json_config_parser.h>
-#include <cc/command_interpreter.h>
 #include <dhcpsrv/cfgmgr.h>
+#include <exceptions/exceptions.h>
 #include <log/logger_support.h>
 #include <log/logger_manager.h>
-#include <cfgrpt/config_report.h>
+#include <log/output_option.h>
+#include <process/cfgrpt/config_report.h>
+#include <process/daemon.h>
+#include <util/filesystem.h>
 
 #include <boost/lexical_cast.hpp>
 
@@ -31,6 +34,9 @@ using namespace std;
 /// instantiates ControlledDhcpv4Srv class that is responsible for establishing
 /// connection with msgq (receiving commands and configuration) and also
 /// creating Dhcpv4 server object as well.
+///
+/// For detailed explanation or relations between main(), ControlledDhcpv4Srv,
+/// Dhcpv4Srv and other classes, see \ref dhcpv4Session.
 
 namespace {
 
@@ -41,26 +47,33 @@ const char* const DHCP4_NAME = "kea-dhcp4";
 /// Note: This function never returns. It terminates the process.
 void
 usage() {
-    cerr << "Kea DHCPv4 server, version " << VERSION << endl;
+    cerr << "Kea DHCPv4 server, "
+         << "version " << VERSION
+         << " (" << PACKAGE_VERSION_TYPE << ")"
+         << endl;
     cerr << endl;
     cerr << "Usage: " << DHCP4_NAME
-         << " -[v|V|W] [-d] [-{c|t} cfgfile] [-p number] [-P number]" << endl;
+         << " -[v|V|W] [-d] [-{c|t|T} cfgfile] [-p number] [-P number]" << endl;
     cerr << "  -v: print version number and exit" << endl;
     cerr << "  -V: print extended version and exit" << endl;
     cerr << "  -W: display the configuration report and exit" << endl;
     cerr << "  -d: debug mode with extra verbosity (former -v)" << endl;
     cerr << "  -c file: specify configuration file" << endl;
     cerr << "  -t file: check the configuration file syntax and exit" << endl;
+    cerr << "  -T file: check the configuration file doing hooks load and extra "
+         << "checks and exit" << endl;
     cerr << "  -p number: specify non-standard server port number 1-65535 "
          << "(useful for testing only)" << endl;
     cerr << "  -P number: specify non-standard client port number 1-65535 "
          << "(useful for testing only)" << endl;
     exit(EXIT_FAILURE);
 }
-} // end of anonymous namespace
+}  // namespace
 
 int
 main(int argc, char* argv[]) {
+    isc::util::file::setUmask();
+
     int ch;
     // The default. Any other values are useful for testing only.
     int server_port_number = DHCP4_SERVER_PORT;
@@ -68,11 +81,12 @@ main(int argc, char* argv[]) {
     int client_port_number = 0;
     bool verbose_mode = false; // Should server be verbose?
     bool check_mode = false;   // Check syntax
+    bool load_hooks = false;   // Check hooks config
 
     // The standard config file
     std::string config_file("");
 
-    while ((ch = getopt(argc, argv, "dvVWc:p:P:t:")) != -1) {
+    while ((ch = getopt(argc, argv, "dvVWc:p:P:t:T:")) != -1) {
         switch (ch) {
         case 'd':
             verbose_mode = true;
@@ -90,15 +104,22 @@ main(int argc, char* argv[]) {
             cout << isc::detail::getConfigReport() << endl;
             return (EXIT_SUCCESS);
 
+        case 'T':
+            load_hooks = true;
+            check_mode = true;
+            config_file = optarg;
+            break;
+
         case 't':
             check_mode = true;
-            // falls through
+            config_file = optarg;
+            break;
 
         case 'c': // config file
             config_file = optarg;
             break;
 
-        case 'p':
+        case 'p': // server port number
             try {
                 server_port_number = boost::lexical_cast<int>(optarg);
             } catch (const boost::bad_lexical_cast &) {
@@ -113,7 +134,7 @@ main(int argc, char* argv[]) {
             }
             break;
 
-        case 'P':
+        case 'P': // client port number
             try {
                 client_port_number = boost::lexical_cast<int>(optarg);
             } catch (const boost::bad_lexical_cast &) {
@@ -138,7 +159,6 @@ main(int argc, char* argv[]) {
         usage();
     }
 
-
     // Configuration file is required.
     if (config_file.empty()) {
         cerr << "Configuration file not specified." << endl;
@@ -150,7 +170,6 @@ main(int argc, char* argv[]) {
 
     if (check_mode) {
         try {
-
             // We need to initialize logging, in case any error messages are to be printed.
             // This is just a test, so we don't care about lockfile.
             setenv("KEA_LOCKFILE_DIR", "none", 0);
@@ -178,9 +197,11 @@ main(int argc, char* argv[]) {
             ControlledDhcpv4Srv server(0);
             ConstElementPtr answer;
 
+            server.setProcName(DHCP4_NAME);
+
             // Now we pass the Dhcp4 configuration to the server, but
             // tell it to check the configuration only (check_only = true)
-            answer = configureDhcp4Server(server, dhcp4, true);
+            answer = configureDhcp4Server(server, dhcp4, true, load_hooks);
 
             int status_code = 0;
             answer = isc::config::parseAnswer(status_code, answer);
@@ -211,7 +232,13 @@ main(int argc, char* argv[]) {
             .arg(client_port_number)
             .arg(verbose_mode ? "yes" : "no");
 
-        LOG_INFO(dhcp4_logger, DHCP4_STARTING).arg(VERSION);
+        LOG_INFO(dhcp4_logger, DHCP4_STARTING)
+            .arg(VERSION)
+            .arg(PACKAGE_VERSION_TYPE);
+
+        if (string(PACKAGE_VERSION_TYPE) == "development") {
+            LOG_WARN(dhcp4_logger, DHCP4_DEVELOPMENT_VERSION);
+        }
 
         // Create the server instance.
         ControlledDhcpv4Srv server(server_port_number, client_port_number);
@@ -229,8 +256,15 @@ main(int argc, char* argv[]) {
             server.init(config_file);
         } catch (const std::exception& ex) {
 
+            // Let's log out what went wrong.
             try {
-                // Let's log out what went wrong.
+                // Log with the current logger, but only if it's not
+                // configured with console output so as to not log twice.
+                if (!dhcp4_logger.hasAppender(isc::log::OutputOption::DEST_CONSOLE)) {
+                    LOG_ERROR(dhcp4_logger, DHCP4_INIT_FAIL).arg(ex.what());
+                }
+
+                // Log on the console as well.
                 isc::log::LoggerManager log_manager;
                 log_manager.process();
                 LOG_ERROR(dhcp4_logger, DHCP4_INIT_FAIL).arg(ex.what());
@@ -248,7 +282,7 @@ main(int argc, char* argv[]) {
         LOG_INFO(dhcp4_logger, DHCP4_STARTED).arg(VERSION);
 
         // And run the main loop of the server.
-        server.run();
+        ret = server.run();
 
         LOG_INFO(dhcp4_logger, DHCP4_SHUTDOWN);
 
@@ -281,6 +315,10 @@ main(int argc, char* argv[]) {
             // Already logged so ignore
         }
         ret = EXIT_FAILURE;
+    } catch (...) {
+        cerr << DHCP4_NAME << ": Fatal error during start up"
+             << endl;
+        ret = EXIT_FAILURE;
     }
 
     return (ret);
