@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2024 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2025 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -55,15 +55,14 @@ Parser4Context::parseCommon() {
             isc_throw(Dhcp4ParseError, "Parser abort");
         }
         scanEnd();
-    }
-    catch (...) {
+    } catch (...) {
         scanEnd();
         throw;
     }
     if (stack_.size() == 1) {
         return (stack_[0]);
     } else {
-        isc_throw(Dhcp4ParseError, "Expected exactly one terminal Element expected, found "
+        isc_throw(Dhcp4ParseError, "Expected exactly one terminal Element, found "
                   << stack_.size());
     }
 }
