@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2024 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2025 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -112,7 +112,7 @@ namespace {
 ///   - 2 subnets: 10.0.0.0/24 guarded by first and 10.0.1.0/24
 ///   - 2 pools: 10.0.0.10-10.0.0.100 and 10.0.1.10-10.0.1.100
 ///   - 1 global reservation for HW address 'aa:bb:cc:dd:ee:ff'
-///     setting the first class
+///     with IP address outside the pool, setting the first class
 ///   - the following class defined: first
 ///
 /// - Configuration 9:
@@ -412,6 +412,7 @@ const char* CONFIGS[] = {
         "}],"
         "\"reservations\": [ {"
         "    \"hw-address\": \"aa:bb:cc:dd:ee:ff\","
+        "    \"ip-address\": \"10.0.1.200\","
         "    \"client-classes\": [ \"first\" ] } ]"
     "}",
 
