@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2025 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -10,17 +10,20 @@
 #include <asiolink/io_service.h>
 #include <cc/command_interpreter.h>
 #include <config/command_mgr.h>
+#include <config/unix_command_config.h>
 #include <config/timeouts.h>
 #include <dhcp/dhcp4.h>
 #include <dhcp/libdhcp++.h>
-#include <dhcp/tests/iface_mgr_test_config.h>
+#include <dhcp/testutils/iface_mgr_test_config.h>
 #include <dhcpsrv/cfgmgr.h>
+#include <dhcpsrv/host_mgr.h>
 #include <dhcpsrv/lease.h>
 #include <dhcpsrv/lease_mgr_factory.h>
 #include <dhcp4/ctrl_dhcp4_srv.h>
 #include <dhcp4/tests/dhcp4_test_utils.h>
 #include <hooks/hooks_manager.h>
 #include <log/logger_support.h>
+#include <process/log_parser.h>
 #include <stats/stats_mgr.h>
 #include <util/multi_threading_mgr.h>
 #include <util/chrono_time_utils.h>
@@ -29,7 +32,6 @@
 #include <testutils/sandbox.h>
 
 #include "marker_file.h"
-#include "test_libraries.h"
 
 #include <boost/scoped_ptr.hpp>
 #include <gtest/gtest.h>
@@ -53,6 +55,7 @@ using namespace isc::hooks;
 using namespace isc::stats;
 using namespace isc::test;
 using namespace isc::util;
+using namespace isc::process;
 namespace ph = std::placeholders;
 
 namespace {
@@ -113,12 +116,8 @@ public:
     ///
     /// Sets socket path to its default value.
     CtrlChannelDhcpv4SrvTest() : interfaces_("\"*\"") {
-        const char* env = getenv("KEA_SOCKET_TEST_DIR");
-        if (env) {
-            socket_path_ = string(env) + "/kea4.sock";
-        } else {
-            socket_path_ = sandbox.join("kea4.sock");
-        }
+        resetLogPath();
+        setSocketTestPath();
         reset();
         IfaceMgr::instance().setTestMode(false);
         IfaceMgr::instance().setDetectCallback(std::bind(&IfaceMgr::checkDetectIfaces,
@@ -127,6 +126,8 @@ public:
 
     /// @brief Destructor
     ~CtrlChannelDhcpv4SrvTest() {
+        resetLogPath();
+        Dhcpv4SrvTest::resetSocketPath();
         LeaseMgrFactory::destroy();
         StatsMgr::instance().removeAll();
 
@@ -135,13 +136,33 @@ public:
         CommandMgr::instance().setConnectionTimeout(TIMEOUT_DHCP_SERVER_RECEIVE_COMMAND);
 
         server_.reset();
+        reset();
         IfaceMgr::instance().setTestMode(false);
         IfaceMgr::instance().setDetectCallback(std::bind(&IfaceMgr::checkDetectIfaces,
                                                IfaceMgr::instancePtr().get(), ph::_1));
         IfaceMgr::instance().clearIfaces();
         IfaceMgr::instance().closeSockets();
         IfaceMgr::instance().detectIfaces();
-    };
+    }
+
+    /// @brief Sets the log path where log output may be written.
+    /// @param explicit_path path to use as the log path.
+    void setLogTestPath(const std::string explicit_path = "") {
+        LogConfigParser::getLogPath(true, (!explicit_path.empty() ?
+                                           explicit_path : TEST_DATA_BUILDDIR));
+    }
+
+    /// @brief Resets the log path to TEST_DATA_BUILDDIR.
+    void resetLogPath() {
+        LogConfigParser::getLogPath(true);
+    }
+
+    /// @brief Sets the path in which the socket can be created.
+    /// @param explicit_path path to use as the socket path.
+    void setSocketTestPath(const std::string explicit_path = "") {
+        Dhcpv4SrvTest::setSocketTestPath(explicit_path);
+        socket_path_ = UnixCommandConfig::getSocketPath() + "/kea4.sock";
+    }
 
     /// @brief Returns pointer to the server's IO service.
     ///
@@ -423,12 +444,12 @@ TEST_F(CtrlChannelDhcpv4SrvTest, commands) {
     int rcode = -1;
 
     // Case 1: send bogus command
-    ConstElementPtr result = ControlledDhcpv4Srv::processCommand("blah", params);
+    ConstElementPtr result = CommandMgr::instance().processCommand(createCommand("blah", params));
     ConstElementPtr comment = parseAnswer(rcode, result);
-    EXPECT_EQ(1, rcode); // expect failure (no such command as blah)
+    EXPECT_EQ(2, rcode); // expect failure (no such command as blah)
 
     // Case 2: send shutdown command without any parameters
-    result = ControlledDhcpv4Srv::processCommand("shutdown", params);
+    result = CommandMgr::instance().processCommand(createCommand("shutdown", params));
     comment = parseAnswer(rcode, result);
     EXPECT_EQ(0, rcode); // expect success
     // Exit value should default to 0.
@@ -438,7 +459,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, commands) {
     ConstElementPtr x(new isc::data::IntElement(77));
     params->set("exit-value", x);
 
-    result = ControlledDhcpv4Srv::processCommand("shutdown", params);
+    result = CommandMgr::instance().processCommand(createCommand("shutdown", params));
     comment = parseAnswer(rcode, result);
     EXPECT_EQ(0, rcode); // expect success
 
@@ -446,96 +467,6 @@ TEST_F(CtrlChannelDhcpv4SrvTest, commands) {
     EXPECT_EQ(77, server_->getExitValue());
 }
 
-// Check that the "libreload" command will reload libraries
-TEST_F(CtrlChannelDhcpv4SrvTest, libreload) {
-    createUnixChannelServer();
-
-    // Ensure no marker files to start with.
-    ASSERT_FALSE(checkMarkerFileExists(LOAD_MARKER_FILE));
-    ASSERT_FALSE(checkMarkerFileExists(UNLOAD_MARKER_FILE));
-
-    // Load two libraries
-    HookLibsCollection libraries;
-    libraries.push_back(make_pair(CALLOUT_LIBRARY_1, ConstElementPtr()));
-    HooksManager::loadLibraries(libraries);
-
-    // Check they are loaded.
-    HookLibsCollection loaded_libraries =
-        HooksManager::getLibraryInfo();
-    ASSERT_TRUE(libraries == loaded_libraries);
-
-    EXPECT_TRUE(checkMarkerFile(LOAD_MARKER_FILE, "1"));
-    EXPECT_FALSE(checkMarkerFileExists(UNLOAD_MARKER_FILE));
-
-    // Now execute the "libreload" command.  This should cause the libraries
-    // to unload and to reload.
-    std::string response;
-    sendUnixCommand("{ \"command\": \"libreload\" }", response);
-    EXPECT_EQ("{ \"result\": 0, "
-              "\"text\": \"Hooks libraries successfully reloaded "
-              "(WARNING: libreload is deprecated).\" }"
-              , response);
-
-    // Check that the libraries have unloaded and reloaded.  The libraries are
-    // unloaded in the reverse order to which they are loaded.  When they load,
-    // they should append information to the loading marker file.
-    EXPECT_TRUE(checkMarkerFile(UNLOAD_MARKER_FILE, "1"));
-    EXPECT_TRUE(checkMarkerFile(LOAD_MARKER_FILE, "11"));
-}
-
-// Check that the "libreload" command will fail to reload libraries which are
-// not compatible when multi-threading is enabled
-TEST_F(CtrlChannelDhcpv4SrvTest, libreloadFailMultiThreading) {
-    createUnixChannelServer();
-
-    // Ensure no marker files to start with.
-    ASSERT_FALSE(checkMarkerFileExists(LOAD_MARKER_FILE));
-    ASSERT_FALSE(checkMarkerFileExists(UNLOAD_MARKER_FILE));
-
-    // Disable multi-threading to temporarily trick the hook manager
-    // into loading single-threaded libraries.
-    MultiThreadingMgr::instance().setMode(false);
-
-    // Load two libraries
-    HookLibsCollection libraries;
-    libraries.push_back(make_pair(CALLOUT_LIBRARY_1, ConstElementPtr()));
-    libraries.push_back(make_pair(CALLOUT_LIBRARY_2, ConstElementPtr()));
-    HooksManager::loadLibraries(libraries);
-
-    // Check they are loaded.
-    HookLibsCollection loaded_libraries =
-        HooksManager::getLibraryInfo();
-    ASSERT_TRUE(libraries == loaded_libraries);
-
-    // ... which also included checking that the marker file created by the
-    // load functions exists and holds the correct value (of "12" - the
-    // first library appends "1" to the file, the second appends "2"). Also
-    // check that the unload marker file does not yet exist.
-    EXPECT_TRUE(checkMarkerFile(LOAD_MARKER_FILE, "12"));
-    EXPECT_FALSE(checkMarkerFileExists(UNLOAD_MARKER_FILE));
-
-    // Enable multi-threading before libreload command which should now fail
-    // as the second library is not multi-threading compatible.
-    MultiThreadingMgr::instance().setMode(true);
-
-    // Now execute the "libreload" command.  This should cause the libraries
-    // to unload and to reload.
-    std::string response;
-    sendUnixCommand("{ \"command\": \"libreload\" }", response);
-    EXPECT_EQ("{ \"result\": 1, "
-              "\"text\": \"Failed to reload hooks libraries"
-              " (WARNING: libreload is deprecated).\" }"
-              , response);
-
-    // Check that the libraries have unloaded and failed to reload.  The
-    // libraries are unloaded in the reverse order to which they are loaded.
-    // When they load, they should append information to the loading marker
-    // file.  Failing to load the second library will also unload the first
-    // library.
-    EXPECT_TRUE(checkMarkerFile(UNLOAD_MARKER_FILE, "211"));
-    EXPECT_TRUE(checkMarkerFile(LOAD_MARKER_FILE, "121"));
-}
-
 // This test checks which commands are registered by the DHCPv4 server.
 TEST_F(CtrlChannelDhcpv4SrvTest, commandsRegistration) {
 
@@ -568,7 +499,6 @@ TEST_F(CtrlChannelDhcpv4SrvTest, commandsRegistration) {
     EXPECT_TRUE(command_list.find("\"config-set\"") != string::npos);
     EXPECT_TRUE(command_list.find("\"config-write\"") != string::npos);
     EXPECT_TRUE(command_list.find("\"leases-reclaim\"") != string::npos);
-    EXPECT_TRUE(command_list.find("\"libreload\"") != string::npos);
     EXPECT_TRUE(command_list.find("\"server-tag-get\"") != string::npos);
     EXPECT_TRUE(command_list.find("\"shutdown\"") != string::npos);
     EXPECT_TRUE(command_list.find("\"statistic-get\"") != string::npos);
@@ -667,13 +597,16 @@ TEST_F(CtrlChannelDhcpv4SrvTest, controlChannelStats) {
     // preparing the schema which check if all statistics are set to zero
     std::ostringstream s;
     s << "{ \"arguments\": { ";
-    for (auto st = initial_stats.begin(); st != initial_stats.end();) {
-        s << "\"" << *st << "\": [ [ 0, \"";
-        s << isc::util::clockToText(StatsMgr::instance().getObservation(*st)->getInteger().second);
-        s << "\" ] ]";
-        if (++st != initial_stats.end()) {
+    bool first = true;
+    for (auto const& st : initial_stats) {
+        if (!first) {
             s << ", ";
+        } else {
+            first = false;
         }
+        s << "\"" << st << "\": [ [ 0, \"";
+        s << isc::util::clockToText(StatsMgr::instance().getObservation(st)->getInteger().second);
+        s << "\" ] ]";
     }
     s << " }, \"result\": 0 }";
 
@@ -734,6 +667,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, controlChannelStats) {
 
 // Check that the "config-set" command will replace current configuration
 TEST_F(CtrlChannelDhcpv4SrvTest, configSet) {
+    setLogTestPath("/dev");
     createUnixChannelServer();
 
     // Define strings to permutate the config arguments
@@ -802,8 +736,9 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configSet) {
         "       ,\"loggers\": [ { \n"
         "            \"name\": \"kea\", \n"
         "            \"severity\": \"FATAL\", \n"
-        "            \"output_options\": [{ \n"
-        "                \"output\": \"/dev/null\" \n"
+        "            \"output-options\": [{ \n"
+        "                \"output\": \"/dev/null\", \n"
+        "                \"maxsize\": 0"
         "            }] \n"
         "        }] \n";
 
@@ -970,6 +905,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configHashGet) {
 
 // Verify that the "config-test" command will do what we expect.
 TEST_F(CtrlChannelDhcpv4SrvTest, configTest) {
+    setLogTestPath("/dev");
     createUnixChannelServer();
 
     // Define strings to permutate the config arguments
@@ -1017,8 +953,9 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configTest) {
         "       ,\"loggers\": [ { \n"
         "            \"name\": \"kea\", \n"
         "            \"severity\": \"FATAL\", \n"
-        "            \"output_options\": [{ \n"
-        "                \"output\": \"/dev/null\" \n"
+        "            \"output-options\": [{ \n"
+        "                \"output\": \"/dev/null\", \n"
+        "                \"maxsize\": 0"
         "            }] \n"
         "        }] \n";
 
@@ -1263,6 +1200,27 @@ TEST_F(CtrlChannelDhcpv4SrvTest, statusGet) {
     EXPECT_EQ(3, found_queue_stats->size());
 }
 
+// Check that status is returned even if LeaseMgr and HostMgr are not created.
+TEST_F(CtrlChannelDhcpv4SrvTest, noManagers) {
+    // Send the status-get command.
+    createUnixChannelServer();
+    LeaseMgrFactory::destroy();
+    HostMgr::create();
+    string response_text;
+    sendUnixCommand(R"({ "command": "status-get" })", response_text);
+    ConstElementPtr response;
+    ASSERT_NO_THROW(response = Element::fromJSON(response_text));
+    ASSERT_TRUE(response);
+    ASSERT_EQ(Element::map, response->getType());
+    ConstElementPtr result(response->get("result"));
+    ASSERT_TRUE(result);
+    ASSERT_EQ(Element::integer, result->getType());
+    EXPECT_EQ(0, result->intValue());
+    ConstElementPtr arguments(response->get("arguments"));
+    ASSERT_TRUE(arguments);
+    ASSERT_EQ(Element::map, arguments->getType());
+}
+
 // Checks that socket status exists in status-get responses.
 TEST_F(CtrlChannelDhcpv4SrvTest, statusGetSockets) {
     // Create dummy interfaces to test socket status.
@@ -1481,7 +1439,6 @@ TEST_F(CtrlChannelDhcpv4SrvTest, listCommands) {
     checkListCommands(rsp, "config-write");
     checkListCommands(rsp, "list-commands");
     checkListCommands(rsp, "leases-reclaim");
-    checkListCommands(rsp, "libreload");
     checkListCommands(rsp, "version-get");
     checkListCommands(rsp, "server-tag-get");
     checkListCommands(rsp, "shutdown");
@@ -1518,6 +1475,9 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configWriteFilename) {
     createUnixChannelServer();
     std::string response;
 
+    // This is normally set by the command line -c parameter.
+    server_->setConfigFile("test1.json");
+
     sendUnixCommand("{ \"command\": \"config-write\", "
                     "\"arguments\": { \"filename\": \"test2.json\" } }", response);
 
@@ -1525,6 +1485,57 @@ TEST_F(CtrlChannelDhcpv4SrvTest, configWriteFilename) {
     ::remove("test2.json");
 }
 
+// Tests if config-write can be called with a valid full path as parameter.
+TEST_F(CtrlChannelDhcpv4SrvTest, configWriteFullPath) {
+    createUnixChannelServer();
+    std::string response;
+
+    // This is normally set by the command line -c parameter.
+    server_->setConfigFile("/tmp/test1.json");
+
+    sendUnixCommand("{ \"command\": \"config-write\", "
+                    "\"arguments\": { \"filename\": \"/tmp/test2.json\" } }", response);
+
+    checkConfigWrite(response, CONTROL_RESULT_SUCCESS, "/tmp/test2.json");
+    ::remove("/tmp/test2.json");
+}
+
+// Tests if config-write raises an error with invalid path as parameter.
+TEST_F(CtrlChannelDhcpv4SrvTest, configWriteBadPath) {
+    createUnixChannelServer();
+    std::string response;
+
+    // This is normally set by the command line -c parameter.
+    server_->setConfigFile("test1.json");
+
+    sendUnixCommand("{ \"command\": \"config-write\", "
+                    "\"arguments\": { \"filename\": \"/tmp/test2.json\" } }", response);
+
+    string expected = "not allowed to write config into /tmp/test2.json: ";
+    expected += "file /tmp/test2.json must be in the same directory ";
+    expected += "as the config file (test1.json)";
+    checkConfigWrite(response, CONTROL_RESULT_ERROR, expected);
+    ::remove("/tmp/test2.json");
+}
+
+// Tests if config-write raises an error with invalid full path as parameter.
+TEST_F(CtrlChannelDhcpv4SrvTest, configWriteBadFullPath) {
+    createUnixChannelServer();
+    std::string response;
+
+    // This is normally set by the command line -c parameter.
+    server_->setConfigFile("/tmp/kea1/test.json");
+
+    sendUnixCommand("{ \"command\": \"config-write\", "
+                    "\"arguments\": { \"filename\": \"/tmp/kea2/test.json\" } }", response);
+
+    string expected = "not allowed to write config into /tmp/kea2/test.json: ";
+    expected += "file /tmp/kea2/test.json must be in the same directory ";
+    expected += "as the config file (/tmp/kea1/test.json)";
+    checkConfigWrite(response, CONTROL_RESULT_ERROR, expected);
+    ::remove("/tmp/kea2/test.json");
+}
+
 // Tests if config-reload attempts to reload a file and reports that the
 // file is missing.
 TEST_F(CtrlChannelDhcpv4SrvTest, configReloadMissingFile) {
@@ -1718,6 +1729,32 @@ TEST_F(CtrlChannelDhcpv4SrvTest, dhcpDisableBadParam) {
     EXPECT_EQ("{ \"result\": 1, \"text\": \"'max-period' must be positive "
               "integer\" }", response);
 
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-disable\","
+                    "    \"arguments\": {"
+                    "        \"origin-id\": \"\""
+                    "    }"
+                    "}", response);
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    EXPECT_EQ("{ \"result\": 1, \"text\": \"'origin-id' argument must be a number\" }", response);
+
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-disable\","
+                    "    \"arguments\": {"
+                    "        \"origin-id\": \"foo\""
+                    "    }"
+                    "}", response);
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    EXPECT_EQ("{ \"result\": 1, \"text\": \"'origin-id' argument must be a number\" }", response);
+
     sendUnixCommand("{"
                     "    \"command\": \"dhcp-disable\","
                     "    \"arguments\": {"
@@ -1765,7 +1802,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, dhcpDisable) {
 
     EXPECT_FALSE(server_->network_state_->isServiceEnabled());
 
-    server_->network_state_->enableService(NetworkState::Origin::USER_COMMAND);
+    server_->network_state_->enableService(NetworkState::USER_COMMAND);
 
     EXPECT_TRUE(server_->network_state_->isServiceEnabled());
 
@@ -1785,7 +1822,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, dhcpDisable) {
 
     EXPECT_FALSE(server_->network_state_->isServiceEnabled());
 
-    server_->network_state_->enableService(NetworkState::Origin::USER_COMMAND);
+    server_->network_state_->enableService(NetworkState::USER_COMMAND);
 
     EXPECT_TRUE(server_->network_state_->isServiceEnabled());
 
@@ -1805,7 +1842,60 @@ TEST_F(CtrlChannelDhcpv4SrvTest, dhcpDisable) {
 
     EXPECT_FALSE(server_->network_state_->isServiceEnabled());
 
-    server_->network_state_->enableService(NetworkState::Origin::HA_COMMAND);
+    server_->network_state_->enableService(NetworkState::HA_REMOTE_COMMAND);
+
+    EXPECT_TRUE(server_->network_state_->isServiceEnabled());
+
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-disable\","
+                    "    \"arguments\": {"
+                    "        \"origin\": 2002"
+                    "    }"
+                    "}", response);
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    cfg = parseAnswer(status, rsp);
+    EXPECT_EQ(CONTROL_RESULT_SUCCESS, status);
+
+    EXPECT_FALSE(server_->network_state_->isServiceEnabled());
+
+    server_->network_state_->enableService(NetworkState::HA_REMOTE_COMMAND+2);
+
+    EXPECT_TRUE(server_->network_state_->isServiceEnabled());
+}
+
+// This test verifies if it is possible to disable DHCP service using
+// the origin-id.
+TEST_F(CtrlChannelDhcpv4SrvTest, dhcpDisableOriginId) {
+    createUnixChannelServer();
+    std::string response;
+
+    EXPECT_TRUE(server_->network_state_->isServiceEnabled());
+
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-disable\","
+                    "    \"arguments\": {"
+                    "        \"origin-id\": 2002,"
+                    "        \"origin\": \"user\""
+                    "    }"
+                    "}", response);
+
+    ConstElementPtr rsp;
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    int status;
+    ConstElementPtr cfg = parseAnswer(status, rsp);
+    EXPECT_EQ(CONTROL_RESULT_SUCCESS, status);
+
+    EXPECT_FALSE(server_->network_state_->isServiceEnabled());
+
+    server_->network_state_->enableService(NetworkState::HA_REMOTE_COMMAND+2);
 
     EXPECT_TRUE(server_->network_state_->isServiceEnabled());
 }
@@ -1837,7 +1927,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, dhcpDisableTemporarily) {
     EXPECT_FALSE(server_->network_state_->isServiceEnabled());
     // And the timer should be scheduled which counts the time to automatic
     // enabling of the service.
-    EXPECT_TRUE(server_->network_state_->isDelayedEnableAll());
+    EXPECT_TRUE(server_->network_state_->isDelayedEnableService());
 }
 
 // This test verifies that enable DHCP service command performs sanity check on
@@ -1845,6 +1935,33 @@ TEST_F(CtrlChannelDhcpv4SrvTest, dhcpDisableTemporarily) {
 TEST_F(CtrlChannelDhcpv4SrvTest, dhcpEnableBadParam) {
     createUnixChannelServer();
     std::string response;
+    ConstElementPtr rsp;
+
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-enable\","
+                    "    \"arguments\": {"
+                    "        \"origin-id\": \"\""
+                    "    }"
+                    "}", response);
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    EXPECT_EQ("{ \"result\": 1, \"text\": \"'origin-id' argument must be a number\" }", response);
+
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-enable\","
+                    "    \"arguments\": {"
+                    "        \"origin-id\": \"foo\""
+                    "    }"
+                    "}", response);
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    EXPECT_EQ("{ \"result\": 1, \"text\": \"'origin-id' argument must be a number\" }", response);
 
     sendUnixCommand("{"
                     "    \"command\": \"dhcp-enable\","
@@ -1852,7 +1969,6 @@ TEST_F(CtrlChannelDhcpv4SrvTest, dhcpEnableBadParam) {
                     "        \"origin\": \"\""
                     "    }"
                     "}", response);
-    ConstElementPtr rsp;
 
     // The response should be a valid JSON.
     EXPECT_NO_THROW(rsp = Element::fromJSON(response));
@@ -1894,7 +2010,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, dhcpEnable) {
 
     EXPECT_TRUE(server_->network_state_->isServiceEnabled());
 
-    server_->network_state_->disableService(NetworkState::Origin::USER_COMMAND);
+    server_->network_state_->disableService(NetworkState::USER_COMMAND);
 
     EXPECT_FALSE(server_->network_state_->isServiceEnabled());
 
@@ -1914,7 +2030,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, dhcpEnable) {
 
     EXPECT_TRUE(server_->network_state_->isServiceEnabled());
 
-    server_->network_state_->disableService(NetworkState::Origin::HA_COMMAND);
+    server_->network_state_->disableService(NetworkState::HA_REMOTE_COMMAND);
 
     EXPECT_FALSE(server_->network_state_->isServiceEnabled());
 
@@ -1933,6 +2049,81 @@ TEST_F(CtrlChannelDhcpv4SrvTest, dhcpEnable) {
     EXPECT_EQ(CONTROL_RESULT_SUCCESS, status);
 
     EXPECT_TRUE(server_->network_state_->isServiceEnabled());
+
+    server_->network_state_->disableService(NetworkState::HA_REMOTE_COMMAND+1);
+
+    EXPECT_FALSE(server_->network_state_->isServiceEnabled());
+
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-enable\","
+                    "    \"arguments\": {"
+                    "        \"origin\": 2001"
+                    "    }"
+                    "}", response);
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    cfg = parseAnswer(status, rsp);
+    EXPECT_EQ(CONTROL_RESULT_SUCCESS, status);
+
+    EXPECT_TRUE(server_->network_state_->isServiceEnabled());
+}
+
+// This test verifies if it is possible to enable DHCP service using
+// the origin-id.
+TEST_F(CtrlChannelDhcpv4SrvTest, dhcpEnableOriginId) {
+    createUnixChannelServer();
+    std::string response;
+
+    ConstElementPtr rsp;
+
+    int status;
+
+    // Disable the service using two distinct origins.
+    server_->network_state_->disableService(NetworkState::HA_REMOTE_COMMAND+1);
+    server_->network_state_->disableService(NetworkState::USER_COMMAND);
+
+    EXPECT_FALSE(server_->network_state_->isServiceEnabled());
+
+    // Enable the service for the 'origin-id' of 2001. The 'origin' should
+    // be ignored.
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-enable\","
+                    "    \"arguments\": {"
+                    "        \"origin-id\": 2001,"
+                    "        \"origin\": \"user\""
+                    "    }"
+                    "}", response);
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    ConstElementPtr cfg = parseAnswer(status, rsp);
+    EXPECT_EQ(CONTROL_RESULT_SUCCESS, status);
+
+    // The service should still be disabled.
+    EXPECT_FALSE(server_->network_state_->isServiceEnabled());
+
+    // Enable the service for the user command.
+    sendUnixCommand("{"
+                    "    \"command\": \"dhcp-enable\","
+                    "    \"arguments\": {"
+                    "        \"origin-id\": 1"
+                    "    }"
+                    "}", response);
+
+    // The response should be a valid JSON.
+    EXPECT_NO_THROW(rsp = Element::fromJSON(response));
+    ASSERT_TRUE(rsp);
+
+    cfg = parseAnswer(status, rsp);
+    EXPECT_EQ(CONTROL_RESULT_SUCCESS, status);
+
+    // The service should be enabled.
+    EXPECT_TRUE(server_->network_state_->isServiceEnabled());
 }
 
 /// Verify that concurrent connections over the control channel can be
@@ -1981,7 +2172,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, longCommand) {
 
     // This is the desired size of the command sent to the server (1MB). The
     // actual size sent will be slightly greater than that.
-    const size_t command_size = 1024 * 1000;
+    const ssize_t command_size = 1024 * 1000;
 
     while (command.tellp() < command_size) {
 
@@ -2094,7 +2285,7 @@ TEST_F(CtrlChannelDhcpv4SrvTest, longResponse) {
 
         // Remember the response size so as we know when we should stop
         // receiving.
-        const size_t long_response_size = reference_response.size();
+        const ssize_t long_response_size = reference_response.size();
 
         // Create the client and connect it to the server.
         boost::scoped_ptr<UnixControlClient> client(new UnixControlClient());
@@ -2163,8 +2354,8 @@ TEST_F(CtrlChannelDhcpv4SrvTest, connectionTimeoutPartialCommand) {
         // Let's wait up to 15s for the server's response. The response
         // should arrive sooner assuming that the timeout mechanism for
         // the server is working properly.
-        const unsigned int timeout = 15;
-        ASSERT_TRUE(client->getResponse(response, timeout));
+        const unsigned int response_timeout = 15;
+        ASSERT_TRUE(client->getResponse(response, response_timeout));
 
         // Explicitly close the client's connection.
         client->disconnectFromServer();
@@ -2213,8 +2404,8 @@ TEST_F(CtrlChannelDhcpv4SrvTest, connectionTimeoutNoData) {
         // Let's wait up to 15s for the server's response. The response
         // should arrive sooner assuming that the timeout mechanism for
         // the server is working properly.
-        const unsigned int timeout = 15;
-        ASSERT_TRUE(client->getResponse(response, timeout));
+        const unsigned int response_timeout = 15;
+        ASSERT_TRUE(client->getResponse(response, response_timeout));
 
         // Explicitly close the client's connection.
         client->disconnectFromServer();
