@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -145,8 +145,11 @@ void Dhcp6to4Ipc::handler(int /* fd */) {
             callout_handle->getArgument("response6", pkt);
         }
 
-        LOG_DEBUG(packet6_logger, DBG_DHCP6_DETAIL_DATA, DHCP6_RESPONSE_DATA)
-            .arg(static_cast<int>(pkt->getType())).arg(pkt->toText());
+        LOG_DEBUG(packet6_logger, DBG_DHCP6_DETAIL_DATA, DHCP6_DHCP4O6_RESPONSE_DATA)
+            .arg(pkt->getLabel())
+            .arg(pkt->getName())
+            .arg(static_cast<int>(pkt->getType()))
+            .arg(pkt->toText());
 
         // Forward packet to the client.
         IfaceMgr::instance().send(pkt);
@@ -155,7 +158,9 @@ void Dhcp6to4Ipc::handler(int /* fd */) {
         Dhcpv6Srv::processStatsSent(pkt);
 
     } catch (const std::exception& e) {
-        LOG_ERROR(packet6_logger, DHCP6_DHCP4O6_SEND_FAIL).arg(e.what());
+        LOG_ERROR(packet6_logger, DHCP6_DHCP4O6_SEND_FAIL)
+            .arg(pkt->getLabel())
+            .arg(e.what());
     }
 }
 
