@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2025 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,6 +7,7 @@
 #include <config.h>
 
 #include <asiolink/io_address.h>
+#include <asiolink/io_service_mgr.h>
 #include <cc/data.h>
 #include <cc/command_interpreter.h>
 #include <config/command_mgr.h>
@@ -43,13 +44,11 @@
 #include <hooks/hooks_parser.h>
 #include <log/logger_support.h>
 #include <process/config_ctl_parser.h>
-#include <util/encode/hex.h>
+#include <util/encode/encode.h>
 #include <util/multi_threading_mgr.h>
-#include <util/strutil.h>
 #include <util/triplet.h>
 
 #include <boost/algorithm/string.hpp>
-#include <boost/foreach.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/scoped_ptr.hpp>
 #include <boost/shared_ptr.hpp>
@@ -74,21 +73,6 @@ using namespace isc::util;
 
 namespace {
 
-/// @brief Checks if specified directory exists.
-///
-/// @param dir_path Path to a directory.
-/// @throw BadValue If the directory does not exist or is not a directory.
-void dirExists(const string& dir_path) {
-    struct stat statbuf;
-    if (stat(dir_path.c_str(), &statbuf) < 0) {
-        isc_throw(BadValue, "Bad directory '" << dir_path
-                  << "': " << strerror(errno));
-    }
-    if ((statbuf.st_mode & S_IFMT) != S_IFDIR) {
-        isc_throw(BadValue, "'" << dir_path << "' is not a directory");
-    }
-}
-
 /// @brief Parser for list of RSOO options
 ///
 /// This parser handles a Dhcp6/relay-supplied-options entry. It contains a
@@ -109,7 +93,7 @@ public:
     /// @param cfg server configuration (RSOO will be stored here)
     void parse(const SrvConfigPtr& cfg, const isc::data::ConstElementPtr& value) {
         try {
-            BOOST_FOREACH(ConstElementPtr source_elem, value->listValue()) {
+            for (auto const& source_elem : value->listValue()) {
                 std::string option_str = source_elem->stringValue();
                 // This option can be either code (integer) or name. Let's try code first
                 int64_t code = 0;
@@ -183,8 +167,15 @@ public:
 
         // Set the data directory for server id file.
         if (global->contains("data-directory")) {
-            CfgMgr::instance().setDataDir(getString(global, "data-directory"),
-                                          false);
+            auto dd = getString(global, "data-directory");
+            if (dd != CfgMgr::instance().getDataDir()) {
+                isc_throw(DhcpConfigError,
+                          "'data-directory' of '" << dd << "' is invalid,"
+                          << " supported path is '"
+                          << CfgMgr::instance().getDataDir() << "'");
+            }
+
+            LOG_WARN(dhcp6_logger, DHCP6_DATA_DIRECTORY_DEPRECATED);
         }
 
         // Set the probation period for decline handling.
@@ -245,10 +236,10 @@ public:
         }
 
         // Let's go through all the networks one by one
-        for (auto net = networks->begin(); net != networks->end(); ++net) {
+        for (auto const& net : *networks) {
 
             // For each network go through all the subnets in it.
-            const Subnet6SimpleCollection* subnets = (*net)->getAllSubnets();
+            const Subnet6SimpleCollection* subnets = net->getAllSubnets();
             if (!subnets) {
                 // Shared network without subnets it weird, but we decided to
                 // accept such configurations.
@@ -256,8 +247,8 @@ public:
             }
 
             // For each subnet, add it to a list of regular subnets.
-            for (auto subnet = subnets->begin(); subnet != subnets->end(); ++subnet) {
-                dest->add(*subnet);
+            for (auto const& subnet : *subnets) {
+                dest->add(subnet);
             }
         }
     }
@@ -305,75 +296,74 @@ public:
         std::set<string> names;
 
         // Let's go through all the networks one by one
-        for (auto net = networks.begin(); net != networks.end(); ++net) {
+        for (auto const& net : networks) {
             string txt;
 
             // Let's check if all subnets have either the same interface
             // or don't have the interface specified at all.
-            string iface = (*net)->getIface();
+            string iface = net->getIface();
 
-            const Subnet6SimpleCollection* subnets = (*net)->getAllSubnets();
+            const Subnet6SimpleCollection* subnets = net->getAllSubnets();
             if (subnets) {
 
                 bool rapid_commit = false;
 
-                // For each subnet, add it to a list of regular subnets.
-                for (auto subnet = subnets->begin(); subnet != subnets->end(); ++subnet) {
+                // Rapid commit must either be enabled or disabled in all subnets
+                // in the shared network.
+                if (subnets->size()) {
+                    // If this is the first subnet, remember the value.
+                    rapid_commit = (*subnets->begin())->getRapidCommit();
+                }
 
-                    // Rapid commit must either be enabled or disabled in all subnets
-                    // in the shared network.
-                    if (subnet == subnets->begin()) {
-                        // If this is the first subnet, remember the value.
-                        rapid_commit = (*subnet)->getRapidCommit();
-                    } else {
-                        // Ok, this is the second or following subnets. The value
-                        // must match what was set in the first subnet.
-                        if (rapid_commit != (*subnet)->getRapidCommit()) {
-                            isc_throw(DhcpConfigError, "All subnets in a shared network "
-                                      "must have the same rapid-commit value. Subnet "
-                                      << (*subnet)->toText()
-                                      << " has specified rapid-commit "
-                                      << ( (*subnet)->getRapidCommit() ? "true" : "false")
-                                      << ", but earlier subnet in the same shared-network"
-                                      << " or the shared-network itself used rapid-commit "
-                                      << (rapid_commit ? "true" : "false"));
-                        }
+                // For each subnet, add it to a list of regular subnets.
+                for (auto const& subnet : *subnets) {
+                    // Ok, this is the second or following subnets. The value
+                    // must match what was set in the first subnet.
+                    if (rapid_commit != subnet->getRapidCommit()) {
+                        isc_throw(DhcpConfigError, "All subnets in a shared network "
+                                  "must have the same rapid-commit value. Subnet "
+                                  << subnet->toText()
+                                  << " has specified rapid-commit "
+                                  << (subnet->getRapidCommit() ? "true" : "false")
+                                  << ", but earlier subnet in the same shared-network"
+                                  << " or the shared-network itself used rapid-commit "
+                                  << (rapid_commit ? "true" : "false"));
                     }
 
                     if (iface.empty()) {
-                        iface = (*subnet)->getIface();
+                        iface = subnet->getIface();
                         continue;
                     }
 
-                    if ((*subnet)->getIface().empty()) {
+                    if (subnet->getIface().empty()) {
                         continue;
                     }
 
-                    if ((*subnet)->getIface() != iface) {
-                        isc_throw(DhcpConfigError, "Subnet " << (*subnet)->toText()
-                                  << " has specified interface " << (*subnet)->getIface()
+                    if (subnet->getIface() != iface) {
+                        isc_throw(DhcpConfigError, "Subnet " << subnet->toText()
+                                  << " has specified interface " << subnet->getIface()
                                   << ", but earlier subnet in the same shared-network"
                                   << " or the shared-network itself used " << iface);
                     }
 
                     // Let's collect the subnets in case we later find out the
                     // subnet doesn't have a mandatory name.
-                    txt += (*subnet)->toText() + " ";
+                    txt += subnet->toText() + " ";
                 }
             }
 
             // Next, let's check name of the shared network.
-            if ((*net)->getName().empty()) {
+            if (net->getName().empty()) {
                 isc_throw(DhcpConfigError, "Shared-network with subnets "
                           << txt << " is missing mandatory 'name' parameter");
             }
 
             // Is it unique?
-            if (names.find((*net)->getName()) != names.end()) {
+            if (names.find(net->getName()) != names.end()) {
                 isc_throw(DhcpConfigError, "A shared-network with "
-                          "name " << (*net)->getName() << " defined twice.");
+                          "name " << net->getName() << " defined twice.");
             }
-            names.insert((*net)->getName());
+            names.insert(net->getName());
 
         }
     }
@@ -431,10 +421,6 @@ void configureCommandChannel() {
 /// @param config_set the configuration being processed
 isc::data::ConstElementPtr
 processDhcp6Config(isc::data::ConstElementPtr config_set) {
-    // Before starting any subnet operations, let's reset the subnet-id counter,
-    // so newly recreated configuration starts with first subnet-id equal 1.
-    Subnet::resetSubnetID();
-
     // Revert any runtime option definitions configured so far and not committed.
     LibDHCP::revertRuntimeOptionDefs();
     // Let's set empty container in case a user hasn't specified any configuration
@@ -460,16 +446,6 @@ processDhcp6Config(isc::data::ConstElementPtr config_set) {
         // default values and will insert derived values as well.
         mutable_cfg = boost::const_pointer_cast<Element>(config_set);
 
-        // Relocate dhcp-ddns parameters that have moved to global scope.
-        // Rule is that a global value overrides the dhcp-ddns value, so
-        // we need to do this before we apply global defaults.
-        // Note this is done for backward compatibility.
-        srv_config->moveDdnsParams(mutable_cfg);
-
-        // Move from reservation mode to new reservations flags.
-        // @todo add warning
-        BaseNetworkParser::moveReservationMode(mutable_cfg);
-
         // Set all default values if not specified by the user.
         SimpleParser6::setAllDefaults(mutable_cfg);
 
@@ -488,13 +464,6 @@ processDhcp6Config(isc::data::ConstElementPtr config_set) {
         // Apply global options in the staging config, e.g. ip-reservations-unique
         global_parser.parseEarly(srv_config, mutable_cfg);
 
-        // Specific check for this global parameter.
-        ConstElementPtr data_directory = mutable_cfg->get("data-directory");
-        if (data_directory) {
-            parameter_name = "data-directory";
-            dirExists(data_directory->stringValue());
-        }
-
         // We need definitions first
         ConstElementPtr option_defs = mutable_cfg->get("option-def");
         if (option_defs) {
@@ -586,7 +555,7 @@ processDhcp6Config(isc::data::ConstElementPtr config_set) {
         if (expiration_cfg) {
             parameter_name = "expired-leases-processing";
             ExpirationConfigParser parser;
-            parser.parse(expiration_cfg);
+            parser.parse(expiration_cfg, CfgMgr::instance().getStagingCfg()->getCfgExpiration());
         }
 
         // The hooks-libraries configuration must be parsed after parsing
@@ -649,7 +618,7 @@ processDhcp6Config(isc::data::ConstElementPtr config_set) {
         if (hosts_databases) {
             parameter_name = "hosts-databases";
             CfgDbAccessPtr cfg_db_access = srv_config->getCfgDbAccess();
-            for (auto it : hosts_databases->listValue()) {
+            for (auto const& it : hosts_databases->listValue()) {
                 db::DbAccessParser parser;
                 std::string access_string;
                 parser.parse(access_string, it);
@@ -689,8 +658,8 @@ processDhcp6Config(isc::data::ConstElementPtr config_set) {
             HostCollection hosts;
             HostReservationsListParser<HostReservationParser6> parser;
             parser.parse(SUBNET_ID_GLOBAL, reservations, hosts);
-            for (auto h = hosts.begin(); h != hosts.end(); ++h) {
-                srv_config->getCfgHosts()->add(*h);
+            for (auto const& h : hosts) {
+                srv_config->getCfgHosts()->add(h);
             }
         }
 
@@ -711,32 +680,15 @@ processDhcp6Config(isc::data::ConstElementPtr config_set) {
 
         ConstElementPtr compatibility = mutable_cfg->get("compatibility");
         if (compatibility) {
-            for (auto kv : compatibility->mapValue()) {
-                if (!kv.second || (kv.second->getType() != Element::boolean)) {
-                    isc_throw(DhcpConfigError,
-                              "compatibility parameter values must be "
-                              << "boolean (" << kv.first << " at "
-                              << kv.second->getPosition() << ")");
-                }
-                if (kv.first == "lenient-option-parsing") {
-                    CfgMgr::instance().getStagingCfg()->setLenientOptionParsing(
-                        kv.second->boolValue());
-                } else {
-                    isc_throw(DhcpConfigError,
-                              "unsupported compatibility parameter: "
-                              << kv.first << " (" << kv.second->getPosition()
-                              << ")");
-                }
-            }
+            CompatibilityParser parser;
+            parser.parse(compatibility, *CfgMgr::instance().getStagingCfg());
         }
 
         // Make parsers grouping.
-        ConfigPair config_pair;
         const std::map<std::string, ConstElementPtr>& values_map =
             mutable_cfg->mapValue();
 
-        BOOST_FOREACH(config_pair, values_map) {
-
+        for (auto const& config_pair : values_map) {
             parameter_name = config_pair.first;
 
             // These are converted to SimpleParser and are handled already above.
@@ -789,7 +741,6 @@ processDhcp6Config(isc::data::ConstElementPtr config_set) {
                  (config_pair.first == "decline-probation-period") ||
                  (config_pair.first == "dhcp4o6-port") ||
                  (config_pair.first == "server-tag") ||
-                 (config_pair.first == "reservation-mode") ||
                  (config_pair.first == "reservations-global") ||
                  (config_pair.first == "reservations-in-subnet") ||
                  (config_pair.first == "reservations-out-of-pool") ||
@@ -808,6 +759,7 @@ processDhcp6Config(isc::data::ConstElementPtr config_set) {
                  (config_pair.first == "ddns-qualifying-suffix") ||
                  (config_pair.first == "ddns-update-on-renew") ||
                  (config_pair.first == "ddns-use-conflict-resolution") ||
+                 (config_pair.first == "ddns-conflict-resolution-mode") ||
                  (config_pair.first == "ddns-ttl-percent") ||
                  (config_pair.first == "store-extended-info") ||
                  (config_pair.first == "statistic-default-sample-count") ||
@@ -900,9 +852,11 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
 
                     CfgDbAccessPtr cfg_db = CfgMgr::instance().getStagingCfg()->getCfgDbAccess();
                     string params = "universe=6 persist=false";
-                    if (cfg_db->getExtendedInfoTablesEnabled()) {
-                        params += " extended-info-tables=true";
-                    }
+                    // The "extended-info-tables" has no effect on -T command
+                    // line parameter so it is omitted on purpose.
+                    // Note that in this case, the current code creates managers
+                    // before hooks are loaded, so it can not be activated by
+                    // the BLQ hook.
                     cfg_db->setAppendedParameters(params);
                     cfg_db->createManagers();
                 } catch (const std::exception& ex) {
@@ -1031,6 +985,7 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
             // change causes problems when trying to roll back.
             HooksManager::prepareUnloadLibraries();
             static_cast<void>(HooksManager::unloadLibraries());
+            IOServiceMgr::instance().clearIOServices();
             const HooksConfig& libraries =
                 CfgMgr::instance().getStagingCfg()->getHooksConfig();
             bool multi_threading_enabled = true;
@@ -1086,6 +1041,17 @@ configureDhcp6Server(Dhcpv6Srv& server, isc::data::ConstElementPtr config_set,
             if (notify_libraries) {
                 return (notify_libraries);
             }
+
+            /// Let postponed hook initializations run.
+            try {
+                // Handle events registered by hooks using external IOService objects.
+                IOServiceMgr::instance().pollIOServices();
+            } catch (const std::exception& ex) {
+                std::ostringstream err;
+                err << "Error initializing hooks: "
+                    << ex.what();
+                return (isc::config::createAnswer(CONTROL_RESULT_ERROR, err.str()));
+            }
         }
 
         return (answer);
