@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -14,3 +14,11 @@ static const int LIBRARY_NUMBER = 2;
 #include <config.h>
 
 #include <dhcp6/tests/callout_library_common.h>
+
+extern "C" {
+
+int (*do_load)(isc::hooks::LibraryHandle& handle);
+
+int (*do_unload)();
+
+}
