@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2025 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -10,12 +10,12 @@
 #include <dhcp/option_int.h>
 #include <dhcp/option_int_array.h>
 #include <dhcp/pkt6.h>
-#include <dhcp/tests/iface_mgr_test_config.h>
+#include <dhcp/testutils/iface_mgr_test_config.h>
 #include <dhcp/opaque_data_tuple.h>
 #include <dhcp/option_string.h>
 #include <dhcp/option_vendor_class.h>
 #include <dhcp/option6_addrlst.h>
-#include <dhcp/tests/pkt_captures.h>
+#include <dhcp/testutils/pkt_captures.h>
 #include <dhcpsrv/cfgmgr.h>
 #include <dhcp6/tests/dhcp6_test_utils.h>
 #include <dhcp6/tests/dhcp6_client.h>
@@ -98,7 +98,8 @@ namespace {
 ///   - Used for the early global reservations lookup / select subnet.
 ///   - 2 subnets: 2001:db8:1::/48 (guarded) and 2001:db8:2::/48
 ///   - 2 pools: 2001:db8:1:1::/64 and 2001:db8:2:1::/64
-///   - 1 global reservation setting the first class
+///   - 1 global reservation with IP address outside the pool, setting
+///     the first class
 ///   - the following class defined: first
 ///
 /// - Configuration 7:
@@ -416,6 +417,7 @@ const char* CONFIGS[] = {
     "\"reservations\": ["
     "{"
     "    \"duid\": \"01:02:03:04\","
+    "    \"ip-addresses\": [ \"2001:db8:1:cafe::1\" ],"
     "    \"client-classes\": [ \"first\" ]"
     "}"
     "],"
@@ -657,19 +659,25 @@ TEST_F(ClassifyTest, matchClassification) {
     AllocEngine::ClientContext6 ctx1;
     bool drop = !srv.earlyGHRLookup(query1, ctx1);
     ASSERT_FALSE(drop);
-    srv.initContext(query1, ctx1, drop);
+    ctx1.subnet_ = srv_.selectSubnet(query1, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx1, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response1 = srv.processSolicit(ctx1);
     AllocEngine::ClientContext6 ctx2;
     drop = !srv.earlyGHRLookup(query2, ctx2);
     ASSERT_FALSE(drop);
-    srv.initContext(query2, ctx2, drop);
+    ctx2.subnet_ = srv_.selectSubnet(query2, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx2, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response2 = srv.processSolicit(ctx2);
     AllocEngine::ClientContext6 ctx3;
     drop = !srv.earlyGHRLookup(query3, ctx3);
     ASSERT_FALSE(drop);
-    srv.initContext(query3, ctx3, drop);
+    ctx3.subnet_ = srv_.selectSubnet(query3, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx3, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response3 = srv.processSolicit(ctx3);
 
@@ -769,19 +777,25 @@ TEST_F(ClassifyTest, required) {
     AllocEngine::ClientContext6 ctx1;
     bool drop = !srv.earlyGHRLookup(query1, ctx1);
     ASSERT_FALSE(drop);
-    srv.initContext(query1, ctx1, drop);
+    ctx1.subnet_ = srv_.selectSubnet(query1, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx1, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response1 = srv.processSolicit(ctx1);
     AllocEngine::ClientContext6 ctx2;
     drop = !srv.earlyGHRLookup(query2, ctx2);
     ASSERT_FALSE(drop);
-    srv.initContext(query2, ctx2, drop);
+    ctx2.subnet_ = srv_.selectSubnet(query2, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx2, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response2 = srv.processSolicit(ctx2);
     AllocEngine::ClientContext6 ctx3;
     drop = !srv.earlyGHRLookup(query3, ctx3);
     ASSERT_FALSE(drop);
-    srv.initContext(query3, ctx3, drop);
+    ctx3.subnet_ = srv_.selectSubnet(query3, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx3, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response3 = srv.processSolicit(ctx3);
 
@@ -878,19 +892,25 @@ TEST_F(ClassifyTest, requiredClassification) {
     AllocEngine::ClientContext6 ctx1;
     bool drop = !srv.earlyGHRLookup(query1, ctx1);
     ASSERT_FALSE(drop);
-    srv.initContext(query1, ctx1, drop);
+    ctx1.subnet_ = srv_.selectSubnet(query1, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx1, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response1 = srv.processSolicit(ctx1);
     AllocEngine::ClientContext6 ctx2;
     drop = !srv.earlyGHRLookup(query2, ctx2);
     ASSERT_FALSE(drop);
-    srv.initContext(query2, ctx2, drop);
+    ctx2.subnet_ = srv_.selectSubnet(query2, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx2, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response2 = srv.processSolicit(ctx2);
     AllocEngine::ClientContext6 ctx3;
     drop = !srv.earlyGHRLookup(query3, ctx3);
     ASSERT_FALSE(drop);
-    srv.initContext(query3, ctx3, drop);
+    ctx3.subnet_ = srv_.selectSubnet(query3, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx3, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response3 = srv.processSolicit(ctx3);
 
@@ -970,7 +990,9 @@ TEST_F(ClassifyTest, subnetClassPriority) {
     AllocEngine::ClientContext6 ctx;
     bool drop = !srv.earlyGHRLookup(query, ctx);
     ASSERT_FALSE(drop);
-    srv.initContext(query, ctx,  drop);
+    ctx.subnet_ = srv_.selectSubnet(query, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response = srv.processSolicit(ctx);
 
@@ -1037,7 +1059,9 @@ TEST_F(ClassifyTest, subnetGlobalPriority) {
     AllocEngine::ClientContext6 ctx;
     bool drop = !srv.earlyGHRLookup(query, ctx);
     ASSERT_FALSE(drop);
-    srv.initContext(query, ctx,  drop);
+    ctx.subnet_ = srv_.selectSubnet(query, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response = srv.processSolicit(ctx);
 
@@ -1113,7 +1137,9 @@ TEST_F(ClassifyTest, classGlobalPriority) {
     AllocEngine::ClientContext6 ctx;
     bool drop = !srv.earlyGHRLookup(query, ctx);
     ASSERT_FALSE(drop);
-    srv.initContext(query, ctx,  drop);
+    ctx.subnet_ = srv_.selectSubnet(query, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response = srv.processSolicit(ctx);
 
@@ -1184,7 +1210,9 @@ TEST_F(ClassifyTest, classGlobalPersistency) {
     AllocEngine::ClientContext6 ctx;
     bool drop = !srv.earlyGHRLookup(query, ctx);
     ASSERT_FALSE(drop);
-    srv.initContext(query, ctx,  drop);
+    ctx.subnet_ = srv_.selectSubnet(query, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response = srv.processSolicit(ctx);
 
@@ -1254,7 +1282,9 @@ TEST_F(ClassifyTest, classNeverSend) {
     AllocEngine::ClientContext6 ctx;
     bool drop = !srv.earlyGHRLookup(query, ctx);
     ASSERT_FALSE(drop);
-    srv.initContext(query, ctx,  drop);
+    ctx.subnet_ = srv_.selectSubnet(query, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response = srv.processSolicit(ctx);
 
@@ -1374,7 +1404,9 @@ TEST_F(ClassifyTest, clientClassifyPool) {
     AllocEngine::ClientContext6 ctx1;
     bool drop = !srv.earlyGHRLookup(query1, ctx1);
     ASSERT_FALSE(drop);
-    srv.initContext(query1, ctx1,  drop);
+    ctx1.subnet_ = srv_.selectSubnet(query1, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx1, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response1 = srv.processSolicit(ctx1);
     ASSERT_TRUE(response1);
@@ -1390,7 +1422,9 @@ TEST_F(ClassifyTest, clientClassifyPool) {
     AllocEngine::ClientContext6 ctx2;
     drop = !srv.earlyGHRLookup(query2, ctx2);
     ASSERT_FALSE(drop);
-    srv.initContext(query2, ctx2,  drop);
+    ctx2.subnet_ = srv_.selectSubnet(query2, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx2, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response2 = srv.processSolicit(ctx2);
     ASSERT_TRUE(response2);
@@ -1406,7 +1440,9 @@ TEST_F(ClassifyTest, clientClassifyPool) {
     AllocEngine::ClientContext6 ctx3;
     drop = !srv.earlyGHRLookup(query3, ctx3);
     ASSERT_FALSE(drop);
-    srv.initContext(query3, ctx3,  drop);
+    ctx3.subnet_ = srv_.selectSubnet(query3, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx3, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response3 = srv.processSolicit(ctx3);
     ASSERT_TRUE(response3);
@@ -1464,7 +1500,9 @@ TEST_F(ClassifyTest, clientClassifyPoolKnown) {
     AllocEngine::ClientContext6 ctx1;
     bool drop = !srv.earlyGHRLookup(query1, ctx1);
     ASSERT_FALSE(drop);
-    srv.initContext(query1, ctx1, drop);
+    ctx1.subnet_ = srv_.selectSubnet(query1, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx1, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response1 = srv.processSolicit(ctx1);
     ASSERT_TRUE(response1);
@@ -1494,7 +1532,9 @@ TEST_F(ClassifyTest, clientClassifyPoolKnown) {
     AllocEngine::ClientContext6 ctx2;
     drop = !srv.earlyGHRLookup(query2, ctx2);
     ASSERT_FALSE(drop);
-    srv.initContext(query2, ctx2, drop);
+    ctx2.subnet_ = srv_.selectSubnet(query2, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx2, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response2 = srv.processSolicit(ctx2);
     ASSERT_TRUE(response2);
@@ -1557,14 +1597,14 @@ TEST_F(ClassifyTest, relayOverrideAndClientClass) {
         "    \"subnet\": \"2001:db8:1::/48\", "
         "    \"client-class\": \"foo\", "
         "    \"relay\": { "
-        "        \"ip-address\": \"2001:db8:3::1\""
+        "        \"ip-addresses\": [ \"2001:db8:3::1\" ]"
         "    }"
         " }, "
         " {  \"pools\": [ { \"pool\": \"2001:db8:2::/64\" } ],"
         "    \"id\": 2, "
         "    \"subnet\": \"2001:db8:2::/48\", "
         "    \"relay\": { "
-        "        \"ip-address\": \"2001:db8:3::1\""
+        "        \"ip-addresses\": [ \"2001:db8:3::1\" ]"
         "    }"
         " } "
         "],"
@@ -1795,19 +1835,25 @@ TEST_F(ClassifyTest, member) {
     AllocEngine::ClientContext6 ctx1;
     bool drop = !srv.earlyGHRLookup(query1, ctx1);
     ASSERT_FALSE(drop);
-    srv.initContext(query1, ctx1, drop);
+    ctx1.subnet_ = srv_.selectSubnet(query1, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx1, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response1 = srv.processSolicit(ctx1);
     AllocEngine::ClientContext6 ctx2;
     drop = !srv.earlyGHRLookup(query2, ctx2);
     ASSERT_FALSE(drop);
-    srv.initContext(query2, ctx2, drop);
+    ctx2.subnet_ = srv_.selectSubnet(query2, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx2, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response2 = srv.processSolicit(ctx2);
     AllocEngine::ClientContext6 ctx3;
     drop = !srv.earlyGHRLookup(query3, ctx3);
     ASSERT_FALSE(drop);
-    srv.initContext(query3, ctx3, drop);
+    ctx3.subnet_ = srv_.selectSubnet(query3, drop);
+    ASSERT_FALSE(drop);
+    srv.initContext(ctx3, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr response3 = srv.processSolicit(ctx3);
 
