@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2025 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -36,13 +36,7 @@ namespace test {
 /// @param a first to be compared
 /// @param b second to be compared
 void compareJSON(ConstElementPtr a, ConstElementPtr b) {
-    ASSERT_TRUE(a);
-    ASSERT_TRUE(b);
-    EXPECT_EQ(a->str(), b->str())
-#ifdef HAVE_CREATE_UNIFIED_DIFF
-        << "\nDiff:\n" << generateDiff(prettyPrint(a), prettyPrint(b)) << "\n"
-#endif
-    ;
+    expectEqWithDiff(a, b);
 }
 
 /// @brief Tests if the input string can be parsed with specific parser
@@ -105,37 +99,37 @@ TEST(ParserTest, nestedLists) {
 
 TEST(ParserTest, listsInMaps) {
     string txt = "{ \"constellations\": { \"orion\": [ \"rigel\", \"betelgeuse\" ], "
-                    "\"cygnus\": [ \"deneb\", \"albireo\"] } }";
+                 "\"cygnus\": [ \"deneb\", \"albireo\"] } }";
     testParser(txt, Parser6Context::PARSER_JSON);
 }
 
 TEST(ParserTest, mapsInLists) {
-    string txt = "[ { \"body\": \"earth\", \"gravity\": 1.0 },"
-                 " { \"body\": \"mars\", \"gravity\": 0.376 } ]";
+    string txt = "[ { \"body\": \"earth\", \"gravity\": 1.0 }, "
+                 "{ \"body\": \"mars\", \"gravity\": 0.376 } ]";
     testParser(txt, Parser6Context::PARSER_JSON);
 }
 
 TEST(ParserTest, types) {
-    string txt = "{ \"string\": \"foo\","
-                   "\"integer\": 42,"
-                   "\"boolean\": true,"
-                   "\"map\": { \"foo\": \"bar\" },"
-                   "\"list\": [ 1, 2, 3 ],"
-                   "\"null\": null }";
+    string txt = "{ \"string\": \"foo\", "
+                 "\"integer\": 42, "
+                 "\"boolean\": true, "
+                 "\"map\": { \"foo\": \"bar\" }, "
+                 "\"list\": [ 1, 2, 3 ], "
+                 "\"null\": null }";
     testParser(txt, Parser6Context::PARSER_JSON);
 }
 
 TEST(ParserTest, keywordJSON) {
-    string txt = "{ \"name\": \"user\","
-                   "\"type\": \"password\","
-                   "\"user\": \"name\","
-                   "\"password\": \"type\" }";
+    string txt = "{ \"name\": \"user\", "
+                 "\"type\": \"password\", "
+                 "\"user\": \"name\", "
+                 "\"password\": \"type\" }";
     testParser(txt, Parser6Context::PARSER_JSON);
 }
 
 TEST(ParserTest, keywordDhcp6) {
      string txt = "{ \"Dhcp6\": { \"interfaces-config\": {"
-                  " \"interfaces\": [ \"type\", \"htype\" ] },\n"
+                  "  \"interfaces\": [ \"type\", \"htype\" ] },\n"
                   "\"preferred-lifetime\": 3000,\n"
                   "\"rebind-timer\": 2000, \n"
                   "\"renew-timer\": 1000, \n"
@@ -144,105 +138,122 @@ TEST(ParserTest, keywordDhcp6) {
                   "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
                   "    \"subnet\": \"2001:db8:1::/48\", "
                   "    \"interface\": \"test\" } ],\n"
-                   "\"valid-lifetime\": 4000 } }";
+                  "\"valid-lifetime\": 4000 } }";
      testParser(txt, Parser6Context::PARSER_DHCP6);
 }
 
 // Tests if bash (#) comments are supported. That's the only comment type that
 // was supported by the old parser.
 TEST(ParserTest, bashComments) {
-    string txt= "{ \"Dhcp6\": { \"interfaces-config\": {"
-                "  \"interfaces\": [ \"*\" ]"
-                "},\n"
-                "\"preferred-lifetime\": 3000,\n"
-                "# this is a comment\n"
-                "\"rebind-timer\": 2000, \n"
-                "# lots of comments here\n"
-                "# and here\n"
-                "\"renew-timer\": 1000, \n"
-                "\"subnet6\": [ { "
-                "    \"id\": 1, "
-                "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
-                "    \"subnet\": \"2001:db8:1::/48\", "
-                "    \"interface\": \"eth0\""
-                " } ],"
-                "\"valid-lifetime\": 4000 } }";
+    string txt = "{ \"Dhcp6\": { \"interfaces-config\": {"
+                 "  \"interfaces\": [ \"*\" ]"
+                 "},\n"
+                 "\"preferred-lifetime\": 3000,\n"
+                 "# this is a comment\n"
+                 "\"rebind-timer\": 2000, \n"
+                 "# lots of comments here\n"
+                 "# and here\n"
+                 "\"renew-timer\": 1000, \n"
+                 "\"subnet6\": [ { "
+                 "    \"id\": 1, "
+                 "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
+                 "    \"subnet\": \"2001:db8:1::/48\", "
+                 "    \"interface\": \"eth0\""
+                 " } ],"
+                 "\"valid-lifetime\": 4000 } }";
     testParser(txt, Parser6Context::PARSER_DHCP6);
 }
 
 // Tests if C++ (//) comments can start anywhere, not just in the first line.
 TEST(ParserTest, cppComments) {
-    string txt= "{ \"Dhcp6\": { \"interfaces-config\": {"
-                "  \"interfaces\": [ \"*\" ]"
-                "},\n"
-                "\"preferred-lifetime\": 3000, // this is a comment \n"
-                "\"rebind-timer\": 2000, // everything after // is ignored\n"
-                "\"renew-timer\": 1000, // this will be ignored, too\n"
-                "\"subnet6\": [ { "
-                "    \"id\": 1, "
-                "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
-                "    \"subnet\": \"2001:db8:1::/48\", "
-                "    \"interface\": \"eth0\""
-                " } ],"
-                "\"valid-lifetime\": 4000 } }";
+    string txt = "{ \"Dhcp6\": { \"interfaces-config\": {"
+                 "  \"interfaces\": [ \"*\" ]"
+                 "},\n"
+                 "\"preferred-lifetime\": 3000, // this is a comment \n"
+                 "\"rebind-timer\": 2000, // everything after // is ignored\n"
+                 "\"renew-timer\": 1000, // this will be ignored, too\n"
+                 "\"subnet6\": [ { "
+                 "    \"id\": 1, "
+                 "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
+                 "    \"subnet\": \"2001:db8:1::/48\", "
+                 "    \"interface\": \"eth0\""
+                 " } ],"
+                 "\"valid-lifetime\": 4000 } }";
     testParser(txt, Parser6Context::PARSER_DHCP6, false);
 }
 
 // Tests if bash (#) comments can start anywhere, not just in the first line.
 TEST(ParserTest, bashCommentsInline) {
-    string txt= "{ \"Dhcp6\": { \"interfaces-config\": {"
-                "  \"interfaces\": [ \"*\" ]"
-                "},\n"
-                "\"preferred-lifetime\": 3000, # this is a comment \n"
-                "\"rebind-timer\": 2000, # everything after # is ignored\n"
-                "\"renew-timer\": 1000, # this will be ignored, too\n"
-                "\"subnet6\": [ { "
-                "    \"id\": 1, "
-                "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
-                "    \"subnet\": \"2001:db8:1::/48\", "
-                "    \"interface\": \"eth0\""
-                " } ],"
-                "\"valid-lifetime\": 4000 } }";
+    string txt = "{ \"Dhcp6\": { \"interfaces-config\": {"
+                 "  \"interfaces\": [ \"*\" ]"
+                 "},\n"
+                 "\"preferred-lifetime\": 3000, # this is a comment \n"
+                 "\"rebind-timer\": 2000, # everything after # is ignored\n"
+                 "\"renew-timer\": 1000, # this will be ignored, too\n"
+                 "\"subnet6\": [ { "
+                 "    \"id\": 1, "
+                 "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
+                 "    \"subnet\": \"2001:db8:1::/48\", "
+                 "    \"interface\": \"eth0\""
+                 " } ],"
+                 "\"valid-lifetime\": 4000 } }";
     testParser(txt, Parser6Context::PARSER_DHCP6, false);
 }
 
 // Tests if multi-line C style comments are handled correctly.
 TEST(ParserTest, multilineComments) {
-    string txt= "{ \"Dhcp6\": { \"interfaces-config\": {"
-                "  \"interfaces\": [ \"*\" ]"
-                "},\n"
-                "\"preferred-lifetime\": 3000, /* this is a C style comment\n"
-                "that\n can \n span \n multiple \n lines */ \n"
-                "\"rebind-timer\": 2000,\n"
-                "\"renew-timer\": 1000, \n"
-                "\"subnet6\": [ { "
-                "    \"id\": 1, "
-                "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
-                "    \"subnet\": \"2001:db8:1::/48\", "
-                "    \"interface\": \"eth0\""
-                " } ],"
-                "\"valid-lifetime\": 4000 } }";
+    string txt = "{ \"Dhcp6\": { \"interfaces-config\": {"
+                 "  \"interfaces\": [ \"*\" ]"
+                 "},\n"
+                 "\"preferred-lifetime\": 3000, /* this is a C style comment\n"
+                 "that\n can \n span \n multiple \n lines */ \n"
+                 "\"rebind-timer\": 2000,\n"
+                 "\"renew-timer\": 1000, \n"
+                 "\"subnet6\": [ { "
+                 "    \"id\": 1, "
+                 "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
+                 "    \"subnet\": \"2001:db8:1::/48\", "
+                 "    \"interface\": \"eth0\""
+                 " } ],"
+                 "\"valid-lifetime\": 4000 } }";
     testParser(txt, Parser6Context::PARSER_DHCP6, false);
 }
 
 // Tests if embedded comments are handled correctly.
 TEST(ParserTest, embbededComments) {
-    string txt= "{ \"Dhcp6\": { \"interfaces-config\": {"
-                "  \"interfaces\": [ \"*\" ]"
-                "},\n"
-                "\"comment\": \"a comment\",\n"
-                "\"preferred-lifetime\": 3000,\n"
-                "\"rebind-timer\": 2000,\n"
-                "\"renew-timer\": 1000, \n"
-                "\"subnet6\": [ { "
-                "    \"id\": 1, "
-                "    \"user-context\": { \"comment\": \"indirect\" },"
-                "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
-                "    \"subnet\": \"2001:db8:1::/48\", "
-                "    \"interface\": \"eth0\""
-                " } ],"
-                "\"user-context\": { \"compatible\": true },"
-                "\"valid-lifetime\": 4000 } }";
+    string txt = "{ \"Dhcp6\": { \"interfaces-config\": {"
+                 "  \"interfaces\": [ \"*\" ]"
+                 "},\n"
+                 "\"comment\": \"a comment\",\n"
+                 "\"preferred-lifetime\": 3000,\n"
+                 "\"rebind-timer\": 2000,\n"
+                 "\"renew-timer\": 1000, \n"
+                 "\"subnet6\": [ { "
+                 "    \"id\": 1, "
+                 "    \"user-context\": { \"comment\": \"indirect\" },"
+                 "    \"pools\": [ { \"pool\": \"2001:db8:1::/64\" } ],"
+                 "    \"subnet\": \"2001:db8:1::/48\", "
+                 "    \"interface\": \"eth0\""
+                 " } ],"
+                 "\"user-context\": { \"compatible\": true },"
+                 "\"valid-lifetime\": 4000 } }";
+    testParser(txt, Parser6Context::PARSER_DHCP6, false);
+}
+
+// Test that output_options is an alias of output-options.
+TEST(ParserTest, outputDashOptions) {
+    string txt = "{ \"Dhcp6\": { \"interfaces-config\": {"
+                 "  \"interfaces\": [ \"*\" ]"
+                 "},\n"
+                 "\"preferred-lifetime\": 3000,\n"
+                 "\"rebind-timer\": 2000,\n"
+                 "\"renew-timer\": 1000, \n"
+                 "\"valid-lifetime\": 4000,\n"
+                 "\"loggers\": [ { "
+                 "    \"name\": \"kea-dhcp6\","
+                 "    \"output_options\": [ { \"output\": \"stdout\" } ],"
+                 "    \"severity\": \"INFO\" } ]\n"
+                 "} }";
     testParser(txt, Parser6Context::PARSER_DHCP6, false);
 }
 
@@ -298,7 +309,6 @@ TEST(ParserTest, file) {
     configs.push_back("comments.json");
     configs.push_back("config-backend.json");
     configs.push_back("dhcpv4-over-dhcpv6.json");
-    configs.push_back("dnr.json");
     configs.push_back("duid.json");
     configs.push_back("global-reservations.json");
     configs.push_back("ha-hot-standby-server1-with-tls.json");
@@ -770,20 +780,20 @@ TEST(ParserTest, mapEntries) {
     loadFile(sample_dir + "reservations.json", sample_json);
     loadFile(sample_dir + "all-keys-netconf.json", sample_json);
     KeywordSet sample_keys = {
-        "hosts-database",
-        "reservation-mode"
+        "data-directory",
+        "hosts-database"
     };
     // Recursively extract keywords.
     static void (*extract)(ConstElementPtr, KeywordSet&) =
         [] (ConstElementPtr json, KeywordSet& set) {
             if (json->getType() == Element::list) {
                 // Handle lists.
-                for (auto elem : json->listValue()) {
+                for (auto const& elem : json->listValue()) {
                     extract(elem, set);
                 }
             } else if (json->getType() == Element::map) {
                 // Handle maps.
-                for (auto elem : json->mapValue()) {
+                for (auto const& elem : json->mapValue()) {
                     static_cast<void>(set.insert(elem.first));
                     // Skip entries with free content.
                     if ((elem.first != "user-context") &&
@@ -799,7 +809,7 @@ TEST(ParserTest, mapEntries) {
     auto print_keys = [](const KeywordSet& keys) {
         string s = "{";
         bool first = true;
-        for (auto key : keys) {
+        for (auto const& key : keys) {
             if (first) {
                 first = false;
                 s += " ";
@@ -850,12 +860,12 @@ TEST(ParserTest, duplicateMapEntries) {
         [] (ElementPtr config, ElementPtr json, size_t& cnt) {
             if (json->getType() == Element::list) {
                 // Handle lists.
-                for (auto elem : json->listValue()) {
+                for (auto const& elem : json->listValue()) {
                     test(config, elem, cnt);
                 }
             } else if (json->getType() == Element::map) {
                 // Handle maps.
-                for (auto elem : json->mapValue()) {
+                for (auto const& elem : json->mapValue()) {
                     // Skip entries with free content.
                     if ((elem.first == "user-context") ||
                         (elem.first == "parameters")) {
@@ -924,7 +934,7 @@ TEST_F(TrailingCommasTest, tests) {
       {
         "debuglevel": 99,
         "name": "kea-dhcp6",
-        "output_options": [
+        "output-options": [
           {
             "output": "stdout",
           },
