@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2019-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -23,9 +23,9 @@
 #include <dhcp6/tests/dhcp6_test_utils.h>
 #include <dhcp6/tests/dhcp6_client.h>
 #include <dhcp6/json_config_parser.h>
-#include <dhcp/tests/pkt_captures.h>
+#include <dhcp/testutils/iface_mgr_test_config.h>
+#include <dhcp/testutils/pkt_captures.h>
 #include <dhcp/docsis3_option_defs.h>
-#include <dhcp/tests/iface_mgr_test_config.h>
 #include <dhcp/option_string.h>
 #include <cc/command_interpreter.h>
 
@@ -178,7 +178,9 @@ public:
         AllocEngine::ClientContext6 ctx;
         bool drop = !srv_.earlyGHRLookup(sol, ctx);
         ASSERT_FALSE(drop);
-        srv_.initContext(sol, ctx, drop);
+        ctx.subnet_ = srv_.selectSubnet(sol, drop);
+        ASSERT_FALSE(drop);
+        srv_.initContext(ctx, drop);
         ASSERT_FALSE(drop);
         Pkt6Ptr adv = srv_.processSolicit(ctx);
 
@@ -206,7 +208,9 @@ public:
         AllocEngine::ClientContext6 ctx2;
         drop = !srv_.earlyGHRLookup(sol, ctx2);
         ASSERT_FALSE(drop);
-        srv_.initContext(sol, ctx2, drop);
+        ctx2.subnet_ = srv_.selectSubnet(sol, drop);
+        ASSERT_FALSE(drop);
+        srv_.initContext(ctx2, drop);
         ASSERT_FALSE(drop);
         adv = srv_.processSolicit(ctx2);
         ASSERT_TRUE(adv);
@@ -444,7 +448,9 @@ public:
         AllocEngine::ClientContext6 ctx;
         bool drop = !srv_.earlyGHRLookup(sol, ctx);
         ASSERT_FALSE(drop);
-        srv_.initContext(sol, ctx, drop);
+        ctx.subnet_ = srv_.selectSubnet(sol, drop);
+        ASSERT_FALSE(drop);
+        srv_.initContext(ctx, drop);
         ASSERT_FALSE(drop);
         Pkt6Ptr adv = srv_.processSolicit(ctx);
 
@@ -680,7 +686,9 @@ public:
         AllocEngine::ClientContext6 ctx;
         bool drop = !srv_.earlyGHRLookup(sol, ctx);
         ASSERT_FALSE(drop);
-        srv_.initContext(sol, ctx, drop);
+        ctx.subnet_ = srv_.selectSubnet(sol, drop);
+        ASSERT_FALSE(drop);
+        srv_.initContext(ctx, drop);
         ASSERT_FALSE(drop);
         Pkt6Ptr adv = srv_.processSolicit(ctx);
 
@@ -811,7 +819,7 @@ public:
         // Check that it contains requested option with the appropriate content.
         OptionPtr suboption(response_vendor_options->getOption(option_));
         ASSERT_TRUE(suboption);
-        vector<uint8_t> binary_suboption = suboption->toBinary(false);
+        vector<uint8_t> binary_suboption = suboption->toBinary();
         string text(binary_suboption.begin(), binary_suboption.end());
         EXPECT_EQ("2001:db8::1234:5678", text);
     }
@@ -1383,7 +1391,9 @@ TEST_F(VendorOptsTest, vendorNeverSend) {
     AllocEngine::ClientContext6 ctx;
     bool drop = !srv_.earlyGHRLookup(sol, ctx);
     ASSERT_FALSE(drop);
-    srv_.initContext(sol, ctx, drop);
+    ctx.subnet_ = srv_.selectSubnet(sol, drop);
+    ASSERT_FALSE(drop);
+    srv_.initContext(ctx, drop);
     ASSERT_FALSE(drop);
     Pkt6Ptr adv = srv_.processSolicit(ctx);
 
@@ -1407,7 +1417,9 @@ TEST_F(VendorOptsTest, vendorNeverSend) {
     AllocEngine::ClientContext6 ctx3;
     drop = !srv_.earlyGHRLookup(sol, ctx3);
     ASSERT_FALSE(drop);
-    srv_.initContext(sol, ctx3, drop);
+    ctx3.subnet_ = srv_.selectSubnet(sol, drop);
+    ASSERT_FALSE(drop);
+    srv_.initContext(ctx3, drop);
     ASSERT_FALSE(drop);
     adv = srv_.processSolicit(ctx3);
     ASSERT_TRUE(adv);
@@ -1692,7 +1704,7 @@ TEST_F(VendorOptsTest, twoVendors) {
     ASSERT_EQ(2, classes.size());
     OptionVendorClassPtr opt_class1234;
     OptionVendorClassPtr opt_class5678;
-    for (auto opt : classes) {
+    for (auto const& opt : classes) {
         ASSERT_EQ(D6O_VENDOR_CLASS, opt.first);
         OptionVendorClassPtr opt_class =
             boost::dynamic_pointer_cast<OptionVendorClass>(opt.second);
@@ -1724,7 +1736,7 @@ TEST_F(VendorOptsTest, twoVendors) {
     ASSERT_EQ(2, options.size());
     OptionVendorPtr opt_opts1234;
     OptionVendorPtr opt_opts5678;
-    for (auto opt : options) {
+    for (auto const& opt : options) {
         ASSERT_EQ(D6O_VENDOR_OPTS, opt.first);
         OptionVendorPtr opt_opts =
             boost::dynamic_pointer_cast<OptionVendor>(opt.second);
@@ -1861,7 +1873,7 @@ TEST_F(VendorOptsTest, threeVendors) {
     OptionVendorPtr opt_opts1234;
     OptionVendorPtr opt_docsis;
     OptionVendorPtr opt_opts5678;
-    for (auto opt : options) {
+    for (auto const& opt : options) {
         ASSERT_EQ(D6O_VENDOR_OPTS, opt.first);
         OptionVendorPtr opt_opts =
             boost::dynamic_pointer_cast<OptionVendor>(opt.second);
