@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -84,12 +84,12 @@ protected:
         cstr_ = base_dir + "/" + "config_file";     // config
 
         v4_hdr_ = "address,hwaddr,client_id,valid_lifetime,expire,subnet_id,"
-                  "fqdn_fwd,fqdn_rev,hostname,state,user_context\n";
+                  "fqdn_fwd,fqdn_rev,hostname,state,user_context,pool_id\n";
 
         v6_hdr_ = "address,duid,valid_lifetime,expire,subnet_id,"
                   "pref_lifetime,lease_type,iaid,prefix_len,fqdn_fwd,"
                   "fqdn_rev,hostname,hwaddr,state,user_context,"
-                  "hwtype,hwaddr_source\n";
+                  "hwtype,hwaddr_source,pool_id\n";
 
         // and remove any outstanding test files
         removeTestFile();
@@ -320,7 +320,6 @@ TEST_F(LFCControllerTest, fileRotate) {
     EXPECT_TRUE(noExistIOF());
     removeTestFile();
 
-
     // Test 3: Create a file for previous and finish but not copy.
     writeFile(xstr_, "4");
     writeFile(fstr_, "6");
@@ -332,7 +331,6 @@ TEST_F(LFCControllerTest, fileRotate) {
     EXPECT_TRUE(noExistIOF());
     removeTestFile();
 
-
     // Test 4: Create a file for copy and finish but not previous.
     writeFile(istr_, "8");
     writeFile(fstr_, "9");
@@ -344,7 +342,6 @@ TEST_F(LFCControllerTest, fileRotate) {
     EXPECT_TRUE(noExistIOF());
     removeTestFile();
 
-
     // Test 5: rerun test 2 but using launch instead of cleanup
     // as we already have a finish file we shouldn't do any extra
     // processing
@@ -399,28 +396,28 @@ TEST_F(LFCControllerTest, launch4) {
     // We have several entries for different leases, the naming is:
     // <lease letter>_<version#>
     string a_1 = "192.0.2.1,06:07:08:09:0a:bc,,"
-                 "200,200,8,1,1,host.example.com,1,\n";
+                 "200,200,8,1,1,host.example.com,1,,0\n";
     string a_2 = "192.0.2.1,06:07:08:09:0a:bc,,"
-                 "200,500,8,1,1,host.example.com,1,\n";
+                 "200,500,8,1,1,host.example.com,1,,0\n";
     string a_3 = "192.0.2.1,06:07:08:09:0a:bc,,"
-                 "200,800,8,1,1,host.example.com,1,{ \"foo\": true }\n";
+                 "200,800,8,1,1,host.example.com,1,{ \"foo\": true },0\n";
 
     string b_1 = "192.0.3.15,dd:de:ba:0d:1b:2e:3e:4f,0a:00:01:04,"
-                 "100,100,7,0,0,,1,{ \"bar\": false }\n";
+                 "100,100,7,0,0,,1,{ \"bar\": false },0\n";
     string b_2 = "192.0.3.15,dd:de:ba:0d:1b:2e:3e:4f,0a:00:01:04,"
-                 "100,135,7,0,0,,1,\n";
+                 "100,135,7,0,0,,1,,0\n";
     string b_3 = "192.0.3.15,dd:de:ba:0d:1b:2e:3e:4f,0a:00:01:04,"
-                 "100,150,7,0,0,,1,\n";
+                 "100,150,7,0,0,,1,,0\n";
 
     // This one should be invalid, no hardware address or client id
     // and state is not declined
     string c_1 = "192.0.2.3,,,"
-                 "200,200,8,1,1,host.example.com,0,\n";
+                 "200,200,8,1,1,host.example.com,0,,0\n";
 
     string d_1 = "192.0.2.5,16:17:18:19:1a:bc,,"
-                 "200,200,8,1,1,host.example.com,1,\n";
+                 "200,200,8,1,1,host.example.com,1,,0\n";
     string d_2 = "192.0.2.5,16:17:18:19:1a:bc,,"
-                 "0,200,8,1,1,host.example.com,1,\n";
+                 "0,200,8,1,1,host.example.com,1,,0\n";
 
     // Subtest 1: both previous and copy available.
     // Create the test previous file
@@ -442,7 +439,6 @@ TEST_F(LFCControllerTest, launch4) {
     EXPECT_TRUE(noExistIOFP());
     removeTestFile();
 
-
     // Subtest 2: only previous available
     // Create the test previous file
     test_str = v4_hdr_ + a_1 + b_1 + c_1 + b_2 + a_2 + d_1;
@@ -461,7 +457,6 @@ TEST_F(LFCControllerTest, launch4) {
     EXPECT_TRUE(noExistIOFP());
     removeTestFile();
 
-
     // Subtest 3: only copy available
     // No previous file
 
@@ -480,7 +475,6 @@ TEST_F(LFCControllerTest, launch4) {
     EXPECT_TRUE(noExistIOFP());
     removeTestFile();
 
-
     // Subtest 4: neither available
     // No previous file
 
@@ -496,7 +490,6 @@ TEST_F(LFCControllerTest, launch4) {
     EXPECT_TRUE(noExistIOFP());
     removeTestFile();
 
-
     // Subtest 5: a file with a lot of errors
     // A previous file with a lot of errors
     astr = "1,\n2,\n3,\n4,\n5,\n6,\n7,\n7,\n8,\n9,\n10,\n";
@@ -555,31 +548,44 @@ TEST_F(LFCControllerTest, launch6) {
     // We have several entries for different leases, the naming is:
     // <lease letter>_<version#>.
     string a_1 = "2001:db8:1::1,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,"
-                 "200,200,8,100,0,7,0,1,1,host.example.com,,1,,,\n";
+                 "200,200,8,100,0,7,0,1,1,host.example.com,,1,,,,0\n";
     string a_2 = "2001:db8:1::1,,200,200,8,100,0,7,0,1,1,"
-                 "host.example.com,,1,,,\n";
+                 "host.example.com,,1,,,,0\n";
     string a_3 = "2001:db8:1::1,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,"
-                 "200,400,8,100,0,7,0,1,1,host.example.com,,1,,,\n";
+                 "200,400,8,100,0,7,0,1,1,host.example.com,,1,,,,0\n";
     string a_4 = "2001:db8:1::1,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,"
                  "0,200,8,100,0,7,0,1,1,host.example.com,,1,"
-                 "{ \"foo\": true },,\n";
+                 "{ \"foo\": true },,,0\n";
 
     string b_1 = "2001:db8:2::10,01:01:01:01:0a:01:02:03:04:05,"
-                 "300,300,6,150,0,8,0,0,0,,,1,{ \"bar\": false },,\n";
+                 "300,300,6,150,0,8,0,0,0,,,1,{ \"bar\": false },,,0\n";
     string b_2 = "2001:db8:2::10,01:01:01:01:0a:01:02:03:04:05,"
-                 "300,800,6,150,0,8,0,0,0,,,1,,,\n";
+                 "300,800,6,150,0,8,0,0,0,,,1,,,,0\n";
     string b_3 = "2001:db8:2::10,01:01:01:01:0a:01:02:03:04:05,"
-                 "300,1000,6,150,0,8,0,0,0,,,1,,,\n";
+                 "300,1000,6,150,0,8,0,0,0,,,1,,,,0\n";
 
     string c_1 = "3000:1::,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,"
-                 "100,200,8,0,2,16,64,0,0,,,1,,,\n";
+                 "100,200,8,0,2,16,64,0,0,,,1,,,,0\n";
     string c_2 = "3000:1::,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,"
-                 "100,400,8,0,2,16,64,0,0,,,1,,,\n";
+                 "100,400,8,0,2,16,64,0,0,,,1,,,,0\n";
 
     string d_1 = "2001:db8:1::3,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,"
-                 "200,600,8,100,0,7,0,1,1,host.example.com,,1,,,\n";
+                 "200,600,8,100,0,7,0,1,1,host.example.com,,1,,,,0\n";
 
-    // Subtest 1: bot previous and copy available
+    // new files have 128 prefixlen for non PD type
+    string a_3_n = "2001:db8:1::1,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,"
+                 "200,400,8,100,0,7,128,1,1,host.example.com,,1,,,,0\n";
+
+    string b_2_n = "2001:db8:2::10,01:01:01:01:0a:01:02:03:04:05,"
+                   "300,800,6,150,0,8,128,0,0,,,1,,,,0\n";
+
+    string b_3_n = "2001:db8:2::10,01:01:01:01:0a:01:02:03:04:05,"
+                   "300,1000,6,150,0,8,128,0,0,,,1,,,,0\n";
+
+    string d_1_n = "2001:db8:1::3,00:01:02:03:04:05:06:0a:0b:0c:0d:0e:0f,"
+                   "200,600,8,100,0,7,128,1,1,host.example.com,,1,,,,0\n";
+
+    // Subtest 1: both previous and copy available
     // Create the test previous file
     test_str = v6_hdr_ + a_1 + b_1 + a_2 + c_1 + a_3 + b_2;
     writeFile(xstr_, test_str);
@@ -594,12 +600,11 @@ TEST_F(LFCControllerTest, launch6) {
     // Compare the results, we expect the last lease for each ip
     // except for A which has expired.
     // We also verify none of the temp or pid files remain.
-    test_str = v6_hdr_ + d_1 + b_3 + c_2;
+    test_str = v6_hdr_ + d_1_n + b_3_n + c_2;
     EXPECT_EQ(readFile(xstr_), test_str);
     EXPECT_TRUE(noExistIOFP());
     removeTestFile();
 
-
     // Subtest 2: only previous available
     // Create the test previous file
     test_str = v6_hdr_ + a_1 + b_1 + a_2 + c_1 + a_3 + b_2;
@@ -612,12 +617,11 @@ TEST_F(LFCControllerTest, launch6) {
 
     // Compare the results, we expect the last lease for each ip.
     // We also verify none of the temp or pid files remain.
-    test_str = v6_hdr_ + a_3 + b_2 + c_1;
+    test_str = v6_hdr_ + a_3_n + b_2_n + c_1;
     EXPECT_EQ(readFile(xstr_), test_str);
     EXPECT_TRUE(noExistIOFP());
     removeTestFile();
 
-
     // Subtest 3: only copy available
     // No previous file
 
@@ -630,12 +634,11 @@ TEST_F(LFCControllerTest, launch6) {
 
     // Compare the results, we expect the last lease for each ip.
     // We also verify none of the temp or pid files remain.
-    test_str = v6_hdr_ + d_1 + b_3 + c_2;
+    test_str = v6_hdr_ + d_1_n + b_3_n + c_2;
     EXPECT_EQ(readFile(xstr_), test_str);
     EXPECT_TRUE(noExistIOFP());
     removeTestFile();
 
-
     // Subtest 4: neither available
     // No previous file
 
@@ -651,7 +654,6 @@ TEST_F(LFCControllerTest, launch6) {
     EXPECT_TRUE(noExistIOFP());
     removeTestFile();
 
-
     // Subtest 5: a file with a lot of errors
     // A previous file with a lot of errors.
     astr = "1,\n2,\n3,\n4,\n5,\n6,\n7,\n7,\n8,\n9,\n10,\n";
