@@ -526,7 +526,7 @@ namespace isc { namespace netconf {
     TOKEN_PARAMETERS = 286,        // "parameters"
     TOKEN_LOGGERS = 287,           // "loggers"
     TOKEN_NAME = 288,              // "name"
-    TOKEN_OUTPUT_OPTIONS = 289,    // "output_options"
+    TOKEN_OUTPUT_OPTIONS = 289,    // "output-options"
     TOKEN_OUTPUT = 290,            // "output"
     TOKEN_DEBUGLEVEL = 291,        // "debuglevel"
     TOKEN_SEVERITY = 292,          // "severity"
@@ -593,7 +593,7 @@ namespace isc { namespace netconf {
         S_PARAMETERS = 31,                       // "parameters"
         S_LOGGERS = 32,                          // "loggers"
         S_NAME = 33,                             // "name"
-        S_OUTPUT_OPTIONS = 34,                   // "output_options"
+        S_OUTPUT_OPTIONS = 34,                   // "output-options"
         S_OUTPUT = 35,                           // "output"
         S_DEBUGLEVEL = 36,                       // "debuglevel"
         S_SEVERITY = 37,                         // "severity"
