@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2025 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,6 +6,7 @@
 
 #include <config.h>
 
+#include <util/filesystem.h>
 #include <perfdhcp/avalanche_scen.h>
 #include <perfdhcp/basic_scen.h>
 #include <perfdhcp/command_options.h>
@@ -19,10 +20,14 @@ using namespace isc::perfdhcp;
 
 int
 main(int argc, char* argv[]) {
-    CommandOptions command_options;
-    std::string diags(command_options.getDiags());
+    isc::util::file::setUmask();
+
     int ret_code = 0;
+    std::string diags;
+    bool parser_error = true;
     try {
+        CommandOptions command_options;
+        diags = command_options.getDiags();
         // If parser returns true it means that user specified
         // 'h' or 'v' command line option. Program shows the
         // help or version message and exits here.
@@ -33,17 +38,7 @@ main(int argc, char* argv[]) {
         if (command_options.parse(argc, argv, true)) {
             return (ret_code);
         }
-    } catch (const isc::Exception& e) {
-        ret_code = 1;
-        command_options.usage();
-        std::cerr << "\nERROR: parsing command line options: "
-                  << e.what() << std::endl;
-        if (diags.find('e') != std::string::npos) {
-            std::cerr << "Fatal error" << std::endl;
-        }
-        return (ret_code);
-    }
-    try{
+        parser_error = false;
         auto scenario = command_options.getScenario();
         PerfSocket socket(command_options);
         if (scenario == Scenario::BASIC) {
@@ -55,7 +50,27 @@ main(int argc, char* argv[]) {
         }
     } catch (const std::exception& e) {
         ret_code = 1;
-        std::cerr << "\nERROR: running perfdhcp: " << e.what() << std::endl;
+        if (!parser_error) {
+            std::cerr << std::endl << "ERROR: running perfdhcp: "
+                      << e.what() << std::endl;
+        } else {
+            CommandOptions::usage();
+            std::cerr << std::endl << "ERROR: parsing command line options: "
+                      << e.what() << std::endl;
+        }
+        if (diags.find('e') != std::string::npos) {
+            std::cerr << "Fatal error" << std::endl;
+        }
+    } catch (...) {
+        ret_code = 1;
+        if (!parser_error) {
+            std::cerr << std::endl << "ERROR: running perfdhcp"
+                      << std::endl;
+        } else {
+            CommandOptions::usage();
+            std::cerr << std::endl << "ERROR: parsing command line options"
+                      << std::endl;
+        }
         if (diags.find('e') != std::string::npos) {
             std::cerr << "Fatal error" << std::endl;
         }
