@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -33,12 +33,20 @@ enum class ExchangeType {
     DO,  ///< DHCPv4 DISCOVER-OFFER
     RA,  ///< DHCPv4 REQUEST-ACK
     RNA, ///< DHCPv4 REQUEST-ACK (renewal)
+    RLA, ///< DHCPv4 RELEASE
     SA,  ///< DHCPv6 SOLICIT-ADVERTISE
     RR,  ///< DHCPv6 REQUEST-REPLY
     RN,  ///< DHCPv6 RENEW-REPLY
     RL   ///< DHCPv6 RELEASE-REPLY
 };
 
+/// \brief Get the DHCP version that fits the exchange type.
+///
+/// \param exchange_type exchange type that will determine the version
+/// \throw isc::BadValue exchange type is unrecognized
+/// \return DHCP version: 4 or 6
+int dhcpVersion(ExchangeType const exchange_type);
+
 /// \brief Return name of the exchange.
 ///
 /// Function returns name of the specified exchange type.
@@ -65,7 +73,8 @@ public:
     /// \param name name of the counter used in log file.
     CustomCounter(const std::string& name) :
         counter_(0),
-        name_(name) { };
+        name_(name) {
+    }
 
     /// \brief Increment operator.
     const CustomCounter& operator++() {
@@ -89,25 +98,31 @@ public:
     ///
     /// Method returns counter value.
     ///
-        /// \return counter value.
-    uint64_t getValue() const { return(counter_); }
+    /// \return counter value.
+    uint64_t getValue() const {
+        return (counter_);
+    }
 
     /// \brief Return counter name.
     ///
     /// Method returns counter name.
     ///
-        /// \return counter name.
-    const std::string& getName() const { return(name_); }
+    /// \return counter name.
+    const std::string& getName() const {
+        return (name_);
+    }
+
 private:
     /// \brief Default constructor.
     ///
     /// Default constructor is private because we don't want client
     /// class to call it because we want client class to specify
     /// counter's name.
-    CustomCounter() { };
+    CustomCounter() : counter_(0) {
+    }
 
     uint64_t counter_;  ///< Counter's value.
-    std::string name_;            ///< Counter's name.
+    std::string name_;  ///< Counter's name.
 };
 
 typedef typename boost::shared_ptr<CustomCounter> CustomCounterPtr;
@@ -194,10 +209,8 @@ public:
     /// \code
     /// PktList packets_collection();
     /// ...  # Add elements to the container
-    /// for (PktListIterator it = packets_collection.begin();
-    ///      it != packets_collection.end();
-    ///      ++it) {
-    ///          boost::shared_ptr<Pkt4> pkt = *it;
+    /// for (auto const& it : packets_collection) {
+    ///          boost::shared_ptr<Pkt4> pkt = it;
     ///          # Do something with packet;
     ///      }
     /// \endcode
@@ -411,7 +424,7 @@ public:
                static_cast<double>(unordered_lookups_));
     }
 
-    /// \brief Return number of unordered sent packets lookups
+    /// \brief Return number of unordered sent packets lookups.
     ///
     /// Method returns number of unordered sent packet lookups.
     /// Unordered lookup is used when received packet was sent
@@ -421,7 +434,7 @@ public:
     /// \return number of unordered lookups.
     uint64_t getUnorderedLookups() const { return(unordered_lookups_); }
 
-    /// \brief Return number of ordered sent packets lookups
+    /// \brief Return number of ordered sent packets lookups.
     ///
     /// Method returns number of ordered sent packet lookups.
     /// Ordered lookup is used when packets are received in the
@@ -432,14 +445,14 @@ public:
     /// \return number of ordered lookups.
     uint64_t getOrderedLookups() const { return(ordered_lookups_); }
 
-    /// \brief Return total number of sent packets
+    /// \brief Return total number of sent packets.
     ///
     /// Method returns total number of sent packets.
     ///
     /// \return number of sent packets.
     uint64_t getSentPacketsNum() const { return(sent_packets_num_); }
 
-    /// \brief Return total number of received packets
+    /// \brief Return total number of received packets.
     ///
     /// Method returns total number of received packets.
     ///
@@ -459,6 +472,30 @@ public:
         return(drops);
     }
 
+    /// \brief Return total number of rejected leases.
+    ///
+    /// Method returns total number of rejected leases.
+    ///
+    /// \return number of rejected leases.
+    uint64_t getRejLeasesNum() const { return(rejected_leases_num_); }
+
+    /// \brief Return total number of non unique addresses.
+    ///
+    /// Method returns total number of non unique addresses.
+    ///
+    /// \return number of non unique addresses.
+    uint64_t getNonUniqueAddrNum() const { return(non_unique_addr_num_); }
+
+    /// \brief Increase number of rejected leases.
+    ///
+    /// Method increases total number of rejected leases by one.
+    void updateRejLeases() { ++rejected_leases_num_; }
+
+    /// \brief Increase number of non unique addresses.
+    ///
+    /// Method increases total number of non unique addresses by one.
+    void updateNonUniqueAddr() { ++non_unique_addr_num_; }
+
     /// \brief Print main statistics for packet exchange.
     ///
     /// Method prints main statistics for particular exchange.
@@ -482,7 +519,9 @@ public:
              << "received packets: " << getRcvdPacketsNum() << endl
              << "drops: " << drops << endl
              << "drops ratio: " << drops_ratio << " %" << endl
-             << "orphans: " << getOrphans() << endl;
+             << "orphans: " << getOrphans() << endl
+             << "rejected leases: " << getRejLeasesNum() << endl
+             << "non unique addresses: " << getNonUniqueAddrNum() << endl;
     }
 
     /// \brief Print round trip time packets statistics.
@@ -528,6 +567,20 @@ public:
         return(std::make_tuple(sent_packets_.begin(), sent_packets_.end()));
     }
 
+    /// \brief Return the list of received leases in CSV format as string.
+    ///
+    /// Depending exchange type, it can apply to
+    /// potential leases received in offers and advertisements,
+    /// committed leases received in acknowledgements and replies,
+    /// renewed or released leases.
+    ///
+    /// \return multiline string of received leases in CSV format
+    std::string receivedLeases() const;
+
+    /// \brief Print the list of received leases.
+    void printLeases() const;
+
+    static int malformed_pkts_;
 
 // Private stuff of ExchangeStats class
 private:
@@ -619,6 +672,11 @@ private:
 
     uint64_t sent_packets_num_;    ///< Total number of sent packets.
     uint64_t rcvd_packets_num_;    ///< Total number of received packets.
+
+    uint64_t non_unique_addr_num_; ///< Total number of non unique addresses
+                                   ///< offered/advertised.
+    uint64_t rejected_leases_num_; ///< Total number of rejected leases
+                                   /// (e.g. NoAddrAvail)
     boost::posix_time::ptime boot_time_; ///< Time when test is started.
 };
 
@@ -628,7 +686,7 @@ typedef boost::shared_ptr<ExchangeStats> ExchangeStatsPtr;
 /// Map containing all specified exchange types.
 typedef typename std::map<ExchangeType, ExchangeStatsPtr> ExchangesMap;
 
-/// Iterator pointing to \ref ExchangesMap
+/// Iterator pointing to \ref ExchangesMap.
 typedef typename ExchangesMap::const_iterator ExchangesMapIterator;
 
 
@@ -718,10 +776,8 @@ public:
     ///
     // \return true, if packet drops occurred.
     bool droppedPackets() const {
-        for (ExchangesMapIterator it = exchanges_.begin();
-             it != exchanges_.end();
-             ++it) {
-            if (it->second->getDroppedPacketsNum() > 0) {
+        for (auto const& it : exchanges_) {
+            if (it.second->getDroppedPacketsNum() > 0) {
                 return (true);
             }
         }
@@ -877,7 +933,7 @@ public:
         return(xchg_stats->getAvgUnorderedLookupSetSize());
     }
 
-    /// \brief Return number of unordered sent packets lookups
+    /// \brief Return number of unordered sent packets lookups.
     ///
     /// Method returns number of unordered sent packet lookups.
     /// Unordered lookup is used when received packet was sent
@@ -892,7 +948,7 @@ public:
         return(xchg_stats->getUnorderedLookups());
     }
 
-    /// \brief Return number of ordered sent packets lookups
+    /// \brief Return number of ordered sent packets lookups.
     ///
     /// Method returns number of ordered sent packet lookups.
     /// Ordered lookup is used when packets are received in the
@@ -908,7 +964,7 @@ public:
         return(xchg_stats->getOrderedLookups());
     }
 
-    /// \brief Return total number of sent packets
+    /// \brief Return total number of sent packets.
     ///
     /// Method returns total number of sent packets for specified
     /// exchange type.
@@ -921,7 +977,7 @@ public:
         return(xchg_stats->getSentPacketsNum());
     }
 
-    /// \brief Return total number of received packets
+    /// \brief Return total number of received packets.
     ///
     /// Method returns total number of received packets for specified
     /// exchange type.
@@ -962,6 +1018,49 @@ public:
         return(xchg_stats->getCollectedNum());
     }
 
+    /// \brief Return total number of rejected leases.
+    ///
+    /// Method returns total number of rejected leases for specified
+    /// exchange type.
+    ///
+    /// \param xchg_type exchange type.
+    /// \throw isc::BadValue if invalid exchange type specified.
+    /// \return number of rejected leases.
+    uint64_t getRejLeasesNum(const ExchangeType xchg_type) const {
+        ExchangeStatsPtr xchg_stats = getExchangeStats(xchg_type);
+        return(xchg_stats->getRejLeasesNum());
+    }
+
+    /// \brief Increase total number of rejected leases.
+    ///
+    /// Method increases total number of rejected leases by one
+    /// for specified exchange type.
+    void updateRejLeases(const ExchangeType xchg_type) {
+        ExchangeStatsPtr xchg_stats = getExchangeStats(xchg_type);
+        xchg_stats->updateRejLeases();
+    }
+
+    /// \brief Increase total number of non unique addresses.
+    ///
+    /// Method increases total number of non unique addresses or
+    /// prefixes by one for specified exchange type.
+    void updateNonUniqueAddrNum(const ExchangeType xchg_type) {
+        ExchangeStatsPtr xchg_stats = getExchangeStats(xchg_type);
+        xchg_stats->updateNonUniqueAddr();
+    }
+
+    /// \brief Return total number of non unique addresses.
+    ///
+    /// Method returns total number of non unique addresses and/or
+    /// prefixes for specified exchange type.
+    ///
+    /// \param xchg_type exchange type.
+    /// \throw isc::BadValue if invalid exchange type specified.
+    /// \return number of non unique addresses.
+    uint64_t getNonUniqueAddrNum(const ExchangeType xchg_type) const {
+        ExchangeStatsPtr xchg_stats = getExchangeStats(xchg_type);
+        return(xchg_stats->getNonUniqueAddrNum());
+    }
 
     /// \brief Get time period since the start of test.
     ///
@@ -995,11 +1094,9 @@ public:
             isc_throw(isc::InvalidOperation,
                       "no exchange type added for tracking");
         }
-        for (ExchangesMapIterator it = exchanges_.begin();
-             it != exchanges_.end();
-             ++it) {
-            ExchangeStatsPtr xchg_stats = it->second;
-            std::cout << "***Statistics for: " << it->first
+        for (auto const& it : exchanges_) {
+            ExchangeStatsPtr xchg_stats = it.second;
+            std::cout << "***Statistics for: " << it.first
                       << "***" << std::endl;
             xchg_stats->printMainStats();
             std::cout << std::endl;
@@ -1013,25 +1110,47 @@ public:
     /// Method prints intermediate statistics for all exchanges.
     /// Statistics includes sent, received and dropped packets
     /// counters.
-    void printIntermediateStats() const {
+    ///
+    /// \param clean_report value to generate easy to parse report.
+    /// \param clean_sep string used as separator if clean_report enabled..
+    void
+    printIntermediateStats(bool clean_report, std::string clean_sep) const {
         std::ostringstream stream_sent;
         std::ostringstream stream_rcvd;
         std::ostringstream stream_drops;
+        std::ostringstream stream_reject;
         std::string sep("");
-        for (ExchangesMapIterator it = exchanges_.begin();
-             it != exchanges_.end(); ++it) {
-
-            if (it != exchanges_.begin()) {
-                sep = "/";
+        bool first = true;
+        for (auto const& it : exchanges_) {
+            if (!first) {
+                if (clean_report) {
+                    sep = clean_sep;
+                } else {
+                    sep = "/";
+                }
+            } else {
+                first = false;
             }
-            stream_sent << sep << it->second->getSentPacketsNum();
-            stream_rcvd << sep << it->second->getRcvdPacketsNum();
-            stream_drops << sep << it->second->getDroppedPacketsNum();
+            stream_sent << sep << it.second->getSentPacketsNum();
+            stream_rcvd << sep << it.second->getRcvdPacketsNum();
+            stream_drops << sep << it.second->getDroppedPacketsNum();
+            stream_reject << sep << it.second->getRejLeasesNum();
+        }
+
+        if (clean_report) {
+            std::cout << stream_sent.str()
+                      << clean_sep << stream_rcvd.str()
+                      << clean_sep << stream_drops.str()
+                      << clean_sep << stream_reject.str()
+                      << std::endl;
+
+        } else {
+            std::cout << "sent: " << stream_sent.str()
+                      << "; received: " << stream_rcvd.str()
+                      << "; drops: " << stream_drops.str()
+                      << "; rejected: " << stream_reject.str()
+                      << std::endl;
         }
-        std::cout << "sent: " << stream_sent.str()
-                  << "; received: " << stream_rcvd.str()
-                  << "; drops: " << stream_drops.str()
-                  << std::endl;
     }
 
     /// \brief Print timestamps of all packets.
@@ -1050,18 +1169,19 @@ public:
             isc_throw(isc::InvalidOperation,
                       "no exchange type added for tracking");
         }
-        for (ExchangesMapIterator it = exchanges_.begin();
-             it != exchanges_.end();
-             ++it) {
-            ExchangeStatsPtr xchg_stats = it->second;
+        for (auto const& it : exchanges_) {
+            ExchangeStatsPtr xchg_stats = it.second;
             std::cout << "***Timestamps for packets: "
-                      << it->first
+                      << it.first
                       << "***" << std::endl;
             xchg_stats->printTimestamps();
             std::cout << std::endl;
         }
     }
 
+    /// \brief Delegate to all exchanges to print their leases.
+    void printLeases() const;
+
     /// \brief Print names and values of custom counters.
     ///
     /// Method prints names and values of custom counters. Custom counters
@@ -1072,24 +1192,24 @@ public:
         if (custom_counters_.empty()) {
             isc_throw(isc::InvalidOperation, "no custom counters specified");
         }
-        for (CustomCountersMapIterator it = custom_counters_.begin();
-             it != custom_counters_.end();
-             ++it) {
-            CustomCounterPtr counter = it->second;
+        for (auto const& it : custom_counters_) {
+            CustomCounterPtr counter = it.second;
             std::cout << counter->getName() << ": " << counter->getValue()
                       << std::endl;
         }
     }
 
-    std::tuple<typename ExchangeStats::PktListIterator, typename ExchangeStats::PktListIterator> getSentPackets(const ExchangeType xchg_type) const {
+    std::tuple<typename ExchangeStats::PktListIterator, typename ExchangeStats::PktListIterator>
+    getSentPackets(const ExchangeType xchg_type) const {
         ExchangeStatsPtr xchg_stats = getExchangeStats(xchg_type);
-        std::tuple<typename ExchangeStats::PktListIterator, typename ExchangeStats::PktListIterator> sent_packets_its = xchg_stats->getSentPackets();
-        return(sent_packets_its);
+        std::tuple<typename ExchangeStats::PktListIterator, typename ExchangeStats::PktListIterator> sent_packets_its =
+                xchg_stats->getSentPackets();
+        return (sent_packets_its);
     }
 
 private:
 
-    /// \brief Return exchange stats object for given exchange type
+    /// \brief Return exchange stats object for given exchange type.
     ///
     /// Method returns exchange stats object for given exchange type.
     ///
