@@ -50,15 +50,19 @@ extern const isc::log::MessageID HA_INIT_OK;
 extern const isc::log::MessageID HA_INVALID_PARTNER_STATE_COMMUNICATION_RECOVERY;
 extern const isc::log::MessageID HA_INVALID_PARTNER_STATE_HOT_STANDBY;
 extern const isc::log::MessageID HA_INVALID_PARTNER_STATE_LOAD_BALANCING;
+extern const isc::log::MessageID HA_LEASE4_SERVER_DECLINE_FAILED;
 extern const isc::log::MessageID HA_LEASES4_COMMITTED_FAILED;
 extern const isc::log::MessageID HA_LEASES4_COMMITTED_NOTHING_TO_UPDATE;
+extern const isc::log::MessageID HA_LEASES4_COMMITTED_NO_RELATIONSHIP;
 extern const isc::log::MessageID HA_LEASES6_COMMITTED_FAILED;
 extern const isc::log::MessageID HA_LEASES6_COMMITTED_NOTHING_TO_UPDATE;
+extern const isc::log::MessageID HA_LEASES6_COMMITTED_NO_RELATIONSHIP;
 extern const isc::log::MessageID HA_LEASES_BACKLOG_COMMUNICATIONS_FAILED;
 extern const isc::log::MessageID HA_LEASES_BACKLOG_FAILED;
 extern const isc::log::MessageID HA_LEASES_BACKLOG_NOTHING_TO_SEND;
 extern const isc::log::MessageID HA_LEASES_BACKLOG_START;
 extern const isc::log::MessageID HA_LEASES_BACKLOG_SUCCESS;
+extern const isc::log::MessageID HA_LEASES_SYNC_APPLIED_LEASES;
 extern const isc::log::MessageID HA_LEASES_SYNC_COMMUNICATIONS_FAILED;
 extern const isc::log::MessageID HA_LEASES_SYNC_FAILED;
 extern const isc::log::MessageID HA_LEASES_SYNC_LEASE_PAGE_RECEIVED;
@@ -100,6 +104,18 @@ extern const isc::log::MessageID HA_STATE_MACHINE_CONTINUED;
 extern const isc::log::MessageID HA_STATE_MACHINE_PAUSED;
 extern const isc::log::MessageID HA_STATE_TRANSITION;
 extern const isc::log::MessageID HA_STATE_TRANSITION_PASSIVE_BACKUP;
+extern const isc::log::MessageID HA_SUBNET4_SELECT_FAILED;
+extern const isc::log::MessageID HA_SUBNET4_SELECT_INVALID_HA_SERVER_NAME;
+extern const isc::log::MessageID HA_SUBNET4_SELECT_NOT_FOR_US;
+extern const isc::log::MessageID HA_SUBNET4_SELECT_NO_RELATIONSHIP_FOR_SUBNET;
+extern const isc::log::MessageID HA_SUBNET4_SELECT_NO_RELATIONSHIP_SELECTOR_FOR_SUBNET;
+extern const isc::log::MessageID HA_SUBNET4_SELECT_NO_SUBNET_SELECTED;
+extern const isc::log::MessageID HA_SUBNET6_SELECT_FAILED;
+extern const isc::log::MessageID HA_SUBNET6_SELECT_INVALID_HA_SERVER_NAME;
+extern const isc::log::MessageID HA_SUBNET6_SELECT_NOT_FOR_US;
+extern const isc::log::MessageID HA_SUBNET6_SELECT_NO_RELATIONSHIP_FOR_SUBNET;
+extern const isc::log::MessageID HA_SUBNET6_SELECT_NO_RELATIONSHIP_SELECTOR_FOR_SUBNET;
+extern const isc::log::MessageID HA_SUBNET6_SELECT_NO_SUBNET_SELECTED;
 extern const isc::log::MessageID HA_SYNC_COMPLETE_NOTIFY_COMMUNICATIONS_FAILED;
 extern const isc::log::MessageID HA_SYNC_COMPLETE_NOTIFY_FAILED;
 extern const isc::log::MessageID HA_SYNC_COMPLETE_NOTIFY_HANDLER_FAILED;
@@ -108,6 +124,7 @@ extern const isc::log::MessageID HA_SYNC_HANDLER_FAILED;
 extern const isc::log::MessageID HA_SYNC_START;
 extern const isc::log::MessageID HA_SYNC_SUCCESSFUL;
 extern const isc::log::MessageID HA_TERMINATED;
+extern const isc::log::MessageID HA_TERMINATED_PARTNER_DID_NOT_RESTART;
 extern const isc::log::MessageID HA_TERMINATED_RESTART_PARTNER;
 
 } // namespace ha
