@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -61,6 +61,7 @@ public:
     using StateType::partner_time_at_skew_;
     using StateType::unsent_update_count_;
     using StateType::getRejectedLeaseUpdatesCountFromContainer;
+    using StateType::partner_state_time_;
 };
 
 /// @brief Type of the NakedCommunicationState for DHCPv4.
@@ -150,6 +151,13 @@ public:
     data::ConstElementPtr
     createValidPassiveBackupJsonConfiguration() const;
 
+    /// @brief Return HA configuration for a hub in a hub-and-spoke model
+    /// in a JSON format.
+    ///
+    /// @return Pointer to the unparsed configuration.
+    data::ConstElementPtr
+    createValidHubJsonConfiguration() const;
+
     /// @brief Return HA configuration with three servers.
     ///
     /// @param ha_mode HA operation mode (default is load balancing).
@@ -162,6 +170,11 @@ public:
     /// @return Pointer to the parsed configuration.
     HAConfigPtr createValidPassiveBackupConfiguration() const;
 
+    /// @brief Return HA configuration for a hub in a hub-and-spoke model.
+    ///
+    /// @return Pointer to the parsed configuration.
+    HAConfigPtr createValidHubConfiguration() const;
+
     /// @brief Checks the status code and message against expected values.
     ///
     /// @param answer Element set containing an integer response and string
@@ -297,6 +310,9 @@ public:
 
     /// @brief Object holding a state of the DHCP service.
     dhcp::NetworkStatePtr network_state_;
+
+    /// @brief Test timer.
+    isc::asiolink::IntervalTimerPtr timer_;
 };
 
 } // end of namespace isc::ha::test
