@@ -1,4 +1,4 @@
-// Copyright (C) 2021-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2021-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -10,6 +10,7 @@
 
 #include <config.h>
 
+#include <asiolink/io_service_mgr.h>
 #include <dhcpsrv/cfgmgr.h>
 #include <hooks/hooks.h>
 #include <process/daemon.h>
@@ -22,6 +23,7 @@
 #include <sstream>
 #include <string>
 
+using namespace isc::asiolink;
 using namespace isc::cb;
 using namespace isc::dhcp;
 using namespace isc::hooks;
@@ -66,16 +68,9 @@ int load(LibraryHandle& /* handle */) {
 ///
 /// @param handle callout handle passed to the callout.
 /// @return 0 on success, 1 otherwise.
-int dhcp4_srv_configured(CalloutHandle& handle) {
-    isc::asiolink::IOServicePtr io_service;
-    handle.getArgument("io_context", io_service);
-    if (!io_service) {
-        const string error("Error: io_context is null");
-        handle.setArgument("error", error);
-        handle.setStatus(isc::hooks::CalloutHandle::NEXT_STEP_DROP);
-        return (1);
-    }
-    isc::dhcp::PgSqlConfigBackendImpl::setIOService(io_service);
+int dhcp4_srv_configured(CalloutHandle& /* handle */) {
+    isc::dhcp::PgSqlConfigBackendImpl::setIOService(IOServicePtr(new IOService()));
+    IOServiceMgr::instance().registerIOService(isc::dhcp::PgSqlConfigBackendImpl::getIOService());
     return (0);
 }
 
@@ -85,16 +80,9 @@ int dhcp4_srv_configured(CalloutHandle& handle) {
 ///
 /// @param handle callout handle passed to the callout.
 /// @return 0 on success, 1 otherwise.
-int dhcp6_srv_configured(CalloutHandle& handle) {
-    isc::asiolink::IOServicePtr io_service;
-    handle.getArgument("io_context", io_service);
-    if (!io_service) {
-        const string error("Error: io_context is null");
-        handle.setArgument("error", error);
-        handle.setStatus(isc::hooks::CalloutHandle::NEXT_STEP_DROP);
-        return (1);
-    }
-    isc::dhcp::PgSqlConfigBackendImpl::setIOService(io_service);
+int dhcp6_srv_configured(CalloutHandle& /* handle */) {
+    isc::dhcp::PgSqlConfigBackendImpl::setIOService(IOServicePtr(new IOService()));
+    IOServiceMgr::instance().registerIOService(isc::dhcp::PgSqlConfigBackendImpl::getIOService());
     return (0);
 }
 
@@ -106,6 +94,12 @@ int unload() {
     // Unregister the factories and remove PostgreSQL backends
     isc::dhcp::PgSqlConfigBackendDHCPv4::unregisterBackendType();
     isc::dhcp::PgSqlConfigBackendDHCPv6::unregisterBackendType();
+    IOServicePtr io_service = isc::dhcp::PgSqlConfigBackendImpl::getIOService();
+    if (io_service) {
+        IOServiceMgr::instance().unregisterIOService(io_service);
+        io_service->stopAndPoll();
+        isc::dhcp::PgSqlConfigBackendImpl::setIOService(IOServicePtr());
+    }
     return (0);
 }
 
