@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2019-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -12,11 +12,8 @@
 
 #include <config.h>
 
-#include <cc/data.h>
-#include <dhcpsrv/cfgmgr.h>
-#include <hooks/hooks_manager.h>
-#include <process/daemon.h>
-#include <testutils/lib_load_test_fixture.h>
+#include <dhcpsrv/testutils/lib_load_test_fixture.h>
+#include <testutils/gtest_utils.h>
 
 #include <gtest/gtest.h>
 #include <errno.h>
