@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -37,7 +37,7 @@ class ConnectionPool;
 /// It is possible to make multiple connections to the server side
 /// socket simultaneously.
 ///
-/// The test should perform IOService::run_one until it finds that
+/// The test should perform IOService::runOne until it finds that
 /// the number of responses sent by the server is greater than
 /// expected. The number of responses sent so far can be retrieved
 /// using @ref TestServerUnixSocket::getResponseNum.
@@ -53,7 +53,7 @@ public:
     /// @param io_service IO service.
     /// @param socket_file_path Socket file path.
     /// @param custom_response Custom response to be sent to the client.
-    TestServerUnixSocket(IOService& io_service,
+    TestServerUnixSocket(const IOServicePtr& io_service,
                          const std::string& socket_file_path,
                          const std::string& custom_response = "");
 
@@ -123,7 +123,7 @@ private:
     void signalRunning();
 
     /// @brief IO service used by the tests.
-    IOService& io_service_;
+    IOServicePtr io_service_;
 
     /// @brief Server endpoint.
     boost::asio::local::stream_protocol::endpoint server_endpoint_;
