@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,7 +13,7 @@
 
 #include <gtest/gtest.h>
 
-#include <util/encode/hex.h>
+#include <util/encode/encode.h>
 
 #include <cryptolink/cryptolink.h>
 #include <cryptolink/crypto_hash.h>
@@ -50,8 +50,7 @@ namespace {
                      size_t len)
     {
         ASSERT_EQ(len, buf.getLength());
-        checkData(static_cast<const uint8_t*>(buf.getData()), expected,
-                  len);
+        checkData(buf.getData(), expected, len);
     }
 
     /// @brief Hash with the convenience functions
