@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,7 +13,7 @@
 
 #include <gtest/gtest.h>
 
-#include <util/encode/hex.h>
+#include <util/encode/encode.h>
 
 #include <cryptolink/cryptolink.h>
 #include <cryptolink/crypto_hmac.h>
@@ -60,8 +60,7 @@ namespace {
                      size_t len)
     {
         ASSERT_EQ(len, buf.getLength());
-        checkData(static_cast<const uint8_t*>(buf.getData()), expected,
-                  len);
+        checkData(buf.getData(), expected, len);
     }
 
     /// @brief Sign and verify with the convenience functions
@@ -462,7 +461,7 @@ doRFC4231Tests(HashAlgorithm hash_algorithm,
         0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c,
         0x0d, 0x0e, 0x0f, 0x10, 0x11, 0x12,
         0x13, 0x14, 0x15, 0x16, 0x17, 0x18,
-        0x19 
+        0x19
     };
     secret_list.push_back(std::string(secret_array,
                                       secret_array + sizeof(secret_array)));
