@@ -1,4 +1,4 @@
-// Copyright (C) 2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2021-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -28,7 +28,7 @@ D2StatTest::~D2StatTest() {
 void
 checkStats(const string& key_name, const StatMap& expected_stats) {
     StatMap key_stats;
-    for (const auto& it : expected_stats) {
+    for (auto const& it : expected_stats) {
         const string& stat_name =
             StatsMgr::generateName("key", key_name, it.first);
         key_stats[stat_name] = it.second;
