@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -114,14 +114,13 @@ IfaceMgr::detectIfaces(bool update_only) {
     freeifaddrs(iflist);
 
     // Interfaces registering
-    for (IfaceLst::const_iterator iface_iter = ifaces.begin();
-        iface_iter != ifaces.end(); ++iface_iter) {
+    for (auto const& iface_iter : ifaces) {
         IfacePtr iface;
         if (update_only) {
-            iface = getIface(iface_iter->first);
+            iface = getIface(iface_iter.first);
         }
         if (!iface) {
-            addInterface(iface_iter->second);
+            addInterface(iface_iter.second);
         }
     }
 }
