@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -11,7 +11,6 @@
 #include <dhcp/libdhcp++.h>
 #include <dhcp/option6_addrlst.h>
 #include <exceptions/exceptions.h>
-#include <util/io_utilities.h>
 
 #include <sstream>
 
@@ -68,15 +67,14 @@ void Option6AddrLst::pack(isc::util::OutputBuffer& buf, bool) const {
     // len field contains length without 4-byte option header
     buf.writeUint16(len() - getHeaderLen());
 
-    for (AddressContainer::const_iterator addr=addrs_.begin();
-         addr!=addrs_.end(); ++addr) {
-        if (!addr->isV6()) {
-            isc_throw(isc::BadValue, addr->toText()
+    for (auto const& addr : addrs_) {
+        if (!addr.isV6()) {
+            isc_throw(isc::BadValue, addr.toText()
                       << " is not an IPv6 address");
         }
         // If an address is IPv6 address it should have assumed
         // length of V6ADDRESS_LEN.
-        buf.writeData(&addr->toBytes()[0], V6ADDRESS_LEN);
+        buf.writeData(&addr.toBytes()[0], V6ADDRESS_LEN);
     }
 }
 
@@ -97,9 +95,8 @@ std::string Option6AddrLst::toText(int indent) const {
     stringstream output;
     output << headerToText(indent) << ":";
 
-    for (AddressContainer::const_iterator addr = addrs_.begin();
-         addr != addrs_.end(); ++addr) {
-        output << " " << *addr;
+    for (auto const& addr : addrs_) {
+        output << " " << addr;
     }
     return (output.str());
 }
