@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -18,6 +18,7 @@
 namespace {
 
 using namespace isc::dhcp;
+using namespace boost::posix_time;
 
 /// @brief Maximum number of attempts to open BPF device.
 const unsigned int MAX_BPF_OPEN_ATTEMPTS = 100;
@@ -26,7 +27,7 @@ const unsigned int MAX_BPF_OPEN_ATTEMPTS = 100;
 /// received on local loopback interface.
 const unsigned int BPF_LOCAL_LOOPBACK_HEADER_LEN = 4;
 
-/// The following structure defines a Berkely Packet Filter program to perform
+/// The following structure defines a Berkeley Packet Filter program to perform
 /// packet filtering. The program operates on Ethernet packets.  To help with
 /// interpretation of the program, for the types of Ethernet packets we are
 /// interested in, the header layout is:
@@ -137,7 +138,7 @@ struct bpf_insn ethernet_ip_udp_filter [] = {
 struct bpf_insn loopback_ip_udp_filter [] = {
     // Make sure this is an IP packet. The pseudo header comprises a 4-byte
     // long value identifying the address family, which should be set to
-    // AF_INET. The default value used here (0xFFFFFFFF) must be overriden
+    // AF_INET. The default value used here (0xFFFFFFFF) must be overridden
     // with htonl(AF_INET) from within the openSocket function.
     // #0
     BPF_STMT(BPF_LD + BPF_W + BPF_ABS, 0),
@@ -226,10 +227,9 @@ PktFilterBPF::openSocket(Iface& iface,
                          const isc::asiolink::IOAddress& addr,
                          const uint16_t port, const bool,
                          const bool) {
-
     // Open fallback socket first. If it fails, it will give us an indication
     // that there is another service (perhaps DHCP server) running.
-    // The function will throw an exception and effectivelly cease opening
+    // The function will throw an exception and effectively cease opening
     // the BPF device below.
     int fallback = openFallbackSocket(addr, port);
 
@@ -310,7 +310,7 @@ PktFilterBPF::openSocket(Iface& iface,
         close(fallback);
         close(sock);
         isc_throw(SocketConfigError, "Unable to obtain the required"
-                  " buffer legth for reads from BPF device");
+                  " buffer length for reads from BPF device");
     }
 
     if (buf_len < sizeof(bpf_hdr)) {
@@ -355,14 +355,14 @@ PktFilterBPF::openSocket(Iface& iface,
     }
 
     // Configure the BPF device to use the immediate mode. This ensures
-    // that the read function returns immediatelly, instead of waiting
+    // that the read function returns immediately, instead of waiting
     // for the kernel to fill up the buffer, which would likely cause
     // read hangs.
     int flag = 1;
     if (ioctl(sock, BIOCIMMEDIATE, &flag) < 0) {
         close(fallback);
         close(sock);
-        isc_throw(SocketConfigError, "Failed to set promiscious mode for"
+        isc_throw(SocketConfigError, "Failed to set promiscuous mode for"
                   " BPF device");
     }
 
@@ -376,6 +376,7 @@ PktFilterBPF::openSocket(Iface& iface,
         close(sock);
         throw;
     }
+
     return (SocketInfo(addr, port, sock, fallback));
 }
 
@@ -411,10 +412,10 @@ PktFilterBPF::receive(Iface& iface, const SocketInfo& socket_info) {
     datalen = read(socket_info.sockfd_, iface.getReadBuffer(),
                    iface.getReadBufferSize());
     // If negative value is returned by read(), it indicates that an
-    // error occured. If returned value is 0, no data was read from the
+    // error occurred. If returned value is 0, no data was read from the
     // socket. In both cases something has gone wrong, because we expect
     // that a chunk of data is there. We signal the lack of data by
-    // returing an empty packet.
+    // returning an empty packet.
     if (datalen <= 0) {
         return Pkt4Ptr();
     }
@@ -537,6 +538,22 @@ PktFilterBPF::receive(Iface& iface, const SocketInfo& socket_info) {
     pkt->setLocalHWAddr(dummy_pkt->getLocalHWAddr());
     pkt->setRemoteHWAddr(dummy_pkt->getRemoteHWAddr());
 
+    // Set time the packet was stored in the buffer.
+#if (defined(BPF_TIMEVAL)) && (BPF_TIMEVAL == timeval32)
+    // Convert to ptime directly to avoid timeval vs
+    // timeval32 definitons under MacOS.
+    time_t time_t_secs = bpfh.bh_tstamp.tv_sec;
+    ptime timestamp = from_time_t(time_t_secs);
+    time_duration usecs(0, 0, 0, bpfh.bh_tstamp.tv_usec);
+    timestamp += usecs;
+    pkt->addPktEvent(PktEvent::SOCKET_RECEIVED, timestamp);
+#else
+    pkt->addPktEvent(PktEvent::SOCKET_RECEIVED, bpfh.bh_tstamp);
+#endif
+
+    // Set time packet was read from the buffer.
+    pkt->addPktEvent(PktEvent::BUFFER_READ);
+
     return (pkt);
 }
 
@@ -586,6 +603,7 @@ PktFilterBPF::send(const Iface& iface, uint16_t sockfd, const Pkt4Ptr& pkt) {
                   << strerror(errno));
     }
 
+    pkt->addPktEvent(PktEvent::RESPONSE_SENT);
     return (0);
 }
 
