@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -270,9 +270,7 @@ TEST(OptionOpaqueDataTuples, pack4) {
         0x66, 0x6F, 0x6F                    // foo
     };
     // Compare the buffer with reference data.
-    EXPECT_EQ(0, memcmp(static_cast<const void*>(ref),
-                        static_cast<const void*>(buf.getData()),
-                        buf.getLength()));
+    EXPECT_EQ(0, memcmp(ref, buf.getData(), buf.getLength()));
 }
 
 // Check that the DHCPv4 option is rendered to the buffer in wire format,
@@ -304,9 +302,7 @@ TEST(OptionOpaqueDataTuples, pack4_with_ltf) {
         0x66, 0x6F, 0x6F                    // foo
     };
     // Compare the buffer with reference data.
-    EXPECT_EQ(0, memcmp(static_cast<const void*>(ref),
-                        static_cast<const void*>(buf.getData()),
-                        buf.getLength()));
+    EXPECT_EQ(0, memcmp(ref, buf.getData(), buf.getLength()));
 }
 
 // Check that the DHCPv6 option is rendered to the buffer in wire format.
@@ -336,9 +332,7 @@ TEST(OptionOpaqueDataTuples, pack6) {
         0x66, 0x6F, 0x6F                    // foo
     };
     // Compare the buffer with reference data.
-    EXPECT_EQ(0, memcmp(static_cast<const void*>(ref),
-                        static_cast<const void*>(buf.getData()),
-                        buf.getLength()));
+    EXPECT_EQ(0, memcmp(ref, buf.getData(), buf.getLength()));
 }
 
 // This function checks that the DHCPv4 option with two opaque data tuples
@@ -536,9 +530,9 @@ TEST(OptionOpaqueDataTuples, unpack6Truncated) {
 // data is parsed correctly.
 TEST(OptionOpaqueDataTuples, unpack4NoTuple) {
     // Prepare data to decode.
-    const uint8_t buf_data[] = {
-    };
-    OptionBuffer buf(buf_data, buf_data + sizeof(buf_data));
+    std::vector<uint8_t> buf_vector;
+    const uint8_t* buf_data = buf_vector.data();
+    OptionBuffer buf(buf_data, buf_data + buf_vector.size());
 
     OptionOpaqueDataTuplesPtr data_tuple;
     ASSERT_NO_THROW(
@@ -555,9 +549,9 @@ TEST(OptionOpaqueDataTuples, unpack4NoTuple) {
 // data is parsed correctly when tuple's length field is coded on 2 octets.
 TEST(OptionOpaqueDataTuples, unpack4NoTuple_with_ltf) {
     // Prepare data to decode.
-    const uint8_t buf_data[] = {
-    };
-    OptionBuffer buf(buf_data, buf_data + sizeof(buf_data));
+    std::vector<uint8_t> buf_vector;
+    const uint8_t* buf_data = buf_vector.data();
+    OptionBuffer buf(buf_data, buf_data + buf_vector.size());
 
     OptionOpaqueDataTuplesPtr data_tuple;
     ASSERT_NO_THROW(
@@ -575,9 +569,9 @@ TEST(OptionOpaqueDataTuples, unpack4NoTuple_with_ltf) {
 // data is parsed correctly.
 TEST(OptionOpaqueDataTuples, unpack6NoTuple) {
     // Prepare data to decode.
-    const uint8_t buf_data[] = {
-    };
-    OptionBuffer buf(buf_data, buf_data + sizeof(buf_data));
+    std::vector<uint8_t> buf_vector;
+    const uint8_t* buf_data = buf_vector.data();
+    OptionBuffer buf(buf_data, buf_data + buf_vector.size());
 
     OptionOpaqueDataTuplesPtr data_tuple;
     ASSERT_NO_THROW(
