@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -46,14 +46,17 @@
 /// communications that is independent of the IO layer mechanisms.  While the
 /// type and details of the IO mechanism are not relevant to either class, it
 /// is presumed to use isc::asiolink library for asynchronous event processing.
-///
 
 #include <asiolink/io_address.h>
 #include <asiolink/io_service.h>
 #include <dhcp_ddns/ncr_msg.h>
 #include <exceptions/exceptions.h>
 
+#include <boost/scoped_ptr.hpp>
+#include <boost/enable_shared_from_this.hpp>
+
 #include <deque>
+#include <mutex>
 
 namespace isc {
 namespace dhcp_ddns {
@@ -68,7 +71,7 @@ enum NameChangeProtocol {
   NCR_TCP
 };
 
-/// @brief Function which converts labels to  NameChangeProtocol enum values.
+/// @brief Function which converts text labels to @ref NameChangeProtocol enums.
 ///
 /// @param protocol_str text to convert to an enum.
 /// Valid string values: "UDP", "TCP"
@@ -79,7 +82,7 @@ enum NameChangeProtocol {
 /// enum value.
 extern NameChangeProtocol stringToNcrProtocol(const std::string& protocol_str);
 
-/// @brief Function which converts NameChangeProtocol enums to text labels.
+/// @brief Function which converts @ref NameChangeProtocol enums to text labels.
 ///
 /// @param protocol enum value to convert to label
 ///
@@ -108,7 +111,6 @@ public:
         isc::Exception(file, line, what) { };
 };
 
-
 /// @brief Abstract interface for receiving NameChangeRequests.
 ///
 /// NameChangeListener provides the means to:
@@ -167,10 +169,10 @@ public:
 
     /// @brief Defines the outcome of an asynchronous NCR receive
     enum Result {
-      SUCCESS,
-      TIME_OUT,
-      STOPPED,
-      ERROR
+        SUCCESS,
+        TIME_OUT,
+        STOPPED,
+        ERROR
     };
 
     /// @brief Abstract class for defining application layer receive callbacks.
@@ -178,8 +180,9 @@ public:
     /// Applications which will receive NameChangeRequests must provide a
     /// derivation of this class to the listener constructor in order to
     /// receive NameChangeRequests.
-    class RequestReceiveHandler {
-      public:
+    class RequestReceiveHandler : public boost::enable_shared_from_this<RequestReceiveHandler> {
+    public:
+
         /// @brief Function operator implementing a NCR receive callback.
         ///
         /// This method allows the application to receive the inbound
@@ -190,19 +193,23 @@ public:
         /// @param ncr is a pointer to the newly received NameChangeRequest if
         /// result is NameChangeListener::SUCCESS.  It is indeterminate other
         /// wise.
+        ///
         /// @throw This method MUST NOT throw.
-        virtual void operator ()(const Result result,
-                                 NameChangeRequestPtr& ncr) = 0;
+        virtual void operator()(const Result result,
+                                NameChangeRequestPtr& ncr) = 0;
 
         virtual ~RequestReceiveHandler() {
         }
     };
 
+    /// @brief Defines a smart pointer to an instance of a request receive handler.
+    typedef boost::shared_ptr<RequestReceiveHandler> RequestReceiveHandlerPtr;
+
     /// @brief Constructor
     ///
     /// @param recv_handler is a pointer the application layer handler to be
     /// invoked each time a NCR is received or a receive error occurs.
-    NameChangeListener(RequestReceiveHandler& recv_handler);
+    NameChangeListener(RequestReceiveHandlerPtr recv_handler);
 
     /// @brief Destructor
     virtual ~NameChangeListener() {
@@ -218,7 +225,7 @@ public:
     ///
     /// @throw NcrListenError if the listener is already "listening" or
     /// in the event the open or doReceive methods fail.
-    void startListening(isc::asiolink::IOService& io_service);
+    void startListening(const isc::asiolink::IOServicePtr& io_service);
 
     /// @brief Closes the IO source and stops listen logic.
     ///
@@ -227,6 +234,7 @@ public:
     void stopListening();
 
 protected:
+
     /// @brief Initiates an asynchronous receive
     ///
     /// Sets context information to indicate that IO is in progress and invokes
@@ -274,7 +282,7 @@ protected:
     ///
     /// @throw If the implementation encounters an error it MUST
     /// throw it as an isc::Exception or derivative.
-    virtual void open(isc::asiolink::IOService& io_service) = 0;
+    virtual void open(const isc::asiolink::IOServicePtr& io_service) = 0;
 
     /// @brief Abstract method which closes the IO source.
     ///
@@ -296,12 +304,15 @@ protected:
     virtual void doReceive() = 0;
 
 public:
+
     /// @brief Returns true if the listener is listening, false otherwise.
     ///
     /// A true value indicates that the IO source has been opened successfully,
     /// and that receive loop logic is active.  This implies that closing the
     /// IO source will interrupt that operation, resulting in a callback
     /// invocation.
+    ///
+    /// @return The listening mode.
     bool amListening() const {
         return (listening_);
     }
@@ -315,6 +326,8 @@ public:
     /// deleted while there is an IO call pending.  This can result in the
     /// IO service attempting to invoke methods on objects that are no longer
     /// valid.
+    ///
+    /// @return The IO pending flag.
     bool isIoPending() const {
         return (io_pending_);
     }
@@ -337,13 +350,12 @@ private:
     bool io_pending_;
 
     /// @brief Application level NCR receive completion handler.
-    RequestReceiveHandler& recv_handler_;
+    RequestReceiveHandlerPtr recv_handler_;
 };
 
 /// @brief Defines a smart pointer to an instance of a listener.
 typedef boost::shared_ptr<NameChangeListener> NameChangeListenerPtr;
 
-
 /// @brief Thrown when a NameChangeSender encounters an error.
 class NcrSenderError : public isc::Exception {
 public:
@@ -372,7 +384,6 @@ public:
         isc::Exception(file, line, what) { };
 };
 
-
 /// @brief Abstract interface for sending NameChangeRequests.
 ///
 /// NameChangeSender provides the means to:
@@ -476,8 +487,9 @@ public:
     /// Applications which will send NameChangeRequests must provide a
     /// derivation of this class to the sender constructor in order to
     /// receive send outcome notifications.
-    class RequestSendHandler {
-      public:
+    class RequestSendHandler : public boost::enable_shared_from_this<RequestSendHandler> {
+    public:
+
         /// @brief Function operator implementing a NCR send callback.
         ///
         /// This method allows the application to receive the outcome of
@@ -496,6 +508,9 @@ public:
         }
     };
 
+    /// @brief Defines a smart pointer to an instance of a request send handler.
+    typedef boost::shared_ptr<RequestSendHandler> RequestSendHandlerPtr;
+
     /// @brief Constructor
     ///
     /// @param send_handler is a pointer the application layer handler to be
@@ -503,8 +518,8 @@ public:
     /// @param send_queue_max is the maximum number of entries allowed in the
     /// send queue.  Once the maximum number is reached, all calls to
     /// sendRequest will fail with an exception.
-    NameChangeSender(RequestSendHandler& send_handler,
-            size_t send_queue_max = MAX_QUEUE_DEFAULT);
+    NameChangeSender(RequestSendHandlerPtr send_handler,
+                     size_t send_queue_max = MAX_QUEUE_DEFAULT);
 
     /// @brief Destructor
     virtual ~NameChangeSender() {
@@ -519,7 +534,7 @@ public:
     ///
     /// @throw NcrSenderError if the sender is already "sending" or
     /// NcrSenderOpenError if the open fails.
-    void startSending(isc::asiolink::IOService & io_service);
+    void startSending(const isc::asiolink::IOServicePtr& io_service);
 
     /// @brief Closes the IO sink and stops send logic.
     ///
@@ -550,7 +565,7 @@ public:
     ///
     /// @throw NcrSenderError if either sender is in send mode, if the number of
     /// messages in the source sender's queue is larger than this sender's
-    /// maxium queue size, or if this sender's queue is not empty.
+    /// maximum queue size, or if this sender's queue is not empty.
     void assumeQueue(NameChangeSender& source_sender);
 
     /// @brief Returns a file descriptor suitable for use with select
@@ -573,13 +588,61 @@ public:
     /// @return true if the sender has at IO ready, false otherwise.
     virtual bool ioReady() = 0;
 
+private:
+
+    /// @brief Prepares the IO for transmission in a thread safe context.
+    ///
+    /// @param io_service is the IOService that will handle IO event processing.
+    void startSendingInternal(const isc::asiolink::IOServicePtr& io_service);
+
+    /// @brief Queues the given request to be sent in a thread safe context.
+    ///
+    /// @param ncr is the NameChangeRequest to send.
+    ///
+    /// @throw NcrSenderQueueFull if the send queue has reached capacity.
+    void sendRequestInternal(NameChangeRequestPtr& ncr);
+
+    /// @brief Move all queued requests from a given sender into the send queue
+    /// in a thread safe context.
+    ///
+    /// @param source_sender from whom the queued messages will be taken
+    ///
+    /// @throw NcrSenderError if this sender's queue is not empty.
+    void assumeQueueInternal(NameChangeSender& source_sender);
+
+    /// @brief Calls the NCR send completion handler registered with the
+    /// sender in a thread safe context.
+    ///
+    /// @param result contains that send outcome status.
+    void invokeSendHandlerInternal(const NameChangeSender::Result result);
+
+    /// @brief Removes the request at the front of the send queue in a thread
+    /// safe context.
+    void skipNextInternal();
+
+    /// @brief Returns the number of entries currently in the send queue in a
+    /// thread safe context.
+    ///
+    /// @return the queue size.
+    size_t getQueueSizeInternal() const;
+
+    /// @brief Returns the entry at a given position in the queue in a thread
+    /// safe context.
+    ///
+    /// @return Pointer reference to the queue entry.
+    ///
+    /// @throw NcrSenderError if the given index is beyond the
+    /// end of the queue.
+    const NameChangeRequestPtr& peekAtInternal(const size_t index) const;
+
 protected:
-    /// @brief Dequeues and sends the next request on the send queue.
+
+    /// @brief Dequeues and sends the next request on the send queue in a thread
+    /// safe context.
     ///
     /// If there is already a send in progress just return. If there is not
     /// a send in progress and the send queue is not empty the grab the next
     /// message on the front of the queue and call doSend().
-    ///
     void sendNext();
 
     /// @brief Calls the NCR send completion handler registered with the
@@ -587,8 +650,8 @@ protected:
     ///
     /// This is the hook by which the sender's caller's NCR send completion
     /// handler is called.  This method MUST be invoked by the derivation's
-    /// implementation of doSend.   Note that if the send was a success,
-    /// the entry at the front of the queue is removed from the queue.
+    /// implementation of doSend.  Note that if the send was a success, the
+    /// entry at the front of the queue is removed from the queue.
     /// If not we leave it there so we can retry it.  After we invoke the
     /// handler we clear the pending ncr value and queue up the next send.
     ///
@@ -619,7 +682,7 @@ protected:
     ///
     /// @throw If the implementation encounters an error it MUST
     /// throw it as an isc::Exception or derivative.
-    virtual void open(isc::asiolink::IOService& io_service) = 0;
+    virtual void open(const isc::asiolink::IOServicePtr& io_service) = 0;
 
     /// @brief Abstract method which closes the IO sink.
     ///
@@ -643,6 +706,7 @@ protected:
     virtual void doSend(NameChangeRequestPtr& ncr) = 0;
 
 public:
+
     /// @brief Removes the request at the front of the send queue
     ///
     /// This method can be used to avoid further retries of a failed
@@ -653,7 +717,7 @@ public:
     /// It is presumed that sends will only fail due to some sort of
     /// communications issue. In the unlikely event that a request is
     /// somehow tainted and causes an send failure based on its content,
-    /// this method provides a means to remove th message.
+    /// this method provides a means to remove the message.
     void skipNext();
 
     /// @brief Flushes all entries in the send queue
@@ -661,6 +725,7 @@ public:
     /// This method can be used to discard all of the NCRs currently in the
     /// the send queue.  Note it may not be called while the sender is in
     /// the sending state.
+    ///
     /// @throw NcrSenderError if called and sender is in sending state.
     void clearSendQueue();
 
@@ -668,6 +733,8 @@ public:
     ///
     /// A true value indicates that the IO sink has been opened successfully,
     /// and that send loop logic is active.
+    ///
+    /// @return The send mode.
     bool amSending() const {
         return (sending_);
     }
@@ -676,11 +743,13 @@ public:
     ///
     /// A true value indicates that a request is actively in the process of
     /// being delivered.
-    bool isSendInProgress() const {
-        return ((ncr_to_send_) ? true : false);
-    }
+    ///
+    /// @return The send in progress flag.
+    bool isSendInProgress() const;
 
     /// @brief Returns the maximum number of entries allowed in the send queue.
+    ///
+    /// @return The queue maximum size.
     size_t getQueueMaxSize() const  {
         return (send_queue_max_);
     }
@@ -696,13 +765,14 @@ public:
     void setQueueMaxSize(const size_t new_max);
 
     /// @brief Returns the number of entries currently in the send queue.
-    size_t getQueueSize() const {
-        return (send_queue_.size());
-    }
+    ///
+    /// @return The queue size.
+    size_t getQueueSize() const;
 
     /// @brief Returns the entry at a given position in the queue.
     ///
     /// Note that the entry is not removed from the queue.
+    ///
     /// @param index the index of the entry in the queue to fetch.
     /// Valid values are 0 (front of the queue) to (queue size - 1).
     ///
@@ -731,16 +801,19 @@ public:
     /// By running only one handler at time, we ensure that NCR IO activity
     /// doesn't starve other processing.  It is unclear how much of a real
     /// threat this poses but for now it is best to err on the side of caution.
-    ///
     virtual void runReadyIO();
 
 protected:
+
     /// @brief Returns a reference to the send queue.
+    ///
+    /// @return The send queue.
     SendQueue& getSendQueue() {
         return (send_queue_);
     }
 
 private:
+
     /// @brief Sets the sending indicator to the given value.
     ///
     /// Note, this method is private as it is used the base class is solely
@@ -748,14 +821,24 @@ private:
     ///
     /// @param value is the new value to assign to the indicator.
     void setSending(bool value) {
-            sending_ = value;
+        sending_ = value;
     }
 
+protected:
+
+    /// @brief Pointer to the IOService currently being used by the sender.
+    /// @note We need to remember the io_service but we receive it by
+    /// reference.  Use a raw pointer to store it.  This value should never be
+    /// exposed and is only valid while in send mode.
+    asiolink::IOServicePtr io_service_;
+
+private:
+
     /// @brief Boolean indicator which tracks sending status.
     bool sending_;
 
-    /// @brief A pointer to regisetered send completion handler.
-    RequestSendHandler& send_handler_;
+    /// @brief A pointer to registered send completion handler.
+    RequestSendHandlerPtr send_handler_;
 
     /// @brief Maximum number of entries permitted in the send queue.
     size_t send_queue_max_;
@@ -766,17 +849,14 @@ private:
     /// @brief Pointer to the request which is in the process of being sent.
     NameChangeRequestPtr ncr_to_send_;
 
-    /// @brief Pointer to the IOService currently being used by the sender.
-    /// @note We need to remember the io_service but we receive it by
-    /// reference.  Use a raw pointer to store it.  This value should never be
-    /// exposed and is only valid while in send mode.
-    asiolink::IOService* io_service_;
+    /// @brief The mutex used to protect internal state.
+    const boost::scoped_ptr<std::mutex> mutex_;
 };
 
 /// @brief Defines a smart pointer to an instance of a sender.
 typedef boost::shared_ptr<NameChangeSender> NameChangeSenderPtr;
 
-} // namespace isc::dhcp_ddns
-} // namespace isc
+}  // namespace dhcp_ddns
+}  // namespace isc
 
 #endif
