@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2019-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,6 +13,7 @@
 #include <dhcpsrv/parsers/simple_parser4.h>
 #include <hooks/callout_handle.h>
 #include <hooks/hooks_manager.h>
+#include <boost/foreach.hpp>
 
 using namespace isc::db;
 using namespace isc::data;
@@ -48,7 +49,11 @@ CBControlDHCPv4::databaseConfigApply(const BackendSelector& backend_selector,
                                      const boost::posix_time::ptime& lb_modification_time,
                                      const AuditEntryCollection& audit_entries) {
 
-    bool globals_fetched = false;
+    auto globals_fetched = false;
+    auto reconfig = audit_entries.empty();
+    auto cb_update = !reconfig;
+    auto current_cfg = CfgMgr::instance().getCurrentCfg();
+    auto staging_cfg = CfgMgr::instance().getStagingCfg();
 
     // Let's first delete all the configuration elements for which DELETE audit
     // entries are found. Although, this may break chronology of the audit in
@@ -57,13 +62,12 @@ CBControlDHCPv4::databaseConfigApply(const BackendSelector& backend_selector,
     // delete this object from the local configuration (which will fail because
     // the object does not exist) and then we will try to fetch it from the
     // database which will return no result.
-    if (!audit_entries.empty()) {
+    if (cb_update) {
 
-        auto cfg = CfgMgr::instance().getCurrentCfg();
         auto external_cfg = CfgMgr::instance().createExternalCfg();
 
         // Get audit entries for deleted global parameters.
-        const auto& index = audit_entries.get<AuditEntryObjectTypeTag>();
+        auto const& index = audit_entries.get<AuditEntryObjectTypeTag>();
         auto range = index.equal_range(boost::make_tuple("dhcp4_global_parameter",
                                                          AuditEntry::ModificationType::DELETE));
         if (range.first != range.second) {
@@ -75,7 +79,7 @@ CBControlDHCPv4::databaseConfigApply(const BackendSelector& backend_selector,
             // database query and the number of global parameters is small.
             data::StampedValueCollection globals;
             globals = getMgr().getPool()->getAllGlobalParameters4(backend_selector, server_selector);
-            addGlobalsToConfig(external_cfg, globals);
+            translateAndAddGlobalsToConfig(external_cfg, globals);
 
             // Add defaults.
             external_cfg->applyDefaultsConfiguredGlobals(SimpleParser4::GLOBAL4_DEFAULTS);
@@ -85,7 +89,7 @@ CBControlDHCPv4::databaseConfigApply(const BackendSelector& backend_selector,
 
             // Now that we successfully fetched the new global parameters, let's
             // remove existing ones and merge them into the current configuration.
-            cfg->clearConfiguredGlobals();
+            current_cfg->clearConfiguredGlobals();
             CfgMgr::instance().mergeIntoCurrentCfg(external_cfg->getSequence());
             globals_fetched = true;
         }
@@ -96,37 +100,37 @@ CBControlDHCPv4::databaseConfigApply(const BackendSelector& backend_selector,
             // audit entry is found.
             range = index.equal_range(boost::make_tuple("dhcp4_option_def",
                                                         AuditEntry::ModificationType::DELETE));
-            for (auto entry = range.first; entry != range.second; ++entry) {
-                cfg->getCfgOptionDef()->del((*entry)->getObjectId());
+            BOOST_FOREACH(auto const& entry, range) {
+                current_cfg->getCfgOptionDef()->del(entry->getObjectId());
             }
 
             // Repeat the same for other configuration elements.
 
             range = index.equal_range(boost::make_tuple("dhcp4_options",
                                                         AuditEntry::ModificationType::DELETE));
-            for (auto entry = range.first; entry != range.second; ++entry) {
-                cfg->getCfgOption()->del((*entry)->getObjectId());
+            BOOST_FOREACH(auto const& entry, range) {
+                current_cfg->getCfgOption()->del(entry->getObjectId());
             }
 
             range = index.equal_range(boost::make_tuple("dhcp4_client_class",
                                                         AuditEntry::ModificationType::DELETE));
-            for (auto entry = range.first; entry != range.second; ++entry) {
-                cfg->getClientClassDictionary()->removeClass((*entry)->getObjectId());
+            BOOST_FOREACH(auto const& entry, range) {
+                current_cfg->getClientClassDictionary()->removeClass(entry->getObjectId());
             }
 
             range = index.equal_range(boost::make_tuple("dhcp4_shared_network",
                                                         AuditEntry::ModificationType::DELETE));
-            for (auto entry = range.first; entry != range.second; ++entry) {
-                cfg->getCfgSharedNetworks4()->del((*entry)->getObjectId());
+            BOOST_FOREACH(auto const& entry, range) {
+                current_cfg->getCfgSharedNetworks4()->del(entry->getObjectId());
             }
 
             range = index.equal_range(boost::make_tuple("dhcp4_subnet",
                                                         AuditEntry::ModificationType::DELETE));
-            for (auto entry = range.first; entry != range.second; ++entry) {
+            BOOST_FOREACH(auto const& entry, range) {
                 // If the deleted subnet belongs to a shared network and the
                 // shared network is not being removed, we need to detach the
                 // subnet from the shared network.
-                auto subnet = cfg->getCfgSubnets4()->getBySubnetId((*entry)->getObjectId());
+                auto subnet = current_cfg->getCfgSubnets4()->getBySubnetId(entry->getObjectId());
                 if (subnet) {
                     // Check if the subnet belongs to a shared network.
                     SharedNetwork4Ptr network;
@@ -136,7 +140,7 @@ CBControlDHCPv4::databaseConfigApply(const BackendSelector& backend_selector,
                         network->del(subnet->getID());
                     }
                     // Actually delete the subnet from the configuration.
-                    cfg->getCfgSubnets4()->del((*entry)->getObjectId());
+                    current_cfg->getCfgSubnets4()->del(entry->getObjectId());
                 }
             }
 
@@ -149,58 +153,60 @@ CBControlDHCPv4::databaseConfigApply(const BackendSelector& backend_selector,
     }
 
     // Create the external config into which we'll fetch backend config data.
-    SrvConfigPtr external_cfg = CfgMgr::instance().createExternalCfg();
+    auto external_cfg = CfgMgr::instance().createExternalCfg();
 
     // First let's fetch the globals and add them to external config.
     AuditEntryCollection updated_entries;
-    if (!globals_fetched && !audit_entries.empty()) {
-        updated_entries = fetchConfigElement(audit_entries, "dhcp4_global_parameter");
-    }
-    if (!globals_fetched && (audit_entries.empty() || !updated_entries.empty())) {
-        data::StampedValueCollection globals;
-        globals = getMgr().getPool()->getModifiedGlobalParameters4(backend_selector, server_selector,
-                                                                   lb_modification_time);
-        addGlobalsToConfig(external_cfg, globals);
-        globals_fetched = true;
+    if (!globals_fetched) {
+        if (cb_update) {
+            updated_entries = fetchConfigElement(audit_entries, "dhcp4_global_parameter");
+        }
+        if (reconfig || !updated_entries.empty()) {
+            data::StampedValueCollection globals;
+            globals = getMgr().getPool()->getModifiedGlobalParameters4(backend_selector, server_selector,
+                                                                       lb_modification_time);
+            translateAndAddGlobalsToConfig(external_cfg, globals);
+            globals_fetched = true;
+        }
     }
 
     // Now we fetch the option definitions and add them.
-    if (!audit_entries.empty()) {
+    if (cb_update) {
         updated_entries = fetchConfigElement(audit_entries, "dhcp4_option_def");
     }
-    if (audit_entries.empty() || !updated_entries.empty()) {
+    if (reconfig || !updated_entries.empty()) {
         OptionDefContainer option_defs =
             getMgr().getPool()->getModifiedOptionDefs4(backend_selector, server_selector,
                                                        lb_modification_time);
-        for (auto option_def = option_defs.begin(); option_def != option_defs.end(); ++option_def) {
-            if (!audit_entries.empty() && !hasObjectId(updated_entries, (*option_def)->getId())) {
+        for (auto const& option_def : option_defs) {
+            if (!audit_entries.empty() && !hasObjectId(updated_entries, option_def->getId())) {
                 continue;
             }
-            external_cfg->getCfgOptionDef()->add(*option_def);
+            external_cfg->getCfgOptionDef()->add(option_def);
         }
     }
 
     // Next fetch the options. They are returned as a container of OptionDescriptors.
-    if (!audit_entries.empty()) {
+    if (cb_update) {
         updated_entries = fetchConfigElement(audit_entries, "dhcp4_options");
     }
-    if (audit_entries.empty() || !updated_entries.empty()) {
+    if (reconfig || !updated_entries.empty()) {
         OptionContainer options = getMgr().getPool()->getModifiedOptions4(backend_selector,
                                                                           server_selector,
                                                                           lb_modification_time);
-        for (auto option = options.begin(); option != options.end(); ++option) {
-            if (!audit_entries.empty() && !hasObjectId(updated_entries, (*option).getId())) {
+        for (auto const& option : options) {
+            if (!audit_entries.empty() && !hasObjectId(updated_entries, option.getId())) {
                 continue;
             }
-            external_cfg->getCfgOption()->add((*option), (*option).space_name_);
+            external_cfg->getCfgOption()->add(option, option.space_name_);
         }
     }
 
     // Fetch client classes. They are returned in a ClientClassDictionary.
-    if (!audit_entries.empty()) {
+    if (cb_update) {
         updated_entries = fetchConfigElement(audit_entries, "dhcp4_client_class");
     }
-    if (audit_entries.empty() || !updated_entries.empty()) {
+    if (reconfig || !updated_entries.empty()) {
         ClientClassDictionary client_classes = getMgr().getPool()->getAllClientClasses4(backend_selector,
                                                                                         server_selector);
         // Match expressions are not initialized for classes returned from the config backend.
@@ -209,61 +215,103 @@ CBControlDHCPv4::databaseConfigApply(const BackendSelector& backend_selector,
 
         // Class options also need to be created when returned from the config backend.
         client_classes.createOptions(external_cfg->getCfgOptionDef());
+        client_classes.encapsulateOptions();
 
         external_cfg->setClientClassDictionary(boost::make_shared<ClientClassDictionary>(client_classes));
     }
 
+    // Allocator selection at the global level can affect subnets and shared networks
+    // for which the allocator hasn't been specified explicitly. Let's see if the
+    // allocator has been specified at the global level.
+    std::string global_allocator;
+    auto allocator = external_cfg->getConfiguredGlobal(CfgGlobals::ALLOCATOR);
+    if (allocator && (allocator->getType() == Element::string)) {
+       global_allocator = allocator->stringValue();
+    }
+
+    // If we're fetching the changes from the config backend we also want
+    // to see if the global allocator has changed. Let's get the currently
+    // used allocator too.
+    auto allocator_changed = false;
+    // We're only affected by the allocator change if this is the update from
+    // the configuration backend.
+    if (cb_update) {
+        auto allocator = CfgMgr::instance().getCurrentCfg()->getConfiguredGlobal(CfgGlobals::ALLOCATOR);
+        if (allocator && (allocator->getType() == Element::string)) {
+            allocator_changed = (global_allocator != allocator->stringValue());
+        }
+    }
+
     // Now fetch the shared networks.
-    if (!audit_entries.empty()) {
+    if (cb_update) {
         updated_entries = fetchConfigElement(audit_entries, "dhcp4_shared_network");
     }
-    if (audit_entries.empty() || !updated_entries.empty()) {
-        SharedNetwork4Collection networks =
-            getMgr().getPool()->getModifiedSharedNetworks4(backend_selector, server_selector,
-                                                           lb_modification_time);
-        for (auto network = networks.begin(); network != networks.end(); ++network) {
-            if (!audit_entries.empty() && !hasObjectId(updated_entries, (*network)->getId())) {
-                continue;
-            }
-            // In order to take advantage of the dynamic inheritance of global
-            // parameters to a shared network we need to set a callback function
-            // for each network to allow for fetching global parameters.
-            (*network)->setFetchGlobalsFn([] () -> ConstElementPtr {
-                return (CfgMgr::instance().getCurrentCfg()->getConfiguredGlobals());
-            });
-            external_cfg->getCfgSharedNetworks4()->add((*network));
+    SharedNetwork4Collection networks;
+    if (allocator_changed || reconfig) {
+        // A change of the allocator or the server reconfiguration can affect all
+        // shared networks. Get all shared networks.
+        networks = getMgr().getPool()->getAllSharedNetworks4(backend_selector, server_selector);
+
+    } else if (!updated_entries.empty()) {
+        // An update from the config backend when the global allocator hasn't changed
+        // means that we only need to handle the modified subnets.
+        networks = getMgr().getPool()->getModifiedSharedNetworks4(backend_selector, server_selector,
+                                                                  lb_modification_time);
+    }
+    // Iterate over all shared networks that may require reconfiguration.
+    for (auto const& network : networks) {
+        if (!allocator_changed && cb_update && !hasObjectId(updated_entries, network->getId())) {
+            continue;
         }
+        // In order to take advantage of the dynamic inheritance of global
+        // parameters to a shared network we need to set a callback function
+        // for each network to allow for fetching global parameters.
+        network->setFetchGlobalsFn([] () -> ConstCfgGlobalsPtr {
+             return (CfgMgr::instance().getCurrentCfg()->getConfiguredGlobals());
+        });
+        network->setDefaultAllocatorType(global_allocator);
+        external_cfg->getCfgSharedNetworks4()->add(network);
     }
 
-    // Next we fetch subnets.
-    if (!audit_entries.empty()) {
+    // Next, fetch the subnets.
+    if (cb_update) {
         updated_entries = fetchConfigElement(audit_entries, "dhcp4_subnet");
     }
-    if (audit_entries.empty() || !updated_entries.empty()) {
-        Subnet4Collection subnets = getMgr().getPool()->getModifiedSubnets4(backend_selector,
-                                                                            server_selector,
-                                                                            lb_modification_time);
-        for (auto subnet = subnets.begin(); subnet != subnets.end(); ++subnet) {
-            if (!audit_entries.empty() && !hasObjectId(updated_entries, (*subnet)->getID())) {
-                continue;
-            }
-            // In order to take advantage of the dynamic inheritance of global
-            // parameters to a subnet we need to set a callback function for each
-            // subnet to allow for fetching global parameters.
-            (*subnet)->setFetchGlobalsFn([] () -> ConstElementPtr {
-                return (CfgMgr::instance().getCurrentCfg()->getConfiguredGlobals());
-            });
-            external_cfg->getCfgSubnets4()->add((*subnet));
+    Subnet4Collection subnets;
+    if (allocator_changed || reconfig) {
+        // A change of the allocator or the server reconfiguration can affect all
+        // subnets. Get all subnets.
+        subnets = getMgr().getPool()->getAllSubnets4(backend_selector, server_selector);
+
+    }  else if (!updated_entries.empty()) {
+        // An update from the config backend when the global allocator hasn't changed
+        // means that we only need to handle the modified subnets.
+        subnets = getMgr().getPool()->getModifiedSubnets4(backend_selector,
+                                                          server_selector,
+                                                          lb_modification_time);
+    }
+    // Iterate over all subnets that may require reconfiguration.
+    for (auto const& subnet : subnets) {
+        if (!allocator_changed && cb_update && !hasObjectId(updated_entries, subnet->getID())) {
+            continue;
         }
+        // In order to take advantage of the dynamic inheritance of global
+        // parameters to a subnet we need to set a callback function for each
+        // subnet to allow for fetching global parameters.
+        subnet->setFetchGlobalsFn([] () -> ConstCfgGlobalsPtr {
+            return (CfgMgr::instance().getCurrentCfg()->getConfiguredGlobals());
+        });
+        subnet->setDefaultAllocatorType(global_allocator);
+        external_cfg->getCfgSubnets4()->add(subnet);
     }
 
-    if (audit_entries.empty()) {
+    if (reconfig) {
         // If we're configuring the server after startup, we do not apply the
         // ip-reservations-unique setting here. It will be applied when the
         // configuration is committed.
-        auto const& cfg = CfgMgr::instance().getStagingCfg();
-        external_cfg->sanityChecksLifetime(*cfg, "valid-lifetime");
+        external_cfg->sanityChecksLifetime(*staging_cfg, "valid-lifetime");
         CfgMgr::instance().mergeIntoStagingCfg(external_cfg->getSequence());
+
     } else {
         if (globals_fetched) {
             // ip-reservations-unique parameter requires special handling because
@@ -282,13 +330,14 @@ CBControlDHCPv4::databaseConfigApply(const BackendSelector& backend_selector,
                 external_cfg->addConfiguredGlobal("ip-reservations-unique", Element::create(true));
             }
         }
-        auto const& cfg = CfgMgr::instance().getCurrentCfg();
-        external_cfg->sanityChecksLifetime(*cfg, "valid-lifetime");
+        external_cfg->sanityChecksLifetime(*current_cfg, "valid-lifetime");
         CfgMgr::instance().mergeIntoCurrentCfg(external_cfg->getSequence());
+        CfgMgr::instance().getCurrentCfg()->getCfgSubnets4()->initAllocatorsAfterConfigure();
     }
+
     LOG_INFO(dhcpsrv_logger, DHCPSRV_CFGMGR_CONFIG4_MERGED);
 
-    if (!audit_entries.empty() &&
+    if (cb_update &&
         HooksManager::calloutsPresent(hooks_.hook_index_cb4_updated_)) {
         CalloutHandlePtr callout_handle = HooksManager::createCalloutHandle();
 
