@@ -1,11 +1,16 @@
-// Copyright (C) 2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+#include <config.h>
+
 #include <dhcp/dhcp6.h>
 #include <dhcpsrv/cfg_rsoo.h>
+#include <boost/lexical_cast.hpp>
+
+using namespace isc::data;
 
 namespace isc {
 namespace dhcp {
@@ -33,6 +38,16 @@ CfgRSOO::enable(const uint16_t code) {
     }
 }
 
+ElementPtr
+CfgRSOO::toElement() const {
+    ElementPtr result = Element::createList();
+    // We can use LibDHCP::getOptionDef(DHCP6_OPTION_SPACE, *opt) too...
+    for (auto const& opt : rsoo_options_) {
+        const std::string& code = boost::lexical_cast<std::string>(opt);
+        result->add(Element::create(code));
+    }
+    return (result);
+}
 
 }
 }
