@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -200,15 +200,49 @@ getModifiedGlobalParameters4(const db::BackendSelector& backend_selector,
     return (parameters);
 }
 
+ClientClassDefPtr
+ConfigBackendPoolDHCPv4::getClientClass4(const BackendSelector& backend_selector,
+                                         const ServerSelector& server_selector,
+                                         const std::string& name) const {
+    ClientClassDefPtr client_class;
+    getPropertyPtrConst<ClientClassDefPtr, const std::string&>
+        (&ConfigBackendDHCPv4::getClientClass4, backend_selector, server_selector,
+         client_class, name);
+    return (client_class);
+}
+
+ClientClassDictionary
+ConfigBackendPoolDHCPv4::getAllClientClasses4(const BackendSelector& backend_selector,
+                                              const ServerSelector& server_selector) const {
+    ClientClassDictionary client_classes;
+    getAllPropertiesConst<ClientClassDictionary>
+        (&ConfigBackendDHCPv4::getAllClientClasses4, backend_selector, server_selector,
+         client_classes);
+    return (client_classes);
+
+}
+
+ClientClassDictionary
+ConfigBackendPoolDHCPv4::getModifiedClientClasses4(const BackendSelector& backend_selector,
+                                                   const ServerSelector& server_selector,
+                                                   const boost::posix_time::ptime& modification_time) const {
+    ClientClassDictionary client_classes;
+    getMultiplePropertiesConst<ClientClassDictionary, const boost::posix_time::ptime&>
+        (&ConfigBackendDHCPv4::getModifiedClientClasses4, backend_selector, server_selector,
+         client_classes, modification_time);
+    return (client_classes);
+}
+
 AuditEntryCollection
 ConfigBackendPoolDHCPv4::
 getRecentAuditEntries(const db::BackendSelector& backend_selector,
                       const db::ServerSelector& server_selector,
-                      const boost::posix_time::ptime& modification_time) const {
+                      const boost::posix_time::ptime& modification_time,
+                      const uint64_t& modification_id) const {
     AuditEntryCollection audit_entries;
     getMultiplePropertiesConst<AuditEntryCollection, const boost::posix_time::ptime&>
         (&ConfigBackendDHCPv4::getRecentAuditEntries, backend_selector,
-         server_selector, audit_entries, modification_time);
+         server_selector, audit_entries, modification_time, modification_id);
     return (audit_entries);
 }
 
@@ -308,6 +342,16 @@ ConfigBackendPoolDHCPv4::createUpdateGlobalParameter4(const BackendSelector& bac
          server_selector, value);
 }
 
+void
+ConfigBackendPoolDHCPv4::createUpdateClientClass4(const BackendSelector& backend_selector,
+                                                  const ServerSelector& server_selector,
+                                                  const ClientClassDefPtr& client_class,
+                                                  const std::string& follow_class_name) {
+    createUpdateDeleteProperty<void, const ClientClassDefPtr&, const std::string&>
+        (&ConfigBackendDHCPv4::createUpdateClientClass4, backend_selector,
+         server_selector, client_class, follow_class_name);
+}
+
 void
 ConfigBackendPoolDHCPv4::createUpdateServer4(const BackendSelector& backend_selector,
                                              const ServerPtr& server) {
@@ -446,6 +490,22 @@ ConfigBackendPoolDHCPv4::deleteAllGlobalParameters4(const BackendSelector& backe
              server_selector));
 }
 
+uint64_t
+ConfigBackendPoolDHCPv4::deleteClientClass4(const BackendSelector& backend_selector,
+                                            const ServerSelector& server_selector,
+                                            const std::string& name) {
+    return (createUpdateDeleteProperty<uint64_t, const std::string&>
+            (&ConfigBackendDHCPv4::deleteClientClass4, backend_selector,
+             server_selector, name));
+}
+
+uint64_t
+ConfigBackendPoolDHCPv4::deleteAllClientClasses4(const BackendSelector& backend_selector,
+                                                 const ServerSelector& server_selector) {
+    return (createUpdateDeleteProperty<uint64_t>
+            (&ConfigBackendDHCPv4::deleteAllClientClasses4, backend_selector, server_selector));
+}
+
 uint64_t
 ConfigBackendPoolDHCPv4::deleteServer4(const BackendSelector& backend_selector,
                                        const ServerTag& server_tag) {
