@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -14,7 +14,9 @@
 #include <asiolink/io_address.h>
 #include <dhcp_ddns/ncr_io.h>
 #include <dhcpsrv/d2_client_cfg.h>
+#include <dhcpsrv/srv_config.h>
 #include <exceptions/exceptions.h>
+#include <util/str.h>
 
 #include <boost/algorithm/string.hpp>
 #include <boost/shared_ptr.hpp>
@@ -39,8 +41,8 @@ namespace dhcp {
 /// @note Handlers are expected not to throw. In the event a handler does
 /// throw invoking code logs the exception and then swallows it.
 typedef
-boost::function<void(const dhcp_ddns::NameChangeSender::Result result,
-                     dhcp_ddns::NameChangeRequestPtr& ncr)> D2ClientErrorHandler;
+std::function<void(const dhcp_ddns::NameChangeSender::Result result,
+                   dhcp_ddns::NameChangeRequestPtr& ncr)> D2ClientErrorHandler;
 
 /// @brief D2ClientMgr isolates Kea from the details of being a D2 client.
 ///
@@ -106,9 +108,9 @@ public:
     /// @brief Determines server flags based on configuration and  client flags.
     ///
     /// This method uses input values for the client's FQDN S and N flags, in
-    /// conjunction with the configuration parameters updates-enabled, override-
-    /// no-updates, and override-client-updates to determine the values that
-    /// should be used for the server's FQDN S and N flags.
+    /// conjunction with the configuration parameters updates-enabled,
+    /// ddns-override-no-updates, and ddns-override-client-updates to determine
+    /// the values that should be used for the server's FQDN S and N flags.
     /// The logic in this method is based upon RFCs 4702 and 4704, and is
     /// shown in the following truth table:
     ///
@@ -147,40 +149,48 @@ public:
     /// @param client_n  N Flag from the client's FQDN
     /// @param server_s [out] S Flag for the server's FQDN
     /// @param server_n [out] N Flag for the server's FQDN
+    /// @param ddns_params DDNS behavioral configuration parameters
     ///
     /// @throw isc::BadValue if client_s and client_n are both 1 as this is
     /// an invalid combination per RFCs.
     void analyzeFqdn(const bool client_s, const bool client_n, bool& server_s,
-                     bool& server_n) const;
+                     bool& server_n, const DdnsParams& ddns_params) const;
 
     /// @brief Builds a FQDN based on the configuration and given IP address.
     ///
-    /// Using the current values for generated-prefix, qualifying-suffix and
-    /// an IP address, this method constructs a fully qualified domain name.
+    /// Using the current values for ddns-generated-prefix,
+    /// ddns-qualifying-suffix and an IP address, this method constructs a fully
+    /// qualified domain name.
     /// It supports both IPv4 and IPv6 addresses.  The format of the name
     /// is as follows:
     ///
-    ///     <generated-prefix>-<ip address>.<qualifying-suffix>.
+    ///     <ddns-generated-prefix>-<ip address>.<ddns-qualifying-suffix>.
     ///
     /// <ip-address> is the result of IOAddress.toText() with the delimiters
     /// ('.' for IPv4 or ':' for IPv6) replaced with a hyphen, '-'.
     ///
     /// @param address IP address from which to derive the name (IPv4 or IPv6)
+    /// @param ddns_params DDNS behavioral configuration parameters
     /// @param trailing_dot A boolean value which indicates whether trailing
     /// dot should be appended (if true) or not (false).
     ///
     /// @return std::string containing the generated name.
     std::string generateFqdn(const asiolink::IOAddress& address,
+                             const DdnsParams& ddns_params,
                              const bool trailing_dot = true) const;
 
     /// @brief Adds a qualifying suffix to a given domain name
     ///
-    /// Constructs a FQDN based on the configured qualifying-suffix and
+    /// Constructs a FQDN based on the configured ddns-qualifying-suffix and
     /// a partial domain name as follows:
     ///
-    ///     <partial_name>.<qualifying-suffix>.
+    ///     <partial_name>.<ddns-qualifying-suffix>.
+    ///
+    /// Note that the qualifying suffix will only be appended if the
+    /// input name does not already end with that suffix.
     ///
     /// @param partial_name domain name to qualify
+    /// @param ddns_params DDNS behavioral configuration parameters
     /// @param trailing_dot A boolean value which when true guarantees the
     /// result will end with a "." and when false that the result will not
     /// end with a "."   Note that this rule is applied even if the qualifying
@@ -188,25 +198,28 @@ public:
     ///
     /// @return std::string containing the qualified name.
     std::string qualifyName(const std::string& partial_name,
+                            const DdnsParams& ddns_params,
                             const bool trailing_dot) const;
 
     /// @brief Set server FQDN flags based on configuration and a given FQDN
     ///
-    /// Templated wrapper around the analyzeFqdn() allowing that method to
+    /// Template wrapper around the analyzeFqdn() allowing that method to
     /// be used for either IPv4 or IPv6 processing.  This methods resets all
     /// of the flags in the response to zero and then sets the S,N, and O
     /// flags.  Any other flags are the responsibility of the invoking layer.
     ///
     /// @param fqdn FQDN option from which to read client (inbound) flags
     /// @param fqdn_resp FQDN option to update with the server (outbound) flags
+    /// @param ddns_params DDNS behavioral configuration parameters
     /// @tparam T FQDN Option class containing the FQDN data such as
     /// dhcp::Option4ClientFqdn or dhcp::Option6ClientFqdn
     template <class T>
-    void adjustFqdnFlags(const T& fqdn, T& fqdn_resp);
+    void adjustFqdnFlags(const T& fqdn, T& fqdn_resp,
+                         const DdnsParams& ddns_params);
 
     /// @brief Get directional update flags based on server FQDN flags
     ///
-    /// Templated convenience method which determines whether forward and
+    /// Template convenience method which determines whether forward and
     /// reverse updates should be performed based on a server response version
     /// of the FQDN flags. The logic is straight forward and currently not
     /// dependent upon configuration specific values:
@@ -226,18 +239,18 @@ public:
 
     /// @brief Set server FQDN name based on configuration and a given FQDN
     ///
-    /// Templated method which adjusts the domain name value and type in
+    /// Template method which adjusts the domain name value and type in
     /// a server FQDN from a client (inbound) FQDN and the current
     /// configuration.  The logic is as follows:
     ///
-    /// If replace-client-name is true or the supplied name is empty, the
+    /// If ddns-replace-client-name is true or the supplied name is empty, the
     /// server FQDN is set to ""/PARTIAL.
     ///
-    /// If replace-client-name is false and the supplied name is a partial
+    /// If ddns-replace-client-name is false and the supplied name is a partial
     /// name the server FQDN is set to the supplied name qualified by
-    /// appending the qualifying-suffix.
+    /// appending the ddns-qualifying-suffix.
     ///
-    /// If replace-client-name is false and the supplied name is a fully
+    /// If ddns-replace-client-name is false and the supplied name is a fully
     /// qualified name, set the server FQDN to the supplied name.
     ///
     /// If hostname-char-set is not empty, the inbound name will be
@@ -248,10 +261,12 @@ public:
     ///
     /// @param fqdn FQDN option from which to get client (inbound) name
     /// @param fqdn_resp FQDN option to update with the adjusted name
+    /// @param ddns_params DDNS behavioral configuration parameters
     /// @tparam T  FQDN Option class containing the FQDN data such as
     /// dhcp::Option4ClientFqdn or dhcp::Option6ClientFqdn
     template <class T>
-    void adjustDomainName(const T& fqdn, T& fqdn_resp);
+    void adjustDomainName(const T& fqdn, T& fqdn_resp,
+                          const DdnsParams& ddns_params);
 
     /// @brief Enables sending NameChangeRequests to kea-dhcp-ddns
     ///
@@ -266,15 +281,15 @@ public:
     /// @param io_service IOService to be used for sender IO event processing
     /// @warning It is up to the invoking layer to ensure the io_service
     /// instance used outlives the D2ClientMgr send mode. When the send mode
-    /// is exited, either explicitly by callind stopSender() or implicitly
-    /// through D2CLientMgr destruction, any ASIO objects such as sockets or
+    /// is exited, either explicitly by calling stopSender() or implicitly
+    /// through D2ClientMgr destruction, any ASIO objects such as sockets or
     /// timers will be closed and released.  If the io_service goes out of scope
     /// first this behavior could be unpredictable.
     ///
     /// @throw D2ClientError if sender instance is null. Underlying layer
     /// may throw NCRSenderExceptions exceptions.
     void startSender(D2ClientErrorHandler error_handler,
-                     isc::asiolink::IOService& io_service);
+                     const isc::asiolink::IOServicePtr& io_service);
 
     /// @brief Enables sending NameChangeRequests to kea-dhcp-ddns
     ///
@@ -365,7 +380,7 @@ public:
     ///
     /// Serves as callback registered for the sender's select-fd with IfaceMgr.
     /// It instructs the sender to execute the next ready IO handler.
-    /// It provides an instance method that can be bound via boost::bind, as
+    /// It provides an instance method that can be bound via std::bind, as
     /// NameChangeSender is abstract.
     void runReadyIO();
 
@@ -380,6 +395,9 @@ public:
     /// it will not accept additional messages.
     void suspendUpdates();
 
+    /// @brief Stop the sender
+    void stop();
+
 protected:
     /// @brief Function operator implementing the NCR sender callback.
     ///
@@ -415,6 +433,7 @@ protected:
     int getRegisteredSelectFd();
 
 private:
+
     /// @brief Container class for DHCP-DDNS configuration parameters.
     D2ClientConfigPtr d2_client_config_;
 
@@ -435,11 +454,11 @@ private:
 
 template <class T>
 void
-D2ClientMgr::adjustFqdnFlags(const T& fqdn, T& fqdn_resp) {
+D2ClientMgr::adjustFqdnFlags(const T& fqdn, T& fqdn_resp, const DdnsParams& ddns_params) {
     bool server_s = false;
     bool server_n = false;
     analyzeFqdn(fqdn.getFlag(T::FLAG_S), fqdn.getFlag(T::FLAG_N),
-                server_s, server_n);
+                server_s, server_n, ddns_params);
 
     // Reset the flags to zero to avoid triggering N and S both 1 check.
     fqdn_resp.resetFlags();
@@ -462,18 +481,19 @@ D2ClientMgr::getUpdateDirections(const T& fqdn_resp,
 
 template <class T>
 void
-D2ClientMgr::adjustDomainName(const T& fqdn, T& fqdn_resp) {
+D2ClientMgr::adjustDomainName(const T& fqdn, T& fqdn_resp, const DdnsParams& ddns_params) {
     // If we're configured to replace it or the supplied name is blank
     // set the response name to blank.
-    if ((d2_client_config_->getReplaceClientNameMode() == D2ClientConfig::RCM_ALWAYS ||
-         d2_client_config_->getReplaceClientNameMode() == D2ClientConfig::RCM_WHEN_PRESENT) ||
+    D2ClientConfig::ReplaceClientNameMode mode = ddns_params.getReplaceClientNameMode();
+    if ((mode == D2ClientConfig::RCM_ALWAYS || mode == D2ClientConfig::RCM_WHEN_PRESENT) ||
         fqdn.getDomainName().empty()) {
         fqdn_resp.setDomainName("", T::PARTIAL);
     } else {
         // Sanitize the name the client sent us, if we're configured to do so.
         std::string client_name = fqdn.getDomainName();
 
-        if (d2_client_config_->getHostnameSanitizer()) {
+        isc::util::str::StringSanitizerPtr sanitizer = ddns_params.getHostnameSanitizer();
+        if (sanitizer) {
             // We need the raw text form, so we can replace escaped chars
             dns::Name tmp(client_name);
             std::string raw_name = tmp.toRawText();
@@ -485,12 +505,15 @@ D2ClientMgr::adjustDomainName(const T& fqdn, T& fqdn_resp) {
             std::vector<std::string> labels;
             boost::algorithm::split(labels, raw_name, boost::is_any_of("."));
             std::stringstream ss;
-            for (auto label = labels.begin(); label != labels.end(); ++label ) {
-                if (label != labels.begin()) {
+            bool first = true;
+            for (auto const& label : labels) {
+                if (!first) {
                     ss << ".";
+                } else {
+                    first = false;
                 }
 
-                ss << d2_client_config_->getHostnameSanitizer()->scrub(*label);
+                ss << sanitizer->scrub(label);
             }
 
             client_name = ss.str();
@@ -498,14 +521,14 @@ D2ClientMgr::adjustDomainName(const T& fqdn, T& fqdn_resp) {
 
         // If the supplied name is partial, qualify it by adding the suffix.
         if (fqdn.getDomainNameType() == T::PARTIAL) {
-            fqdn_resp.setDomainName(qualifyName(client_name,true), T::FULL);
-        }
-        else  {
+            fqdn_resp.setDomainName(qualifyName(client_name, ddns_params, true), T::FULL);
+        } else  {
             fqdn_resp.setDomainName(client_name, T::FULL);
         }
     }
 }
 
+
 /// @brief Defines a pointer for D2ClientMgr instances.
 typedef boost::shared_ptr<D2ClientMgr> D2ClientMgrPtr;
 
