@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,9 +13,9 @@
 #include <dhcpsrv/lease_mgr.h>
 #include <exceptions/exceptions.h>
 #include <stats/stats_mgr.h>
-#include <util/encode/hex.h>
+#include <util/encode/encode.h>
+#include <util/str.h>
 
-#include <boost/foreach.hpp>
 #include <boost/algorithm/string.hpp>
 
 #include <algorithm>
@@ -38,8 +38,6 @@ using namespace std;
 namespace isc {
 namespace dhcp {
 
-IOServicePtr LeaseMgr::io_service_ = IOServicePtr();
-
 LeasePageSize::LeasePageSize(const size_t page_size)
     : page_size_(page_size) {
 
@@ -108,9 +106,8 @@ LeaseMgr::recountLeaseStats4() {
     const Subnet4Collection* subnets =
         CfgMgr::instance().getCurrentCfg()->getCfgSubnets4()->getAll();
 
-    for (Subnet4Collection::const_iterator subnet = subnets->begin();
-         subnet != subnets->end(); ++subnet) {
-        SubnetID subnet_id = (*subnet)->getID();
+    for (auto const& subnet : *subnets) {
+        SubnetID subnet_id = subnet->getID();
         stats_mgr.setValue(StatsMgr::generateName("subnet", subnet_id,
                                                   "assigned-addresses"),
                            zero);
@@ -131,7 +128,7 @@ LeaseMgr::recountLeaseStats4() {
             stats_mgr.setValue(name_rec, zero);
         }
 
-        for (const auto& pool : (*subnet)->getPools(Lease::TYPE_V4)) {
+        for (auto const& pool : subnet->getPools(Lease::TYPE_V4)) {
             const std::string name_aa(StatsMgr::generateName("subnet", subnet_id,
                                                              StatsMgr::generateName("pool", pool->getID(),
                                                                                     "assigned-addresses")));
@@ -322,9 +319,8 @@ LeaseMgr::recountLeaseStats6() {
     const Subnet6Collection* subnets =
         CfgMgr::instance().getCurrentCfg()->getCfgSubnets6()->getAll();
 
-    for (Subnet6Collection::const_iterator subnet = subnets->begin();
-         subnet != subnets->end(); ++subnet) {
-        SubnetID subnet_id = (*subnet)->getID();
+    for (auto const& subnet : *subnets) {
+        SubnetID subnet_id = subnet->getID();
         stats_mgr.setValue(StatsMgr::generateName("subnet", subnet_id,
                                                   "assigned-nas"),
                            zero);
@@ -355,7 +351,7 @@ LeaseMgr::recountLeaseStats6() {
                 zero);
         }
 
-        for (const auto& pool : (*subnet)->getPools(Lease::TYPE_NA)) {
+        for (auto const& pool : subnet->getPools(Lease::TYPE_NA)) {
             const std::string& name_anas(StatsMgr::generateName("subnet", subnet_id,
                                                                 StatsMgr::generateName("pool", pool->getID(),
                                                                                        "assigned-nas")));
@@ -385,7 +381,7 @@ LeaseMgr::recountLeaseStats6() {
             }
         }
 
-        for (const auto& pool : (*subnet)->getPools(Lease::TYPE_PD)) {
+        for (auto const& pool : subnet->getPools(Lease::TYPE_PD)) {
             const std::string& name_apds(StatsMgr::generateName("subnet", subnet_id,
                                                                 StatsMgr::generateName("pd-pool", pool->getID(),
                                                                                        "assigned-pds")));
@@ -639,7 +635,7 @@ LeaseMgr::upgradeLease4ExtendedInfo(const Lease4Ptr& lease,
 
             OptionPtr remote_id = rai->getOption(RAI_OPTION_REMOTE_ID);
             if (remote_id) {
-                vector<uint8_t> bytes = remote_id->toBinary(false);
+                vector<uint8_t> bytes = remote_id->toBinary();
                 if (bytes.size() > 0) {
                     upgraded->set("remote-id",
                                   Element::create(encode::encodeHex(bytes)));
@@ -724,7 +720,7 @@ LeaseMgr::upgradeLease4ExtendedInfo(const Lease4Ptr& lease,
         }
 
         verifying = "relay-agent-info";
-        for (auto elem : extended_info->mapValue()) {
+        for (auto const& elem : extended_info->mapValue()) {
             if ((elem.first != "sub-options") &&
                 (elem.first != "remote-id") &&
                 (elem.first != "relay-id") &&
@@ -898,7 +894,7 @@ LeaseMgr::upgradeLease6ExtendedInfo(const Lease6Ptr& lease,
                 if (remote_id_it != opts.end()) {
                     OptionPtr remote_id = remote_id_it->second;
                     if (remote_id) {
-                        vector<uint8_t> bytes = remote_id->toBinary(false);
+                        vector<uint8_t> bytes = remote_id->toBinary();
                         if (bytes.size() > 0) {
                             relay->set("remote-id",
                                        Element::create(encode::encodeHex(bytes)));
@@ -1062,7 +1058,7 @@ LeaseMgr::upgradeLease6ExtendedInfo(const Lease6Ptr& lease,
             }
 
             verifying = (upgraded ? "relays" : "relay-info");
-            for (auto elem : relay->mapValue()) {
+            for (auto const& elem : relay->mapValue()) {
                 if ((elem.first != "hop") &&
                     (elem.first != "link") &&
                     (elem.first != "peer") &&
@@ -1266,5 +1262,15 @@ LeaseMgr::addExtendedInfo6(const Lease6Ptr& lease) {
     return (added);
 }
 
+size_t
+LeaseMgr::byRelayId6size() const {
+    return (0);
+}
+
+size_t
+LeaseMgr::byRemoteId6size() const {
+    return (0);
+}
+
 } // namespace isc::dhcp
 } // namespace isc
