@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -20,6 +20,7 @@
 #include <dhcpsrv/cfg_mac_source.h>
 #include <dhcpsrv/srv_config.h>
 #include <dhcpsrv/parsers/base_network_parser.h>
+#include <dhcpsrv/parsers/option_data_parser.h>
 #include <cc/simple_parser.h>
 #include <exceptions/exceptions.h>
 #include <util/optional.h>
@@ -54,7 +55,7 @@ typedef boost::shared_ptr<OptionStorage> OptionStoragePtr;
 template<typename ValueType>
 class ValueStorage {
 public:
-    /// @brief  Stores the the parameter, its value and the position in the
+    /// @brief  Stores the parameter, its value and the position in the
     /// store.
     ///
     /// If the parameter does not exist in the store, then it will be added,
@@ -221,8 +222,6 @@ public:
     void parse(SrvConfig& srv_cfg, isc::data::ConstElementPtr value);
 };
 
-typedef std::pair<isc::dhcp::OptionDefinitionPtr, std::string> OptionDefinitionTuple;
-
 /// @brief Parser for a single option definition.
 ///
 /// This parser creates an instance of a single option definition.
@@ -236,11 +235,10 @@ public:
     /// @brief Parses an entry that describes single option definition.
     ///
     /// @param option_def a configuration entry to be parsed.
-    /// @return tuple (option definition, option space) of the parsed structure
+    /// @return option definition of the parsed structure.
     ///
     /// @throw DhcpConfigError if parsing was unsuccessful.
-    OptionDefinitionTuple
-    parse(isc::data::ConstElementPtr option_def);
+    OptionDefinitionPtr parse(isc::data::ConstElementPtr option_def);
 
 private:
     /// @brief Address family: @c AF_INET or @c AF_INET6.
@@ -304,10 +302,13 @@ public:
     /// @param pools is the storage in which to store the parsed pool
     /// @param pool_structure a single entry on a list of pools
     /// @param address_family AF_INET (for DHCPv4) or AF_INET6 (for DHCPv6).
+    /// @param encapsulate_options a boolean parameter indicating if the
+    /// parsed options should be encapsulated with suboptions.
     /// @throw isc::dhcp::DhcpConfigError when pool parsing fails
     virtual void parse(PoolStoragePtr pools,
                        isc::data::ConstElementPtr pool_structure,
-                       const uint16_t address_family);
+                       const uint16_t address_family,
+                       bool encapsulate_options = true);
 
 protected:
     /// @brief Creates a Pool object given a IPv4 prefix and the prefix length.
@@ -328,6 +329,18 @@ protected:
     virtual PoolPtr poolMaker(isc::asiolink::IOAddress &min,
                               isc::asiolink::IOAddress &max,
                               int32_t ptype = 0) = 0;
+
+    /// @brief Returns an instance of the @c OptionDataListParser to
+    /// be used in parsing the option-data structure.
+    ///
+    /// This function can be overridden in the child classes to supply
+    /// a custom parser for option data.
+    ///
+    /// @param address_family AF_INET (for DHCPv4) or AF_INET6 (for DHCPv6).
+    ///
+    /// @return an instance of the @c OptionDataListParser.
+    virtual boost::shared_ptr<OptionDataListParser>
+    createOptionDataListParser(const uint16_t address_family) const;
 };
 
 /// @brief Parser for IPv4 pool definitions.
@@ -379,13 +392,27 @@ public:
     ///
     /// @param pools is the storage in which to store the parsed pools.
     /// @param pools_list a list of pool structures
+    /// @param encapsulate_options a boolean parameter indicating if the
+    /// parsed options should be encapsulated with suboptions.
     /// @throw isc::dhcp::DhcpConfigError when pool parsing fails
     virtual void parse(PoolStoragePtr pools,
-                       isc::data::ConstElementPtr pools_list) = 0;
+                       isc::data::ConstElementPtr pools_list,
+                       bool encapsulate_options) = 0;
+
+protected:
+
+    /// @brief Returns an instance of the @c PoolParser to be used in
+    /// parsing the address pools.
+    ///
+    /// This function can be overridden in the child classes to supply
+    /// a custom parser for the pools.
+    ///
+    /// @return an instance of the @c PoolParser.
+    virtual boost::shared_ptr<PoolParser> createPoolConfigParser() const = 0;
 };
 
 /// @brief Specialization of the pool list parser for DHCPv4
-class Pools4ListParser : PoolsListParser {
+class Pools4ListParser : public PoolsListParser {
 public:
 
     /// @brief parses the actual structure
@@ -394,8 +421,22 @@ public:
     ///
     /// @param pools storage container in which to store the parsed pool.
     /// @param pools_list a list of pool structures
+    /// @param encapsulate_options a boolean parameter indicating if the
+    /// parsed options should be encapsulated with suboptions.
     /// @throw isc::dhcp::DhcpConfigError when pool parsing fails
-    void parse(PoolStoragePtr pools, data::ConstElementPtr pools_list);
+    void parse(PoolStoragePtr pools, data::ConstElementPtr pools_list,
+               bool encapsulate_options = true);
+
+protected:
+
+    /// @brief Returns an instance of the @c Pool4Parser to be used in
+    /// parsing the address pools.
+    ///
+    /// This function can be overridden in the child classes to supply
+    /// a custom parser for the pools.
+    ///
+    /// @return an instance of the @c Pool4Parser.
+    virtual boost::shared_ptr<PoolParser> createPoolConfigParser() const;
 };
 
 /// @brief parser for additional relay information
@@ -474,7 +515,9 @@ public:
     /// @brief constructor
     ///
     /// @param family address family: @c AF_INET or @c AF_INET6
-    explicit SubnetConfigParser(uint16_t family);
+    /// @param check_iface Check if the specified interface exists in
+    /// the system.
+    explicit SubnetConfigParser(uint16_t family, bool check_iface = true);
 
     /// @brief virtual destructor (does nothing)
     virtual ~SubnetConfigParser() { }
@@ -486,10 +529,13 @@ protected:
     /// Subnet6ConfigParser) classes.
     ///
     /// @param subnet pointer to the content of subnet definition
+    /// @param encapsulate_options a boolean parameter indicating if the
+    /// parsed options should be encapsulated with suboptions.
     /// @return a pointer to newly created subnet
     ///
     /// @throw isc::DhcpConfigError if subnet configuration parsing failed.
-    SubnetPtr parse(isc::data::ConstElementPtr subnet);
+    SubnetPtr parse(isc::data::ConstElementPtr subnet,
+                    bool encapsulate_options);
 
     /// @brief Instantiates the subnet based on a given IP prefix and prefix
     /// length.
@@ -500,7 +546,7 @@ protected:
     virtual void initSubnet(isc::data::ConstElementPtr params,
                             isc::asiolink::IOAddress addr, uint8_t len) = 0;
 
-private:
+protected:
 
     /// @brief Create a new subnet using a data from child parsers.
     ///
@@ -509,7 +555,24 @@ private:
     /// failed.
     void createSubnet(isc::data::ConstElementPtr data);
 
-protected:
+    /// @brief Returns an instance of the @c OptionDataListParser to
+    /// be used in parsing the option-data structure.
+    ///
+    /// This function can be overridden in the child classes to supply
+    /// a custom parser for option data.
+    ///
+    /// @return an instance of the @c OptionDataListParser.
+    virtual boost::shared_ptr<OptionDataListParser> createOptionDataListParser() const;
+
+    /// @brief Returns an instance of the @c PoolsListParser to be used
+    /// in parsing the address pools.
+    ///
+    /// This function can be overridden in the child classes to supply
+    /// a custom parser for the pools.
+    ///
+    /// @return an instance of the @c PoolsListParser.
+    virtual boost::shared_ptr<PoolsListParser>
+    createPoolsListParser() const = 0;
 
     /// Storage for pools belonging to this subnet.
     PoolStoragePtr pools_;
@@ -523,8 +586,8 @@ protected:
     /// Pointer to relay information
     isc::dhcp::Network::RelayInfoPtr relay_info_;
 
-    /// Pointer to the options configuration.
-    CfgOptionPtr options_;
+    /// Check if the specified interface exists in the system.
+    bool check_iface_;
 };
 
 /// @anchor Subnet4ConfigParser
@@ -538,14 +601,20 @@ public:
     /// @brief Constructor
     ///
     /// stores global scope parameters, options, option definitions.
-    Subnet4ConfigParser();
+    ///
+    /// @param check_iface Check if the specified interface exists in
+    /// the system.
+    Subnet4ConfigParser(bool check_iface = true);
 
     /// @brief Parses a single IPv4 subnet configuration and adds to the
     /// Configuration Manager.
     ///
     /// @param subnet A new subnet being configured.
+    /// @param encapsulate_options a boolean parameter indicating if the
+    /// parsed options should be encapsulated with suboptions.
     /// @return a pointer to created Subnet4 object
-    Subnet4Ptr parse(data::ConstElementPtr subnet);
+    Subnet4Ptr parse(data::ConstElementPtr subnet,
+                     bool encapsulate_options = true);
 
 protected:
 
@@ -557,6 +626,23 @@ protected:
     /// @param len is the prefix length
     void initSubnet(data::ConstElementPtr params,
                     asiolink::IOAddress addr, uint8_t len);
+
+    /// @brief Verifies the host reservation address is in the subnet range
+    ///
+    /// @param subnet pointer to the subnet
+    /// @param host pointer to the host reservation
+    /// @throw DhcpConfigError when the address is not in the subnet range.
+    void validateResv(const Subnet4Ptr& subnet, ConstHostPtr host);
+
+    /// @brief Returns an instance of the @c Pools4ListParser to be used
+    /// in parsing the address pools.
+    ///
+    /// This function can be overridden in the child classes to supply
+    /// a custom parser for the pools.
+    ///
+    /// @return an instance of the @c Pools4ListParser.
+    virtual boost::shared_ptr<PoolsListParser>
+    createPoolsListParser() const;
 };
 
 /// @brief this class parses list of DHCP4 subnets
@@ -567,6 +653,16 @@ protected:
 class Subnets4ListConfigParser : public isc::data::SimpleParser {
 public:
 
+    /// @brief constructor
+    ///
+    /// @param check_iface Check if the specified interface exists in
+    /// the system.
+    Subnets4ListConfigParser(bool check_iface = true);
+
+    /// @brief Virtual destructor.
+    virtual ~Subnets4ListConfigParser() {
+    }
+
     /// @brief parses contents of the list
     ///
     /// Iterates over all entries on the list, parses its content
@@ -575,16 +671,36 @@ public:
     ///
     /// @param cfg Pointer to server configuration.
     /// @param subnets_list pointer to a list of IPv4 subnets
+    /// @param encapsulate_options a boolean parameter indicating if the
+    /// parsed options should be encapsulated with suboptions.
     /// @return number of subnets created
-    size_t parse(SrvConfigPtr cfg, data::ConstElementPtr subnets_list);
+    size_t parse(SrvConfigPtr cfg, data::ConstElementPtr subnets_list,
+                 bool encapsulate_options = true);
 
     /// @brief Parses contents of the subnet4 list.
     ///
     /// @param [out] subnets Container where parsed subnets will be stored.
     /// @param subnets_list pointer to a list of IPv4 subnets
+    /// @param encapsulate_options a boolean parameter indicating if the
+    /// parsed options should be encapsulated with suboptions.
     /// @return Number of subnets created.
     size_t parse(Subnet4Collection& subnets,
-                 data::ConstElementPtr subnets_list);
+                 data::ConstElementPtr subnets_list,
+                 bool encapsulate_options = true);
+
+protected:
+
+    /// @brief Returns an instance of the @c Subnet4ConfigParser to be
+    /// used in parsing the subnets.
+    ///
+    /// This function can be overridden in the child classes to supply
+    /// a custom parser for the subnets.
+    ///
+    /// @return an instance of the @c Subnet4ConfigParser.
+    virtual boost::shared_ptr<Subnet4ConfigParser> createSubnetConfigParser() const;
+
+    /// Check if the specified interface exists in the system.
+    bool check_iface_;
 };
 
 /// @brief Parser for IPv6 pool definitions.
@@ -620,7 +736,7 @@ protected:
 };
 
 /// @brief Specialization of the pool list parser for DHCPv6
-class Pools6ListParser : PoolsListParser {
+class Pools6ListParser : public PoolsListParser {
 public:
 
     /// @brief parses the actual structure
@@ -629,8 +745,22 @@ public:
     ///
     /// @param pools storage container in which to store the parsed pool.
     /// @param pools_list a list of pool structures
+    /// @param encapsulate_options a boolean parameter indicating if the
+    /// parsed options should be encapsulated with suboptions.
     /// @throw isc::dhcp::DhcpConfigError when pool parsing fails
-    void parse(PoolStoragePtr pools, data::ConstElementPtr pools_list);
+    void parse(PoolStoragePtr pools, data::ConstElementPtr pools_list,
+               bool encapsulate_options = true);
+
+protected:
+
+    /// @brief Returns an instance of the @c Pool6Parser to be used in
+    /// parsing the address pools.
+    ///
+    /// This function can be overridden in the child classes to supply
+    /// a custom parser for the pools.
+    ///
+    /// @return an instance of the @c Pool6Parser.
+    virtual boost::shared_ptr<PoolParser> createPoolConfigParser() const;
 };
 
 /// @brief Parser for IPv6 prefix delegation definitions.
@@ -657,6 +787,10 @@ public:
     ///
     PdPoolParser();
 
+    /// @brief Virtual destructor.
+    virtual ~PdPoolParser() {
+    }
+
     /// @brief Builds a prefix delegation pool from the given configuration
     ///
     /// This function parses configuration entries and creates an instance
@@ -665,18 +799,28 @@ public:
     /// @param pools storage container in which to store the parsed pool.
     /// @param pd_pool_ pointer to an element that holds configuration entries
     /// that define a prefix delegation pool.
+    /// @param encapsulate_options a boolean parameter indicating if the
+    /// parsed options should be encapsulated with suboptions.
     ///
     /// @throw DhcpConfigError if configuration parsing fails.
-    void parse(PoolStoragePtr pools, data::ConstElementPtr pd_pool_);
+    void parse(PoolStoragePtr pools, data::ConstElementPtr pd_pool_,
+               bool encapsulate_options = true);
 
-private:
+protected:
+
+    /// @brief Returns an instance of the @c OptionDataListParser to
+    /// be used in parsing the option-data structure.
+    ///
+    /// This function can be overridden in the child classes to supply
+    /// a custom parser for option data.
+    ///
+    /// @return an instance of the @c OptionDataListParser.
+    virtual boost::shared_ptr<OptionDataListParser>
+    createOptionDataListParser() const;
 
     /// Pointer to the created pool object.
     isc::dhcp::Pool6Ptr pool_;
 
-    /// A storage for pool specific option values.
-    CfgOptionPtr options_;
-
     /// @brief User context (optional, may be null)
     ///
     /// User context is arbitrary user data, to be used by hooks.
@@ -693,9 +837,13 @@ private:
 /// This parser iterates over a list of prefix delegation pool entries and
 /// creates pool instances for each one. If the parsing is successful, the
 /// collection of pools is committed to the provided storage.
-class PdPoolsListParser : public PoolsListParser {
+class PdPoolsListParser : public isc::data::SimpleParser {
 public:
 
+    /// @brief Virtual destructor.
+    virtual ~PdPoolsListParser() {
+    }
+
     /// @brief Parse configuration entries.
     ///
     /// This function parses configuration entries and creates instances
@@ -707,6 +855,18 @@ public:
     ///
     /// @throw DhcpConfigError if configuration parsing fails.
     void parse(PoolStoragePtr pools, data::ConstElementPtr pd_pool_list);
+
+protected:
+
+    /// @brief Returns an instance of the @c PdPoolParser to be used in
+    /// parsing the prefix delegation pools.
+    ///
+    /// This function can be overridden in the child classes to supply
+    /// a custom parser for the pools.
+    ///
+    /// @return an instance of the @c PdPool6Parser.
+    virtual boost::shared_ptr<PdPoolParser>
+    createPdPoolConfigParser() const;
 };
 
 /// @anchor Subnet6ConfigParser
@@ -721,14 +881,20 @@ public:
     /// @brief Constructor
     ///
     /// stores global scope parameters, options, option definitions.
-    Subnet6ConfigParser();
+    ///
+    /// @param check_iface Check if the specified interface exists in
+    /// the system.
+    Subnet6ConfigParser(bool check_iface = true);
 
     /// @brief Parses a single IPv6 subnet configuration and adds to the
     /// Configuration Manager.
     ///
     /// @param subnet A new subnet being configured.
+    /// @param encapsulate_options a boolean parameter indicating if the
+    /// parsed options should be encapsulated with suboptions.
     /// @return a pointer to created Subnet6 object
-    Subnet6Ptr parse(data::ConstElementPtr subnet);
+    Subnet6Ptr parse(data::ConstElementPtr subnet,
+                     bool encapsulate_options = true);
 
 protected:
     /// @brief Issues a DHCP6 server specific warning regarding duplicate subnet
@@ -738,8 +904,8 @@ protected:
     /// @param addr is the subnet address
     /// @todo A means to know the correct logger and perhaps a common
     /// message would allow this message to be emitted by the base class.
-    virtual void duplicate_option_warning(uint32_t code,
-                                         asiolink::IOAddress& addr);
+    virtual void duplicateOptionWarning(uint32_t code,
+                                        asiolink::IOAddress& addr);
 
     /// @brief Instantiates the IPv6 Subnet based on a given IPv6 address
     /// and prefix length.
@@ -749,6 +915,33 @@ protected:
     /// @param len is the prefix length
     void initSubnet(isc::data::ConstElementPtr params,
                     isc::asiolink::IOAddress addr, uint8_t len);
+
+    /// @brief Verifies host reservation addresses are in the subnet range
+    ///
+    /// @param subnet pointer to the subnet
+    /// @param host pointer to the host reservation
+    /// @throw DhcpConfigError when an address is not in the subnet range.
+    void validateResvs(const Subnet6Ptr& subnet, ConstHostPtr host);
+
+    /// @brief Returns an instance of the @c Pools6ListParser to be used
+    /// in parsing the address pools.
+    ///
+    /// This function can be overridden in the child classes to supply
+    /// a custom parser for the pools.
+    ///
+    /// @return an instance of the @c Pools6ListParser.
+    virtual boost::shared_ptr<PoolsListParser>
+    createPoolsListParser() const;
+
+    /// @brief Returns an instance of the @c PdPools6ListParser to be used
+    /// in parsing the prefix delegation pools.
+    ///
+    /// This function can be overridden in the child classes to supply
+    /// a custom parser for the pools.
+    ///
+    /// @return an instance of the @c PdPools6ListParser.
+    virtual boost::shared_ptr<PdPoolsListParser>
+    createPdPoolsListParser() const;
 };
 
 
@@ -760,6 +953,16 @@ protected:
 class Subnets6ListConfigParser : public isc::data::SimpleParser {
 public:
 
+    /// @brief constructor
+    ///
+    /// @param check_iface Check if the specified interface exists in
+    /// the system.
+    Subnets6ListConfigParser(bool check_iface = true);
+
+    /// @brief Virtual destructor.
+    virtual ~Subnets6ListConfigParser() {
+    }
+
     /// @brief parses contents of the list
     ///
     /// Iterates over all entries on the list, parses its content
@@ -768,25 +971,46 @@ public:
     ///
     /// @param cfg configuration (parsed subnets will be stored here)
     /// @param subnets_list pointer to a list of IPv6 subnets
+    /// @param encapsulate_options a boolean parameter indicating if the
+    /// parsed options should be encapsulated with suboptions.
+    ///
     /// @throw DhcpConfigError if CfgMgr rejects the subnet (e.g. subnet-id is a duplicate)
-    size_t parse(SrvConfigPtr cfg, data::ConstElementPtr subnets_list);
+    size_t parse(SrvConfigPtr cfg, data::ConstElementPtr subnets_list,
+                 bool encapsulate_options = true);
 
     /// @brief Parses contents of the subnet6 list.
     ///
     /// @param [out] subnets Container where parsed subnets will be stored.
     /// @param subnets_list pointer to a list of IPv6 subnets
+    /// @param encapsulate_options a boolean parameter indicating if the
+    /// parsed options should be encapsulated with suboptions.
+    ///
     /// @return Number of subnets created.
     size_t parse(Subnet6Collection& subnets,
-                 data::ConstElementPtr subnets_list);
+                 data::ConstElementPtr subnets_list,
+                 bool encapsulate_options = true);
 
+protected:
+
+    /// @brief Returns an instance of the @c Subnet6ConfigParser to be
+    /// used in parsing the subnets.
+    ///
+    /// This function can be overridden in the child classes to supply
+    /// a custom parser for the subnets.
+    ///
+    /// @return an instance of the @c Subnet6ConfigParser.
+    virtual boost::shared_ptr<Subnet6ConfigParser> createSubnetConfigParser() const;
+
+    /// Check if the specified interface exists in the system.
+    bool check_iface_;
 };
 
-/// @brief Parser for  D2ClientConfig
+/// @brief Parser for D2ClientConfig
 ///
 /// This class parses the configuration element "dhcp-ddns" common to the
 /// config files for both dhcp4 and dhcp6. It creates an instance of a
 /// D2ClientConfig.
-class D2ClientConfigParser : public  isc::data::SimpleParser {
+class D2ClientConfigParser : public isc::data::SimpleParser {
 public:
 
     /// @brief Parses a given dhcp-ddns element into D2ClientConfig.
@@ -796,7 +1020,6 @@ public:
     /// The elements currently supported are (see isc::dhcp::D2ClientConfig
     /// for details on each):
     /// -# enable-updates
-    /// -# qualifying-suffix
     /// -# server-ip
     /// -# server-port
     /// -# sender-ip
@@ -804,10 +1027,6 @@ public:
     /// -# max-queue-size
     /// -# ncr-protocol
     /// -# ncr-format
-    /// -# override-no-update
-    /// -# override-client-update
-    /// -# replace-client-name
-    /// -# generated-prefix
     ///
     /// @return returns a pointer to newly created D2ClientConfig.
     D2ClientConfigPtr parse(isc::data::ConstElementPtr d2_client_cfg);
@@ -858,7 +1077,16 @@ private:
     getMode(isc::data::ConstElementPtr scope, const std::string& name);
 };
 
-}; // end of isc::dhcp namespace
-}; // end of isc namespace
+class CompatibilityParser : public isc::data::SimpleParser {
+public:
+    /// @brief Parse compatibility flags
+    ///
+    /// @param cfg The configuration element to be parsed
+    /// @param srv_cfg The configuration where the parameters are stored
+    void parse(isc::data::ConstElementPtr cfg, isc::dhcp::SrvConfig& srv_cfg);
+};
+
+} // end of isc::dhcp namespace
+} // end of isc namespace
 
 #endif // DHCP_PARSERS_H
