@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -45,10 +45,8 @@ SharedNetwork4Parser::parse(const data::ConstElementPtr& shared_network_data,
         std::string name = getString(shared_network_data, "name");
         shared_network.reset(new SharedNetwork4(name));
 
-        // Move from reservation mode to new reservations flags.
         ElementPtr mutable_params;
         mutable_params = boost::const_pointer_cast<Element>(shared_network_data);
-        BaseNetworkParser::moveReservationMode(mutable_params);
 
         // Parse parameters common to all Network derivations.
         NetworkPtr network = boost::dynamic_pointer_cast<Network>(shared_network);
@@ -88,9 +86,8 @@ SharedNetwork4Parser::parse(const data::ConstElementPtr& shared_network_data,
             parser->parse(subnets, json);
 
             // Add all returned subnets into shared network.
-            for (auto subnet = subnets.cbegin(); subnet != subnets.cend();
-                 ++subnet) {
-                shared_network->add(*subnet);
+            for (auto const& subnet : subnets) {
+                shared_network->add(subnet);
             }
         }
 
@@ -170,14 +167,13 @@ SharedNetwork4Parser::parse(const data::ConstElementPtr& shared_network_data,
         if (shared_network_data->contains("require-client-classes")) {
             const std::vector<data::ElementPtr>& class_list =
                 shared_network_data->get("require-client-classes")->listValue();
-            for (auto cclass = class_list.cbegin();
-                 cclass != class_list.cend(); ++cclass) {
-                if (((*cclass)->getType() != Element::string) ||
-                    (*cclass)->stringValue().empty()) {
+            for (auto const& cclass : class_list) {
+                if ((cclass->getType() != Element::string) ||
+                    cclass->stringValue().empty()) {
                     isc_throw(DhcpConfigError, "invalid class name ("
-                              << (*cclass)->getPosition() << ")");
+                              << cclass->getPosition() << ")");
                 }
-                shared_network->requireClientClass((*cclass)->stringValue());
+                shared_network->requireClientClass(cclass->stringValue());
             }
         }
 
@@ -255,10 +251,8 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data,
         std::string name = getString(shared_network_data, "name");
         shared_network.reset(new SharedNetwork6(name));
 
-        // Move from reservation mode to new reservations flags.
         ElementPtr mutable_params;
         mutable_params = boost::const_pointer_cast<Element>(shared_network_data);
-        BaseNetworkParser::moveReservationMode(mutable_params);
 
         // Parse parameters common to all Network derivations.
         NetworkPtr network = boost::dynamic_pointer_cast<Network>(shared_network);
@@ -344,14 +338,13 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data,
         if (shared_network_data->contains("require-client-classes")) {
             const std::vector<data::ElementPtr>& class_list =
                 shared_network_data->get("require-client-classes")->listValue();
-            for (auto cclass = class_list.cbegin();
-                 cclass != class_list.cend(); ++cclass) {
-                if (((*cclass)->getType() != Element::string) ||
-                    (*cclass)->stringValue().empty()) {
+            for (auto const& cclass : class_list) {
+                if ((cclass->getType() != Element::string) ||
+                    cclass->stringValue().empty()) {
                     isc_throw(DhcpConfigError, "invalid class name ("
-                              << (*cclass)->getPosition() << ")");
+                              << cclass->getPosition() << ")");
                 }
-                shared_network->requireClientClass((*cclass)->stringValue());
+                shared_network->requireClientClass(cclass->stringValue());
             }
         }
 
@@ -364,9 +357,8 @@ SharedNetwork6Parser::parse(const data::ConstElementPtr& shared_network_data,
             parser->parse(subnets, json);
 
             // Add all returned subnets into shared network.
-            for (auto subnet = subnets.cbegin(); subnet != subnets.cend();
-                 ++subnet) {
-                shared_network->add(*subnet);
+            for (auto const& subnet : subnets) {
+                shared_network->add(subnet);
             }
         }
 
