@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -11,11 +11,17 @@
 #include <cc/data.h>
 #include <cc/user_context.h>
 #include <dhcp/option_space_container.h>
+#include <dhcpsrv/allocator.h>
+#include <dhcpsrv/allocation_state.h>
 #include <dhcpsrv/lease.h>
 #include <dhcpsrv/network.h>
 #include <dhcpsrv/pool.h>
 #include <dhcpsrv/subnet_id.h>
-#include <dhcpsrv/triplet.h>
+#include <util/bigints.h>
+#include <util/dhcp_space.h>
+#include <util/triplet.h>
+
+#include <boost/enable_shared_from_this.hpp>
 #include <boost/multi_index/mem_fun.hpp>
 #include <boost/multi_index/indexed_by.hpp>
 #include <boost/multi_index/ordered_index.hpp>
@@ -24,6 +30,7 @@
 #include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/pointer_cast.hpp>
 #include <boost/shared_ptr.hpp>
+
 #include <cstdint>
 #include <map>
 #include <utility>
@@ -34,10 +41,14 @@ namespace dhcp {
 class Subnet : public virtual Network {
 public:
 
-    /// @brief checks if specified address is in range
+    /// @brief checks if specified address is in range.
+    ///
+    /// @param addr this address will be checked if it is included in a specific
+    ///        range
+    /// @return true if address is in range, false otherwise
     bool inRange(const isc::asiolink::IOAddress& addr) const;
 
-    /// @brief checks if the specified address is in pools
+    /// @brief checks if the specified address is in pools.
     ///
     /// Note the difference between inRange() and inPool() for addresses
     /// (i.e. *not* prefixes). For a given subnet (e.g. 2001::/64) there
@@ -53,7 +64,7 @@ public:
     /// @return true if the address is in any of the pools
     bool inPool(Lease::Type type, const isc::asiolink::IOAddress& addr) const;
 
-    /// @brief checks if the specified address is in allowed pools
+    /// @brief checks if the specified address is in allowed pools.
     ///
     /// This takes also into account client classes
     ///
@@ -66,48 +77,14 @@ public:
                 const isc::asiolink::IOAddress& addr,
                 const ClientClasses& client_classes) const;
 
-    /// @brief returns the last address that was tried from this subnet
+     /// @brief Returns unique ID for that subnet.
     ///
-    /// This method returns the last address that was attempted to be allocated
-    /// from this subnet. This is used as helper information for the next
-    /// iteration of the allocation algorithm.
-    ///
-    /// @todo: Define map<SubnetID, ClientClass, IOAddress> somewhere in the
-    ///        AllocEngine::IterativeAllocator and keep the data there
-    ///
-    /// @param type lease type to be returned
-    /// @return address/prefix that was last tried from this subnet
-    isc::asiolink::IOAddress getLastAllocated(Lease::Type type) const;
-
-    /// @brief Returns the timestamp when the @c setLastAllocated function
-    /// was called.
-    ///
-    /// @param lease_type Lease type for which last allocation timestamp should
-    /// be returned.
-    ///
-    /// @return Time when a lease of a specified type has been allocated from
-    /// this subnet. The negative infinity time is returned if a lease type is
-    /// not recognized (which is unlikely).
-    boost::posix_time::ptime getLastAllocatedTime(const Lease::Type& lease_type) const;
-
-    /// @brief sets the last address that was tried from this subnet
-    ///
-    /// This method sets the last address that was attempted to be allocated
-    /// from this subnet. This is used as helper information for the next
-    /// iteration of the allocation algorithm.
-    ///
-    /// @todo: Define map<SubnetID, ClientClass, IOAddress> somewhere in the
-    ///        AllocEngine::IterativeAllocator and keep the data there
-    /// @param addr address/prefix to that was tried last
-    /// @param type lease type to be set
-    void setLastAllocated(Lease::Type type,
-                          const isc::asiolink::IOAddress& addr);
-
-    /// @brief Returns unique ID for that subnet
     /// @return unique ID for that subnet
-    SubnetID getID() const { return (id_); }
+    SubnetID getID() const {
+        return (id_);
+    }
 
-    /// @brief Returns subnet parameters (prefix and prefix length)
+    /// @brief Returns subnet parameters (prefix and prefix length).
     ///
     /// @return (prefix, prefix length) pair
     std::pair<isc::asiolink::IOAddress, uint8_t> get() const {
@@ -132,19 +109,19 @@ public:
     ///
     /// @param pool pool to be added
     ///
-    /// @throw isc::BadValue if the pool type is invalid, the pool
-    /// is not an IA_PD pool and the address range of this pool does not
-    /// match the subnet prefix, or the pool overlaps with an existing pool
-    /// within the subnet.
+    /// @throw isc::BadValue if the pool type is invalid, in the case of address
+    /// pools if the address range of the pool does not match the subnet prefix,
+    /// or if the pool overlaps with an existing pool within the subnet.
     void addPool(const PoolPtr& pool);
 
-    /// @brief Deletes all pools of specified type
+    /// @brief Deletes all pools of specified type.
     ///
     /// This method is used for testing purposes only
+    ///
     /// @param type type of pools to be deleted
     void delPools(Lease::Type type);
 
-    /// @brief Returns a pool that specified address belongs to
+    /// @brief Returns a pool that specified address belongs to.
     ///
     /// This method uses binary search to retrieve the pool. Thus, the number
     /// of comparisons performed by this method is logarithmic in the number
@@ -153,7 +130,7 @@ public:
     /// If there is no pool that the address belongs to (hint is invalid), other
     /// pool of specified type will be returned.
     ///
-    /// With anypool set to true, this is means give me a pool, preferably
+    /// With anypool set to true, this means give me a pool, preferably
     /// the one that addr belongs to. With anypool set to false, it means
     /// give me a pool that addr belongs to (or NULL if here is no such pool)
     ///
@@ -165,9 +142,9 @@ public:
     const PoolPtr getPool(Lease::Type type, const isc::asiolink::IOAddress& addr,
                           bool anypool = true) const;
 
-    /// @brief Returns a pool that specified address belongs to with classes
+    /// @brief Returns a pool that specified address belongs to with classes.
     ///
-    /// Variant using only pools allowing given classes
+    /// Variant using only pools allowing given classes.
     ///
     /// @param type pool type that the pool is looked for
     /// @param client_classes client class list which must be allowed
@@ -176,7 +153,7 @@ public:
                           const ClientClasses& client_classes,
                           const isc::asiolink::IOAddress& addr) const;
 
-    /// @brief Returns a pool without any address specified
+    /// @brief Returns a pool without any address specified.
     ///
     /// @param type pool type that the pool is looked for
     /// @return returns one of the pools defined
@@ -184,13 +161,13 @@ public:
         return (getPool(type, default_pool()));
     }
 
-    /// @brief Returns the default address that will be used for pool selection
+    /// @brief Returns the default address that will be used for pool selection.
     ///
     /// It must be implemented in derived classes (should return :: for Subnet6
-    /// and 0.0.0.0 for Subnet4)
+    /// and 0.0.0.0 for Subnet4).
     virtual isc::asiolink::IOAddress default_pool() const = 0;
 
-    /// @brief Returns all pools (const variant)
+    /// @brief Returns all pools (const variant).
     ///
     /// The reference is only valid as long as the object that returned it.
     ///
@@ -198,34 +175,42 @@ public:
     /// @return a collection of all pools
     const PoolCollection& getPools(Lease::Type type) const;
 
-    /// @brief Returns the number of possible leases for specified lease type
+    /// @brief Returns the number of possible leases for specified lease type.
     ///
     /// @param type type of the lease
-    uint64_t getPoolCapacity(Lease::Type type) const;
+    isc::util::uint128_t getPoolCapacity(Lease::Type type) const;
 
     /// @brief Returns the number of possible leases for specified lease type
     /// allowed for a client which belongs to classes.
     ///
     /// @param type type of the lease
-    /// @param client_classes List of classes the client belongs to.
-    uint64_t getPoolCapacity(Lease::Type type,
-                             const ClientClasses& client_classes) const;
+    /// @param client_classes list of classes the client belongs to
+    /// @return number of leases matching lease type and classes
+    isc::util::uint128_t getPoolCapacity(Lease::Type type,
+                                         const ClientClasses& client_classes) const;
+
+    /// @brief Returns the number of possible leases for specified lease type
+    /// allowed for a client which belongs to classes and matching selection
+    /// criteria relative to provided hint prefix length.
+    ///
+    /// @param type type of the lease
+    /// @param client_classes list of classes the client belongs to
+    /// @param prefix_length_match type which indicates the selection criteria
+    /// for the pools relative to the provided hint prefix length
+    /// @param hint_prefix_length the hint prefix length that the client
+    /// provided
+    /// @return number of leases matching lease type and classes
+    isc::util::uint128_t getPoolCapacity(Lease::Type type,
+                                         const ClientClasses& client_classes,
+                                         Allocator::PrefixLenMatchType prefix_length_match,
+                                         uint8_t hint_prefix_length) const;
 
     /// @brief Returns textual representation of the subnet (e.g.
-    /// "2001:db8::/64")
+    /// "2001:db8::/64").
     ///
     /// @return textual representation
     virtual std::string toText() const;
 
-    /// @brief Resets subnet-id counter to its initial value (1)
-    ///
-    /// This should be called during reconfiguration, before any new
-    /// subnet objects are created. It will ensure that the subnet_id will
-    /// be consistent between reconfigures.
-    static void resetSubnetID() {
-        static_id_ = 1;
-    }
-
     /// @brief Retrieves pointer to a shared network associated with a subnet.
     ///
     /// By implementing it as a template function we overcome a need to
@@ -260,6 +245,8 @@ public:
     }
 
     /// @brief Returns shared network name.
+    ///
+    /// @return shared network name
     std::string getSharedNetworkName() const {
         return (shared_network_name_);
     }
@@ -280,7 +267,7 @@ public:
         shared_network_name_ = shared_network_name;
     }
 
-    /// @brief Returns all pools (non-const variant)
+    /// @brief Returns all pools (non-const variant).
     ///
     /// The reference is only valid as long as the object that returned it.
     ///
@@ -288,57 +275,73 @@ public:
     /// @return a collection of all pools
     PoolCollection& getPoolsWritable(Lease::Type type);
 
+    /// @brief Returns lease allocator instance.
+    ///
+    /// An allocator is responsible for selecting leases from the subnet's
+    /// pools. Each subnet has one allocator common for all pools belonging
+    /// to the subnet. The allocation engine uses this function to get the
+    /// current subnet allocator and uses it to select and offer an address.
+    ///
+    /// @param type lease type for which the allocator instance should be
+    /// returned.
+    /// @return Allocator instance.
+    AllocatorPtr getAllocator(Lease::Type type) const;
+
+    /// @brief Sets new allocator instance.
+    ///
+    /// If the server is configured to use a different allocator for the
+    /// subnet, it can set the current allocator with this function.
+    ///
+    /// @param type lease type for which the allocator is set.
+    /// @param allocator new allocator instance.
+    void setAllocator(Lease::Type type, const AllocatorPtr& allocator);
+
+    /// @brief Returns subnet-specific allocation state.
+    ///
+    /// The actual type of the state depends on the allocator type.
+    ///
+    /// @param type lease type for which the allocation state is returned.
+    /// @return allocation state.
+    SubnetAllocationStatePtr getAllocationState(Lease::Type type) const;
+
+    /// @brief Sets subnet-specific allocation state.
+    ///
+    /// @param type lease type for which the allocation state is set.
+    /// @param allocation_state allocation state instance.
+    void setAllocationState(Lease::Type type, const SubnetAllocationStatePtr& allocation_state);
+
+    /// @brief Instantiates the allocators and their states.
+    ///
+    /// It determines the types of the allocators to create using the list of
+    /// the allocator types specified with the @c Network::setAllocatorType method.
+    ///
+    /// This function is called from the subnet parsers and after fetching
+    /// the subnet configuration from a configuration backend.
+    virtual void createAllocators() = 0;
+
+    /// @brief Calls @c initAfterConfigure for each allocator.
+    void initAllocatorsAfterConfigure();
+
 protected:
 
-    /// @brief Protected constructor
+    /// @brief Protected constructor.
     //
     /// By making the constructor protected, we make sure that no one will
     /// ever instantiate that class. Subnet4 and Subnet6 should be used instead.
     ///
-    /// This constructor assigns a new subnet-id (see @ref generateNextID).
-    /// This subnet-id has unique value that is strictly monotonously increasing
-    /// for each subnet, until it is explicitly reset back to 1 during
-    /// reconfiguration process.
-    ///
     /// @param prefix subnet prefix
     /// @param len prefix length for the subnet
-    /// @param id arbitrary subnet id, value of 0 triggers autogeneration
-    /// of subnet id
+    /// @param id arbitrary subnet id between 0 and 2^32-1 excluded.
     Subnet(const isc::asiolink::IOAddress& prefix, uint8_t len,
            const SubnetID id);
 
-    /// @brief virtual destructor
+    /// @brief virtual destructor.
     ///
     /// A virtual destructor is needed because other classes
     /// derive from this class.
     virtual ~Subnet() { };
 
-    /// @brief keeps the subnet-id value
-    ///
-    /// It is incremented every time a new Subnet object is created. It is reset
-    /// (@ref resetSubnetID) every time reconfiguration
-    /// occurs.
-    ///
-    /// Static value initialized in subnet.cc.
-    static SubnetID static_id_;
-
-    /// @brief returns the next unique Subnet-ID
-    ///
-    /// This method generates and returns the next unique subnet-id.
-    /// It is a strictly monotonously increasing value (1,2,3,...) for
-    /// each new Subnet object created. It can be explicitly reset
-    /// back to 1 during reconfiguration (@ref resetSubnetID).
-    ///
-    /// @return the next unique Subnet-ID
-    static SubnetID generateNextID() {
-        if (static_id_ == SUBNET_ID_MAX) {
-            resetSubnetID();
-        }
-
-        return (static_id_++);
-    }
-
-    /// @brief Checks if used pool type is valid
+    /// @brief Checks if used pool type is valid.
     ///
     /// Allowed type for Subnet4 is Pool::TYPE_V4.
     /// Allowed types for Subnet6 are Pool::TYPE_{IA,TA,PD}.
@@ -348,17 +351,36 @@ protected:
     /// @throw BadValue if invalid value is used
     virtual void checkType(Lease::Type type) const = 0;
 
-    /// @brief Returns a sum of possible leases in all pools
+    /// @brief Returns a sum of possible leases in all pools.
+    ///
     /// @param pools list of pools
     /// @return sum of possible leases
-    uint64_t sumPoolCapacity(const PoolCollection& pools) const;
+    isc::util::uint128_t sumPoolCapacity(const PoolCollection& pools) const;
 
-    /// @brief Returns a sum of possible leases in all pools allowing classes
+    /// @brief Returns a sum of possible leases in all pools allowing classes.
+    ///
     /// @param pools list of pools
     /// @param client_classes list of classes
     /// @return sum of possible/allowed leases
-    uint64_t sumPoolCapacity(const PoolCollection& pools,
-                             const ClientClasses& client_classes) const;
+    isc::util::uint128_t sumPoolCapacity(const PoolCollection& pools,
+                                         const ClientClasses& client_classes) const;
+
+    /// @brief Returns a sum of possible leases in all pools allowing classes
+    /// and matching selection criteria relative to provided hint prefix length.
+    ///
+    /// @note This function should be called only for PD pools.
+    ///
+    /// @param pools list of pools
+    /// @param client_classes list of classes
+    /// @param prefix_length_match type which indicates the selection criteria
+    /// for the pools relative to the provided hint prefix length
+    /// @param hint_prefix_length the hint prefix length that the client
+    /// provided
+    /// @return sum of possible/allowed leases
+    isc::util::uint128_t sumPoolCapacity(const PoolCollection& pools,
+                                         const ClientClasses& client_classes,
+                                         Allocator::PrefixLenMatchType prefix_length_match,
+                                         uint8_t hint_prefix_length) const;
 
     /// @brief Checks if the specified pool overlaps with an existing pool.
     ///
@@ -367,7 +389,7 @@ protected:
     /// it overlaps with any existing pool within this subnet.
     ///
     /// @return true if pool overlaps with an existing pool of a specified
-    /// type.
+    /// type, false otherwise
     bool poolOverlaps(const Lease::Type& pool_type, const PoolPtr& pool) const;
 
     /// @brief Unparse a subnet object.
@@ -375,6 +397,12 @@ protected:
     /// @return A pointer to unparsed subnet configuration.
     virtual data::ElementPtr toElement() const;
 
+    virtual std::string getLabel() const {
+        std::stringstream ss;
+        ss << "subnet-id " << id_;
+        return (ss.str());
+    }
+
     /// @brief Converts subnet prefix to a pair of prefix/length pair.
     ///
     /// IPv4 and IPv6 specific conversion functions should apply extra checks
@@ -392,51 +420,29 @@ protected:
     /// a Subnet4 or Subnet6.
     SubnetID id_;
 
-    /// @brief collection of IPv4 or non-temporary IPv6 pools in that subnet
+    /// @brief collection of IPv4 or non-temporary IPv6 pools in that subnet.
     PoolCollection pools_;
 
-    /// @brief collection of IPv6 temporary address pools in that subnet
+    /// @brief collection of IPv6 temporary address pools in that subnet.
     PoolCollection pools_ta_;
 
-    /// @brief collection of IPv6 prefix pools in that subnet
+    /// @brief collection of IPv6 prefix pools in that subnet.
     PoolCollection pools_pd_;
 
-    /// @brief a prefix of the subnet
+    /// @brief a prefix of the subnet.
     isc::asiolink::IOAddress prefix_;
 
-    /// @brief a prefix length of the subnet
+    /// @brief a prefix length of the subnet.
     uint8_t prefix_len_;
 
-    /// @brief last allocated address
-    ///
-    /// This is the last allocated address that was previously allocated from
-    /// this particular subnet. Some allocation algorithms (e.g. iterative) use
-    /// that value, others do not. It should be noted that although the value
-    /// is usually correct, there are cases when it is invalid, e.g. after
-    /// removing a pool, restarting or changing allocation algorithms. For
-    /// that purpose it should be only considered a help that should not be
-    /// fully trusted.
-    isc::asiolink::IOAddress last_allocated_ia_;
-
-    /// @brief last allocated temporary address
-    ///
-    /// See @ref last_allocated_ia_ for details.
-    isc::asiolink::IOAddress last_allocated_ta_;
-
-    /// @brief last allocated IPv6 prefix
-    ///
-    /// See @ref last_allocated_ia_ for details.
-    isc::asiolink::IOAddress last_allocated_pd_;
-
-    /// @brief Timestamp indicating when a lease of a specified type has been
-    /// last allocated from this subnet.
-    std::map<Lease::Type, boost::posix_time::ptime> last_allocated_time_;
-
-    /// @brief Name of the network interface (if connected directly)
-    std::string iface_;
-
     /// @brief Shared network name.
     std::string shared_network_name_;
+
+    /// @brief Lease allocators used by the subnet.
+    std::map<Lease::Type, AllocatorPtr> allocators_;
+
+    /// @brief Holds subnet-specific allocation state.
+    std::map<Lease::Type, SubnetAllocationStatePtr> allocation_states_;
 };
 
 /// @brief A generic pointer to either Subnet4 or Subnet6 object
@@ -456,10 +462,10 @@ typedef boost::shared_ptr<Subnet4> Subnet4Ptr;
 /// This class represents an IPv4 subnet.
 /// @note Subnet and Network use virtual inheritance to avoid
 /// a diamond issue with UserContext
-class Subnet4 : public Subnet, public Network4 {
+class Subnet4 : public Subnet, public Network4, public boost::enable_shared_from_this<Subnet4> {
 public:
 
-    /// @brief Constructor with all parameters
+    /// @brief Constructor with all parameters.
     ///
     /// This constructor calls Subnet::Subnet, where subnet-id is generated.
     ///
@@ -468,13 +474,12 @@ public:
     /// @param t1 renewal timer (in seconds)
     /// @param t2 rebind timer (in seconds)
     /// @param valid_lifetime preferred lifetime of leases (in seconds)
-    /// @param id arbitrary subnet id, default value of 0 triggers
-    /// autogeneration of subnet id
+    /// @param id arbitrary subnet id, no default value
     Subnet4(const isc::asiolink::IOAddress& prefix, uint8_t length,
-            const Triplet<uint32_t>& t1,
-            const Triplet<uint32_t>& t2,
-            const Triplet<uint32_t>& valid_lifetime,
-            const SubnetID id = 0);
+            const util::Triplet<uint32_t>& t1,
+            const util::Triplet<uint32_t>& t2,
+            const util::Triplet<uint32_t>& valid_lifetime,
+            const SubnetID id);
 
     /// @brief Factory function creating an instance of the @c Subnet4.
     ///
@@ -483,21 +488,24 @@ public:
     /// unloaded before the object is destroyed. This ensures that the
     /// ownership of the object by the Kea process is retained.
     ///
+    /// It associates the subnet with the default, iterative, allocator.
+    /// Therefore, using this function should be preferred over the
+    /// constructor whenever the subnet needs a default allocator.
+    ///
     /// @param prefix Subnet4 prefix
     /// @param length prefix length
     /// @param t1 renewal timer (in seconds)
     /// @param t2 rebind timer (in seconds)
     /// @param valid_lifetime preferred lifetime of leases (in seconds)
-    /// @param id arbitrary subnet id, default value of 0 triggers
-    /// autogeneration of subnet id
+    /// @param id arbitrary subnet id, no default value
     ///
     /// @return Pointer to the @c Subnet4 instance.
     static Subnet4Ptr
     create(const isc::asiolink::IOAddress& prefix, uint8_t length,
-           const Triplet<uint32_t>& t1,
-           const Triplet<uint32_t>& t2,
-           const Triplet<uint32_t>& valid_lifetime,
-           const SubnetID id = 0);
+           const util::Triplet<uint32_t>& t1,
+           const util::Triplet<uint32_t>& t2,
+           const util::Triplet<uint32_t>& valid_lifetime,
+           const SubnetID id);
 
     /// @brief Returns next subnet within shared network.
     ///
@@ -563,6 +571,15 @@ public:
     /// @return A pointer to unparsed subnet configuration.
     virtual data::ElementPtr toElement() const;
 
+    /// @brief Instantiates the allocator and its state.
+    ///
+    /// It uses the type of the allocator specified with the
+    /// @c Network::setAllocatorType method.
+    ///
+    /// This function is called from the subnet parsers and after fetching
+    /// the subnet configuration from a configuration backend.
+    virtual void createAllocators();
+
     /// @brief Converts subnet prefix to a pair of prefix/length pair.
     ///
     /// @param prefix Prefix to be parsed.
@@ -572,13 +589,20 @@ public:
 
 private:
 
-    /// @brief Returns default address for pool selection
+    /// @brief Deleted copy constructor.
+    Subnet4(const Subnet4&) = delete;
+
+    /// @brief Deleted assignment operator.
+    Subnet4& operator=(const Subnet4&) = delete;
+
+    /// @brief Returns default address for pool selection.
+    ///
     /// @return ANY IPv4 address
     virtual isc::asiolink::IOAddress default_pool() const {
         return (isc::asiolink::IOAddress("0.0.0.0"));
     }
 
-    /// @brief Checks if used pool type is valid
+    /// @brief Checks if used pool type is valid.
     ///
     /// Allowed type for Subnet4 is Pool::TYPE_V4.
     ///
@@ -603,10 +627,10 @@ typedef boost::shared_ptr<Subnet6> Subnet6Ptr;
 /// This class represents an IPv6 subnet.
 /// @note Subnet and Network use virtual inheritance to avoid
 /// a diamond issue with UserContext
-class Subnet6 : public Subnet, public Network6 {
+class Subnet6 : public Subnet, public Network6, public boost::enable_shared_from_this<Subnet6> {
 public:
 
-    /// @brief Constructor with all parameters
+    /// @brief Constructor with all parameters.
     ///
     /// This constructor calls Subnet::Subnet, where subnet-id is generated.
     ///
@@ -616,14 +640,14 @@ public:
     /// @param t2 rebind timer (in seconds)
     /// @param preferred_lifetime preferred lifetime of leases (in seconds)
     /// @param valid_lifetime preferred lifetime of leases (in seconds)
-    /// @param id arbitrary subnet id, default value of 0 triggers
+    /// @param id arbitrary subnet id, no default value
     /// autogeneration of subnet id
     Subnet6(const isc::asiolink::IOAddress& prefix, uint8_t length,
-            const Triplet<uint32_t>& t1,
-            const Triplet<uint32_t>& t2,
-            const Triplet<uint32_t>& preferred_lifetime,
-            const Triplet<uint32_t>& valid_lifetime,
-            const SubnetID id = 0);
+            const util::Triplet<uint32_t>& t1,
+            const util::Triplet<uint32_t>& t2,
+            const util::Triplet<uint32_t>& preferred_lifetime,
+            const util::Triplet<uint32_t>& valid_lifetime,
+            const SubnetID id);
 
     /// @brief Factory function creating an instance of the @c Subnet4.
     ///
@@ -632,23 +656,26 @@ public:
     /// unloaded before the object is destroyed. This ensures that the
     /// ownership of the object by the Kea process is retained.
     ///
+    /// It associates the subnet with the default, iterative, allocator.
+    /// Therefore, using this function should be preferred over the
+    /// constructor whenever the subnet needs a default allocator.
+    ///
     /// @param prefix Subnet6 prefix
     /// @param length prefix length
     /// @param t1 renewal timer (in seconds)
     /// @param t2 rebind timer (in seconds)
     /// @param preferred_lifetime preferred lifetime of leases (in seconds)
     /// @param valid_lifetime preferred lifetime of leases (in seconds)
-    /// @param id arbitrary subnet id, default value of 0 triggers
-    /// autogeneration of subnet id
+    /// @param id arbitrary subnet id, no default value
     ///
     /// @return Pointer to the @c Subnet6 instance.
     static Subnet6Ptr
     create(const isc::asiolink::IOAddress& prefix, uint8_t length,
-           const Triplet<uint32_t>& t1,
-           const Triplet<uint32_t>& t2,
-           const Triplet<uint32_t>& preferred_lifetime,
-           const Triplet<uint32_t>& valid_lifetime,
-           const SubnetID id = 0);
+           const util::Triplet<uint32_t>& t1,
+           const util::Triplet<uint32_t>& t2,
+           const util::Triplet<uint32_t>& preferred_lifetime,
+           const util::Triplet<uint32_t>& valid_lifetime,
+           const SubnetID id);
 
     /// @brief Returns next subnet within shared network.
     ///
@@ -693,6 +720,15 @@ public:
     virtual bool
     clientSupported(const isc::dhcp::ClientClasses& client_classes) const;
 
+    /// @brief Instantiates the allocators and their states.
+    ///
+    /// It determines the types of the allocators to create using the list of
+    /// the allocator types specified with the @c Network::setAllocatorType method.
+    ///
+    /// This function is called from the subnet parsers and after fetching
+    /// the subnet configuration from a configuration backend.
+    virtual void createAllocators();
+
     /// @brief Unparse a subnet object.
     ///
     /// @return A pointer to unparsed subnet configuration.
@@ -707,7 +743,14 @@ public:
 
 private:
 
+    /// @brief Deleted copy constructor.
+    Subnet6(const Subnet6&) = delete;
+
+    /// @brief Deleted assignment operator.
+    Subnet6& operator=(const Subnet6&) = delete;
+
     /// @brief Returns default address for pool selection
+    ///
     /// @return ANY IPv6 address
     virtual isc::asiolink::IOAddress default_pool() const {
         return (isc::asiolink::IOAddress("::"));
@@ -727,9 +770,6 @@ private:
 ///
 //@{
 
-/// @brief Tag for the random access index.
-struct SubnetRandomAccessIndexTag { };
-
 /// @brief Tag for the index for searching by subnet identifier.
 struct SubnetSubnetIdIndexTag { };
 
@@ -742,7 +782,31 @@ struct SubnetServerIdIndexTag { };
 /// @brief Tag for the index for searching by subnet modification time.
 struct SubnetModificationTimeIndexTag { };
 
-/// @brief A collection of @c Subnet4 objects
+/// @brief A simple collection of @c Subnet4 objects
+///
+/// This container provides a set of indexes which can be used to retrieve
+/// subnets by subnet identifier and subnet prefix.
+///
+/// The random access index is used for direct iteration over the collection.
+typedef boost::multi_index_container<
+    // Multi index container holds pointers to the subnets.
+    Subnet4Ptr,
+    // The following holds all indexes.
+    boost::multi_index::indexed_by<
+        // First index allows for searching using subnet identifier.
+        boost::multi_index::ordered_unique<
+            boost::multi_index::tag<SubnetSubnetIdIndexTag>,
+            boost::multi_index::const_mem_fun<Subnet, SubnetID, &Subnet::getID>
+        >,
+        // Second index allows for searching using an output from toText function.
+        boost::multi_index::ordered_unique<
+            boost::multi_index::tag<SubnetPrefixIndexTag>,
+            boost::multi_index::const_mem_fun<Subnet, std::string, &Subnet::toText>
+        >
+    >
+> Subnet4SimpleCollection;
+
+/// @brief A collection of @c Subnet4 objects.
 ///
 /// This container provides a set of indexes which can be used to retrieve
 /// subnets by various properties.
@@ -770,30 +834,25 @@ typedef boost::multi_index_container<
     Subnet4Ptr,
     // The following holds all indexes.
     boost::multi_index::indexed_by<
-        // First is the random access index allowing for accessing
-        // objects just like we'd do with a vector.
-        boost::multi_index::random_access<
-            boost::multi_index::tag<SubnetRandomAccessIndexTag>
-        >,
-        // Second index allows for searching using subnet identifier.
+        // First index allows for searching using subnet identifier.
         boost::multi_index::ordered_unique<
             boost::multi_index::tag<SubnetSubnetIdIndexTag>,
             boost::multi_index::const_mem_fun<Subnet, SubnetID, &Subnet::getID>
         >,
-        // Third index allows for searching using an output from toText function.
+        // Second index allows for searching using an output from toText function.
         boost::multi_index::ordered_unique<
             boost::multi_index::tag<SubnetPrefixIndexTag>,
             boost::multi_index::const_mem_fun<Subnet, std::string, &Subnet::toText>
         >,
 
-        // Fourth index allows for searching using an output from getServerId.
+        // Third index allows for searching using an output from getServerId.
         boost::multi_index::ordered_non_unique<
             boost::multi_index::tag<SubnetServerIdIndexTag>,
             boost::multi_index::const_mem_fun<Network4, asiolink::IOAddress,
                                               &Network4::getServerId>
         >,
 
-        // Fifth index allows for searching using subnet modification time.
+        // Forth index allows for searching using subnet modification time.
         boost::multi_index::ordered_non_unique<
             boost::multi_index::tag<SubnetModificationTimeIndexTag>,
             boost::multi_index::const_mem_fun<data::BaseStampedElement,
@@ -803,6 +862,30 @@ typedef boost::multi_index_container<
     >
 > Subnet4Collection;
 
+/// @brief A simple collection of @c Subnet6 objects
+///
+/// This container provides a set of indexes which can be used to retrieve
+/// subnets by subnet identifier and subnet prefix.
+///
+/// The random access index is used for direct iteration over the collection.
+typedef boost::multi_index_container<
+    // Multi index container holds pointers to the subnets.
+    Subnet6Ptr,
+    // The following holds all indexes.
+    boost::multi_index::indexed_by<
+        // First index allows for searching using subnet identifier.
+        boost::multi_index::ordered_unique<
+            boost::multi_index::tag<SubnetSubnetIdIndexTag>,
+            boost::multi_index::const_mem_fun<Subnet, SubnetID, &Subnet::getID>
+        >,
+        // Second index allows for searching using an output from toText function.
+        boost::multi_index::ordered_unique<
+            boost::multi_index::tag<SubnetPrefixIndexTag>,
+            boost::multi_index::const_mem_fun<Subnet, std::string, &Subnet::toText>
+        >
+    >
+> Subnet6SimpleCollection;
+
 /// @brief A collection of @c Subnet6 objects
 ///
 /// This container provides a set of indexes which can be used to retrieve
@@ -830,22 +913,17 @@ typedef boost::multi_index_container<
     Subnet6Ptr,
     // The following holds all indexes.
     boost::multi_index::indexed_by<
-        // First is the random access index allowing for accessing
-        // objects just like we'd do with a vector.
-        boost::multi_index::random_access<
-            boost::multi_index::tag<SubnetRandomAccessIndexTag>
-        >,
-        // Second index allows for searching using subnet identifier.
+        // First index allows for searching using subnet identifier.
         boost::multi_index::ordered_unique<
             boost::multi_index::tag<SubnetSubnetIdIndexTag>,
             boost::multi_index::const_mem_fun<Subnet, SubnetID, &Subnet::getID>
         >,
-        // Third index allows for searching using an output from toText function.
+        // Second index allows for searching using an output from toText function.
         boost::multi_index::ordered_unique<
             boost::multi_index::tag<SubnetPrefixIndexTag>,
             boost::multi_index::const_mem_fun<Subnet, std::string, &Subnet::toText>
         >,
-        // Fourth index allows for searching using subnet modification time.
+        // Third index allows for searching using subnet modification time.
         boost::multi_index::ordered_non_unique<
             boost::multi_index::tag<SubnetModificationTimeIndexTag>,
             boost::multi_index::const_mem_fun<data::BaseStampedElement,
@@ -860,7 +938,8 @@ typedef boost::multi_index_container<
 ///
 /// @tparam ReturnPtrType Type of the returned object, i.e. @c Subnet4Ptr
 /// or @c Subnet6Ptr.
-/// @tparam CollectionType One of the @c Subnet4Collection or @c Subnet6Collection.
+/// @tparam CollectionType One of the @c Subnet4SimpleCollection,
+/// @c Subnet4Collection, @c Subnet6SimpleCollection or @c Subnet6Collection.
 template<typename ReturnPtrType, typename CollectionType>
 class SubnetFetcher {
 public:
@@ -889,10 +968,34 @@ using SubnetFetcher4 = SubnetFetcher<Subnet4Ptr, Subnet4Collection>;
 
 /// @brief Type of the @c SubnetFetcher used for IPv6.
 using SubnetFetcher6 = SubnetFetcher<Subnet6Ptr, Subnet6Collection>;
-
-
 //@}
 
+/// @brief adapters for linking templates to qualified names
+/// @{
+namespace {
+
+template <isc::util::DhcpSpace D>
+struct AdapterSubnet {};
+
+template <>
+struct AdapterSubnet<isc::util::DHCPv4> {
+    using type = Subnet4;
+};
+
+template <>
+struct AdapterSubnet<isc::util::DHCPv6> {
+    using type = Subnet6;
+};
+
+}  // namespace
+
+template <isc::util::DhcpSpace D>
+using SubnetT = typename AdapterSubnet<D>::type;
+
+template <isc::util::DhcpSpace D>
+using SubnetTPtr = boost::shared_ptr<SubnetT<D>>;
+/// @}
+
 } // end of isc::dhcp namespace
 } // end of isc namespace
 
