@@ -1,10 +1,11 @@
-// Copyright (C) 2014-2020 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
+
 #include <asiolink/io_address.h>
 #include <dhcp/duid.h>
 #include <dhcp/hwaddr.h>
@@ -12,7 +13,10 @@
 #include <dhcpsrv/cfg_hosts_util.h>
 #include <dhcpsrv/host.h>
 #include <dhcpsrv/cfgmgr.h>
+#include <testutils/gtest_utils.h>
+
 #include <gtest/gtest.h>
+
 #include <sstream>
 #include <set>
 
@@ -273,6 +277,34 @@ TEST_F(CfgHostsTest, getAll6BySubnet) {
     }
 }
 
+// This test checks that hosts with the same reserved address can be retrieved
+// from the host configuration.
+TEST_F(CfgHostsTest, getAll6ByAddress) {
+    CfgHosts cfg;
+    // Add 25 hosts identified by DUID in the same subnet.
+    for (unsigned i = 0; i < 25; ++i) {
+        HostPtr host = HostPtr(new Host(duids_[i]->toText(), "duid",
+                                        SUBNET_ID_UNUSED, SubnetID(i+1),
+                                        IOAddress("0.0.0.0")));
+        host->addReservation(IPv6Resrv(IPv6Resrv::TYPE_NA,
+                                       increase(IOAddress("2001:db8:1::1"),
+                                                i % 5)));
+        cfg.add(host);
+    }
+
+    // Try to retrieve all added reservations with IP equals 2001:db8:1::1.
+    auto hosts = cfg.getAll6(IOAddress("2001:db8:1::1"));
+    EXPECT_EQ(5, hosts.size());
+    for (unsigned i = 0; i < 5; ++i) {
+        EXPECT_EQ(1 + 5 * i, hosts[i]->getIPv6SubnetID());
+        IPv6ResrvRange reservations =
+            hosts[i]->getIPv6Reservations(IPv6Resrv::TYPE_NA);
+        ASSERT_EQ(1, std::distance(reservations.first, reservations.second));
+        EXPECT_EQ(IOAddress("2001:db8:1::1"),
+                  reservations.first->second.getPrefix());
+    }
+}
+
 // This test checks that hosts in the same subnet can be retrieved from
 // the host configuration by pages.
 TEST_F(CfgHostsTest, getPage4) {
@@ -443,15 +475,233 @@ TEST_F(CfgHostsTest, getAll4ByAddress) {
 
     HostCollection hosts = cfg.getAll4(IOAddress("192.0.2.10"));
     std::set<uint32_t> subnet_ids;
-    for (HostCollection::const_iterator host = hosts.begin(); host != hosts.end();
-         ++host) {
-        subnet_ids.insert((*host)->getIPv4SubnetID());
+    for (auto const& host : hosts) {
+        subnet_ids.insert(host->getIPv4SubnetID());
     }
     ASSERT_EQ(25, subnet_ids.size());
     EXPECT_EQ(1, *subnet_ids.begin());
     EXPECT_EQ(25, *subnet_ids.rbegin());
 }
 
+// This test checks that the IPv4 reservation for the specified IPv4 address can
+// be deleted.
+TEST_F(CfgHostsTest, deleteForIPv4) {
+    CfgHosts cfg;
+    // Add hosts.
+    IOAddress address("10.0.0.42");
+    SubnetID subnet_id(42);
+    size_t host_count = 10;
+
+    for (size_t i = 0; i < host_count; i++)
+    {
+        cfg.add(HostPtr(new Host(hwaddrs_[i]->toText(false),
+                                "hw-address",
+                                subnet_id, SUBNET_ID_UNUSED,
+                                increase(address, i))));
+    }
+
+    // Get all inserted hosts.
+    HostCollection hosts_by_subnet = cfg.getAll4(subnet_id);
+    HostCollection hosts_by_address = cfg.getAll4(address);
+    // Make sure the hosts and IP reservations were added.
+    ASSERT_EQ(host_count, hosts_by_subnet.size());
+    ASSERT_EQ(1, hosts_by_address.size());
+
+    // Delete one host.
+    EXPECT_TRUE(cfg.del(subnet_id, address));
+
+    // Check if the host is actually deleted.
+    hosts_by_subnet = cfg.getAll4(subnet_id);
+    hosts_by_address = cfg.getAll4(address);
+    EXPECT_EQ(host_count-1, hosts_by_subnet.size());
+    EXPECT_EQ(0, hosts_by_address.size());
+}
+
+// This test checks that the IPv6 reservation for the specified subnet ID and
+// IPv6 address can be deleted.
+TEST_F(CfgHostsTest, deleteForIPv6) {
+    CfgHosts cfg;
+    // Add hosts.
+    IOAddress address("2001:db8:1::1");
+    size_t host_count = 10;
+    SubnetID subnet_id(42);
+
+    for (size_t i = 0; i < host_count; i++)
+    {
+        HostPtr host = HostPtr(new Host(duids_[i]->toText(), "duid",
+                                        SUBNET_ID_UNUSED, subnet_id,
+                                        IOAddress::IPV4_ZERO_ADDRESS()));
+        host->addReservation(IPv6Resrv(IPv6Resrv::TYPE_NA,
+                                       increase(IOAddress(address), i)));
+        cfg.add(host);
+    }
+
+
+    // Get all inserted hosts.
+    auto hosts_by_subnet_and_address = cfg.getAll6(subnet_id, address);
+    auto hosts_by_subnet = cfg.getAll6(subnet_id);
+    // Make sure the hosts and IP reservations were added.
+    ASSERT_EQ(1, hosts_by_subnet_and_address.size());
+    ASSERT_EQ(host_count, hosts_by_subnet.size());
+
+    // Delete one host.
+    EXPECT_TRUE(cfg.del(subnet_id, address));
+
+    // Check if the host is actually deleted.
+    hosts_by_subnet_and_address = cfg.getAll6(subnet_id, address);
+    hosts_by_subnet = cfg.getAll6(subnet_id);
+    EXPECT_EQ(0, hosts_by_subnet_and_address.size());
+    EXPECT_EQ(host_count-1, hosts_by_subnet.size());
+}
+
+// This test checks that false is returned for deleting the IPv4 reservation
+// that doesn't exist.
+TEST_F(CfgHostsTest, deleteForMissingIPv4) {
+    CfgHosts cfg;
+
+    // Delete non-existent host.
+    EXPECT_FALSE(cfg.del(SubnetID(42), IOAddress(("10.0.0.42"))));
+}
+
+// This test checks that false is returned for deleting the IPv6 reservation
+// that doesn't exist.
+TEST_F(CfgHostsTest, deleteForMissingIPv6) {
+    CfgHosts cfg;
+
+    // Delete non-existent host.
+    EXPECT_FALSE(cfg.del(SubnetID(42), IOAddress(("2001:db8:1::1"))));
+}
+
+// This test checks that the reservation for the specified IPv4 subnet and
+// identifier can be deleted.
+TEST_F(CfgHostsTest, del4) {
+    CfgHosts cfg;
+
+    // Add hosts.
+    size_t host_count = 20;
+    size_t host_id = 5;
+    SubnetID subnet_id(42);
+    IOAddress address("10.0.0.1");
+
+    // Add half of the hosts with the same subnet ID but differ with DUID and
+    // address.
+    for (size_t i = 0; i < host_count / 2; i++) {
+        HostPtr host = HostPtr(new Host(duids_[i]->toText(), "duid",
+                                        subnet_id, SUBNET_ID_UNUSED,
+                                        increase(address, i)));
+        cfg.add(host);
+    }
+    // Add half of the hosts with the same subnet DUID and address but
+    // differ with address.
+    for (size_t i = 0; i < host_count / 2; i++) {
+        HostPtr host = HostPtr(new Host(duids_[host_id]->toText(), "duid",
+                                        SubnetID(subnet_id + i + 1), SUBNET_ID_UNUSED,
+                                        increase(address, host_id)));
+        cfg.add(host);
+    }
+
+
+    // Get all inserted hosts.
+    HostCollection hosts_by_subnet = cfg.getAll4(subnet_id);
+    HostCollection hosts_by_address = cfg.getAll4(increase(address, host_id));
+    HostPtr host = cfg.get4(subnet_id, Host::IdentifierType::IDENT_DUID,
+                                             &duids_[host_id]->getDuid()[0],
+                                             duids_[host_id]->getDuid().size());
+    // Make sure the hosts and IP reservations were added.
+    ASSERT_EQ(host_count / 2, hosts_by_subnet.size());
+    ASSERT_EQ(host_count / 2 + 1, hosts_by_address.size());
+    ASSERT_TRUE(host);
+
+    // Delete one host.
+    EXPECT_TRUE(cfg.del4(subnet_id, Host::IdentifierType::IDENT_DUID,
+                         &duids_[host_id]->getDuid()[0], duids_[host_id]->getDuid().size()));
+
+    // Check if the host is actually deleted.
+    hosts_by_subnet = cfg.getAll4(subnet_id);
+    hosts_by_address = cfg.getAll4(increase(address, host_id));
+    host = cfg.get4(subnet_id, Host::IdentifierType::IDENT_DUID,
+                                             &duids_[host_id]->getDuid()[0],
+                                             duids_[host_id]->getDuid().size());
+    EXPECT_EQ((host_count / 2)-1, hosts_by_subnet.size());
+    EXPECT_EQ(host_count / 2, hosts_by_address.size());
+    EXPECT_FALSE(host);
+}
+
+// This test checks that the host and its reservations for the specified IPv6
+// subnet and identifier can be deleted.
+TEST_F(CfgHostsTest, del6) {
+    CfgHosts cfg;
+
+    // Add hosts.
+    size_t host_count = 20;
+    size_t host_id = 5;
+    SubnetID subnet_id(42);
+    IOAddress address("2001:db8:1::1");
+
+    // Add half of the hosts with the same subnet ID but differ with DUID and
+    // address.
+    for (size_t i = 0; i < host_count / 2; i++) {
+        HostPtr host = HostPtr(new Host(duids_[i]->toText(), "duid",
+                                        SUBNET_ID_UNUSED, subnet_id,
+                                        IOAddress::IPV4_ZERO_ADDRESS()));
+        host->addReservation(IPv6Resrv(IPv6Resrv::TYPE_NA,
+                                       increase(IOAddress(address), i)));
+        cfg.add(host);
+    }
+    // Add half of the hosts with the same subnet DUID and address but
+    // differ with address.
+    for (size_t i = 0; i < host_count / 2; i++) {
+        HostPtr host = HostPtr(new Host(duids_[host_id]->toText(), "duid",
+                                        SUBNET_ID_UNUSED, SubnetID(subnet_id + i + 1),
+                                        IOAddress::IPV4_ZERO_ADDRESS()));
+        host->addReservation(IPv6Resrv(IPv6Resrv::TYPE_NA,
+                                       increase(address, host_id)));
+        cfg.add(host);
+    }
+
+
+    // Get all inserted hosts.
+    HostCollection hosts_by_subnet = cfg.getAll6(subnet_id);
+    HostCollection hosts_by_address = cfg.getAll6(increase(address, host_id));
+    HostPtr host = cfg.get6(subnet_id, Host::IdentifierType::IDENT_DUID,
+                                             &duids_[host_id]->getDuid()[0],
+                                             duids_[host_id]->getDuid().size());
+    // Make sure the hosts and IP reservations were added.
+    ASSERT_EQ(host_count / 2, hosts_by_subnet.size());
+    ASSERT_EQ(host_count / 2 + 1, hosts_by_address.size());
+    ASSERT_TRUE(host);
+
+    // Delete one host.
+    EXPECT_TRUE(cfg.del6(subnet_id, Host::IdentifierType::IDENT_DUID,
+                         &duids_[host_id]->getDuid()[0], duids_[host_id]->getDuid().size()));
+
+    // Check if the host is actually deleted.
+    hosts_by_subnet = cfg.getAll6(subnet_id);
+    hosts_by_address = cfg.getAll6(increase(address, host_id));
+    host = cfg.get6(subnet_id, Host::IdentifierType::IDENT_DUID,
+                                             &duids_[host_id]->getDuid()[0],
+                                             duids_[host_id]->getDuid().size());
+    EXPECT_EQ((host_count / 2)-1, hosts_by_subnet.size());
+    EXPECT_EQ(host_count / 2, hosts_by_address.size());
+    EXPECT_FALSE(host);
+}
+
+// This test checks that false is returned for deleting the IPv4 host that
+// doesn't exist.
+TEST_F(CfgHostsTest, del4MissingHost) {
+    CfgHosts cfg;
+    EXPECT_FALSE(cfg.del4(SubnetID(42), Host::IdentifierType::IDENT_DUID,
+                         &duids_[0]->getDuid()[0], duids_[0]->getDuid().size()));
+}
+
+// This test checks that false is returned for deleting the IPv6 host that
+// doesn't exist.
+TEST_F(CfgHostsTest, del6MissingHost) {
+    CfgHosts cfg;
+    EXPECT_FALSE(cfg.del6(SubnetID(42), Host::IdentifierType::IDENT_DUID,
+                         &duids_[0]->getDuid()[0], duids_[0]->getDuid().size()));
+}
+
 // This test checks that all reservations for the specified IPv4 subnet can
 // be deleted.
 TEST_F(CfgHostsTest, deleteAll4) {
@@ -469,15 +719,14 @@ TEST_F(CfgHostsTest, deleteAll4) {
                                  "hw-address",
                                  SubnetID(1 + i % 2), SUBNET_ID_UNUSED,
                                  IOAddress::IPV4_ZERO_ADDRESS(),
-                                 "hostname")));
+                                 s.str())));
     }
 
     // Get all inserted hosts.
     HostCollection hosts = cfg.getAll4(IOAddress::IPV4_ZERO_ADDRESS());
     std::set<uint32_t> subnet_ids;
-    for (HostCollection::const_iterator host = hosts.begin(); host != hosts.end();
-         ++host) {
-        subnet_ids.insert((*host)->getIPv4SubnetID());
+    for (auto const& host : hosts) {
+        subnet_ids.insert(host->getIPv4SubnetID());
     }
     // Make sure there are two unique subnets: 1 and 2.
     ASSERT_EQ(2, subnet_ids.size());
@@ -490,9 +739,8 @@ TEST_F(CfgHostsTest, deleteAll4) {
     // Gather the host counts again.
     subnet_ids.clear();
     hosts = cfg.getAll4(IOAddress::IPV4_ZERO_ADDRESS());
-    for (HostCollection::const_iterator host = hosts.begin(); host != hosts.end();
-         ++host) {
-        subnet_ids.insert((*host)->getIPv4SubnetID());
+    for (auto const& host : hosts) {
+        subnet_ids.insert(host->getIPv4SubnetID());
     }
     // We should only have hosts for one subnet and it should be the subnet
     // with ID of 1.
@@ -715,7 +963,7 @@ TEST_F(CfgHostsTest, deleteAll6) {
                       reservations.first->second.getPrefix());
 
         } else {
-            // All hosts for subnet id 2 should be gone.
+            // All hosts for subnet id 1 should be gone.
             EXPECT_FALSE(host);
         }
     }
@@ -1200,5 +1448,62 @@ TEST_F(CfgHostsTest, duplicatesSubnet6DUID) {
                                              "foo.example.com"))));
 }
 
+// Checks that updates work correctly.
+TEST_F(CfgHostsTest, update) {
+    CfgHosts cfg;
 
-} // end of anonymous namespace
+    HostPtr const host(boost::make_shared<Host>(duids_[0]->toText(), "duid", SUBNET_ID_UNUSED,
+                                                SubnetID(1), IOAddress("0.0.0.0"),
+                                                "foo.example.com"));
+
+    // Updating a host that doesn't exist should throw.
+    EXPECT_THROW_MSG(cfg.update(host), HostNotFound, "Host not updated (not found).");
+
+    // There should be no hosts.
+    HostCollection hosts(cfg.getAll6(SubnetID(1)));
+    EXPECT_EQ(0, hosts.size());
+
+    // Add a host.
+    EXPECT_NO_THROW(cfg.add(host));
+
+    // The host should be in the config.
+    hosts = cfg.getAll6(SubnetID(1));
+    ASSERT_EQ(1, hosts.size());
+    EXPECT_EQ("duid=010203040500 ipv6_subnet_id=1 hostname=foo.example.com "
+              "ipv4_reservation=(no) siaddr=(no) sname=(empty) file=(empty) "
+              "key=(empty) ipv6_reservations=(none)", hosts[0]->toText());
+
+    // Update the host. Change nothing.
+    EXPECT_NO_THROW(cfg.update(host));
+
+    // The same host should be in the config.
+    hosts = cfg.getAll6(SubnetID(1));
+    ASSERT_EQ(1, hosts.size());
+    EXPECT_EQ("duid=010203040500 ipv6_subnet_id=1 hostname=foo.example.com "
+              "ipv4_reservation=(no) siaddr=(no) sname=(empty) file=(empty) "
+              "key=(empty) ipv6_reservations=(none)", hosts[0]->toText());
+
+    // Update the host with new hostname.
+    host->setHostname("bar.example.com");
+    EXPECT_NO_THROW(cfg.update(host));
+
+    // The change should be reflected in the config.
+    hosts = cfg.getAll6(SubnetID(1));
+    ASSERT_EQ(1, hosts.size());
+    EXPECT_EQ("duid=010203040500 ipv6_subnet_id=1 hostname=bar.example.com "
+              "ipv4_reservation=(no) siaddr=(no) sname=(empty) file=(empty) "
+              "key=(empty) ipv6_reservations=(none)", hosts[0]->toText());
+
+    // Remove hostname from host.
+    host->setHostname("");
+    EXPECT_NO_THROW(cfg.update(host));
+
+    // The change should be reflected in the config.
+    hosts = cfg.getAll6(SubnetID(1));
+    ASSERT_EQ(1, hosts.size());
+    EXPECT_EQ("duid=010203040500 ipv6_subnet_id=1 hostname=(empty) "
+              "ipv4_reservation=(no) siaddr=(no) sname=(empty) file=(empty) "
+              "key=(empty) ipv6_reservations=(none)", hosts[0]->toText());
+}
+
+}  // namespace
