@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,9 +6,9 @@
 
 #include <config.h>
 #include <dhcp/dhcp4.h>
-#include <dhcp/tests/iface_mgr_test_config.h>
-#include <dhcp/tests/pkt_filter_test_stub.h>
-#include <dhcp/tests/pkt_filter6_test_stub.h>
+#include <dhcp/testutils/iface_mgr_test_config.h>
+#include <dhcp/testutils/pkt_filter_test_stub.h>
+#include <dhcp/testutils/pkt_filter6_test_stub.h>
 #include <dhcpsrv/cfg_iface.h>
 #include <asiolink/io_service.h>
 #include <asiolink/asio_wrapper.h>
@@ -117,12 +117,13 @@ CfgIfaceTest::unicastOpen(const std::string& iface_name) const {
 
 void
 CfgIfaceTest::doWait(const long timeout) {
-    asiolink::IntervalTimer timer(*io_service_);
+    asiolink::IntervalTimer timer(io_service_);
     timer.setup([this]() {
         io_service_->stop();
     }, timeout, asiolink::IntervalTimer::ONE_SHOT);
     io_service_->run();
-    io_service_->get_io_service().reset();
+    io_service_->stop();
+    io_service_->restart();
 }
 
 // This test checks that the interface names can be explicitly selected
