@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -34,7 +34,7 @@ void checkMergedNetwork(const CfgSharedNetworks4& networks, const std::string& n
     ASSERT_EQ(exp_valid, network->getValid().get()) << " network valid lifetime wrong";
     const Subnet4SimpleCollection* subnets = network->getAllSubnets();
     ASSERT_EQ(exp_subnets.size(), subnets->size()) << " wrong number of subnets";
-    for (auto exp_id : exp_subnets) {
+    for (auto const& exp_id : exp_subnets) {
         ASSERT_TRUE(network->getSubnet(exp_id))
                     << " did not find expected subnet: " << exp_id;
     }
@@ -370,6 +370,7 @@ TEST(CfgSharedNetworks4Test, mergeNetworks) {
     OptionStringPtr opstr = boost::dynamic_pointer_cast<OptionString>(desc.option_);
     ASSERT_TRUE(opstr);
     EXPECT_EQ("Yay!", opstr->getValue());
+    EXPECT_TRUE(network->getCfgOption()->isEncapsulated());
 
     // No changes to network2.
     ASSERT_NO_FATAL_FAILURE(checkMergedNetwork(cfg_to, "network2", Triplet<uint32_t>(200),
