@@ -1,4 +1,4 @@
-// Copyright (C) 2022-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2022-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -61,7 +61,6 @@ public:
     }
 
     /// @brief Exposes protected methods and members.
-    using LeaseMgr::setExtendedInfoTablesEnabled;
     using Memfile_LeaseMgr::relay_id6_;
     using Memfile_LeaseMgr::remote_id6_;
     using Memfile_LeaseMgr::deleteExtendedInfo6;
@@ -845,8 +844,6 @@ MemfileExtendedInfoTest::testGetLeases6ByRelayId() {
     IOAddress lease_addr0(ADDRESS6[0]);
     IOAddress lease_addr1(ADDRESS6[1]);
     IOAddress lease_addr2(ADDRESS6[2]);
-    IOAddress link_addr(ADDRESS6[4]);
-    IOAddress other_link_addr("2001:db8:1::4");
     IOAddress zero = IOAddress::IPV6_ZERO_ADDRESS();
     vector<uint8_t> relay_id_data0 = createFromString(DUIDS[0]);
     DUID relay_id0(relay_id_data0);
@@ -865,34 +862,14 @@ MemfileExtendedInfoTest::testGetLeases6ByRelayId() {
     EXPECT_EQ(6, lease_mgr_->relay_id6_.size());
 
     Lease6Collection got;
-    // Unknown relay id #2, no link: nothing.
+    // Unknown relay id #2: nothing.
     EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRelayId(relay_id2,
-                                                          zero,
-                                                          0,
                                                           zero,
                                                           LeasePageSize(100)));
     EXPECT_EQ(0, got.size());
 
-    // Unknown relay id #2, link: nothing.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRelayId(relay_id2,
-                                                          link_addr,
-                                                          64,
-                                                          zero,
-                                                          LeasePageSize(100)));
-    EXPECT_EQ(0, got.size());
-
-    // Relay id #0, other link: nothing.
+    // Relay id #0: 3 entries but 2 addresses.
     EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRelayId(relay_id0,
-                                                          other_link_addr,
-                                                          64,
-                                                          zero,
-                                                          LeasePageSize(100)));
-    EXPECT_EQ(0, got.size());
-
-    // Relay id #0, no link: 3 entries but 2 addresses.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRelayId(relay_id0,
-                                                          zero,
-                                                          0,
                                                           zero,
                                                           LeasePageSize(100)));
     ASSERT_EQ(2, got.size());
@@ -903,10 +880,8 @@ MemfileExtendedInfoTest::testGetLeases6ByRelayId() {
     ASSERT_TRUE(lease);
     EXPECT_EQ(lease_addr1, lease->addr_);
 
-    // Relay id #1, no link, partial: 2 entries.
+    // Relay id #1, partial: 2 entries.
     EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRelayId(relay_id1,
-                                                          zero,
-                                                          0,
                                                           zero,
                                                           LeasePageSize(2)));
     ASSERT_EQ(2, got.size());
@@ -919,76 +894,12 @@ MemfileExtendedInfoTest::testGetLeases6ByRelayId() {
 
     // Relay id #1, no link, partial from previous: 1 entry.
     EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRelayId(relay_id1,
-                                                          zero,
-                                                          0,
                                                           lease->addr_,
                                                           LeasePageSize(2)));
     ASSERT_EQ(1, got.size());
     lease = got[0];
     ASSERT_TRUE(lease);
     EXPECT_EQ(lease_addr2, lease->addr_);
-
-    // Add another entry for last tests.
-    EXPECT_NO_THROW(lease_mgr_->addRelayId6(lease_addr0, relay_id_data1));
-    EXPECT_EQ(7, lease_mgr_->relay_id6_.size());
-
-    // Relay id #1, link: 3 entries.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRelayId(relay_id1,
-                                                          link_addr,
-                                                          64,
-                                                          zero,
-                                                          LeasePageSize(100)));
-    ASSERT_EQ(3, got.size());
-    lease = got[0];
-    ASSERT_TRUE(lease);
-    EXPECT_EQ(lease_addr0, lease->addr_);
-    lease = got[1];
-    ASSERT_TRUE(lease);
-    EXPECT_EQ(lease_addr1, lease->addr_);
-    lease = got[2];
-    ASSERT_TRUE(lease);
-    EXPECT_EQ(lease_addr2, lease->addr_);
-
-    // Relay id #1, link, initial partial: 1 entry.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRelayId(relay_id1,
-                                                          link_addr,
-                                                          64,
-                                                          zero,
-                                                          LeasePageSize(1)));
-    ASSERT_EQ(1, got.size());
-    lease = got[0];
-    ASSERT_TRUE(lease);
-    EXPECT_EQ(lease_addr0, lease->addr_);
-
-    // Relay id #1, link, next partial: 1 entry.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRelayId(relay_id1,
-                                                          link_addr,
-                                                          64,
-                                                          lease->addr_,
-                                                          LeasePageSize(1)));
-    ASSERT_EQ(1, got.size());
-    lease = got[0];
-    ASSERT_TRUE(lease);
-    EXPECT_EQ(lease_addr1, lease->addr_);
-
-    // Relay id #1, link, next partial: 1 entry.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRelayId(relay_id1,
-                                                          link_addr,
-                                                          64,
-                                                          lease->addr_,
-                                                          LeasePageSize(1)));
-    ASSERT_EQ(1, got.size());
-    lease = got[0];
-    ASSERT_TRUE(lease);
-    EXPECT_EQ(lease_addr2, lease->addr_);
-
-    // Relay id #1, link, final partial: nothing.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRelayId(relay_id1,
-                                                          link_addr,
-                                                          64,
-                                                          lease->addr_,
-                                                          LeasePageSize(1)));
-    EXPECT_EQ(0, got.size());
 }
 
 TEST_F(MemfileExtendedInfoTest, getLeases6ByRelayId) {
@@ -1012,8 +923,6 @@ MemfileExtendedInfoTest::testGetLeases6ByRemoteId() {
     IOAddress lease_addr0(ADDRESS6[0]);
     IOAddress lease_addr1(ADDRESS6[1]);
     IOAddress lease_addr2(ADDRESS6[2]);
-    IOAddress link_addr(ADDRESS6[4]);
-    IOAddress other_link_addr("2001:db8:1::4");
     IOAddress zero = IOAddress::IPV6_ZERO_ADDRESS();
     vector<uint8_t> remote_id0 = createFromString(DUIDS[0]);
     vector<uint8_t> remote_id1 = createFromString(DUIDS[1]);
@@ -1029,34 +938,14 @@ MemfileExtendedInfoTest::testGetLeases6ByRemoteId() {
     EXPECT_EQ(6, lease_mgr_->remote_id6_.size());
 
     Lease6Collection got;
-    // Unknown remote id #2, no link: nothing.
+    // Unknown remote id #2: nothing.
     EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRemoteId(remote_id2,
-                                                           zero,
-                                                           0,
                                                            zero,
                                                            LeasePageSize(10)));
     EXPECT_EQ(0, got.size());
 
-    // Unknown remote id #2, link: nothing.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRemoteId(remote_id2,
-                                                           link_addr,
-                                                           64,
-                                                           zero,
-                                                           LeasePageSize(10)));
-    EXPECT_EQ(0, got.size());
-
-    // Remote id #0, other link: nothing.
+    // Remote id #0: 3 entries but 2 addresses.
     EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRemoteId(remote_id0,
-                                                           other_link_addr,
-                                                           64,
-                                                           zero,
-                                                           LeasePageSize(10)));
-    EXPECT_EQ(0, got.size());
-
-    // Remote id #0, no link: 3 entries but 2 addresses.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRemoteId(remote_id0,
-                                                           zero,
-                                                           0,
                                                            zero,
                                                            LeasePageSize(10)));
     ASSERT_EQ(2, got.size());
@@ -1067,10 +956,8 @@ MemfileExtendedInfoTest::testGetLeases6ByRemoteId() {
     ASSERT_TRUE(lease);
     EXPECT_EQ(lease_addr1, lease->addr_);
 
-    // Remote id #1, no link, partial: 2 entries.
+    // Remote id #1, partial: 2 entries.
     EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRemoteId(remote_id1,
-                                                           zero,
-                                                           0,
                                                            zero,
                                                            LeasePageSize(2)));
     ASSERT_EQ(2, got.size());
@@ -1081,78 +968,14 @@ MemfileExtendedInfoTest::testGetLeases6ByRemoteId() {
     ASSERT_TRUE(lease);
     EXPECT_EQ(lease_addr1, lease->addr_);
 
-    // Remote id #1, no link, partial from previous: 1 entry.
+    // Remote id #1, partial from previous: 1 entry.
     EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRemoteId(remote_id1,
-                                                           zero,
-                                                           0,
                                                            lease->addr_,
                                                            LeasePageSize(2)));
     ASSERT_EQ(1, got.size());
     lease = got[0];
     ASSERT_TRUE(lease);
     EXPECT_EQ(lease_addr2, lease->addr_);
-
-    // Add another entry for last tests.
-    EXPECT_NO_THROW(lease_mgr_->addRemoteId6(lease_addr0, remote_id1));
-    EXPECT_EQ(7, lease_mgr_->remote_id6_.size());
-
-    // Remote id #1, link: 3 entries.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRemoteId(remote_id1,
-                                                           link_addr,
-                                                           64,
-                                                           zero,
-                                                           LeasePageSize(10)));
-    ASSERT_EQ(3, got.size());
-    lease = got[0];
-    ASSERT_TRUE(lease);
-    EXPECT_EQ(lease_addr0, lease->addr_);
-    lease = got[1];
-    ASSERT_TRUE(lease);
-    EXPECT_EQ(lease_addr1, lease->addr_);
-    lease = got[2];
-    ASSERT_TRUE(lease);
-    EXPECT_EQ(lease_addr2, lease->addr_);
-
-    // Remote id #1, link, initial partial: 1 entry.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRemoteId(remote_id1,
-                                                           link_addr,
-                                                           64,
-                                                           zero,
-                                                           LeasePageSize(1)));
-    ASSERT_EQ(1, got.size());
-    lease = got[0];
-    ASSERT_TRUE(lease);
-    EXPECT_EQ(lease_addr0, lease->addr_);
-
-    // Remote id #1, link, next partial: 1 entry.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRemoteId(remote_id1,
-                                                           link_addr,
-                                                           64,
-                                                           lease->addr_,
-                                                           LeasePageSize(1)));
-    ASSERT_EQ(1, got.size());
-    lease = got[0];
-    ASSERT_TRUE(lease);
-    EXPECT_EQ(lease_addr1, lease->addr_);
-
-    // Remote id #1, link, next partial: 1 entry.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRemoteId(remote_id1,
-                                                           link_addr,
-                                                           64,
-                                                           lease->addr_,
-                                                           LeasePageSize(1)));
-    ASSERT_EQ(1, got.size());
-    lease = got[0];
-    ASSERT_TRUE(lease);
-    EXPECT_EQ(lease_addr2, lease->addr_);
-
-    // Remote id #1, link, final partial: nothing.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByRemoteId(remote_id1,
-                                                           link_addr,
-                                                           64,
-                                                           lease->addr_,
-                                                           LeasePageSize(1)));
-    EXPECT_EQ(0, got.size());
 }
 
 TEST_F(MemfileExtendedInfoTest, getLeases6ByRemoteId) {
@@ -1164,32 +987,33 @@ TEST_F(MemfileExtendedInfoTest, getLeases6ByRemoteIdMultiThreading) {
     testGetLeases6ByRemoteId();
 }
 
-/// @brief Verifies that getLeases6ByLink works as expected.
+/// @brief Verifies that paged getLeases6 by subnet id works as expected.
 void
 MemfileExtendedInfoTest::testGetLeases6ByLink() {
     // Lease manager is created with empty tables.
     start(Memfile_LeaseMgr::V6);
     initLease6();
 
+    // Put leases in the subnet 1.
+    for (size_t i = 0; i < leases6.size(); ++i) {
+        Lease6Ptr lease(new Lease6(*leases6[i]));
+        leases6[i] = lease;
+        lease->subnet_id_ = 1;
+        EXPECT_NO_THROW(lease_mgr_->updateLease6(leases6[i]));
+    }
+
     // Create parameter values.
-    IOAddress link_addr(ADDRESS6[4]);
-    IOAddress other_link_addr("2001:db8:1::4");
     IOAddress zero = IOAddress::IPV6_ZERO_ADDRESS();
 
     Lease6Collection got;
     // Other link: nothing.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByLink(other_link_addr,
-                                                       64,
-                                                       zero,
-                                                       LeasePageSize(10)));
+    EXPECT_NO_THROW(got = lease_mgr_->getLeases6(10, zero,
+                                                 LeasePageSize(10)));
     EXPECT_EQ(0, got.size());
 
     // Link: 8 entries.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByLink(link_addr,
-                                                       64,
-                                                       zero,
-                                                       LeasePageSize(10)));
-
+    EXPECT_NO_THROW(got = lease_mgr_->getLeases6(1, zero,
+                                                 LeasePageSize(10)));
     ASSERT_EQ(8, got.size());
     Lease6Ptr lease;
     for (size_t i = 0; i < 8; ++i) {
@@ -1199,10 +1023,8 @@ MemfileExtendedInfoTest::testGetLeases6ByLink() {
     }
 
     // Link: initial partial: 4 entries.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByLink(link_addr,
-                                                       64,
-                                                       zero,
-                                                       LeasePageSize(4)));
+    EXPECT_NO_THROW(got = lease_mgr_->getLeases6(1, zero,
+                                                 LeasePageSize(4)));
     ASSERT_EQ(4, got.size());
     for (size_t i = 0; i < 4; ++i) {
         lease = got[i];
@@ -1211,10 +1033,8 @@ MemfileExtendedInfoTest::testGetLeases6ByLink() {
     }
 
     // Link: next partial: 4 entries.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByLink(link_addr,
-                                                       64,
-                                                       lease->addr_,
-                                                       LeasePageSize(4)));
+    EXPECT_NO_THROW(got = lease_mgr_->getLeases6(1, lease->addr_,
+                                                 LeasePageSize(4)));
     ASSERT_EQ(4, got.size());
     for (size_t i = 0; i < 4; ++i) {
         lease = got[i];
@@ -1223,10 +1043,8 @@ MemfileExtendedInfoTest::testGetLeases6ByLink() {
     }
 
     // Link: further partial: nothing.
-    EXPECT_NO_THROW(got = lease_mgr_->getLeases6ByLink(link_addr,
-                                                       64,
-                                                       lease->addr_,
-                                                       LeasePageSize(4)));
+    EXPECT_NO_THROW(got = lease_mgr_->getLeases6(1, lease->addr_,
+                                                 LeasePageSize(4)));
     EXPECT_EQ(0, got.size());
 }
 
