@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -447,26 +447,26 @@ TEST_F(PgSqlHostDataSourceTest, globalSubnetId6MultiThreading) {
 }
 
 /// @brief Verifies that IPv4 host reservation with options can have a max value
-/// for  dhcp4_subnet id
+/// for dhcp4_subnet id
 TEST_F(PgSqlHostDataSourceTest, maxSubnetId4) {
     testMaxSubnetId4();
 }
 
 /// @brief Verifies that IPv4 host reservation with options can have a max value
-/// for  dhcp4_subnet id
+/// for dhcp4_subnet id
 TEST_F(PgSqlHostDataSourceTest, maxSubnetId4MultiThreading) {
     MultiThreadingTest mt(true);
     testMaxSubnetId4();
 }
 
 /// @brief Verifies that IPv6 host reservation with options can have a max value
-/// for  dhcp6_subnet id
+/// for dhcp6_subnet id
 TEST_F(PgSqlHostDataSourceTest, maxSubnetId6) {
     testMaxSubnetId6();
 }
 
 /// @brief Verifies that IPv6 host reservation with options can have a max value
-/// for  dhcp6_subnet id
+/// for dhcp6_subnet id
 TEST_F(PgSqlHostDataSourceTest, maxSubnetId6MultiThreading) {
     MultiThreadingTest mt(true);
     testMaxSubnetId6();
@@ -1639,6 +1639,13 @@ TEST_F(PgSQLHostMgrTest, del) {
     testDeleteByIDAndAddress(*getCfgHosts(), HostMgr::instance());
 }
 
+// This test verifies that the reservation can be deleted from database
+// by providing subnet ID and address, and other reservations in the subnet
+// remain undeleted.
+TEST_F(PgSQLHostMgrTest, delOneHost) {
+    testDeleteOneHostByIDAndAddress(HostMgr::instance());
+}
+
 // This test verifies that the IPv4 reservations can be deleted from a
 // configuration file and a database by subnet ID and identifier.
 TEST_F(PgSQLHostMgrTest, del4) {
@@ -1658,6 +1665,54 @@ TEST_F(PgSQLHostMgrTest, setIPReservationsUnique) {
     EXPECT_TRUE(HostMgr::instance().setIPReservationsUnique(false));
 }
 
+/// @brief Verifies that loss of connectivity to PostgreSQL is handled correctly.
+TEST_F(PgSQLHostMgrDbLostCallbackTest, testRetryOpenDbLostAndRecoveredCallback) {
+    MultiThreadingTest mt(false);
+    testRetryOpenDbLostAndRecoveredCallback();
+}
+
+/// @brief Verifies that loss of connectivity to PostgreSQL is handled correctly.
+TEST_F(PgSQLHostMgrDbLostCallbackTest, testRetryOpenDbLostAndRecoveredCallbackMultiThreading) {
+    MultiThreadingTest mt(true);
+    testRetryOpenDbLostAndRecoveredCallback();
+}
+
+/// @brief Verifies that loss of connectivity to PostgreSQL is handled correctly.
+TEST_F(PgSQLHostMgrDbLostCallbackTest, testRetryOpenDbLostAndFailedCallback) {
+    MultiThreadingTest mt(false);
+    testRetryOpenDbLostAndFailedCallback();
+}
+
+/// @brief Verifies that loss of connectivity to PostgreSQL is handled correctly.
+TEST_F(PgSQLHostMgrDbLostCallbackTest, testRetryOpenDbLostAndFailedCallbackMultiThreading) {
+    MultiThreadingTest mt(true);
+    testRetryOpenDbLostAndFailedCallback();
+}
+
+/// @brief Verifies that loss of connectivity to PostgreSQL is handled correctly.
+TEST_F(PgSQLHostMgrDbLostCallbackTest, testRetryOpenDbLostAndRecoveredAfterTimeoutCallback) {
+    MultiThreadingTest mt(false);
+    testRetryOpenDbLostAndRecoveredAfterTimeoutCallback();
+}
+
+/// @brief Verifies that loss of connectivity to PostgreSQL is handled correctly.
+TEST_F(PgSQLHostMgrDbLostCallbackTest, testRetryOpenDbLostAndRecoveredAfterTimeoutCallbackMultiThreading) {
+    MultiThreadingTest mt(true);
+    testRetryOpenDbLostAndRecoveredAfterTimeoutCallback();
+}
+
+/// @brief Verifies that loss of connectivity to PostgreSQL is handled correctly.
+TEST_F(PgSQLHostMgrDbLostCallbackTest, testRetryOpenDbLostAndFailedAfterTimeoutCallback) {
+    MultiThreadingTest mt(false);
+    testRetryOpenDbLostAndFailedAfterTimeoutCallback();
+}
+
+/// @brief Verifies that loss of connectivity to PostgreSQL is handled correctly.
+TEST_F(PgSQLHostMgrDbLostCallbackTest, testRetryOpenDbLostAndFailedAfterTimeoutCallbackMultiThreading) {
+    MultiThreadingTest mt(true);
+    testRetryOpenDbLostAndFailedAfterTimeoutCallback();
+}
+
 /// @brief Verifies that db lost callback is not invoked on an open failure
 TEST_F(PgSQLHostMgrDbLostCallbackTest, testNoCallbackOnOpenFailure) {
     MultiThreadingTest mt(false);
