@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -12,7 +12,7 @@
 #include <dhcp/option.h>
 #include <dhcp/option4_addrlst.h>
 #include <dhcp/option6_addrlst.h>
-#include <dhcp/tests/iface_mgr_test_config.h>
+#include <dhcp/testutils/iface_mgr_test_config.h>
 #include <dhcpsrv/cfg_option.h>
 #include <dhcpsrv/parsers/shared_network_parser.h>
 #include <testutils/gtest_utils.h>
@@ -92,10 +92,9 @@ public:
         ASSERT_EQ(test.addresses_.size(), network.getRelayAddresses().size());
 
         // Are the expected addresses in the list?
-        for (auto exp_address = test.addresses_.begin(); exp_address != test.addresses_.end();
-             ++exp_address) {
-            EXPECT_TRUE(network.hasRelayAddress(*exp_address))
-                        << " expected address: " << (*exp_address).toText() << " not found" ;
+        for (auto const& exp_address : test.addresses_) {
+            EXPECT_TRUE(network.hasRelayAddress(exp_address))
+                        << " expected address: " << exp_address.toText() << " not found" ;
         }
     }
 
@@ -384,24 +383,6 @@ TEST_F(SharedNetwork4ParserTest, relayInfoTests) {
 
     // Create the vector of test scenarios.
     std::vector<RelayTest> tests = {
-        {
-            "valid ip-address #1",
-            "{ \"ip-address\": \"192.168.2.1\" }",
-            true,
-            { asiolink::IOAddress("192.168.2.1") }
-        },
-        {
-            "invalid ip-address #1",
-            "{ \"ip-address\": \"not an address\" }",
-            false,
-            { }
-        },
-        {
-            "invalid ip-address #2",
-            "{ \"ip-address\": \"2001:db8::1\" }",
-            false,
-            { }
-        },
         {
             "valid ip-addresses #1",
             "{ \"ip-addresses\": [ ] }",
@@ -433,16 +414,7 @@ TEST_F(SharedNetwork4ParserTest, relayInfoTests) {
             { }
         },
         {
-            "invalid both ip-address and ip-addresses",
-            "{"
-            " \"ip-address\": \"192.168.2.1\", "
-            " \"ip-addresses\": [ \"192.168.2.1\", \"192.168.2.2\" ]"
-            " }",
-            false,
-            { }
-        },
-        {
-            "invalid neither ip-address nor ip-addresses",
+            "invalid no ip-addresses",
             "{}",
             false,
             { }
@@ -451,11 +423,9 @@ TEST_F(SharedNetwork4ParserTest, relayInfoTests) {
 
     // Iterate over the test scenarios, verifying each prescribed
     // outcome.
-    for (auto test = tests.begin(); test != tests.end(); ++test) {
-        {
-            SCOPED_TRACE((*test).description_);
-            relayTest(*test);
-        }
+    for (auto const& test : tests) {
+        SCOPED_TRACE(test.description_);
+        relayTest(test);
     }
 }
 
@@ -963,24 +933,6 @@ TEST_F(SharedNetwork6ParserTest, relayInfoTests) {
 
     // Create the vector of test scenarios.
     std::vector<RelayTest> tests = {
-        {
-            "valid ip-address #1",
-            "{ \"ip-address\": \"2001:db8::1\" }",
-            true,
-            { asiolink::IOAddress("2001:db8::1") }
-        },
-        {
-            "invalid ip-address #1",
-            "{ \"ip-address\": \"not an address\" }",
-            false,
-            { }
-        },
-        {
-            "invalid ip-address #2",
-            "{ \"ip-address\": \"192.168.2.1\" }",
-            false,
-            { }
-        },
         {
             "valid ip-addresses #1",
             "{ \"ip-addresses\": [ ] }",
@@ -1012,16 +964,7 @@ TEST_F(SharedNetwork6ParserTest, relayInfoTests) {
             { }
         },
         {
-            "invalid both ip-address and ip-addresses",
-            "{"
-            " \"ip-address\": \"2001:db8::1\", "
-            " \"ip-addresses\": [ \"2001:db8::1\", \"2001:db8::2\" ]"
-            " }",
-            false,
-            { }
-        },
-        {
-            "invalid neither ip-address nor ip-addresses",
+            "invalid no ip-addresses",
             "{}",
             false,
             { }
@@ -1030,11 +973,9 @@ TEST_F(SharedNetwork6ParserTest, relayInfoTests) {
 
     // Iterate over the test scenarios, verifying each prescribed
     // outcome.
-    for (auto test = tests.begin(); test != tests.end(); ++test) {
-        {
-            SCOPED_TRACE((*test).description_);
-            relayTest(*test);
-        }
+    for (auto const& test : tests) {
+        SCOPED_TRACE(test.description_);
+        relayTest(test);
     }
 }
 
