@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,7 +6,7 @@
 
 #include <config.h>
 
-#include <dhcp/tests/iface_mgr_test_config.h>
+#include <dhcp/testutils/iface_mgr_test_config.h>
 #include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/client_class_def.h>
 #include <dhcpsrv/srv_config.h>
@@ -14,10 +14,12 @@
 #include <process/logging_info.h>
 #include <testutils/gtest_utils.h>
 #include <testutils/test_to_element.h>
+#include <util/str.h>
 
 #include <gtest/gtest.h>
 
 using namespace isc::asiolink;
+using namespace isc::dhcp_ddns;
 using namespace isc::dhcp;
 using namespace isc::data;
 using namespace isc::process;
@@ -550,21 +552,21 @@ TEST_F(SrvConfigTest, configuredGlobals) {
 
     // Maps and lists should be excluded.
     auto globals = srv_globals->valuesMap();
-    for (auto global = globals.begin(); global != globals.end(); ++global) {
-        if (global->first == "comment") {
-            ASSERT_EQ(Element::string, global->second->getType());
-            EXPECT_EQ("okay", global->second->stringValue());
-        } else if (global->first == "valid-lifetime") {
-            ASSERT_EQ(Element::integer, global->second->getType());
-            EXPECT_EQ(444, global->second->intValue());
-        } else if (global->first == "store-extended-info") {
-            ASSERT_EQ(Element::boolean, global->second->getType());
-            EXPECT_TRUE(global->second->boolValue());
-        } else if (global->first == "t1-percent") {
-            ASSERT_EQ(Element::real, global->second->getType());
-            EXPECT_EQ(1.234, global->second->doubleValue());
+    for (auto const& global : globals) {
+        if (global.first == "comment") {
+            ASSERT_EQ(Element::string, global.second->getType());
+            EXPECT_EQ("okay", global.second->stringValue());
+        } else if (global.first == "valid-lifetime") {
+            ASSERT_EQ(Element::integer, global.second->getType());
+            EXPECT_EQ(444, global.second->intValue());
+        } else if (global.first == "store-extended-info") {
+            ASSERT_EQ(Element::boolean, global.second->getType());
+            EXPECT_TRUE(global.second->boolValue());
+        } else if (global.first == "t1-percent") {
+            ASSERT_EQ(Element::real, global.second->getType());
+            EXPECT_EQ(1.234, global.second->doubleValue());
         } else {
-            ADD_FAILURE() << "unexpected element found:" << global->first;
+            ADD_FAILURE() << "unexpected element found:" << global.first;
         }
     }
 
@@ -1112,9 +1114,13 @@ TEST_F(SrvConfigTest, mergeGlobals4) {
     cfg_from.addConfiguredGlobal("ip-reservations-unique", Element::create(false));
 
     // Add some configured globals:
-    cfg_to.addConfiguredGlobal("dhcp4o6-port", Element::create(999));
-    cfg_to.addConfiguredGlobal("server-tag", Element::create("use_this_server"));
-    cfg_to.addConfiguredGlobal("reservations-lookup-first", Element::create(true));
+    cfg_from.addConfiguredGlobal("dhcp4o6-port", Element::create(999));
+    cfg_from.addConfiguredGlobal("server-tag", Element::create("use_this_server"));
+    cfg_from.addConfiguredGlobal("reservations-lookup-first", Element::create(true));
+    ElementPtr mt = Element::createMap();
+    cfg_from.addConfiguredGlobal("multi-threading", mt);
+    mt->set("enable-multi-threading", Element::create(false));
+    mt->set("thread-pool-size", Element::create(256));
 
     // Now let's merge.
     ASSERT_NO_THROW(cfg_to.merge(cfg_from));
@@ -1139,16 +1145,23 @@ TEST_F(SrvConfigTest, mergeGlobals4) {
     // ip-reservations-unique
     EXPECT_FALSE(cfg_to.getCfgDbAccess()->getIPReservationsUnique());
 
+    // multi-threading
+    EXPECT_TRUE(cfg_to.getDHCPMultiThreading());
+
     // Next we check the explicitly "configured" globals.
     // The list should be all of the "to" + "from", with the
     // latter overwriting the former.
     std::string exp_globals =
         "{ \n"
-        "   \"decline-probation-period\": 300,  \n"
-        "   \"dhcp4o6-port\": 999,  \n"
-        "   \"ip-reservations-unique\": false,  \n"
-        "   \"server-tag\": \"use_this_server\",  \n"
-        "   \"reservations-lookup-first\": true"
+        "   \"decline-probation-period\": 300, \n"
+        "   \"dhcp4o6-port\": 999, \n"
+        "   \"ip-reservations-unique\": false, \n"
+        "   \"server-tag\": \"use_this_server\", \n"
+        "   \"reservations-lookup-first\": true,"
+        "   \"multi-threading\": { \"enable-multi-threading\": false, \n"
+        "                          \"packet-queue-size\": 64, \n"
+        "                          \"thread-pool-size\": 256 \n"
+        "    } \n"
         "} \n";
 
     ConstElementPtr expected_globals;
@@ -1194,9 +1207,13 @@ TEST_F(SrvConfigTest, mergeGlobals6) {
     cfg_from.addConfiguredGlobal("ip-reservations-unique", Element::create(false));
 
     // Add some configured globals:
-    cfg_to.addConfiguredGlobal("dhcp4o6-port", Element::create(999));
-    cfg_to.addConfiguredGlobal("server-tag", Element::create("use_this_server"));
-    cfg_to.addConfiguredGlobal("reservations-lookup-first", Element::create(true));
+    cfg_from.addConfiguredGlobal("dhcp4o6-port", Element::create(999));
+    cfg_from.addConfiguredGlobal("server-tag", Element::create("use_this_server"));
+    cfg_from.addConfiguredGlobal("reservations-lookup-first", Element::create(true));
+    ElementPtr mt = Element::createMap();
+    cfg_from.addConfiguredGlobal("multi-threading", mt);
+    mt->set("enable-multi-threading", Element::create(false));
+    mt->set("thread-pool-size", Element::create(256));
 
     // Now let's merge.
     ASSERT_NO_THROW(cfg_to.merge(cfg_from));
@@ -1218,16 +1235,23 @@ TEST_F(SrvConfigTest, mergeGlobals6) {
     // ip-reservations-unique
     EXPECT_FALSE(cfg_to.getCfgDbAccess()->getIPReservationsUnique());
 
+    // multi-threading
+    EXPECT_TRUE(cfg_to.getDHCPMultiThreading());
+
     // Next we check the explicitly "configured" globals.
     // The list should be all of the "to" + "from", with the
     // latter overwriting the former.
     std::string exp_globals =
         "{ \n"
-        "   \"decline-probation-period\": 300,  \n"
-        "   \"dhcp4o6-port\": 999,  \n"
-        "   \"ip-reservations-unique\": false,  \n"
-        "   \"server-tag\": \"use_this_server\",  \n"
-        "   \"reservations-lookup-first\": true"
+        "   \"decline-probation-period\": 300, \n"
+        "   \"dhcp4o6-port\": 999, \n"
+        "   \"ip-reservations-unique\": false, \n"
+        "   \"server-tag\": \"use_this_server\", \n"
+        "   \"reservations-lookup-first\": true, \n"
+        "   \"multi-threading\": { \"enable-multi-threading\": false, \n"
+        "                          \"packet-queue-size\": 64, \n"
+        "                          \"thread-pool-size\": 256 \n"
+        "    } \n"
         "} \n";
 
     ConstElementPtr expected_globals;
@@ -1291,150 +1315,6 @@ TEST_F(SrvConfigTest, mergeEmptyClientClasses) {
     EXPECT_TRUE(cfg_to.getClientClassDictionary()->findClass("bar"));
 }
 
-// Validates SrvConfig::moveDdnsParams by ensuring that deprecated dhcp-ddns
-// parameters are:
-// 1. Translated to their global counterparts if they do not exist globally
-// 2. Removed from the dhcp-ddns element
-TEST_F(SrvConfigTest, moveDdnsParamsTest) {
-    DdnsParamsPtr params;
-
-    CfgMgr::instance().setFamily(AF_INET);
-
-    struct Scenario {
-        std::string description;
-        std::string input_cfg;
-        std::string exp_cfg;
-    };
-
-    std::vector<Scenario> scenarios {
-        {
-            "scenario 1, move with no global conflicts",
-            // input_cfg
-            "{\n"
-            "   \"dhcp-ddns\": {\n"
-            "       \"enable-updates\": true, \n"
-            "       \"server-ip\" : \"192.0.2.0\",\n"
-            "       \"server-port\" : 3432,\n"
-            "       \"sender-ip\" : \"192.0.2.1\",\n"
-            "       \"sender-port\" : 3433,\n"
-            "       \"max-queue-size\" : 2048,\n"
-            "       \"ncr-protocol\" : \"UDP\",\n"
-            "       \"ncr-format\" : \"JSON\",\n"
-            "       \"user-context\": { \"foo\": \"bar\" },\n"
-            "       \"override-no-update\": true,\n"
-            "       \"override-client-update\": false,\n"
-            "       \"replace-client-name\": \"always\",\n"
-            "       \"generated-prefix\": \"prefix\",\n"
-            "       \"qualifying-suffix\": \"suffix.com.\",\n"
-            "       \"hostname-char-set\": \"[^A-Z]\",\n"
-            "       \"hostname-char-replacement\": \"x\"\n"
-            "   }\n"
-            "}\n",
-            // exp_cfg
-            "{\n"
-            "   \"dhcp-ddns\": {\n"
-            "       \"enable-updates\": true, \n"
-            "       \"server-ip\" : \"192.0.2.0\",\n"
-            "       \"server-port\" : 3432,\n"
-            "       \"sender-ip\" : \"192.0.2.1\",\n"
-            "       \"sender-port\" : 3433,\n"
-            "       \"max-queue-size\" : 2048,\n"
-            "       \"ncr-protocol\" : \"UDP\",\n"
-            "       \"ncr-format\" : \"JSON\",\n"
-            "       \"user-context\": { \"foo\": \"bar\" }\n"
-            "   },\n"
-            "   \"ddns-override-no-update\": true,\n"
-            "   \"ddns-override-client-update\": false,\n"
-            "   \"ddns-replace-client-name\": \"always\",\n"
-            "   \"ddns-generated-prefix\": \"prefix\",\n"
-            "   \"ddns-qualifying-suffix\": \"suffix.com.\",\n"
-            "   \"hostname-char-set\": \"[^A-Z]\",\n"
-            "   \"hostname-char-replacement\": \"x\"\n"
-            "}\n"
-        },
-        {
-            "scenario 2, globals already exist for all movable params",
-            // input_cfg
-            "{\n"
-            "   \"dhcp-ddns\" : {\n"
-            "       \"enable-updates\": true, \n"
-            "       \"override-no-update\": true,\n"
-            "       \"override-client-update\": true,\n"
-            "       \"replace-client-name\": \"always\",\n"
-            "       \"generated-prefix\": \"prefix\",\n"
-            "       \"qualifying-suffix\": \"suffix.com.\",\n"
-            "       \"hostname-char-set\": \"[^A-Z]\",\n"
-            "       \"hostname-char-replacement\": \"x\"\n"
-            "   },\n"
-            "   \"ddns-override-no-update\": false,\n"
-            "   \"ddns-override-client-update\": false,\n"
-            "   \"ddns-replace-client-name\": \"when-present\",\n"
-            "   \"ddns-generated-prefix\": \"org_prefix\",\n"
-            "   \"ddns-qualifying-suffix\": \"org_suffix.com.\",\n"
-            "   \"hostname-char-set\": \"[^a-z]\",\n"
-            "   \"hostname-char-replacement\": \"y\"\n"
-            "}\n",
-            // exp_cfg
-            "{\n"
-            "   \"dhcp-ddns\" : {\n"
-            "       \"enable-updates\": true\n"
-            "   },\n"
-            "   \"ddns-override-no-update\": false,\n"
-            "   \"ddns-override-client-update\": false,\n"
-            "   \"ddns-replace-client-name\": \"when-present\",\n"
-            "   \"ddns-generated-prefix\": \"org_prefix\",\n"
-            "   \"ddns-qualifying-suffix\": \"org_suffix.com.\",\n"
-            "   \"hostname-char-set\": \"[^a-z]\",\n"
-            "   \"hostname-char-replacement\": \"y\"\n"
-            "}\n"
-        },
-        {
-            "scenario 3, nothing to move",
-            // input_cfg
-            "{\n"
-            "   \"dhcp-ddns\" : {\n"
-            "       \"enable-updates\": true, \n"
-            "       \"server-ip\" : \"192.0.2.0\",\n"
-            "       \"server-port\" : 3432,\n"
-            "       \"sender-ip\" : \"192.0.2.1\"\n"
-            "   }\n"
-            "}\n",
-            // exp_output
-            "{\n"
-            "   \"dhcp-ddns\" : {\n"
-            "       \"enable-updates\": true, \n"
-            "       \"server-ip\" : \"192.0.2.0\",\n"
-            "       \"server-port\" : 3432,\n"
-            "       \"sender-ip\" : \"192.0.2.1\"\n"
-            "   }\n"
-            "}\n"
-        }
-    };
-
-    for (auto scenario : scenarios) {
-        SrvConfig conf(32);
-        ElementPtr input_cfg;
-        ConstElementPtr exp_cfg;
-        {
-            SCOPED_TRACE(scenario.description);
-            // Parse the input cfg into a mutable Element map.
-            ASSERT_NO_THROW(input_cfg = boost::const_pointer_cast<Element>
-                            (Element::fromJSON(scenario.input_cfg)))
-                            << "input_cfg didn't parse, test is broken";
-
-            // Parse the expected cfg into an Element map.
-            ASSERT_NO_THROW(exp_cfg = Element::fromJSON(scenario.exp_cfg))
-                            << "exp_cfg didn't parse, test is broken";
-
-            // Now call moveDdnsParams.
-            ASSERT_NO_THROW(SrvConfig::moveDdnsParams(input_cfg));
-
-            // Make sure the resultant configuration is as expected.
-            EXPECT_TRUE(input_cfg->equals(*exp_cfg));
-        }
-    }
-}
-
 // Verifies that the scoped values for DDNS parameters can be fetched
 // for a given Subnet4.
 TEST_F(SrvConfigTest, getDdnsParamsTest4) {
@@ -1452,8 +1332,8 @@ TEST_F(SrvConfigTest, getDdnsParamsTest4) {
     // Configure global host sanitizing.
     conf.addConfiguredGlobal("hostname-char-set", Element::create("[^A-Z]"));
     conf.addConfiguredGlobal("hostname-char-replacement", Element::create("x"));
-    // Enable conflict resolution globally.
-    conf.addConfiguredGlobal("ddns-use-conflict-resolution", Element::create(true));
+    // Specify conflict resolution mode globally.
+    conf.addConfiguredGlobal("ddns-conflict-resolution-mode", Element::create("check-with-dhcid"));
     // Configure TTL percent globally.
     conf.addConfiguredGlobal("ddns-ttl-percent", Element::create(20.0));
 
@@ -1494,7 +1374,7 @@ TEST_F(SrvConfigTest, getDdnsParamsTest4) {
     subnet2->setDdnsQualifyingSuffix("example.com.");
     subnet2->setHostnameCharSet("");
     subnet2->setDdnsUpdateOnRenew(true);
-    subnet2->setDdnsUseConflictResolution(false);
+    subnet2->setDdnsConflictResolutionMode("no-check-with-dhcid");
     subnet2->setDdnsTtlPercent(Optional<double>(40.0));
 
     // Get DDNS params for subnet1.
@@ -1510,7 +1390,7 @@ TEST_F(SrvConfigTest, getDdnsParamsTest4) {
     EXPECT_EQ("[^A-Z]", params->getHostnameCharSet());
     EXPECT_EQ("x", params->getHostnameCharReplacement());
     EXPECT_FALSE(params->getUpdateOnRenew());
-    EXPECT_TRUE(params->getUseConflictResolution());
+    EXPECT_EQ("check-with-dhcid", params->getConflictResolutionMode());
     EXPECT_FALSE(params->getTtlPercent().unspecified());
     EXPECT_EQ(20.0, params->getTtlPercent().get());
 
@@ -1534,7 +1414,7 @@ TEST_F(SrvConfigTest, getDdnsParamsTest4) {
     EXPECT_EQ("", params->getHostnameCharSet());
     EXPECT_EQ("x", params->getHostnameCharReplacement());
     EXPECT_TRUE(params->getUpdateOnRenew());
-    EXPECT_FALSE(params->getUseConflictResolution());
+    EXPECT_EQ("no-check-with-dhcid", params->getConflictResolutionMode());
     EXPECT_FALSE(params->getTtlPercent().unspecified());
     EXPECT_EQ(40.0, params->getTtlPercent().get());
 
@@ -1591,7 +1471,8 @@ TEST_F(SrvConfigTest, getDdnsParamsNoSubnetTest4) {
     conf.addConfiguredGlobal("hostname-char-set", Element::create("[^A-Z]"));
     conf.addConfiguredGlobal("hostname-char-replacement", Element::create("x"));
     conf.addConfiguredGlobal("ddns-update-on-renew", Element::create(true));
-    conf.addConfiguredGlobal("ddns-use-conflict-resolution", Element::create(false));
+    conf.addConfiguredGlobal("ddns-conflict-resolution-mode",
+                             Element::create("no-check-with-dhcid"));
     conf.addConfiguredGlobal("ddns-ttl-percent", Element::create(77.0));
 
     // Get DDNS params for no subnet.
@@ -1608,7 +1489,7 @@ TEST_F(SrvConfigTest, getDdnsParamsNoSubnetTest4) {
     EXPECT_TRUE(params->getHostnameCharSet().empty());
     EXPECT_TRUE(params->getHostnameCharReplacement().empty());
     EXPECT_FALSE(params->getUpdateOnRenew());
-    EXPECT_TRUE(params->getUseConflictResolution());
+    EXPECT_EQ("check-with-dhcid", params->getConflictResolutionMode());
     EXPECT_TRUE(params->getTtlPercent().unspecified());
 }
 
@@ -1629,8 +1510,10 @@ TEST_F(SrvConfigTest, getDdnsParamsTest6) {
     // Configure global host sanitizing.
     conf.addConfiguredGlobal("hostname-char-set", Element::create("[^A-Z]"));
     conf.addConfiguredGlobal("hostname-char-replacement", Element::create("x"));
-    // Enable conflict resolution globally.
-    conf.addConfiguredGlobal("ddns-use-conflict-resolution", Element::create(true));
+    // Specify conflict resolution mode globally.
+    conf.addConfiguredGlobal("ddns-conflict-resolution-mode",
+                             Element::create("check-with-dhcid"));
+
     // Configure TTL percent globally.
     conf.addConfiguredGlobal("ddns-ttl-percent", Element::create(25.0));
 
@@ -1671,7 +1554,7 @@ TEST_F(SrvConfigTest, getDdnsParamsTest6) {
     subnet2->setDdnsQualifyingSuffix("example.com.");
     subnet2->setHostnameCharSet("");
     subnet2->setDdnsUpdateOnRenew(true);
-    subnet2->setDdnsUseConflictResolution(false);
+    subnet2->setDdnsConflictResolutionMode("no-check-with-dhcid");
     subnet2->setDdnsTtlPercent(Optional<double>(45.0));
 
     // Get DDNS params for subnet1.
@@ -1687,7 +1570,7 @@ TEST_F(SrvConfigTest, getDdnsParamsTest6) {
     EXPECT_EQ("[^A-Z]", params->getHostnameCharSet());
     EXPECT_EQ("x", params->getHostnameCharReplacement());
     EXPECT_FALSE(params->getUpdateOnRenew());
-    EXPECT_TRUE(params->getUseConflictResolution());
+    EXPECT_EQ("check-with-dhcid", params->getConflictResolutionMode());
     EXPECT_FALSE(params->getTtlPercent().unspecified());
     EXPECT_EQ(25.0, params->getTtlPercent().get());
 
@@ -1711,7 +1594,7 @@ TEST_F(SrvConfigTest, getDdnsParamsTest6) {
     EXPECT_EQ("", params->getHostnameCharSet());
     EXPECT_EQ("x", params->getHostnameCharReplacement());
     EXPECT_TRUE(params->getUpdateOnRenew());
-    EXPECT_FALSE(params->getUseConflictResolution());
+    EXPECT_EQ("no-check-with-dhcid", params->getConflictResolutionMode());
     EXPECT_FALSE(params->getTtlPercent().unspecified());
     EXPECT_EQ(45.0, params->getTtlPercent().get());
 
@@ -1768,7 +1651,8 @@ TEST_F(SrvConfigTest, getDdnsParamsNoSubnetTest6) {
     conf.addConfiguredGlobal("hostname-char-set", Element::create("[^A-Z]"));
     conf.addConfiguredGlobal("hostname-char-replacement", Element::create("x"));
     conf.addConfiguredGlobal("ddns-update-on-renew", Element::create(true));
-    conf.addConfiguredGlobal("ddns-use-conflict-resolution", Element::create(false));
+    conf.addConfiguredGlobal("ddns-conflict-resolution-mode",
+                             Element::create("no-check-with-dhcid"));
     conf.addConfiguredGlobal("ddns-ttl-percent", Element::create(77.0));
 
     // Get DDNS params for no subnet.
@@ -1785,7 +1669,7 @@ TEST_F(SrvConfigTest, getDdnsParamsNoSubnetTest6) {
     EXPECT_TRUE(params->getHostnameCharSet().empty());
     EXPECT_TRUE(params->getHostnameCharReplacement().empty());
     EXPECT_FALSE(params->getUpdateOnRenew());
-    EXPECT_TRUE(params->getUseConflictResolution());
+    EXPECT_EQ("check-with-dhcid", params->getConflictResolutionMode());
     EXPECT_TRUE(params->getTtlPercent().unspecified());
 }
 
