@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -82,6 +82,18 @@ public:
     LabelSequence ls9, ls10, ls11, ls12;
 };
 
+// Check the assignment operator.
+TEST_F(LabelSequenceTest, assign) {
+    // Create the label sequence with example.org (n1 name).
+    LabelSequence ls(n1);
+    EXPECT_TRUE(ls == ls1);
+
+    // Assign the label sequence to example.com (n2 name).
+    ls = ls2;
+    EXPECT_FALSE(ls == ls1);
+    EXPECT_TRUE(ls == ls2);
+}
+
 // Basic equality tests
 TEST_F(LabelSequenceTest, equals_sensitive) {
     EXPECT_TRUE(ls1.equals(ls1, true));
@@ -629,6 +641,23 @@ TEST_F(LabelSequenceTest, toText) {
     EXPECT_EQ("0.1", ls_long2.toText());
 }
 
+// The following verifies that toRawText() returns a string
+// actual characters in place of escape sequences.  We do not
+// bother with an exhaustive set of tests here as this is
+// not a primary use case.
+TEST_F(LabelSequenceTest, toRawText) {
+    Name n("a bc.$exa(m)ple.@org");
+    LabelSequence l(n);
+    EXPECT_EQ("a bc.$exa(m)ple.@org", l.toRawText(true));
+    EXPECT_EQ("a bc.$exa(m)ple.@org.", l.toRawText(false));
+
+    // toRawText is not supposed to do any sanity checks.
+    // Let's try with a very weird name.
+    Name n2("xtra\tchars\n.in.name");
+    LabelSequence l2(n2);
+    EXPECT_EQ("xtra\tchars\n.in.name.", l2.toRawText(false));
+}
+
 // The following are test data used in the getHash test below.  Normally
 // we use example/documentation domain names for testing, but in this case
 // we'd specifically like to use more realistic data, and are intentionally
@@ -639,21 +668,21 @@ const char* const root_servers[] = {
     "d.root-servers.net", "e.root-servers.net", "f.root-servers.net",
     "g.root-servers.net", "h.root-servers.net", "i.root-servers.net",
     "j.root-servers.net", "k.root-servers.net", "l.root-servers.net",
-    "m.root-servers.net", NULL
+    "m.root-servers.net", 0
 };
 
 const char* const jp_servers[] = {
     "a.dns.jp", "b.dns.jp", "c.dns.jp", "d.dns.jp", "e.dns.jp",
-    "f.dns.jp", "g.dns.jp", NULL
+    "f.dns.jp", "g.dns.jp", 0
 };
 const char* const cn_servers[] = {
     "a.dns.cn", "b.dns.cn", "c.dns.cn", "d.dns.cn", "e.dns.cn",
-    "ns.cernet.net", NULL
+    "ns.cernet.net", 0
 };
 const char* const ca_servers[] = {
     "k.ca-servers.ca", "e.ca-servers.ca", "a.ca-servers.ca", "z.ca-servers.ca",
     "tld.isc-sns.net", "c.ca-servers.ca", "j.ca-servers.ca", "l.ca-servers.ca",
-    "sns-pb.isc.org", "f.ca-servers.ca", NULL
+    "sns-pb.isc.org", "f.ca-servers.ca", 0
 };
 
 // A helper function used in the getHash test below.
@@ -666,7 +695,7 @@ hashDistributionCheck(const char* const* servers) {
     // Store all test names and their super domain names (excluding the
     // "root" label) in the set, calculates their hash values, and increments
     // the counter for the corresponding hash "bucket".
-    for (size_t i = 0; servers[i] != NULL; ++i) {
+    for (size_t i = 0; servers[i]; ++i) {
         const Name name(servers[i]);
         for (size_t l = 0; l < name.getLabelCount() - 1; ++l) {
             pair<set<Name>::const_iterator, bool> ret =
@@ -788,9 +817,8 @@ TEST_F(LabelSequenceTest, serialize) {
     isc::util::OutputBuffer ob(0);
     n_maxlabel.toWire(ob);
     expected_data7.insert(expected_data7.end(),
-                          static_cast<const uint8_t*>(ob.getData()),
-                          static_cast<const uint8_t*>(ob.getData()) +
-                          ob.getLength());
+                          ob.getVector().cbegin(),
+                          ob.getVector().cend());
     expected.push_back(DataPair(expected_data7.size(), &expected_data7[0]));
 
     // For each data set, serialize the labels and compare the data to the
@@ -846,7 +874,7 @@ TEST_F(LabelSequenceTest, serialize) {
 // These checks are enabled only in debug mode in the LabelSequence
 // class.
 TEST_F(LabelSequenceTest, badDeserialize) {
-    EXPECT_THROW(LabelSequence(NULL), isc::BadValue);
+    EXPECT_THROW(LabelSequence(0), isc::BadValue);
     const uint8_t zero_offsets[] = { 0 };
     EXPECT_THROW(LabelSequence ls(zero_offsets), isc::BadValue);
     const uint8_t toomany_offsets[] = { Name::MAX_LABELS + 1 };
