@@ -19,6 +19,7 @@ extern const isc::log::MessageID EVAL_DEBUG_INT32TOTEXT;
 extern const isc::log::MessageID EVAL_DEBUG_INT8TOTEXT;
 extern const isc::log::MessageID EVAL_DEBUG_IPADDRESS;
 extern const isc::log::MessageID EVAL_DEBUG_IPADDRESSTOTEXT;
+extern const isc::log::MessageID EVAL_DEBUG_LCASE;
 extern const isc::log::MessageID EVAL_DEBUG_MEMBER;
 extern const isc::log::MessageID EVAL_DEBUG_NOT;
 extern const isc::log::MessageID EVAL_DEBUG_OPTION;
@@ -28,6 +29,10 @@ extern const isc::log::MessageID EVAL_DEBUG_PKT4;
 extern const isc::log::MessageID EVAL_DEBUG_PKT6;
 extern const isc::log::MessageID EVAL_DEBUG_RELAY6;
 extern const isc::log::MessageID EVAL_DEBUG_RELAY6_RANGE;
+extern const isc::log::MessageID EVAL_DEBUG_SPLIT;
+extern const isc::log::MessageID EVAL_DEBUG_SPLIT_DELIM_EMPTY;
+extern const isc::log::MessageID EVAL_DEBUG_SPLIT_EMPTY;
+extern const isc::log::MessageID EVAL_DEBUG_SPLIT_FIELD_OUT_OF_RANGE;
 extern const isc::log::MessageID EVAL_DEBUG_STRING;
 extern const isc::log::MessageID EVAL_DEBUG_SUBSTRING;
 extern const isc::log::MessageID EVAL_DEBUG_SUBSTRING_EMPTY;
@@ -35,6 +40,7 @@ extern const isc::log::MessageID EVAL_DEBUG_SUBSTRING_RANGE;
 extern const isc::log::MessageID EVAL_DEBUG_SUB_OPTION;
 extern const isc::log::MessageID EVAL_DEBUG_SUB_OPTION_NO_OPTION;
 extern const isc::log::MessageID EVAL_DEBUG_TOHEXSTRING;
+extern const isc::log::MessageID EVAL_DEBUG_UCASE;
 extern const isc::log::MessageID EVAL_DEBUG_UINT16TOTEXT;
 extern const isc::log::MessageID EVAL_DEBUG_UINT32TOTEXT;
 extern const isc::log::MessageID EVAL_DEBUG_UINT8TOTEXT;
