@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -26,9 +26,7 @@ namespace {
 class EvalContextTest : public ::testing::Test {
 public:
     /// @brief constructor to initialize members
-    EvalContextTest() : ::testing::Test(),
-    universe_(Option::V4), parsed_(false)
-    { }
+    EvalContextTest() : ::testing::Test(), universe_(Option::V4), parsed_(false) { }
 
     /// @brief checks if the given token is a string with the expected value
     void checkTokenString(const TokenPtr& token, const std::string& expected) {
@@ -47,6 +45,46 @@ public:
         EXPECT_EQ(expected, values.top());
     }
 
+    /// @brief checks if the given token is a constant lower case string with
+    /// the expected value
+    void checkTokenLowerCase(const TokenPtr& token, const std::string& expected) {
+        ASSERT_TRUE(token);
+        boost::shared_ptr<TokenLowerCase> str =
+            boost::dynamic_pointer_cast<TokenLowerCase>(token);
+        ASSERT_TRUE(str);
+
+        Pkt4Ptr pkt4(new Pkt4(DHCPDISCOVER, 12345));
+        ValueStack values;
+
+        values.push(expected);
+
+        EXPECT_NO_THROW(token->evaluate(*pkt4, values));
+
+        ASSERT_EQ(1, values.size());
+
+        EXPECT_EQ(expected, values.top());
+    }
+
+    /// @brief checks if the given token is a constant upper case string with
+    /// the expected value
+    void checkTokenUpperCase(const TokenPtr& token, const std::string& expected) {
+        ASSERT_TRUE(token);
+        boost::shared_ptr<TokenUpperCase> str =
+            boost::dynamic_pointer_cast<TokenUpperCase>(token);
+        ASSERT_TRUE(str);
+
+        Pkt4Ptr pkt4(new Pkt4(DHCPDISCOVER, 12345));
+        ValueStack values;
+
+        values.push(expected);
+
+        EXPECT_NO_THROW(token->evaluate(*pkt4, values));
+
+        ASSERT_EQ(1, values.size());
+
+        EXPECT_EQ(expected, values.top());
+    }
+
     /// @brief checks if the given token is a hex string with the expected value
     void checkTokenHexString(const TokenPtr& token,
                              const std::string& expected) {
@@ -816,9 +854,33 @@ TEST_F(EvalContextTest, string) {
 
     TokenPtr tmp1  = eval.expression.at(0);
     TokenPtr tmp2  = eval.expression.at(1);
+    TokenPtr tmp3  = eval.expression.at(2);
 
     checkTokenString(tmp1, "foo");
     checkTokenString(tmp2, "bar");
+    checkTokenEq(tmp3);
+}
+
+// Test the parsing of a string terminal
+TEST_F(EvalContextTest, stringComplex) {
+    EvalContext eval(Option::V4);
+
+    char data[] = "'12345~!@#$%^&*()_+{}[];:<>/?\\67890\t \0\b\r\f' == "
+            "'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'";
+    EXPECT_NO_THROW(parsed_ = eval.parseString(string(data, sizeof(data) - 1)));
+    EXPECT_TRUE(parsed_);
+
+    ASSERT_EQ(3, eval.expression.size());
+
+    TokenPtr tmp1  = eval.expression.at(0);
+    TokenPtr tmp2  = eval.expression.at(1);
+    TokenPtr tmp3  = eval.expression.at(2);
+
+    char l_data[] = "12345~!@#$%^&*()_+{}[];:<>/?\\67890\t \0\b\r\f";
+    char r_data[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
+    checkTokenString(tmp1, string(l_data, sizeof(l_data) - 1));
+    checkTokenString(tmp2, string(r_data, sizeof(r_data) - 1));
+    checkTokenEq(tmp3);
 }
 
 // Test the parsing of a basic expression using integers
@@ -1017,7 +1079,7 @@ TEST_F(EvalContextTest, relay4Option) {
 
     EvalContext eval(Option::V4);
     EXPECT_NO_THROW(parsed_ =
-                    eval.parseString("relay4[13].hex == 'thirteen'"));
+        eval.parseString("relay4[13].hex == 'thirteen'"));
     EXPECT_TRUE(parsed_);
     ASSERT_EQ(3, eval.expression.size());
 
@@ -1300,7 +1362,7 @@ TEST_F(EvalContextTest, logicalPrecedence) {
     EvalContext evaloa(Option::V4);
     EXPECT_NO_THROW(parsed_ =
         evaloa.parseString("option[123].exists or option[123].exists "
-                         "and option[123].exists"));
+                           "and option[123].exists"));
     EXPECT_TRUE(parsed_);
     ASSERT_EQ(5, evaloa.expression.size());
     token = evaloa.expression.at(4);
@@ -1329,7 +1391,7 @@ TEST_F(EvalContextTest, logicalParentheses) {
     EvalContext evaloa(Option::V4);
     EXPECT_NO_THROW(parsed_ =
         evaloa.parseString("(option[123].exists or option[123].exists) "
-                         "and option[123].exists"));
+                           "and option[123].exists"));
     EXPECT_TRUE(parsed_);
     ASSERT_EQ(5, evaloa.expression.size());
     token = evaloa.expression.at(4);
@@ -1510,6 +1572,94 @@ TEST_F(EvalContextTest, toHexString) {
     checkTokenToHexString(tmp3);
 }
 
+// Test the parsing of lcase expression
+TEST_F(EvalContextTest, lcase) {
+    EvalContext eval(Option::V4);
+
+    EXPECT_NO_THROW(parsed_ = eval.parseString("lcase('LoWeR') == 'lower'"));
+    EXPECT_TRUE(parsed_);
+
+    ASSERT_EQ(4, eval.expression.size());
+
+    TokenPtr tmp1 = eval.expression.at(0);
+    TokenPtr tmp2 = eval.expression.at(1);
+    TokenPtr tmp3 = eval.expression.at(2);
+    TokenPtr tmp4 = eval.expression.at(3);
+
+    checkTokenString(tmp1, "LoWeR");
+    checkTokenLowerCase(tmp2, "lower");
+    checkTokenString(tmp3, "lower");
+    checkTokenEq(tmp4);
+}
+
+// Test the parsing of complex lcase expression
+TEST_F(EvalContextTest, lcaseComplex) {
+    EvalContext eval(Option::V4);
+
+    char data[] = "lcase('12345~!@#$%^&*()_+LoWeR{}[];:<>/?\\67890\t \0\b\r\f') == "
+            "'12345~!@#$%^&*()_+lower{}[];:<>/?\\67890\t \0\b\r\f'";
+    EXPECT_NO_THROW(parsed_ = eval.parseString(string(data, sizeof(data) - 1)));
+    EXPECT_TRUE(parsed_);
+
+    ASSERT_EQ(4, eval.expression.size());
+
+    TokenPtr tmp1 = eval.expression.at(0);
+    TokenPtr tmp2 = eval.expression.at(1);
+    TokenPtr tmp3 = eval.expression.at(2);
+    TokenPtr tmp4 = eval.expression.at(3);
+
+    char expected_data[] = "12345~!@#$%^&*()_+LoWeR{}[];:<>/?\\67890\t \0\b\r\f";
+    char expected_updated_data[] = "12345~!@#$%^&*()_+lower{}[];:<>/?\\67890\t \0\b\r\f";
+    checkTokenString(tmp1, string(expected_data, sizeof(expected_data) - 1));
+    checkTokenLowerCase(tmp2, string(expected_updated_data, sizeof(expected_updated_data) - 1));
+    checkTokenString(tmp3, string(expected_updated_data, sizeof(expected_updated_data) - 1));
+    checkTokenEq(tmp4);
+}
+
+// Test the parsing of ucase expression
+TEST_F(EvalContextTest, ucase) {
+    EvalContext eval(Option::V4);
+
+    EXPECT_NO_THROW(parsed_ = eval.parseString("ucase('uPpEr') == 'UPPER'"));
+    EXPECT_TRUE(parsed_);
+
+    ASSERT_EQ(4, eval.expression.size());
+
+    TokenPtr tmp1 = eval.expression.at(0);
+    TokenPtr tmp2 = eval.expression.at(1);
+    TokenPtr tmp3 = eval.expression.at(2);
+    TokenPtr tmp4 = eval.expression.at(3);
+
+    checkTokenString(tmp1, "uPpEr");
+    checkTokenUpperCase(tmp2, "UPPER");
+    checkTokenString(tmp3, "UPPER");
+    checkTokenEq(tmp4);
+}
+
+// Test the parsing of complex ucase expression
+TEST_F(EvalContextTest, ucaseComplex) {
+    EvalContext eval(Option::V4);
+
+    char data[] = "ucase('12345~!@#$%^&*()_+uPpEr{}[];:<>/?\\67890\t \0\b\r\f') == "
+            "'12345~!@#$%^&*()_+UPPER{}[];:<>/?\\67890\t \0\b\r\f'";
+    EXPECT_NO_THROW(parsed_ = eval.parseString(string(data, sizeof(data) - 1)));
+    EXPECT_TRUE(parsed_);
+
+    ASSERT_EQ(4, eval.expression.size());
+
+    TokenPtr tmp1 = eval.expression.at(0);
+    TokenPtr tmp2 = eval.expression.at(1);
+    TokenPtr tmp3 = eval.expression.at(2);
+    TokenPtr tmp4 = eval.expression.at(3);
+
+    char expected_data[] = "12345~!@#$%^&*()_+uPpEr{}[];:<>/?\\67890\t \0\b\r\f";
+    char expected_updated_data[] = "12345~!@#$%^&*()_+UPPER{}[];:<>/?\\67890\t \0\b\r\f";
+    checkTokenString(tmp1, string(expected_data, sizeof(expected_data) - 1));
+    checkTokenUpperCase(tmp2, string(expected_updated_data, sizeof(expected_updated_data) - 1));
+    checkTokenString(tmp3, string(expected_updated_data, sizeof(expected_updated_data) - 1));
+    checkTokenEq(tmp4);
+}
+
 // Test the parsing of an addrtotext expression
 TEST_F(EvalContextTest, addressToText) {
     {
