@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -9,8 +9,8 @@
 #include <eval/token.h>
 #include <eval/eval_log.h>
 #include <eval/eval_context.h>
-#include <util/encode/hex.h>
-#include <util/io_utilities.h>
+#include <util/encode/encode.h>
+#include <util/io.h>
 #include <asiolink/io_address.h>
 #include <dhcp/pkt4.h>
 #include <dhcp/pkt6.h>
@@ -20,6 +20,7 @@
 #include <dhcp/option_vendor.h>
 #include <dhcp/option_vendor_class.h>
 
+#include <boost/algorithm/string.hpp>
 #include <boost/algorithm/string/classification.hpp>
 #include <boost/algorithm/string/split.hpp>
 
@@ -36,12 +37,13 @@ using namespace std;
 using isc::util::encode::toHex;
 
 void
-TokenString::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenString::evaluate(Pkt& pkt, ValueStack& values) {
     // Literals only push, nothing to pop
     values.push(value_);
 
     // Log what we pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_STRING)
+        .arg(pkt.getLabel())
         .arg('\'' + value_ + '\'');
 }
 
@@ -74,15 +76,54 @@ TokenHexString::TokenHexString(const string& str) : value_("") {
 }
 
 void
-TokenHexString::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenHexString::evaluate(Pkt& pkt, ValueStack& values) {
     // Literals only push, nothing to pop
     values.push(value_);
 
     // Log what we pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_HEXSTRING)
+        .arg(pkt.getLabel())
         .arg(toHex(value_));
 }
 
+void
+TokenLowerCase::evaluate(Pkt& pkt, ValueStack& values) {
+    if (values.size() == 0) {
+        isc_throw(EvalBadStack, "Incorrect empty stack.");
+    }
+
+    string op = values.top();
+
+    values.pop();
+    string result(boost::algorithm::to_lower_copy(op));
+    values.push(result);
+
+    // Log what we pushed
+    LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_LCASE)
+        .arg(pkt.getLabel())
+        .arg('\'' + op + '\'')
+        .arg('\'' + result + '\'');
+}
+
+void
+TokenUpperCase::evaluate(Pkt& pkt, ValueStack& values) {
+    if (values.size() == 0) {
+        isc_throw(EvalBadStack, "Incorrect empty stack.");
+    }
+
+    string op = values.top();
+
+    values.pop();
+    string result(boost::algorithm::to_upper_copy(op));
+    values.push(result);
+
+    // Log what we pushed
+    LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_UCASE)
+        .arg(pkt.getLabel())
+        .arg('\'' + op + '\'')
+        .arg('\'' + result + '\'');
+}
+
 TokenIpAddress::TokenIpAddress(const string& addr) : value_("") {
     // Transform IP address into binary format
     vector<uint8_t> binary;
@@ -99,17 +140,18 @@ TokenIpAddress::TokenIpAddress(const string& addr) : value_("") {
 }
 
 void
-TokenIpAddress::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenIpAddress::evaluate(Pkt& pkt, ValueStack& values) {
     // Literals only push, nothing to pop
     values.push(value_);
 
     // Log what we pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_IPADDRESS)
+        .arg(pkt.getLabel())
         .arg(toHex(value_));
 }
 
 void
-TokenIpAddressToText::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenIpAddressToText::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() == 0) {
         isc_throw(EvalBadStack, "Incorrect empty stack.");
     }
@@ -139,11 +181,12 @@ TokenIpAddressToText::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
     // Log what we pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_IPADDRESSTOTEXT)
+        .arg(pkt.getLabel())
         .arg(op);
 }
 
 void
-TokenInt8ToText::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenInt8ToText::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() == 0) {
         isc_throw(EvalBadStack, "Incorrect empty stack.");
     }
@@ -168,11 +211,12 @@ TokenInt8ToText::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
     // Log what we pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_INT8TOTEXT)
+        .arg(pkt.getLabel())
         .arg(op);
 }
 
 void
-TokenInt16ToText::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenInt16ToText::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() == 0) {
         isc_throw(EvalBadStack, "Incorrect empty stack.");
     }
@@ -198,11 +242,12 @@ TokenInt16ToText::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
     // Log what we pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_INT16TOTEXT)
+        .arg(pkt.getLabel())
         .arg(op);
 }
 
 void
-TokenInt32ToText::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenInt32ToText::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() == 0) {
         isc_throw(EvalBadStack, "Incorrect empty stack.");
     }
@@ -228,11 +273,12 @@ TokenInt32ToText::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
     // Log what we pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_INT32TOTEXT)
+        .arg(pkt.getLabel())
         .arg(op);
 }
 
 void
-TokenUInt8ToText::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenUInt8ToText::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() == 0) {
         isc_throw(EvalBadStack, "Incorrect empty stack.");
     }
@@ -257,11 +303,12 @@ TokenUInt8ToText::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
     // Log what we pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_UINT8TOTEXT)
+        .arg(pkt.getLabel())
         .arg(op);
 }
 
 void
-TokenUInt16ToText::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenUInt16ToText::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() == 0) {
         isc_throw(EvalBadStack, "Incorrect empty stack.");
     }
@@ -287,11 +334,12 @@ TokenUInt16ToText::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
     // Log what we pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_UINT16TOTEXT)
+        .arg(pkt.getLabel())
         .arg(op);
 }
 
 void
-TokenUInt32ToText::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenUInt32ToText::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() == 0) {
         isc_throw(EvalBadStack, "Incorrect empty stack.");
     }
@@ -317,6 +365,7 @@ TokenUInt32ToText::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
     // Log what we pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_UINT32TOTEXT)
+        .arg(pkt.getLabel())
         .arg(op);
 }
 
@@ -354,10 +403,12 @@ TokenOption::evaluate(Pkt& pkt, ValueStack& values) {
     // of the requested option.
     if (representation_type_ == HEXADECIMAL) {
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_OPTION)
+            .arg(pkt.getLabel())
             .arg(option_code_)
             .arg(toHex(opt_str));
     } else {
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_OPTION)
+            .arg(pkt.getLabel())
             .arg(option_code_)
             .arg('\'' + opt_str + '\'');
     }
@@ -470,6 +521,7 @@ TokenPkt::evaluate(Pkt& pkt, ValueStack& values) {
 
     // Log what we pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_PKT)
+        .arg(pkt.getLabel())
         .arg(type_str)
         .arg(print_hex ? toHex(value) : value);
 }
@@ -548,6 +600,7 @@ TokenPkt4::evaluate(Pkt& pkt, ValueStack& values) {
 
     // Log what we pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_PKT4)
+        .arg(pkt.getLabel())
         .arg(type_str)
         .arg(toHex(value));
 }
@@ -587,6 +640,7 @@ TokenPkt6::evaluate(Pkt& pkt, ValueStack& values) {
 
     // Log what we pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_PKT6)
+        .arg(pkt.getLabel())
         .arg(type_str)
         .arg(toHex(value));
 }
@@ -630,9 +684,10 @@ TokenRelay6Field::evaluate(Pkt& pkt, ValueStack& values) {
             values.push("");
             // Log what we pushed
             LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_RELAY6_RANGE)
-              .arg(type_str)
-              .arg(int(nest_level_))
-              .arg("0x");
+                .arg(pkt.getLabel())
+                .arg(type_str)
+                .arg(int(nest_level_))
+                .arg("0x");
             return;
         }
     } catch (const std::bad_cast&) {
@@ -648,13 +703,14 @@ TokenRelay6Field::evaluate(Pkt& pkt, ValueStack& values) {
 
     // Log what we pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_RELAY6)
+        .arg(pkt.getLabel())
         .arg(type_str)
         .arg(int(nest_level_))
         .arg(toHex(value));
 }
 
 void
-TokenEqual::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenEqual::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() < 2) {
         isc_throw(EvalBadStack, "Incorrect stack order. Expected at least "
                   "2 values for == operator, got " << values.size());
@@ -672,13 +728,14 @@ TokenEqual::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
     // Log what we popped and pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_EQUAL)
+        .arg(pkt.getLabel())
         .arg(toHex(op1))
         .arg(toHex(op2))
         .arg('\'' + values.top() + '\'');
 }
 
 void
-TokenSubstring::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenSubstring::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() < 3) {
         isc_throw(EvalBadStack, "Incorrect stack order. Expected at least "
                   "3 values for substring operator, got " << values.size());
@@ -697,6 +754,7 @@ TokenSubstring::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
         // Log what we popped and pushed
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_SUBSTRING_EMPTY)
+            .arg(pkt.getLabel())
             .arg(len_str)
             .arg(start_str)
             .arg("0x")
@@ -737,6 +795,7 @@ TokenSubstring::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
         // Log what we popped and pushed
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_SUBSTRING_RANGE)
+            .arg(pkt.getLabel())
             .arg(len_str)
             .arg(start_str)
             .arg(toHex(string_str))
@@ -766,6 +825,7 @@ TokenSubstring::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
     // Log what we popped and pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_SUBSTRING)
+        .arg(pkt.getLabel())
         .arg(len_str)
         .arg(start_str)
         .arg(toHex(string_str))
@@ -773,7 +833,7 @@ TokenSubstring::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 }
 
 void
-TokenSplit::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenSplit::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() < 3) {
         isc_throw(EvalBadStack, "Incorrect stack order. Expected at least "
                   "3 values for split operator, got " << values.size());
@@ -793,6 +853,7 @@ TokenSplit::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
         // Log what we popped and pushed
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_SPLIT_EMPTY)
+            .arg(pkt.getLabel())
             .arg(field_str)
             .arg(delim_str)
             .arg(string_str)
@@ -817,6 +878,7 @@ TokenSplit::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
         // Log what we popped and pushed
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_SPLIT_DELIM_EMPTY)
+            .arg(pkt.getLabel())
             .arg(field_str)
             .arg(delim_str)
             .arg(string_str)
@@ -836,6 +898,7 @@ TokenSplit::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
         // Log what we popped and pushed
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_SPLIT_FIELD_OUT_OF_RANGE)
+            .arg(pkt.getLabel())
             .arg(field_str)
             .arg(delim_str)
             .arg(string_str)
@@ -848,6 +911,7 @@ TokenSplit::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
     // Log what we popped and pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_SPLIT)
+            .arg(pkt.getLabel())
             .arg(field_str)
             .arg(delim_str)
             .arg(string_str)
@@ -855,7 +919,7 @@ TokenSplit::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 }
 
 void
-TokenConcat::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenConcat::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() < 2) {
         isc_throw(EvalBadStack, "Incorrect stack order. Expected at least "
                   "2 values for concat, got " << values.size());
@@ -871,13 +935,14 @@ TokenConcat::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
     // Log what we popped and pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_CONCAT)
+        .arg(pkt.getLabel())
         .arg(toHex(op1))
         .arg(toHex(op2))
         .arg(toHex(values.top()));
 }
 
 void
-TokenIfElse::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenIfElse::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() < 3) {
         isc_throw(EvalBadStack, "Incorrect stack order. Expected at least "
                   "3 values for ifelse, got " << values.size());
@@ -900,11 +965,13 @@ TokenIfElse::evaluate(Pkt& /*pkt*/, ValueStack& values) {
     // Log what we popped and pushed
     if (val) {
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_IFELSE_TRUE)
+            .arg(pkt.getLabel())
             .arg('\'' + cond + '\'')
             .arg(toHex(iffalse))
             .arg(toHex(iftrue));
     } else {
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_IFELSE_FALSE)
+            .arg(pkt.getLabel())
             .arg('\'' +cond + '\'')
             .arg(toHex(iftrue))
             .arg(toHex(iffalse));
@@ -912,7 +979,7 @@ TokenIfElse::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 }
 
 void
-TokenToHexString::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenToHexString::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() < 2) {
         isc_throw(EvalBadStack, "Incorrect stack order. Expected at least "
                   "2 values for hexstring, got " << values.size());
@@ -939,13 +1006,14 @@ TokenToHexString::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
     // Log what we popped and pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_TOHEXSTRING)
+        .arg(pkt.getLabel())
         .arg(toHex(binary))
         .arg(separator)
         .arg(tmp.str());
 }
 
 void
-TokenNot::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenNot::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() == 0) {
         isc_throw(EvalBadStack, "Incorrect empty stack.");
     }
@@ -962,12 +1030,13 @@ TokenNot::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
     // Log what we popped and pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_NOT)
+        .arg(pkt.getLabel())
         .arg('\'' + op + '\'')
         .arg('\'' + values.top() + '\'');
 }
 
 void
-TokenAnd::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenAnd::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() < 2) {
         isc_throw(EvalBadStack, "Incorrect stack order. Expected at least "
                   "2 values for and operator, got " << values.size());
@@ -988,13 +1057,14 @@ TokenAnd::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
     // Log what we popped and pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_AND)
+        .arg(pkt.getLabel())
         .arg('\'' + op1 + '\'')
         .arg('\'' + op2 + '\'')
         .arg('\'' + values.top() + '\'');
 }
 
 void
-TokenOr::evaluate(Pkt& /*pkt*/, ValueStack& values) {
+TokenOr::evaluate(Pkt& pkt, ValueStack& values) {
     if (values.size() < 2) {
         isc_throw(EvalBadStack, "Incorrect stack order. Expected at least "
                   "2 values for or operator, got " << values.size());
@@ -1015,6 +1085,7 @@ TokenOr::evaluate(Pkt& /*pkt*/, ValueStack& values) {
 
     // Log what we popped and pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_OR)
+        .arg(pkt.getLabel())
         .arg('\'' + op1 + '\'')
         .arg('\'' + op2 + '\'')
         .arg('\'' + values.top() + '\'');
@@ -1030,6 +1101,7 @@ TokenMember::evaluate(Pkt& pkt, ValueStack& values) {
 
     // Log what we pushed
     LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_MEMBER)
+        .arg(pkt.getLabel())
         .arg(client_class_)
         .arg('\'' + values.top() + '\'');
 }
@@ -1074,6 +1146,7 @@ void TokenVendor::evaluate(Pkt& pkt, ValueStack& values) {
         // There's no vendor option, give up.
         std::string txt = pushFailure(values);
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_VENDOR_NO_OPTION)
+            .arg(pkt.getLabel())
             .arg(code)
             .arg(txt);
         return;
@@ -1084,6 +1157,7 @@ void TokenVendor::evaluate(Pkt& pkt, ValueStack& values) {
         // than we're looking for. (0 means accept any vendor-id)
         std::string txt = pushFailure(values);
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_VENDOR_ENTERPRISE_ID_MISMATCH)
+            .arg(pkt.getLabel())
             .arg(vendor_id_)
             .arg(vendor->getVendorId())
             .arg(txt);
@@ -1099,6 +1173,7 @@ void TokenVendor::evaluate(Pkt& pkt, ValueStack& values) {
         memcpy(&txt[0], &value, sizeof(uint32_t));
         values.push(txt);
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_VENDOR_ENTERPRISE_ID)
+            .arg(pkt.getLabel())
             .arg(vendor->getVendorId())
             .arg(util::encode::encodeHex(std::vector<uint8_t>(txt.begin(),
                                                               txt.end())));
@@ -1113,6 +1188,7 @@ void TokenVendor::evaluate(Pkt& pkt, ValueStack& values) {
         // We already passed all the checks: the option is there and has specified
         // enterprise-id.
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_VENDOR_EXISTS)
+            .arg(pkt.getLabel())
             .arg(vendor->getVendorId())
             .arg("true");
         values.push("true");
@@ -1179,6 +1255,7 @@ void TokenVendorClass::evaluate(Pkt& pkt, ValueStack& values) {
         // There's no vendor class option, give up.
         std::string txt = pushFailure(values);
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_VENDOR_CLASS_NO_OPTION)
+            .arg(pkt.getLabel())
             .arg(code)
             .arg(txt);
         return;
@@ -1189,6 +1266,7 @@ void TokenVendorClass::evaluate(Pkt& pkt, ValueStack& values) {
         // than we're looking for. (0 means accept any vendor-id)
         std::string txt = pushFailure(values);
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_VENDOR_CLASS_ENTERPRISE_ID_MISMATCH)
+            .arg(pkt.getLabel())
             .arg(vendor_id_)
             .arg(vendor->getVendorId())
             .arg(txt);
@@ -1204,6 +1282,7 @@ void TokenVendorClass::evaluate(Pkt& pkt, ValueStack& values) {
         memcpy(&txt[0], &value, sizeof(uint32_t));
         values.push(txt);
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_VENDOR_CLASS_ENTERPRISE_ID)
+            .arg(pkt.getLabel())
             .arg(vendor->getVendorId())
             .arg(util::encode::encodeHex(std::vector<uint8_t>(txt.begin(),
                                                               txt.end())));
@@ -1217,6 +1296,7 @@ void TokenVendorClass::evaluate(Pkt& pkt, ValueStack& values) {
         // We already passed all the checks: the option is there and has specified
         // enterprise-id.
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_VENDOR_CLASS_EXISTS)
+            .arg(pkt.getLabel())
             .arg(vendor->getVendorId())
             .arg("true");
         values.push("true");
@@ -1228,6 +1308,7 @@ void TokenVendorClass::evaluate(Pkt& pkt, ValueStack& values) {
             // The index specified is out of bounds, e.g. there are only
             // 2 tuples and index specified is 5.
             LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_VENDOR_CLASS_DATA_NOT_FOUND)
+                .arg(pkt.getLabel())
                 .arg(index_)
                 .arg(vendor->getVendorId())
                 .arg(max)
@@ -1241,6 +1322,7 @@ void TokenVendorClass::evaluate(Pkt& pkt, ValueStack& values) {
         string txt(buf.begin(), buf.end());
 
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, EVAL_DEBUG_VENDOR_CLASS_DATA)
+            .arg(pkt.getLabel())
             .arg(index_)
             .arg(max)
             .arg(txt);
@@ -1307,11 +1389,13 @@ TokenSubOption::evaluate(Pkt& pkt, ValueStack& values) {
     // of the requested parent option and sub-option.
     if (representation_type_ == HEXADECIMAL) {
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, msgid)
+            .arg(pkt.getLabel())
             .arg(option_code_)
             .arg(sub_option_code_)
             .arg(toHex(txt));
     } else {
         LOG_DEBUG(eval_logger, EVAL_DBG_STACK, msgid)
+            .arg(pkt.getLabel())
             .arg(option_code_)
             .arg(sub_option_code_)
             .arg('\'' + txt + '\'');
