@@ -1,9 +1,11 @@
-// Copyright (C) 2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+#include <config.h>
+
 #include <hooks/callout_handle_associate.h>
 #include <hooks/hooks_manager.h>
 
@@ -23,5 +25,10 @@ CalloutHandleAssociate::getCalloutHandle() {
     return (callout_handle_);
 }
 
+void
+CalloutHandleAssociate::resetCalloutHandle() {
+    callout_handle_.reset();
+}
+
 } // end of namespace isc::hooks
 } // end of namespace isc
