@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -139,7 +139,6 @@ int callout_seven(CalloutHandle&) {
 }
 
 // The next functions are duplicates of some of the above, but return an error.
-
 int callout_one_error(CalloutHandle& handle) {
     (void) callout_one(handle);
     return (1);
@@ -160,14 +159,38 @@ int callout_four_error(CalloutHandle& handle) {
     return (1);
 }
 
-};  // extern "C"
+// The next functions are duplicates of some of the above, but throw exceptions.
+int callout_one_exception(CalloutHandle& handle) {
+    (void) callout_one(handle);
+    throw 0;
+    return (1);
+}
+
+int callout_two_exception(CalloutHandle& handle) {
+    (void) callout_two(handle);
+    throw 0;
+    return (1);
+}
+
+int callout_three_exception(CalloutHandle& handle) {
+    (void) callout_three(handle);
+    throw 0;
+    return (1);
+}
+
+int callout_four_exception(CalloutHandle& handle) {
+    (void) callout_four(handle);
+    throw 0;
+    return (1);
+}
+
+}  // extern "C"
 
 // *** Callout Tests ***
 //
 // The next set of tests check that callouts can be called.
 
 // Constructor - check that we trap bad parameters.
-
 TEST_F(CalloutManagerTest, BadConstructorParameters) {
     boost::scoped_ptr<CalloutManager> cm;
 
@@ -176,7 +199,6 @@ TEST_F(CalloutManagerTest, BadConstructorParameters) {
 }
 
 // Check the number of libraries is reported successfully.
-
 TEST_F(CalloutManagerTest, NumberOfLibraries) {
     boost::scoped_ptr<CalloutManager> cm;
 
@@ -196,7 +218,6 @@ TEST_F(CalloutManagerTest, NumberOfLibraries) {
 }
 
 // Check that we can only set the current library index to the correct values.
-
 TEST_F(CalloutManagerTest, CheckLibraryIndex) {
     // Check valid indexes.  As the callout manager is sized for 10 libraries,
     // we expect:
@@ -220,17 +241,13 @@ TEST_F(CalloutManagerTest, CheckLibraryIndex) {
 }
 
 // Check that we can only register callouts on valid hook names.
-
 TEST_F(CalloutManagerTest, ValidHookNames) {
-    getCalloutManager()->setLibraryIndex(0);
-    EXPECT_NO_THROW(getCalloutManager()->registerCallout("alpha", callout_one));
-    EXPECT_THROW(getCalloutManager()->registerCallout("unknown", callout_one),
+    EXPECT_NO_THROW(getCalloutManager()->registerCallout("alpha", callout_one, 0));
+    EXPECT_THROW(getCalloutManager()->registerCallout("unknown", callout_one, 0),
                                                       NoSuchHook);
 }
 
-
 // Check we can register callouts appropriately.
-
 TEST_F(CalloutManagerTest, RegisterCallout) {
     // Ensure that no callouts are attached to any of the hooks.
     EXPECT_FALSE(getCalloutManager()->calloutsPresent(alpha_index_));
@@ -238,10 +255,8 @@ TEST_F(CalloutManagerTest, RegisterCallout) {
 
     // Set up so that hooks "alpha" and "beta" have callouts attached from a
     // different libraries.
-    getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->registerCallout("alpha", callout_one);
-    getCalloutManager()->setLibraryIndex(1);
-    getCalloutManager()->registerCallout("beta", callout_two);
+    getCalloutManager()->registerCallout("alpha", callout_one, 0);
+    getCalloutManager()->registerCallout("beta", callout_two, 1);
 
     // Check all is as expected.
     EXPECT_TRUE(getCalloutManager()->calloutsPresent(alpha_index_));
@@ -260,11 +275,9 @@ TEST_F(CalloutManagerTest, RegisterCallout) {
     EXPECT_EQ(2, callout_value_);
 
     // Register some more callouts from different libraries on hook "alpha".
-    getCalloutManager()->setLibraryIndex(2);
-    getCalloutManager()->registerCallout("alpha", callout_three);
-    getCalloutManager()->registerCallout("alpha", callout_four);
-    getCalloutManager()->setLibraryIndex(3);
-    getCalloutManager()->registerCallout("alpha", callout_five);
+    getCalloutManager()->registerCallout("alpha", callout_three, 2);
+    getCalloutManager()->registerCallout("alpha", callout_four, 2);
+    getCalloutManager()->registerCallout("alpha", callout_five, 3);
 
     // Check it is as expected.
     callout_value_ = 0;
@@ -279,23 +292,20 @@ TEST_F(CalloutManagerTest, RegisterCallout) {
 
     // Add another callout to hook "alpha" from library index 2 - this should
     // appear at the end of the callout list for that library.
-    getCalloutManager()->setLibraryIndex(2);
-    getCalloutManager()->registerCallout("alpha", callout_six);
+    getCalloutManager()->registerCallout("alpha", callout_six, 2);
     callout_value_ = 0;
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(13465, callout_value_);
 
     // Add a callout from library index 1 - this should appear between the
     // callouts from library index 0 and library index 2.
-    getCalloutManager()->setLibraryIndex(1);
-    getCalloutManager()->registerCallout("alpha", callout_seven);
+    getCalloutManager()->registerCallout("alpha", callout_seven, 1);
     callout_value_ = 0;
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(173465, callout_value_);
 }
 
 // Check the "calloutsPresent()" method.
-
 TEST_F(CalloutManagerTest, CalloutsPresent) {
     // Ensure that no callouts are attached to any of the hooks.
     EXPECT_FALSE(getCalloutManager()->calloutsPresent(alpha_index_));
@@ -308,16 +318,11 @@ TEST_F(CalloutManagerTest, CalloutsPresent) {
     // exact callouts attached to a hook are not relevant - only the fact
     // that some callouts are).  Chose the libraries for which the callouts
     // are registered randomly.
-    getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->registerCallout("alpha", callout_one);
-
-    getCalloutManager()->setLibraryIndex(1);
-    getCalloutManager()->registerCallout("alpha", callout_two);
-    getCalloutManager()->registerCallout("beta", callout_two);
-
-    getCalloutManager()->setLibraryIndex(3);
-    getCalloutManager()->registerCallout("alpha", callout_three);
-    getCalloutManager()->registerCallout("delta", callout_four);
+    getCalloutManager()->registerCallout("alpha", callout_one, 0);
+    getCalloutManager()->registerCallout("alpha", callout_two, 1);
+    getCalloutManager()->registerCallout("beta", callout_two, 1);
+    getCalloutManager()->registerCallout("alpha", callout_three, 3);
+    getCalloutManager()->registerCallout("delta", callout_four, 3);
 
     // Check all is as expected.
     EXPECT_TRUE(getCalloutManager()->calloutsPresent(alpha_index_));
@@ -331,7 +336,6 @@ TEST_F(CalloutManagerTest, CalloutsPresent) {
 }
 
 // Test that calling a hook with no callouts on it returns success.
-
 TEST_F(CalloutManagerTest, CallNoCallouts) {
     // Ensure that no callouts are attached to any of the hooks.
     EXPECT_FALSE(getCalloutManager()->calloutsPresent(alpha_index_));
@@ -348,7 +352,6 @@ TEST_F(CalloutManagerTest, CallNoCallouts) {
 // Test that the callouts are called in the correct order (i.e. the callouts
 // from the first library in the order they were registered, then the callouts
 // from the second library in the order they were registered etc.)
-
 TEST_F(CalloutManagerTest, CallCalloutsSuccess) {
     // Ensure that no callouts are attached to any of the hooks.
     EXPECT_FALSE(getCalloutManager()->calloutsPresent(alpha_index_));
@@ -358,26 +361,19 @@ TEST_F(CalloutManagerTest, CallCalloutsSuccess) {
 
     // Each library contributes one callout on hook "alpha".
     callout_value_ = 0;
-    getCalloutManager()->setLibraryIndex(1);
-    getCalloutManager()->registerCallout("alpha", callout_one);
-    getCalloutManager()->setLibraryIndex(1);
-    getCalloutManager()->registerCallout("alpha", callout_two);
-    getCalloutManager()->setLibraryIndex(2);
-    getCalloutManager()->registerCallout("alpha", callout_three);
-    getCalloutManager()->setLibraryIndex(3);
-    getCalloutManager()->registerCallout("alpha", callout_four);
+    getCalloutManager()->registerCallout("alpha", callout_one, 1);
+    getCalloutManager()->registerCallout("alpha", callout_two, 1);
+    getCalloutManager()->registerCallout("alpha", callout_three, 2);
+    getCalloutManager()->registerCallout("alpha", callout_four, 3);
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(1234, callout_value_);
 
     // Do a random selection of callouts on hook "beta".
     callout_value_ = 0;
-    getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->registerCallout("beta", callout_one);
-    getCalloutManager()->registerCallout("beta", callout_three);
-    getCalloutManager()->setLibraryIndex(1);
-    getCalloutManager()->registerCallout("beta", callout_two);
-    getCalloutManager()->setLibraryIndex(3);
-    getCalloutManager()->registerCallout("beta", callout_four);
+    getCalloutManager()->registerCallout("beta", callout_one, 0);
+    getCalloutManager()->registerCallout("beta", callout_three, 0);
+    getCalloutManager()->registerCallout("beta", callout_two, 1);
+    getCalloutManager()->registerCallout("beta", callout_four, 3);
     getCalloutManager()->callCallouts(beta_index_, getCalloutHandle());
     EXPECT_EQ(1324, callout_value_);
 
@@ -388,11 +384,10 @@ TEST_F(CalloutManagerTest, CallCalloutsSuccess) {
 }
 
 // Test that the callouts are called in order, but that callouts occurring
-// after a callout that returns an error are not called.
+// after a callout that returns an error are called.
 //
 // (Note: in this test, the callouts that return an error set the value of
 // callout_value_ before they return the error code.)
-
 TEST_F(CalloutManagerTest, CallCalloutsError) {
     // Ensure that no callouts are attached to any of the hooks.
     EXPECT_FALSE(getCalloutManager()->calloutsPresent(alpha_index_));
@@ -403,68 +398,118 @@ TEST_F(CalloutManagerTest, CallCalloutsError) {
     // Each library contributing one callout on hook "alpha". The first callout
     // returns an error (after adding its value to the result).
     callout_value_ = 0;
-    getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->registerCallout("alpha", callout_one_error);
-    getCalloutManager()->setLibraryIndex(1);
-    getCalloutManager()->registerCallout("alpha", callout_two);
-    getCalloutManager()->setLibraryIndex(2);
-    getCalloutManager()->registerCallout("alpha", callout_three);
-    getCalloutManager()->setLibraryIndex(3);
-    getCalloutManager()->registerCallout("alpha", callout_four);
+    getCalloutManager()->registerCallout("alpha", callout_one_error, 0);
+    getCalloutManager()->registerCallout("alpha", callout_two, 1);
+    getCalloutManager()->registerCallout("alpha", callout_three, 2);
+    getCalloutManager()->registerCallout("alpha", callout_four, 3);
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(1234, callout_value_);
 
     // Each library contributing multiple callouts on hook "beta". The last
     // callout on the first library returns an error.
     callout_value_ = 0;
-    getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->registerCallout("beta", callout_one);
-    getCalloutManager()->registerCallout("beta", callout_one_error);
-    getCalloutManager()->setLibraryIndex(1);
-    getCalloutManager()->registerCallout("beta", callout_two);
-    getCalloutManager()->registerCallout("beta", callout_two);
-    getCalloutManager()->registerCallout("beta", callout_three);
-    getCalloutManager()->registerCallout("beta", callout_three);
-    getCalloutManager()->setLibraryIndex(3);
-    getCalloutManager()->registerCallout("beta", callout_four);
-    getCalloutManager()->registerCallout("beta", callout_four);
+    getCalloutManager()->registerCallout("beta", callout_one, 0);
+    getCalloutManager()->registerCallout("beta", callout_one_error, 0);
+    getCalloutManager()->registerCallout("beta", callout_two, 1);
+    getCalloutManager()->registerCallout("beta", callout_two, 1);
+    getCalloutManager()->registerCallout("beta", callout_three, 1);
+    getCalloutManager()->registerCallout("beta", callout_three, 1);
+    getCalloutManager()->registerCallout("beta", callout_four, 3);
+    getCalloutManager()->registerCallout("beta", callout_four, 3);
     getCalloutManager()->callCallouts(beta_index_, getCalloutHandle());
     EXPECT_EQ(11223344, callout_value_);
 
     // A callout in a random position in the callout list returns an error.
     callout_value_ = 0;
-    getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->registerCallout("gamma", callout_one);
-    getCalloutManager()->registerCallout("gamma", callout_one);
-    getCalloutManager()->setLibraryIndex(1);
-    getCalloutManager()->registerCallout("gamma", callout_two);
-    getCalloutManager()->registerCallout("gamma", callout_two);
-    getCalloutManager()->setLibraryIndex(3);
-    getCalloutManager()->registerCallout("gamma", callout_four_error);
-    getCalloutManager()->registerCallout("gamma", callout_four);
+    getCalloutManager()->registerCallout("gamma", callout_one, 0);
+    getCalloutManager()->registerCallout("gamma", callout_one, 0);
+    getCalloutManager()->registerCallout("gamma", callout_two, 1);
+    getCalloutManager()->registerCallout("gamma", callout_two, 1);
+    getCalloutManager()->registerCallout("gamma", callout_four_error, 3);
+    getCalloutManager()->registerCallout("gamma", callout_four, 3);
     getCalloutManager()->callCallouts(gamma_index_, getCalloutHandle());
     EXPECT_EQ(112244, callout_value_);
 
     // The last callout on a hook returns an error.
     callout_value_ = 0;
-    getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->registerCallout("delta", callout_one);
-    getCalloutManager()->registerCallout("delta", callout_one);
-    getCalloutManager()->setLibraryIndex(1);
-    getCalloutManager()->registerCallout("delta", callout_two);
-    getCalloutManager()->registerCallout("delta", callout_two);
-    getCalloutManager()->setLibraryIndex(2);
-    getCalloutManager()->registerCallout("delta", callout_three);
-    getCalloutManager()->registerCallout("delta", callout_three);
-    getCalloutManager()->setLibraryIndex(3);
-    getCalloutManager()->registerCallout("delta", callout_four);
-    getCalloutManager()->registerCallout("delta", callout_four_error);
+    getCalloutManager()->registerCallout("delta", callout_one, 0);
+    getCalloutManager()->registerCallout("delta", callout_one, 0);
+    getCalloutManager()->registerCallout("delta", callout_two, 1);
+    getCalloutManager()->registerCallout("delta", callout_two, 1);
+    getCalloutManager()->registerCallout("delta", callout_three, 2);
+    getCalloutManager()->registerCallout("delta", callout_three, 2);
+    getCalloutManager()->registerCallout("delta", callout_four, 3);
+    getCalloutManager()->registerCallout("delta", callout_four_error, 3);
     getCalloutManager()->callCallouts(delta_index_, getCalloutHandle());
     EXPECT_EQ(11223344, callout_value_);
 }
 
-// Now test that we can deregister a single callout on a hook.
+// Test that the callouts are called in order, but that callouts occurring
+// after a callout that throws exception are called.
+//
+// (Note: in this test, the callouts that return an error set the value of
+// callout_value_ before they throw exception.)
+TEST_F(CalloutManagerTest, CallCalloutsException) {
+    // Ensure that no callouts are attached to any of the hooks.
+    EXPECT_FALSE(getCalloutManager()->calloutsPresent(alpha_index_));
+    EXPECT_FALSE(getCalloutManager()->calloutsPresent(beta_index_));
+    EXPECT_FALSE(getCalloutManager()->calloutsPresent(gamma_index_));
+    EXPECT_FALSE(getCalloutManager()->calloutsPresent(delta_index_));
 
+    // Each library contributing one callout on hook "alpha". The first callout
+    // returns an error (after adding its value to the result).
+    callout_value_ = 0;
+    getCalloutManager()->registerCallout("alpha", callout_one_exception, 0);
+    getCalloutManager()->registerCallout("alpha", callout_two, 1);
+    getCalloutManager()->registerCallout("alpha", callout_three, 2);
+    getCalloutManager()->registerCallout("alpha", callout_four, 3);
+    getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
+    EXPECT_EQ(1234, callout_value_);
+    EXPECT_EQ(getCalloutHandle().getStatus(), CalloutHandle::NEXT_STEP_DROP);
+
+    // Each library contributing multiple callouts on hook "beta". The last
+    // callout on the first library returns an error.
+    callout_value_ = 0;
+    getCalloutManager()->registerCallout("beta", callout_one, 0);
+    getCalloutManager()->registerCallout("beta", callout_one_exception, 0);
+    getCalloutManager()->registerCallout("beta", callout_two, 1);
+    getCalloutManager()->registerCallout("beta", callout_two, 1);
+    getCalloutManager()->registerCallout("beta", callout_three, 1);
+    getCalloutManager()->registerCallout("beta", callout_three, 1);
+    getCalloutManager()->registerCallout("beta", callout_four, 3);
+    getCalloutManager()->registerCallout("beta", callout_four, 3);
+    getCalloutManager()->callCallouts(beta_index_, getCalloutHandle());
+    EXPECT_EQ(11223344, callout_value_);
+    EXPECT_EQ(getCalloutHandle().getStatus(), CalloutHandle::NEXT_STEP_DROP);
+
+    // A callout in a random position in the callout list returns an error.
+    callout_value_ = 0;
+    getCalloutManager()->registerCallout("gamma", callout_one, 0);
+    getCalloutManager()->registerCallout("gamma", callout_one, 0);
+    getCalloutManager()->registerCallout("gamma", callout_two, 1);
+    getCalloutManager()->registerCallout("gamma", callout_two, 1);
+    getCalloutManager()->registerCallout("gamma", callout_four_exception, 3);
+    getCalloutManager()->registerCallout("gamma", callout_four, 3);
+    getCalloutManager()->callCallouts(gamma_index_, getCalloutHandle());
+    EXPECT_EQ(112244, callout_value_);
+    EXPECT_EQ(getCalloutHandle().getStatus(), CalloutHandle::NEXT_STEP_DROP);
+
+    // The last callout on a hook returns an error.
+    callout_value_ = 0;
+    getCalloutManager()->registerCallout("delta", callout_one, 0);
+    getCalloutManager()->registerCallout("delta", callout_one, 0);
+    getCalloutManager()->registerCallout("delta", callout_two, 1);
+    getCalloutManager()->registerCallout("delta", callout_two, 1);
+    getCalloutManager()->registerCallout("delta", callout_three, 2);
+    getCalloutManager()->registerCallout("delta", callout_three, 2);
+    getCalloutManager()->registerCallout("delta", callout_four, 3);
+    getCalloutManager()->registerCallout("delta", callout_four_exception, 3);
+    getCalloutManager()->callCallouts(delta_index_, getCalloutHandle());
+    EXPECT_EQ(11223344, callout_value_);
+    EXPECT_EQ(getCalloutHandle().getStatus(), CalloutHandle::NEXT_STEP_DROP);
+}
+
+// Now test that we can deregister a single callout on a hook.
 TEST_F(CalloutManagerTest, DeregisterSingleCallout) {
     // Ensure that no callouts are attached to any of the hooks.
     EXPECT_FALSE(getCalloutManager()->calloutsPresent(alpha_index_));
@@ -474,23 +519,20 @@ TEST_F(CalloutManagerTest, DeregisterSingleCallout) {
 
     // Add a callout to hook "alpha" and check it is added correctly.
     callout_value_ = 0;
-    getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->registerCallout("alpha", callout_two);
+    getCalloutManager()->registerCallout("alpha", callout_two, 0);
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(2, callout_value_);
 
     // Remove it and check that the no callouts are present.  We have to reset
     // the current library index here as it was invalidated by the call
     // to callCallouts().
-    getCalloutManager()->setLibraryIndex(0);
     EXPECT_TRUE(getCalloutManager()->calloutsPresent(alpha_index_));
-    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_two));
+    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_two, 0));
     EXPECT_FALSE(getCalloutManager()->calloutsPresent(alpha_index_));
 }
 
 // Now test that we can deregister a single callout on a hook that has multiple
 // callouts from the same library.
-
 TEST_F(CalloutManagerTest, DeregisterSingleCalloutSameLibrary) {
     // Ensure that no callouts are attached to any of the hooks.
     EXPECT_FALSE(getCalloutManager()->calloutsPresent(alpha_index_));
@@ -500,33 +542,28 @@ TEST_F(CalloutManagerTest, DeregisterSingleCalloutSameLibrary) {
 
     // Add multiple callouts to hook "alpha".
     callout_value_ = 0;
-    getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->registerCallout("alpha", callout_one);
-    getCalloutManager()->registerCallout("alpha", callout_two);
-    getCalloutManager()->registerCallout("alpha", callout_three);
-    getCalloutManager()->registerCallout("alpha", callout_four);
+    getCalloutManager()->registerCallout("alpha", callout_one, 0);
+    getCalloutManager()->registerCallout("alpha", callout_two, 0);
+    getCalloutManager()->registerCallout("alpha", callout_three, 0);
+    getCalloutManager()->registerCallout("alpha", callout_four, 0);
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(1234, callout_value_);
 
     // Remove the callout_two callout.  We have to reset the current library
     // index here as it was invalidated by the call to callCallouts().
-    getCalloutManager()->setLibraryIndex(0);
-    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_two));
+    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_two, 0));
     callout_value_ = 0;
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(134, callout_value_);
 
     // Try removing it again.
-    getCalloutManager()->setLibraryIndex(0);
-    EXPECT_FALSE(getCalloutManager()->deregisterCallout("alpha", callout_two));
+    EXPECT_FALSE(getCalloutManager()->deregisterCallout("alpha", callout_two, 0));
     callout_value_ = 0;
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(134, callout_value_);
-
 }
 
 // Check we can deregister multiple callouts from the same library.
-
 TEST_F(CalloutManagerTest, DeregisterMultipleCalloutsSameLibrary) {
     // Ensure that no callouts are attached to any of the hooks.
     EXPECT_FALSE(getCalloutManager()->calloutsPresent(alpha_index_));
@@ -536,44 +573,39 @@ TEST_F(CalloutManagerTest, DeregisterMultipleCalloutsSameLibrary) {
 
     // Each library contributes one callout on hook "alpha".
     callout_value_ = 0;
-    getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->registerCallout("alpha", callout_one);
-    getCalloutManager()->registerCallout("alpha", callout_two);
-    getCalloutManager()->registerCallout("alpha", callout_one);
-    getCalloutManager()->registerCallout("alpha", callout_two);
-    getCalloutManager()->registerCallout("alpha", callout_three);
-    getCalloutManager()->registerCallout("alpha", callout_four);
-    getCalloutManager()->registerCallout("alpha", callout_three);
-    getCalloutManager()->registerCallout("alpha", callout_four);
+    getCalloutManager()->registerCallout("alpha", callout_one, 0);
+    getCalloutManager()->registerCallout("alpha", callout_two, 0);
+    getCalloutManager()->registerCallout("alpha", callout_one, 0);
+    getCalloutManager()->registerCallout("alpha", callout_two, 0);
+    getCalloutManager()->registerCallout("alpha", callout_three, 0);
+    getCalloutManager()->registerCallout("alpha", callout_four, 0);
+    getCalloutManager()->registerCallout("alpha", callout_three, 0);
+    getCalloutManager()->registerCallout("alpha", callout_four, 0);
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(12123434, callout_value_);
 
     // Remove the callout_two callouts.  We have to reset the current library
     // index here as it was invalidated by the call to callCallouts().
-    getCalloutManager()->setLibraryIndex(0);
-    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_two));
+    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_two, 0));
     callout_value_ = 0;
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(113434, callout_value_);
 
     // Try removing multiple callouts that includes one at the end of the
     // list of callouts.
-    getCalloutManager()->setLibraryIndex(0);
-    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_four));
+    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_four, 0));
     callout_value_ = 0;
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(1133, callout_value_);
 
     // ... and from the start.
-    getCalloutManager()->setLibraryIndex(0);
-    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_one));
+    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_one, 0));
     callout_value_ = 0;
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(33, callout_value_);
 
     // ... and the remaining callouts.
-    getCalloutManager()->setLibraryIndex(0);
-    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_three));
+    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_three, 0));
     callout_value_ = 0;
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(0, callout_value_);
@@ -582,7 +614,6 @@ TEST_F(CalloutManagerTest, DeregisterMultipleCalloutsSameLibrary) {
 }
 
 // Check we can deregister multiple callouts from multiple libraries.
-
 TEST_F(CalloutManagerTest, DeregisterMultipleCalloutsMultipleLibraries) {
     // Ensure that no callouts are attached to any of the hooks.
     EXPECT_FALSE(getCalloutManager()->calloutsPresent(alpha_index_));
@@ -592,57 +623,47 @@ TEST_F(CalloutManagerTest, DeregisterMultipleCalloutsMultipleLibraries) {
 
     // Each library contributes two callouts to hook "alpha".
     callout_value_ = 0;
-    getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->registerCallout("alpha", callout_one);
-    getCalloutManager()->registerCallout("alpha", callout_two);
-    getCalloutManager()->setLibraryIndex(1);
-    getCalloutManager()->registerCallout("alpha", callout_three);
-    getCalloutManager()->registerCallout("alpha", callout_four);
-    getCalloutManager()->setLibraryIndex(2);
-    getCalloutManager()->registerCallout("alpha", callout_five);
-    getCalloutManager()->registerCallout("alpha", callout_two);
+    getCalloutManager()->registerCallout("alpha", callout_one, 0);
+    getCalloutManager()->registerCallout("alpha", callout_two, 0);
+    getCalloutManager()->registerCallout("alpha", callout_three, 1);
+    getCalloutManager()->registerCallout("alpha", callout_four, 1);
+    getCalloutManager()->registerCallout("alpha", callout_five, 2);
+    getCalloutManager()->registerCallout("alpha", callout_two, 2);
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(123452, callout_value_);
 
     // Remove the callout_two callout from library 0.  It should not affect
     // the second callout_two callout registered by library 2.
-    getCalloutManager()->setLibraryIndex(0);
-    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_two));
+    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_two, 0));
     callout_value_ = 0;
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(13452, callout_value_);
 }
 
 // Check we can deregister all callouts from a single library.
-
 TEST_F(CalloutManagerTest, DeregisterAllCallouts) {
     // Ensure that no callouts are attached to hook one.
     EXPECT_FALSE(getCalloutManager()->calloutsPresent(alpha_index_));
 
     // Each library contributes two callouts to hook "alpha".
     callout_value_ = 0;
-    getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->registerCallout("alpha", callout_one);
-    getCalloutManager()->registerCallout("alpha", callout_two);
-    getCalloutManager()->setLibraryIndex(1);
-    getCalloutManager()->registerCallout("alpha", callout_three);
-    getCalloutManager()->registerCallout("alpha", callout_four);
-    getCalloutManager()->setLibraryIndex(2);
-    getCalloutManager()->registerCallout("alpha", callout_five);
-    getCalloutManager()->registerCallout("alpha", callout_six);
+    getCalloutManager()->registerCallout("alpha", callout_one, 0);
+    getCalloutManager()->registerCallout("alpha", callout_two, 0);
+    getCalloutManager()->registerCallout("alpha", callout_three, 1);
+    getCalloutManager()->registerCallout("alpha", callout_four, 1);
+    getCalloutManager()->registerCallout("alpha", callout_five, 2);
+    getCalloutManager()->registerCallout("alpha", callout_six, 2);
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(123456, callout_value_);
 
     // Remove all callouts from library index 1.
-    getCalloutManager()->setLibraryIndex(1);
-    EXPECT_TRUE(getCalloutManager()->deregisterAllCallouts("alpha"));
+    EXPECT_TRUE(getCalloutManager()->deregisterAllCallouts("alpha", 1));
     callout_value_ = 0;
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(1256, callout_value_);
 
     // Remove all callouts from library index 2.
-    getCalloutManager()->setLibraryIndex(2);
-    EXPECT_TRUE(getCalloutManager()->deregisterAllCallouts("alpha"));
+    EXPECT_TRUE(getCalloutManager()->deregisterAllCallouts("alpha", 2));
     callout_value_ = 0;
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(12, callout_value_);
@@ -651,7 +672,6 @@ TEST_F(CalloutManagerTest, DeregisterAllCallouts) {
 // Check that we can register/deregister callouts on different libraries
 // and different hooks, and that the callout instances are regarded as
 // unique and do not affect one another.
-
 TEST_F(CalloutManagerTest, MultipleCalloutsLibrariesHooks) {
     // Ensure that no callouts are attached to any of the hooks.
     EXPECT_FALSE(getCalloutManager()->calloutsPresent(alpha_index_));
@@ -661,27 +681,22 @@ TEST_F(CalloutManagerTest, MultipleCalloutsLibrariesHooks) {
 
     // Register callouts on the alpha hook.
     callout_value_ = 0;
-    getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->registerCallout("alpha", callout_one);
-    getCalloutManager()->registerCallout("alpha", callout_two);
-    getCalloutManager()->setLibraryIndex(1);
-    getCalloutManager()->registerCallout("alpha", callout_three);
-    getCalloutManager()->registerCallout("alpha", callout_four);
-    getCalloutManager()->setLibraryIndex(2);
-    getCalloutManager()->registerCallout("alpha", callout_five);
-    getCalloutManager()->registerCallout("alpha", callout_two);
+    getCalloutManager()->registerCallout("alpha", callout_one, 0);
+    getCalloutManager()->registerCallout("alpha", callout_two, 0);
+    getCalloutManager()->registerCallout("alpha", callout_three, 1);
+    getCalloutManager()->registerCallout("alpha", callout_four, 1);
+    getCalloutManager()->registerCallout("alpha", callout_five, 2);
+    getCalloutManager()->registerCallout("alpha", callout_two, 2);
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(123452, callout_value_);
 
     // Register the same callouts on the beta hook, and check that those
     // on the alpha hook are not affected.
     callout_value_ = 0;
-    getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->registerCallout("beta", callout_five);
-    getCalloutManager()->registerCallout("beta", callout_one);
-    getCalloutManager()->setLibraryIndex(2);
-    getCalloutManager()->registerCallout("beta", callout_four);
-    getCalloutManager()->registerCallout("beta", callout_three);
+    getCalloutManager()->registerCallout("beta", callout_five, 0);
+    getCalloutManager()->registerCallout("beta", callout_one, 0);
+    getCalloutManager()->registerCallout("beta", callout_four, 2);
+    getCalloutManager()->registerCallout("beta", callout_three, 2);
     getCalloutManager()->callCallouts(beta_index_, getCalloutHandle());
     EXPECT_EQ(5143, callout_value_);
 
@@ -691,8 +706,7 @@ TEST_F(CalloutManagerTest, MultipleCalloutsLibrariesHooks) {
     EXPECT_EQ(123452, callout_value_);
 
     // Remove callout four from beta and check that alpha is not affected.
-    getCalloutManager()->setLibraryIndex(2);
-    EXPECT_TRUE(getCalloutManager()->deregisterCallout("beta", callout_four));
+    EXPECT_TRUE(getCalloutManager()->deregisterCallout("beta", callout_four, 2));
 
     callout_value_ = 0;
     getCalloutManager()->callCallouts(beta_index_, getCalloutHandle());
@@ -711,7 +725,6 @@ TEST_F(CalloutManagerTest, MultipleCalloutsLibrariesHooks) {
 // only register and deregister callouts within its library) require that
 // the CalloutHandle object pass the appropriate LibraryHandle to the
 // callout.  These tests are done in the handles_unittest tests.
-
 TEST_F(CalloutManagerTest, LibraryHandleRegistration) {
     // Ensure that no callouts are attached to any of the hooks.
     EXPECT_FALSE(getCalloutManager()->calloutsPresent(alpha_index_));
@@ -719,15 +732,11 @@ TEST_F(CalloutManagerTest, LibraryHandleRegistration) {
     // Set up so that hooks "alpha" and "beta" have callouts attached from a
     // different libraries.
     getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->getLibraryHandle().registerCallout("alpha",
-                                                            callout_one);
-    getCalloutManager()->getLibraryHandle().registerCallout("alpha",
-                                                            callout_two);
+    getCalloutManager()->getLibraryHandle().registerCallout("alpha", callout_one);
+    getCalloutManager()->getLibraryHandle().registerCallout("alpha", callout_two);
     getCalloutManager()->setLibraryIndex(1);
-    getCalloutManager()->getLibraryHandle().registerCallout("alpha",
-                                                            callout_three);
-    getCalloutManager()->getLibraryHandle().registerCallout("alpha",
-                                                            callout_four);
+    getCalloutManager()->getLibraryHandle().registerCallout("alpha", callout_three);
+    getCalloutManager()->getLibraryHandle().registerCallout("alpha", callout_four);
 
     // Check all is as expected.
     EXPECT_TRUE(getCalloutManager()->calloutsPresent(alpha_index_));
@@ -743,21 +752,18 @@ TEST_F(CalloutManagerTest, LibraryHandleRegistration) {
 
     // Deregister a callout on library index 0 (after we check we can't
     // deregister it through library index 1).
-    getCalloutManager()->setLibraryIndex(1);
-    EXPECT_FALSE(getCalloutManager()->deregisterCallout("alpha", callout_two));
+    EXPECT_FALSE(getCalloutManager()->deregisterCallout("alpha", callout_two, 1));
     callout_value_ = 0;
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(1234, callout_value_);
 
-    getCalloutManager()->setLibraryIndex(0);
-    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_two));
+    EXPECT_TRUE(getCalloutManager()->deregisterCallout("alpha", callout_two, 0));
     callout_value_ = 0;
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(134, callout_value_);
 
     // Deregister all callouts on library index 1.
-    getCalloutManager()->setLibraryIndex(1);
-    EXPECT_TRUE(getCalloutManager()->deregisterAllCallouts("alpha"));
+    EXPECT_TRUE(getCalloutManager()->deregisterAllCallouts("alpha", 1));
     callout_value_ = 0;
     getCalloutManager()->callCallouts(alpha_index_, getCalloutHandle());
     EXPECT_EQ(1, callout_value_);
@@ -771,11 +777,11 @@ TEST_F(CalloutManagerTest, LibraryHandleAlternateConstructor) {
 
     // Set up so that hooks "alpha" and "beta" have callouts attached from a
     // different libraries.
-    LibraryHandle lh0(getCalloutManager().get(), 0);
+    LibraryHandle lh0(*getCalloutManager().get(), 0);
     lh0.registerCallout("alpha", callout_one);
     lh0.registerCallout("alpha", callout_two);
 
-    LibraryHandle lh1(getCalloutManager().get(), 1);
+    LibraryHandle lh1(*getCalloutManager().get(), 1);
     lh1.registerCallout("alpha", callout_three);
     lh1.registerCallout("alpha", callout_four);
 
@@ -812,7 +818,6 @@ TEST_F(CalloutManagerTest, LibraryHandleAlternateConstructor) {
 
 // Check that the pre- and post- user callout library handles work
 // appropriately with no user libraries.
-
 TEST_F(CalloutManagerTest, LibraryHandlePrePostNoLibraries) {
     // Create a local callout manager and callout handle to reflect no libraries
     // being loaded.
@@ -844,7 +849,6 @@ TEST_F(CalloutManagerTest, LibraryHandlePrePostNoLibraries) {
 }
 
 // Repeat the tests with one user library.
-
 TEST_F(CalloutManagerTest, LibraryHandlePrePostUserLibrary) {
 
     // Setup the pre-, library and post callouts.
@@ -854,7 +858,7 @@ TEST_F(CalloutManagerTest, LibraryHandlePrePostUserLibrary) {
                                                                 callout_one);
 
     // ... and set up a callout in between, on library number 2.
-    LibraryHandle lh1(getCalloutManager().get(), 2);
+    LibraryHandle lh1(*getCalloutManager().get(), 2);
     lh1.registerCallout("alpha", callout_five);
 
     // Check all is as expected.
@@ -870,7 +874,6 @@ TEST_F(CalloutManagerTest, LibraryHandlePrePostUserLibrary) {
 }
 
 // Test that control command handlers can be installed as callouts.
-
 TEST_F(CalloutManagerTest, LibraryHandleRegisterCommandHandler) {
     CalloutHandle handle(getCalloutManager());
 
@@ -882,15 +885,11 @@ TEST_F(CalloutManagerTest, LibraryHandleRegisterCommandHandler) {
     // 'command-two' should also be called.
 
     getCalloutManager()->setLibraryIndex(0);
-    getCalloutManager()->getLibraryHandle().registerCommandCallout("command-one",
-                                                                   callout_one);
-    getCalloutManager()->getLibraryHandle().registerCommandCallout("command-one",
-                                                                   callout_four);
+    getCalloutManager()->getLibraryHandle().registerCommandCallout("command-one", callout_one);
+    getCalloutManager()->getLibraryHandle().registerCommandCallout("command-one", callout_four);
     getCalloutManager()->setLibraryIndex(1);
-    getCalloutManager()->getLibraryHandle().registerCommandCallout("command-one",
-                                                                   callout_two);
-    getCalloutManager()->getLibraryHandle().registerCommandCallout("command-two",
-                                                                   callout_three);
+    getCalloutManager()->getLibraryHandle().registerCommandCallout("command-one", callout_two);
+    getCalloutManager()->getLibraryHandle().registerCommandCallout("command-two", callout_three);
 
     // Command handlers are installed for commands: 'command-one' and 'command-two'.
     EXPECT_TRUE(getCalloutManager()->commandHandlersPresent("command-one"));
@@ -938,9 +937,8 @@ TEST_F(CalloutManagerTest, VectorSize) {
     EXPECT_EQ(s + 1, getCalloutManager()->getHookLibsVectorSize());
 }
 
-
 // The setting of the hook index is checked in the handles_unittest
 // set of tests, as access restrictions mean it is not easily tested
 // on its own.
 
-} // Anonymous namespace
+} // namespace
