@@ -1,6 +1,4 @@
-// File created from log_messages.mes on Tue Jan 24 2017 20:17
-
-#include <config.h>
+// File created from ../../../src/lib/log/log_messages.mes
 
 #include <cstddef>
 #include <log/message_types.h>
