@@ -1,4 +1,4 @@
-// Copyright (C) 2016-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016-2025 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,9 +6,16 @@
 
 #include <config.h>
 
+#include <asiolink/io_service.h>
+#include <asiolink/process_spawn.h>
+#include <database/database_connection.h>
 #include <database/db_exceptions.h>
 #include <database/db_log.h>
 #include <pgsql/pgsql_connection.h>
+#include <util/filesystem.h>
+
+#include <exception>
+#include <unordered_map>
 
 // PostgreSQL errors should be tested based on the SQL state code.  Each state
 // code is 5 decimal, ASCII, digits, the first two define the category of
@@ -28,11 +35,14 @@
 
 #include <sstream>
 
+using namespace isc::asiolink;
 using namespace std;
 
 namespace isc {
 namespace db {
 
+std::string PgSqlConnection::KEA_ADMIN_ = KEA_ADMIN;
+
 // Default connection timeout
 
 /// @todo: migrate this default timeout to src/bin/dhcpX/simple_parserX.cc
@@ -134,14 +144,22 @@ PgSqlConnection::~PgSqlConnection() {
 }
 
 std::pair<uint32_t, uint32_t>
-PgSqlConnection::getVersion(const ParameterMap& parameters) {
+PgSqlConnection::getVersion(const ParameterMap& parameters,
+                            const IOServiceAccessorPtr& ac,
+                            const DbCallback& cb,
+                            const string& timer_name,
+                            unsigned int id) {
     // Get a connection.
-    PgSqlConnection conn(parameters);
+    PgSqlConnection conn(parameters, ac, cb);
+
+    if (!timer_name.empty()) {
+        conn.makeReconnectCtl(timer_name, id);
+    }
 
     // Open the database.
     conn.openDatabaseInternal(false);
 
-    const char* version_sql =  "SELECT version, minor FROM schema_version;";
+    const char* version_sql = "SELECT version, minor FROM schema_version;";
     PgSqlResult r(PQexec(conn.conn_, version_sql));
     if (PQresultStatus(r) != PGRES_TUPLES_OK) {
         isc_throw(DbOperationError, "unable to execute PostgreSQL statement <"
@@ -157,6 +175,115 @@ PgSqlConnection::getVersion(const ParameterMap& parameters) {
     return (make_pair(version, minor));
 }
 
+void
+PgSqlConnection::ensureSchemaVersion(const ParameterMap& parameters,
+                                     const DbCallback& cb,
+                                     const string& timer_name) {
+    // retry-on-startup?
+    bool const retry(parameters.count("retry-on-startup") &&
+                     parameters.at("retry-on-startup") == "true");
+
+    IOServiceAccessorPtr ac(new IOServiceAccessor(&DatabaseConnection::getIOService));
+    pair<uint32_t, uint32_t> schema_version;
+    try {
+        schema_version = getVersion(parameters, ac, cb, retry ? timer_name : string());
+    } catch (DbOpenError const& exception) {
+        throw;
+    } catch (DbOpenErrorWithRetry const& exception) {
+        throw;
+    } catch (exception const& exception) {
+        // This failure may occur for a variety of reasons. We are looking at
+        // initializing schema as the only potential mitigation. We could narrow
+        // down on the error that would suggest an uninitialized schema
+        // which would sound something along the lines of
+        // "table schema_version does not exist", but we do not necessarily have
+        // to. If the error had another cause, it will fail again during
+        // initialization or during the subsequent version retrieval and that is
+        // fine, and the error should still be relevant.
+        initializeSchema(parameters);
+
+        // Retrieve again because the initial retrieval failed.
+        schema_version = getVersion(parameters, ac, cb, retry ? timer_name : string());
+    }
+
+    // Check that the versions match.
+    pair<uint32_t, uint32_t> const expected_version(PGSQL_SCHEMA_VERSION_MAJOR,
+                                                    PGSQL_SCHEMA_VERSION_MINOR);
+    if (schema_version != expected_version) {
+        isc_throw(DbOpenError, "PostgreSQL schema version mismatch: expected version: "
+                                   << expected_version.first << "." << expected_version.second
+                                   << ", found version: " << schema_version.first << "."
+                                   << schema_version.second);
+    }
+}
+
+void
+PgSqlConnection::initializeSchema(const ParameterMap& parameters) {
+    if (parameters.count("readonly") && parameters.at("readonly") == "true") {
+        // The readonly flag is historically used for host backends. Still, if
+        // enabled, it is a strong indication that we should not meDDLe with it.
+        return;
+    }
+
+    if (!isc::util::file::isFile(KEA_ADMIN_)) {
+        // It can happen for kea-admin to not exist, especially with
+        // packages that install it in a separate package.
+        return;
+    }
+
+    // Convert parameters.
+    auto const tupl(toKeaAdminParameters(parameters));
+    vector<string> kea_admin_parameters(get<0>(tupl));
+    ProcessEnvVars const vars(get<1>(tupl));
+    kea_admin_parameters.insert(kea_admin_parameters.begin(), "db-init");
+
+    // Run.
+    ProcessSpawn kea_admin(ProcessSpawn::SYNC, KEA_ADMIN_, kea_admin_parameters, vars,
+                           /* inherit_env = */ true);
+    DB_LOG_INFO(PGSQL_INITIALIZE_SCHEMA).arg(kea_admin.getCommandLine());
+    pid_t const pid(kea_admin.spawn());
+    if (kea_admin.isRunning(pid)) {
+        isc_throw(SchemaInitializationFailed, "kea-admin still running");
+    }
+    int const exit_code(kea_admin.getExitStatus(pid));
+    if (exit_code != 0) {
+        isc_throw(SchemaInitializationFailed, "Expected exit code 0 for kea-admin. Got " << exit_code);
+    }
+}
+
+tuple<vector<string>, vector<string>>
+PgSqlConnection::toKeaAdminParameters(ParameterMap const& params) {
+    vector<string> result{"pgsql"};
+    ProcessEnvVars vars;
+    for (auto const& p : params) {
+        string const& keyword(p.first);
+        string const& value(p.second);
+
+        // These Kea parameters are the same as the kea-admin parameters.
+        if (keyword == "user" ||
+            keyword == "password" ||
+            keyword == "host" ||
+            keyword == "port" ||
+            keyword == "name") {
+            result.push_back("--" + keyword);
+            result.push_back(value);
+            continue;
+        }
+
+        // These Kea parameters do not have a direct kea-admin equivalent.
+        // But they do have a psql client environment variable equivalent.
+        // We pass them to kea-admin.
+        static unordered_map<string, string> conversions{
+            {"connect-timeout", "PGCONNECT_TIMEOUT"},
+            // {"tcp-user-timeout", "N/A"},
+        };
+        if (conversions.count(keyword)) {
+            vars.push_back(conversions.at(keyword) + "=" + value);
+        }
+    }
+    return make_tuple(result, vars);
+}
+
 void
 PgSqlConnection::prepareStatement(const PgSqlTaggedStatement& statement) {
     // Prepare all statements queries with all known fields datatype
@@ -294,6 +421,21 @@ PgSqlConnection::openDatabaseInternal(bool logging) {
         // to release it, but grab the error message first.
         std::string error_message = PQerrorMessage(new_conn);
         PQfinish(new_conn);
+
+        auto const& rec = reconnectCtl();
+        if (rec && DatabaseConnection::retry_) {
+            // Start the connection recovery.
+            startRecoverDbConnection();
+
+            std::ostringstream s;
+
+            s << " (scheduling retry " << rec->retryIndex() + 1 << " of " << rec->maxRetries() << " in " << rec->retryInterval() << " milliseconds)";
+
+            error_message += s.str();
+
+            isc_throw(DbOpenErrorWithRetry, error_message);
+        }
+
         isc_throw(DbOpenError, error_message);
     }
 
