@@ -1,4 +1,4 @@
-// Copyright (C) 2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2023-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -28,19 +28,9 @@ namespace {
 /// @brief IP address to which service is bound.
 const std::string SERVER_ADDRESS = "127.0.0.1";
 
-/// @brief IPv6 address to whch service is bound.
-const std::string IPV6_SERVER_ADDRESS = "::1";
-
 /// @brief Port number to which service is bound.
 const unsigned short SERVER_PORT = 18123;
 
-/// @brief Request Timeout used in most of the tests (ms).
-const long REQUEST_TIMEOUT = 10000;
-
-/// @brief Connection idle timeout used in tests where idle connections
-/// are tested (ms).
-const long SHORT_REQUEST_TIMEOUT = 200;
-
 /// @brief Connection idle timeout used in most of the tests (ms).
 const long IDLE_TIMEOUT = 10000;
 
@@ -59,7 +49,7 @@ public:
     ///
     /// Starts test timer which detects timeouts.
     TlsListenerTest()
-        : io_service_(), test_timer_(io_service_),
+        : io_service_(new IOService()), test_timer_(io_service_),
           run_io_service_timer_(io_service_),
           clients_(), clients_done_(0) {
         test_timer_.setup(std::bind(&TlsListenerTest::timeoutHandler, this, true),
@@ -71,9 +61,12 @@ public:
     ///
     /// Removes active clients.
     virtual ~TlsListenerTest() {
-        for (auto client : clients_) {
+        for (auto const& client : clients_) {
             client->close();
         }
+
+        test_timer_.cancel();
+        io_service_->stopAndPoll();
     }
 
     /// @brief Fetch the server TLS context.
@@ -139,7 +132,7 @@ public:
         if (fail_on_timeout) {
             ADD_FAILURE() << "Timeout occurred while running the test!";
         }
-        io_service_.stop();
+        io_service_->stop();
     }
 
     /// @brief Callback function each client invokes when done.
@@ -151,7 +144,7 @@ public:
         ++clients_done_;
         if (clients_done_ >= clients_.size()) {
             // They're all done or dead. Stop the service.
-            io_service_.stop();
+            io_service_->stop();
         }
     }
 
@@ -160,7 +153,8 @@ public:
     /// @param timeout Optional value specifying for how long the io service
     /// should be ran.
     void runIOService(long timeout = 0) {
-        io_service_.get_io_service().reset();
+        io_service_->stop();
+        io_service_->restart();
 
         if (timeout > 0) {
             run_io_service_timer_.setup(std::bind(&TlsListenerTest::timeoutHandler,
@@ -168,9 +162,8 @@ public:
                                         timeout,
                                         IntervalTimer::ONE_SHOT);
         }
-        io_service_.run();
-        io_service_.get_io_service().reset();
-        io_service_.poll();
+        io_service_->run();
+        io_service_->stopAndPoll(false);
     }
 
     /// @brief Filter that denies every other connection.
@@ -192,7 +185,7 @@ public:
     }
 
     /// @brief IO service used in the tests.
-    IOService io_service_;
+    IOServicePtr io_service_;
 
     /// @brief Asynchronous timer service to detect timeouts.
     IntervalTimer test_timer_;
@@ -244,7 +237,7 @@ TEST_F(TlsListenerTest, listen) {
     ASSERT_EQ(expected_entries, listener.audit_trail_->getConnectionTrail(1));
 
     listener.stop();
-    io_service_.poll();
+    io_service_->poll();
 }
 
 // This test verifies that a TLS connection is denied to a client
@@ -305,7 +298,7 @@ TEST_F(TlsListenerTest, splitReads) {
     EXPECT_FALSE(client->expectedEof());
 
     listener.stop();
-    io_service_.poll();
+    io_service_->poll();
 }
 
 // This test verifies that a TLS connection can be established and used to
@@ -334,7 +327,7 @@ TEST_F(TlsListenerTest, idleTimeoutTest) {
     EXPECT_TRUE(client->expectedEof());
 
     listener.stop();
-    io_service_.poll();
+    io_service_->poll();
 }
 
 // This test verifies that TLS connections with multiple clients.
@@ -359,7 +352,7 @@ TEST_F(TlsListenerTest, multipleClientsListen) {
     ASSERT_EQ(num_clients, clients_.size());
 
     size_t connection_id = 1;
-    for (auto client : clients_) {
+    for (auto const& client : clients_) {
         EXPECT_TRUE(client->receiveDone());
         EXPECT_FALSE(client->expectedEof());
         // Create the list of expected entries.
@@ -375,7 +368,7 @@ TEST_F(TlsListenerTest, multipleClientsListen) {
     }
 
     listener.stop();
-    io_service_.poll();
+    io_service_->poll();
 }
 
 // Verify that the listener handles multiple requests for multiple
@@ -403,7 +396,7 @@ TEST_F(TlsListenerTest, multipleRequetsPerClients) {
     std::list<std::string>expected_responses{ "echo one", "echo two",
                                               "echo three", "good bye"};
     size_t connection_id = 1;
-    for (auto client : clients_) {
+    for (auto const& client : clients_) {
         EXPECT_TRUE(client->receiveDone());
         EXPECT_FALSE(client->expectedEof());
         EXPECT_EQ(expected_responses, client->getResponses());
@@ -428,7 +421,7 @@ TEST_F(TlsListenerTest, multipleRequetsPerClients) {
     }
 
     listener.stop();
-    io_service_.poll();
+    io_service_->poll();
 }
 
 // Verify that connection filtering can eliminate specific connections.
@@ -458,7 +451,7 @@ TEST_F(TlsListenerTest, filterClientsTest) {
     ASSERT_EQ(num_clients, clients_.size());
 
     size_t i = 0;
-    for (auto client : clients_) {
+    for (auto const& client : clients_) {
         if (i % 2 == 0) {
             // These clients should have been accepted and received responses.
             EXPECT_TRUE(client->receiveDone());
@@ -492,7 +485,7 @@ TEST_F(TlsListenerTest, filterClientsTest) {
     }
 
     listener.stop();
-    io_service_.poll();
+    io_service_->poll();
 }
 
 }
