@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -47,19 +47,7 @@ AdaptorSubnet::updateRelay(ElementPtr subnet) {
         return;
     }
     ConstElementPtr addresses = relay->get("ip-addresses");
-    if (!addresses) {
-        ConstElementPtr address = relay->get("ip-address");
-        if (!address) {
-            subnet->remove("relay");
-            return;
-        }
-        ElementPtr addr = Element::create(address->stringValue());
-        ElementPtr addrs = Element::createList();
-        addrs->add(addr);
-        ElementPtr updated = Element::createMap();
-        updated->set("ip-addresses", addrs);
-        subnet->set("relay", updated);
-    } else if (addresses->size() == 0) {
+    if (!addresses || addresses->size() == 0) {
         subnet->remove("relay");
     }
 }
