@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2021 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2022 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,7 +8,6 @@
 #define ISC_TRANSLATOR_PD_POOL_H 1
 
 #include <yang/translator_option_data.h>
-#include <list>
 
 namespace isc {
 namespace yang {
@@ -116,23 +115,34 @@ namespace yang {
 /// - ietf-dhcpv6-server (partial support)
 class TranslatorPdPool : virtual public TranslatorOptionDataList {
 public:
-
     /// @brief Constructor.
     ///
     /// @param session Sysrepo session.
     /// @param model Model name.
-    TranslatorPdPool(sysrepo::S_Session session, const std::string& model);
+    TranslatorPdPool(sysrepo::Session session, const std::string& model);
 
     /// @brief Destructor.
-    virtual ~TranslatorPdPool();
+    virtual ~TranslatorPdPool() = default;
 
-    /// @brief Get and translate a pd-pool from YANG to JSON.
+    /// @brief Translate a pd-pool from YANG to JSON.
+    ///
+    /// @param data_node the YANG node representing the PD pool
+    ///
+    /// @return the JSON representation of the PD pool
+    ///
+    /// @throw BadValue on pd-pool without well formed prefix.
+    /// @throw NetconfError when sysrepo raises an error.
+    isc::data::ElementPtr getPdPool(libyang::DataNode const& data_node);
+
+    /// @brief Translate a pd-pool from YANG to JSON.
     ///
     /// @param xpath The xpath of the pd-pool.
+    ///
     /// @return JSON representation of the pd-pool.
-    /// @throw SysrepoError when sysrepo raises an error.
+    ///
     /// @throw BadValue on pd-pool without well formed prefix.
-    isc::data::ElementPtr getPdPool(const std::string& xpath);
+    /// @throw NetconfError when sysrepo raises an error.
+    isc::data::ElementPtr getPdPoolFromAbsoluteXpath(std::string const& xpath);
 
     /// @brief Translate and set pd-pool from JSON to YANG.
     ///
@@ -143,10 +153,12 @@ public:
 protected:
     /// @brief getPdPool for ietf-dhcpv6-server.
     ///
-    /// @param xpath The xpath of the pd-pool.
+    /// @param data_node the YANG node representing the PD pool
+    ///
     /// @return JSON representation of the pd-pool.
-    /// @throw SysrepoError when sysrepo raises an error.
-    isc::data::ElementPtr getPdPoolIetf6(const std::string& xpath);
+    ///
+    /// @throw NetconfError when sysrepo raises an error.
+    isc::data::ElementPtr getPdPoolIetf6(libyang::DataNode const& data_node);
 
     /// @brief setPdPool for ietf-dhcpv6-server.
     ///
@@ -158,10 +170,12 @@ protected:
 
     /// @brief getPdPool for kea-dhcp6-server.
     ///
-    /// @param xpath The xpath of the pd-pool.
+    /// @param data_node the YANG node representing the PD pool
+    ///
     /// @return JSON representation of the pd-pool.
-    /// @throw SysrepoError when sysrepo raises an error.
-    isc::data::ElementPtr getPdPoolKea(const std::string& xpath);
+    ///
+    /// @throw NetconfError when sysrepo raises an error.
+    isc::data::ElementPtr getPdPoolKea(libyang::DataNode const& data_node);
 
     /// @brief setPdPool for kea-dhcp6-server.
     ///
@@ -169,7 +183,7 @@ protected:
     /// @param elem The JSON element.
     void setPdPoolKea(const std::string& xpath,
                       isc::data::ConstElementPtr elem);
-};
+};  // TranslatorPdPool
 
 /// @brief A translator class for converting a pd-pool list between
 /// YANG and JSON.
@@ -179,21 +193,32 @@ protected:
 /// - ietf-dhcpv6-server (partial support)
 class TranslatorPdPools : virtual public TranslatorPdPool {
 public:
-
     /// @brief Constructor.
     ///
     /// @param session Sysrepo session.
     /// @param model Model name.
-    TranslatorPdPools(sysrepo::S_Session session, const std::string& model);
+    TranslatorPdPools(sysrepo::Session session, const std::string& model);
 
     /// @brief Destructor.
-    virtual ~TranslatorPdPools();
+    virtual ~TranslatorPdPools() = default;
 
-    /// @brief Get and translate pd-pools from YANG to JSON.
+    /// @brief Translate pd-pools from YANG to JSON.
+    ///
+    /// @param data_node the YANG node representing the list of PD pools
+    ///
+    /// @return the JSON representation of the list of PD pools
+    ///
+    /// @throw NetconfError when sysrepo raises an error.
+    isc::data::ElementPtr getPdPools(libyang::DataNode const& data_node);
+
+    /// @brief Translate pd-pools from YANG to JSON.
     ///
     /// @param xpath The xpath of the pd-pool list.
-    /// @throw SysrepoError when sysrepo raises an error.
-    isc::data::ElementPtr getPdPools(const std::string& xpath);
+    ///
+    /// @return the JSON representation of the list of PD pools
+    ///
+    /// @throw NetconfError when sysrepo raises an error.
+    isc::data::ElementPtr getPdPoolsFromAbsoluteXpath(std::string const& xpath);
 
     /// @brief Translate and set pd-pools from JSON to YANG.
     ///
@@ -204,9 +229,10 @@ public:
 protected:
     /// @brief getPdPools common part.
     ///
-    /// @param xpath The xpath of the pd-pool list.
-    /// @throw SysrepoError when sysrepo raises an error.
-    isc::data::ElementPtr getPdPoolsCommon(const std::string& xpath);
+    /// @param data_node the YANG node representing the list of PD pools
+    ///
+    /// @throw NetconfError when sysrepo raises an error.
+    isc::data::ElementPtr getPdPoolsCommon(libyang::DataNode const& data_node);
 
     /// @brief setPdPools using pool-id.
     ///
@@ -222,9 +248,9 @@ protected:
     /// @throw BadValue on pd-pool without prefix or prefix length.
     void setPdPoolsPrefix(const std::string& xpath,
                           isc::data::ConstElementPtr elem);
-};
+};  // TranslatorPdPools
 
 }  // namespace yang
 }  // namespace isc
 
-#endif // ISC_TRANSLATOR_PD_POOL_H
+#endif  // ISC_TRANSLATOR_PD_POOL_H
