@@ -1,4 +1,4 @@
-// Copyright (C) 2018-2023 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2018-2024 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -114,6 +114,7 @@ TranslatorSubnet::getSubnetKea(DataNode const& data_node) {
     checkAndGetLeaf(result, data_node, "ddns-ttl-percent");
     checkAndGetLeaf(result, data_node, "ddns-update-on-renew");
     checkAndGetLeaf(result, data_node, "ddns-use-conflict-resolution");
+    checkAndGetLeaf(result, data_node, "ddns-conflict-resolution-mode");
     checkAndGetLeaf(result, data_node, "hostname-char-replacement");
     checkAndGetLeaf(result, data_node, "hostname-char-set");
     checkAndGetLeaf(result, data_node, "interface");
@@ -122,7 +123,6 @@ TranslatorSubnet::getSubnetKea(DataNode const& data_node) {
     checkAndGetLeaf(result, data_node, "rebind-timer");
     checkAndGetLeaf(result, data_node, "renew-timer");
     checkAndGetLeaf(result, data_node, "require-client-classes");
-    checkAndGetLeaf(result, data_node, "reservation-mode");
     checkAndGetLeaf(result, data_node, "reservations-global");
     checkAndGetLeaf(result, data_node, "reservations-in-subnet");
     checkAndGetLeaf(result, data_node, "reservations-out-of-pool");
@@ -204,7 +204,11 @@ TranslatorSubnet::setSubnet(string const& xpath, ConstElementPtr elem) {
 
 void
 TranslatorSubnet::setSubnetIetf6(string const& xpath, ConstElementPtr elem) {
-    /// Skip key "id".
+    // Set the list element. This is important in case we have no other elements except the key.
+    setItem(xpath, ElementPtr(), LeafBaseType::Unknown);
+
+    // Skip key "id" since it was set with the list element in the call above
+    // with the LeafBaseType::Unknown parameter.
 
     setMandatoryDivergingLeaf(elem, xpath, "subnet", "network-prefix", LeafBaseType::String);
 
@@ -236,7 +240,11 @@ TranslatorSubnet::setSubnetIetf6(string const& xpath, ConstElementPtr elem) {
 
 void
 TranslatorSubnet::setSubnetKea(string const& xpath, ConstElementPtr elem) {
-    /// Skip key "id".
+    // Set the list element. This is important in case we have no other elements except the key.
+    setItem(xpath, ElementPtr(), LeafBaseType::Unknown);
+
+    // Skip key "id" since it was set with the list element in the call above
+    // with the LeafBaseType::Unknown parameter.
 
     ConstElementPtr subnet = elem->get("subnet");
     if (!subnet) {
@@ -258,6 +266,7 @@ TranslatorSubnet::setSubnetKea(string const& xpath, ConstElementPtr elem) {
     checkAndSetLeaf(elem, xpath, "ddns-ttl-percent", LeafBaseType::Dec64);
     checkAndSetLeaf(elem, xpath, "ddns-update-on-renew", LeafBaseType::Bool);
     checkAndSetLeaf(elem, xpath, "ddns-use-conflict-resolution", LeafBaseType::Bool);
+    checkAndSetLeaf(elem, xpath, "ddns-conflict-resolution-mode", LeafBaseType::Enum);
     checkAndSetLeaf(elem, xpath, "hostname-char-replacement", LeafBaseType::String);
     checkAndSetLeaf(elem, xpath, "hostname-char-set", LeafBaseType::String);
     checkAndSetLeaf(elem, xpath, "interface", LeafBaseType::String);
@@ -265,7 +274,6 @@ TranslatorSubnet::setSubnetKea(string const& xpath, ConstElementPtr elem) {
     checkAndSetLeaf(elem, xpath, "min-valid-lifetime", LeafBaseType::Uint32);
     checkAndSetLeaf(elem, xpath, "rebind-timer", LeafBaseType::Uint32);
     checkAndSetLeaf(elem, xpath, "renew-timer", LeafBaseType::Uint32);
-    checkAndSetLeaf(elem, xpath, "reservation-mode", LeafBaseType::Enum);
     checkAndSetLeaf(elem, xpath, "reservations-global", LeafBaseType::Bool);
     checkAndSetLeaf(elem, xpath, "reservations-in-subnet", LeafBaseType::Bool);
     checkAndSetLeaf(elem, xpath, "reservations-out-of-pool", LeafBaseType::Bool);
@@ -286,11 +294,8 @@ TranslatorSubnet::setSubnetKea(string const& xpath, ConstElementPtr elem) {
     }
     ConstElementPtr relay = elem->get("relay");
     if (relay) {
-        ConstElementPtr address = relay->get("ip-address");
         ConstElementPtr addresses = relay->get("ip-addresses");
-        if (address) {
-            setItem(xpath + "/relay/ip-addresses", address, LeafBaseType::String);
-        } else if (addresses && !addresses->empty()) {
+        if (addresses && !addresses->empty()) {
             for (ElementPtr const& addr : addresses->listValue()) {
                 setItem(xpath + "/relay/ip-addresses", addr, LeafBaseType::String);
             }
